Lua_CenterList = class("Lua_CenterList", Lua_DisplayObject)
function Lua_CenterList:ctor(p, width, height, showMode)
	self.rectWidth = width;
	self.rectHeight = height;
	self.rectMinX = -width/2;
	self.rectMaxX = width/2;
	self.rectMinY = -height/2;
	self.rectMaxY = height/2;

	self.cellMinX = 0;
	self.cellMaxX = 0;
	self.cellMinY = 0;
	self.cellMaxY = 0;

	self.cellSpacingX = 0;
	self.cellSpacingY = 0;

    self.type = 1;
	self.showModel = showMode;

	self.startMoveTime = 0;
	self.moveTime = 0.2;
	self.isMoveing = false;
	self.curMoveDistance = 0;--当前已移动的距离
	self.moveDistance = 0;--需要移动的距离

	self.startScale = 1;
	self.changeScale = 1;
	self.curIndex = 1;

	self.dataArr = Array();
	self.showArr = Array();
	self.hideArr = Array();
	self.showDataArr = Array();
	
	self:superCall("ctor", p);
end

function Lua_CenterList:dispose()
	if self.dataArr ~= nil then
		self.dataArr:dispose();
		self.dataArr:clear();
		self.dataArr = nil;
	end
	if self.showArr ~= nil then
		self.showArr:dispose();
		self.showArr:clear();
		self.showArr = nil;
	end
	if self.hideArr ~= nil then
		self.hideArr:dispose();
		self.hideArr:clear();
		self.hideArr = nil;
	end
	if self.showDataArr ~= nil then
		self.showDataArr:dispose();
		self.showDataArr:clear();
		self.showDataArr = nil;
	end
	self.pressItem = nil;
	self.owner = nil;
	self.callBack = nil;
	self:stopTouchTimer();
	self:stopMoveTimer();
	self:superCall("dispose");
end

function Lua_CenterList:buildShowItem()
	self:superCall("buildShowItem");
	self:setRectSize(self.rectWidth, self.rectHeight)
	self.maskContainer = UICreate.createUIMaskContainer(self.content, nil, nil, 0, 0, 0, 0);
	self.maskContainer:setAnchorMin(0, 0);
	self.maskContainer:setAnchorMax(1, 1);
	self.maskContainer:setSubRectPivot(0.5, 0.5);
	self.maskContainer:setColor(0, 0, 0, 1);
	self.maskContainer.clickScale = false;
	self.maskContainer:addMousDownFuntion(self, self.onMouseDownFun);
	self.maskContainer:addMouseUpFuntion(self, self.onMouseUpFun);
	self.maskContainer:addMask();
end

function Lua_CenterList:setCellList(cell, width, height, pivot)
	self.cell = cell;
	self.cellWidth = width;
	self.cellHeight = height;
	pivot = pivot or 4;
	self.cellPivotX, self.cellPivotY = self:getPivotXY(pivot);
	self.blockSizeX = width + self.cellSpacingX;
	self.blockSizeY = height + self.cellSpacingY;
end

function Lua_CenterList:getPivotXY(pivot)
	if value == 0 then
		return 0, 1;
	elseif value == 1 then
		return 0.5, 1;
	elseif value == 2 then
		return 1, 1;
	elseif value == 3 then
		return 0, 0.5;
	elseif value == 4 then
		return 0.5, 0.5;
	elseif value == 5 then
		return 1, 0.5;
	elseif value == 6 then
		return 0, 0;
	elseif value == 7 then
		return 0.5, 0;
	elseif value == 8 then
		return 1, 0;
	end
	return 0.5, 0.5
end

function Lua_CenterList:setCellSpacing(cellSpacingX, cellSpacingY)
	self.cellSpacingX = cellSpacingX;
	self.cellSpacingY = cellSpacingY;
	self.blockSizeX = self.cellWidth + self.cellSpacingX;
	self.blockSizeY = self.cellHeight + self.cellSpacingY;
end

function Lua_CenterList:setShowCount(num)
	self.cellSpacingX = (self.rectWidth - self.cellWidth * num) / (num - 1);
	self.cellSpacingY = (self.rectHeight - self.cellHeight * num) / (num - 1);
	self.blockSizeX = self.cellWidth + self.cellSpacingX;
	self.blockSizeY = self.cellHeight + self.cellSpacingY;
end

--pos位置偏移,rot旋转偏移,scale非中间item的缩放值,startScale中间item的缩放值
function Lua_CenterList:setTransformData(pos, rot, scale, startScale)
	self.changeScale = scale;
	self.changeRot = rot;
	self.changePos = pos;
	self.startScale = startScale;
end

function Lua_CenterList:setCallBack(owner, callBack)
	self.owner = owner;
	self.callBack = callBack;
end

function Lua_CenterList:setArrayData(data, bUpdatePos)
	if data == nil then
		return;
	end
	if data.typeof and data:typeof() == "Array" then
		self.dataLen = data:getSize();
	else
		self.dataLen = #data;
	end
	local _data
	local num = self.dataArr:getSize() - self.dataLen
	while num > 0 do
		_data = self.dataArr:remove()
		_data:dispose()
		num = num - 1
	end
	while num < 0 do
		_data = Lua_CenterData.new(self)
		self.dataArr:add(_data)
		num = num + 1
	end
	for i = 1, self.dataLen do
		_data = self.dataArr[i]
		_data:setData(i, data[i]);
	end
    bUpdatePos = bUpdatePos == nil and true or false;
    if bUpdatePos then
		self:updateChildPos()
	end
end

function Lua_CenterList:updateChildPos()
	if self.showModel == Lua_AxisDefine.Horizontal then
		if self.type == 1 then
			self.startPos = self.rectMinX + self.cellPivotX * self.cellWidth;
		else
			self.startPos = 0;
		end
		local posX = 0;
		local posY = 0;
		local _dataArr = self.dataArr;
		for i = 1, self.dataLen do
			posX = self.startPos + (i - 1) * self.blockSizeX;
			_dataArr[i]:setLocation(posX, posY);
		end
		self.cellMaxX = self.startPos + (self.dataLen - 1) * self.blockSizeX;
		self.cellMinX = -self.cellMaxX;
	else
		if self.type == 1 then
			self.startPos = self.rectMaxY - (1 - self.cellPivotY) * self.cellHeight;
		else
			self.startPos = 0;
		end
		local posX = 0;
		local posY = 0;
		local _dataArr = self.dataArr;
		for i = 1, self.dataLen do
			posX = self.startPos - (i - 1) * self.blockSizeY;
			_dataArr[i]:setLocation(posX, posY);
		end
		self.cellMinY = self.startPos - (self.dataLen - 1) * self.blockSizeY;
		self.cellMaxY = -self.cellMinY;
	end
	self:updateDragItem(0);
end

function Lua_CenterList:updateDragItem(val)
	local data;
	local posX = 0;
	local posY = 0;
	local rot = 0;
	local scale = 1;
	local siblingIndex = 0;
	
	if self.showModel == Lua_AxisDefine.Horizontal then
		local len = self.dataArr:getSize();
		if self.dataArr[len].posX + val > self.cellMaxX then
			val = self.cellMaxX - self.dataArr[len].posX;
		elseif self.dataArr[1].posX + val < self.cellMinX then
			val = self.cellMinX - self.dataArr[1].posX;
		end
		for i = 1, self.dataLen do
			data = self.dataArr[i];
			posX = data.posX + val;
            local total = (self.rectWidth - self.cellWidth) / 2;
            local rate = Mathf.Abs(posX) / total;
			if posX < 0 then
				if self.changePos ~= nil then
					posY = self.changePos * rate;
				end
				if self.changeRot ~= nil then
					rot = self.changeRot * rate;
				end
			elseif posX > 0 then
				if self.changePos ~= nil then
					posY = self.changePos * rate;
				end
				if self.changeRot ~= nil then
					rot = -self.changeRot * rate;
				end
			else
				rot = 0;
				posY = 0;
			end
			if posX > -self.cellWidth * (1 - self.cellPivotX) and posX < self.cellWidth * self.cellPivotX then
				self.curIndex = data.index;
				scale = self.startScale or 1
			else
				scale = self.changeScale;
			end

			if self:checkItemInBound(posX, posY) then
				data:setLocation(posX, posY);
				data:createItemFromPool();
				data:setSiblingIndex(siblingIndex);
				data:setRot(0, 0, rot);
				data:setScale(scale, scale, 1);
				siblingIndex = siblingIndex + 1;
				if self.showDataArr:indexOf(data) == -1 then
					self.showDataArr:add(data);
				end
			else
				data:setLocation(posX, posY);
				data:setRot(0, 0, rot);
				data:setScale(scale, scale, 1);
				data:returnItemToPool();
				if self.showDataArr:indexOf(data) ~= -1 then
					self.showDataArr:removeItem(data);
				end
			end
		end
	else
		for i = 1, self.dataLen do
			data = self.dataArr[i];
			posY = data.posY + val;
            local total = (self.rectHeight - self.cellHeight) / 2;
            local rate = Mathf.Abs(posY) / total;
			if posY < 0 then
				if self.changePos ~= nil then
					posX = self.changePos * rate;
				end
				if self.changePos ~= nil then
					rot = -self.changeRot * rate;
				end
			elseif posY > 0 then
				if self.changePos ~= nil then
					posX = self.changePos * rate;
				end
				if self.changePos ~= nil then
					rot = self.changeRot * rate;
				end
			end
			if posY > -self.cellHeight * (1 - self.cellPivotY) and posY < self.cellHeight * self.cellPivotY then
				self.curIndex = data.index;
				scale = self.startScale or 1
			else
				scale = self.changeScale;
			end

			if self:checkItemInBound(posX, posY) then
				data:setLocation(posX, posY);
				data:createItemFromPool();
				data:setSiblingIndex(siblingIndex);
				data:setRot(0, 0, rot)
				data:setScale(scale, scale, 1);
				siblingIndex = siblingIndex + 1;
				if self.showDataArr:indexOf(data) == -1 then
					self.showDataArr:add(data);
				end
			else
				data:setLocation(posX, posY);
				data:setRot(0, 0, rot)
				data:setScale(scale, scale, 1);
				data:returnItemToPool();
				if self.showDataArr:indexOf(data) ~= -1 then
					self.showDataArr:removeItem(data);
				end
			end
		end
	end
end

function Lua_CenterList:checkClick()
	self.maskContainer:refreshTouchPosInverseSubTf()
	local posX = self.maskContainer.touchSubRectPx
	local posY = self.maskContainer.touchSubRectPy
	local len = self.showArr:getSize()
	for i = 1, len do
		if self.showArr[i]:checkInBound(posX, posY) then
			return self.showArr[i];
		end
	end
	return nil;
end

function Lua_CenterList:onMouseDownFun()
	self.clickMouseTime = getTime();
	self.clickMousePos = self.mousePosition;
	self.curMousePos = self.clickMousePos;
	self:stopMoveTimer();
	self:stopTouchTimer()
	self:startTouchTimer();
	self.pressItem = self:checkClick();
	if self.pressItem ~= nil then
		self.pressItem:onMouseDownHandle();
	end
end

function Lua_CenterList:onMouseUpFun()
	self:stopTouchTimer();
	if self.pressItem ~= nil then
		self.pressItem:onMouseUpHandle();
	end
	local dis = self.mousePosition - self.clickMousePos;
	if Mathf.Abs(dis) <= 2 then
		if self.pressItem ~= nil then
			if self.pressItem.cellIndex ~= self.curIndex then
				self:moveToIndex(self.pressItem.cellIndex);
			end
			self.pressItem:onPressClick();
		end
	else
		local len = self.showDataArr:getSize();
		local data;
		self.showDataArr:sort("siblingIndex", false);
		self.moveDistance = 0;
		if self.showModel == Lua_AxisDefine.Horizontal then
			if dis > 0 then
				data = self.showDataArr[len];
				if data.posX < self.rectMaxX - (1 - self.cellPivotX) * self.cellWidth then
					self.moveDistance = self.rectMaxX - (1 - self.cellPivotX) * self.cellWidth - data.posX;
				elseif data.posX > self.rectMaxX - (1 - self.cellPivotX) * self.cellWidth then
					self.moveDistance = self.rectMaxX - (1 - self.cellPivotX) * self.cellWidth + self.blockSizeX - data.posX;
				end
			elseif dis < 0 then
				data = self.showDataArr[1];
				if data.posX > self.rectMinX + self.cellPivotX * self.cellWidth then
					self.moveDistance = self.rectMinX + self.cellPivotX * self.cellWidth - data.posX;
				elseif data.posX < self.rectMinX + self.cellPivotX * self.cellWidth then
					self.moveDistance = self.rectMinX + self.cellPivotX * self.cellWidth - self.blockSizeX - data.posX;
				end
			end
			if self.dataArr[self.dataLen].posX + self.moveDistance > self.cellMaxX then
				self.moveDistance = self.cellMaxX - self.dataArr[self.dataLen].posX;
			elseif self.dataArr[1].posX + self.moveDistance < self.cellMinX then
				self.moveDistance = self.cellMinX - self.dataArr[1].posX;
			end
		else
			if dis > 0 then
				data = self.dataArr[1];
				if data.posY < self.rectMaxY - (1 - self.cellPivotY) * self.cellHeight then
					self.moveDistance = self.rectMaxY - (1 - self.cellPivotY) * self.cellHeight - data.posX;
				elseif data.posX > self.rectMaxX - (1 - self.cellPivotX) * self.cellWidth then
					self.moveDistance = self.rectMaxY - (1 - self.cellPivotY) * self.cellHeight + self.blockSizeY - data.posY;
				end
			elseif dis < 0 then
				data = self.dataArr[len];
				if data.posY > self.rectMinY + self.cellPivotY * self.cellHeight then
					self.moveDistance = self.rectMinY + self.cellPivotY * self.cellHeight - data.posY;
				elseif data.posY < self.rectMinY + self.cellPivotY * self.cellHeight then
					self.moveDistance = self.rectMinY + self.cellPivotY * self.cellHeight - self.blockSizeY - data.posY;
				end
			end
			if self.dataArr[self.dataLen].posY + self.moveDistance < self.cellMinY then
				self.moveDistance = self.cellMinY - self.dataArr[self.dataLen].posY;
			elseif self.dataArr[1].posY + self.moveDistance > self.cellMaxY then
				self.moveDistance = self.cellMaxY - self.dataArr[1].posY;
			end
		end
		if self.moveDistance ~= 0 then
			self:stopMoveTimer();
			self:startMoveTimer();
		else
			if self.callBack ~= nil and self.owner ~= nil then
				self.callBack(self.owner);
			end
		end
	end
end

function Lua_CenterList.Get:mousePosition()
	if self.showModel == Lua_AxisDefine.Horizontal then
		return CommonUtil.mousePositionX;
	else
		return CommonUtil.mousePositionY;
	end
end

function Lua_CenterList:startTouchTimer()
	if self.bTouchTimer ~= true then
		self.bTouchTimer = true;
		self:addTimer("OnTouchTimer", 0.01, self, self.onTouchTimer);
	end
end

function Lua_CenterList:stopTouchTimer()
	if self.bTouchTimer == true then
		self.bTouchTimer = false;
		Lua_CommonUtil.removeTimer(self, self.onTouchTimer);
	end
end

function Lua_CenterList:onTouchTimer()
	self:updateDragItem(self.mousePosition - self.curMousePos);
	self.curMousePos = self.mousePosition;
end

function Lua_CenterList:startMoveTimer()
	if self.bMoveTimer ~= true then
		self.bMoveTimer = true;
		self.isMoveing = true;
		self.startMoveTime = getTime();
		self.curMoveDistance = 0;
		self:addTimer("OnMoveTimer", 0.01, self, self.onMoveTimer);
	end
end

function Lua_CenterList:stopMoveTimer()
	if self.bMoveTimer == true then
		self.bMoveTimer = false;
		self.isMoveing = false;
		Lua_CommonUtil.removeTimer(self, self.onMoveTimer);

		if self.callBack ~= nil and self.owner ~= nil then
			self.callBack(self.owner);
		end
	end
end

function Lua_CenterList:onMoveTimer()
	local dt = getTime() - self.startMoveTime;
	if dt >= self.moveTime then
		self:stopMoveTimer();
	end
	local lastMoveDistance = self.curMoveDistance;
	self.curMoveDistance = Mathf.Lerp(self.curMoveDistance, self.moveDistance, dt / self.moveTime);
	self:updateDragItem(self.curMoveDistance - lastMoveDistance);
end

function Lua_CenterList:moveToIndex(index, immediately)
	if index > self.dataLen then
		index = self.dataLen;
	elseif index < 1 then
		index = 1;
	end
	if self.showModel == Lua_AxisDefine.Horizontal then
		local dis = self.startPos - self.dataArr[index].posX;
		if immediately then
			self:updateDragItem(dis);
		else
			self.moveDistance = dis;
			self:stopMoveTimer();
			self:startMoveTimer();
		end
	else
		local dis = self.startPos - self.dataArr[index].posY;
		if immediately then
			self:updateDragItem(dis);
		else
			self.moveDistance = dis;
			self:stopMoveTimer();
			self:startMoveTimer();
		end
	end
end

function Lua_CenterList:checkItemInBound(x, y)
	local minX = self.rectMinX - (1 - self.cellPivotX) * self.cellWidth * self.changeScale;
	local maxX = self.rectMaxX + self.cellPivotX * self.cellWidth * self.changeScale;
	local minY = self.rectMinY - (1 - self.cellPivotY) * self.cellHeight * self.changeScale;
	local maxY = self.rectMaxY + self.cellPivotY * self.cellHeight * self.changeScale;
	if x >= minX and x <= maxX and y >= minY and y <= maxY then
		return true;
	end
	return false;
end

function Lua_CenterList:createItem()
	local item;
	local len = self.hideArr:getSize()
	if len <= 0 then
		item = self.cell.new(self.maskContainer.sub);
		item:setRectSize(self.cellWidth, self.cellHeight);
	else
		item = self.hideArr:remove(len);
	end
	self.showArr:add(item);
	item:setEnable(true);
	return item;
end

function Lua_CenterList:returnItem(item)
	if self.showArr:indexOf(item) ~= -1 then
		self.showArr:removeItem(item);
		self.hideArr:add(item);
		item:setEnable(false);
	end
end












Lua_CenterData = class("Lua_CenterData")
function Lua_CenterData:ctor(ownerList)
	self.ownerList = ownerList;
	self.posX = 0;
	self.posY = 0;
	self.posZ = 0;
	self.rotX = 0;
	self.rotY = 0;
	self.rotZ = 0;
	self.scaleX = 1;
	self.scaleY = 1;
	self.scaleZ = 1;
end

function Lua_CenterData:setData(index, data)
	self.data = data;
	self.index = index;
	if self.item ~= nil then
		self.item:setData(data);
	end
end

function Lua_CenterData:setLocation(x, y, z)
	x = Mathf.Floor(x + 0.5);
	y = Mathf.Floor(y + 0.5);
	self.posX = x or 0;
	self.posY = y or 0;
	self.posZ = z or 0;
	if self.item ~= nil then
		self.item:setLocation(self.posX, self.posY, self.posZ);
	end
end

function Lua_CenterData:setRot(x, y, z)
	self.rotX = x;
	self.rotY = y;
	self.rotZ = z;
	if self.item ~= nil then
		self.item:setRot(x, y, z);
	end
end

function Lua_CenterData:setScale(x, y, z)
	self.scaleX = x;
	self.scaleY = y;
	self.scaleZ = z;
	if self.item ~= nil then
		self.item:setScale(x, y, z);
	end
end

function Lua_CenterData:setSiblingIndex(siblingIndex)
	self.siblingIndex = siblingIndex;
	self.item:setSiblingIndex(self.siblingIndex);
end

function Lua_CenterData:createItemFromPool()
	if self.item == nil then
		self.item = self.ownerList:createItem();
	end
	self.item:setLocation(self.posX, self.posY, self.posZ);
	self.item:setRot(self.rotX, self.rotY, self.rotZ);
	self.item:setScale(self.scaleX, self.scaleY, self.scaleZ);
	self.item:setCellIndex(self.index);
	self.item:setData(self.data);
end

function Lua_CenterData:returnItemToPool()
	if self.item ~= nil then
		self.ownerList:returnItem(self.item);
		self.item = nil;
	end
end

function Lua_CenterData:dispose()
	self.data = nil;
	self.ownerList = nil;
	if self.item ~= nil then
		self.item:dispose();
		self.item = nil;
	end
end













Lua_CenterCell = class("Lua_CenterCell", Lua_DisplayObject)
function Lua_CenterCell:ctor(p, skin, frameBuild)
	self:superCall("ctor", p, skin, frameBuild);
end

function Lua_CenterCell:setRot(x, y, z)
	self:superCall("setRot", x, y, z);
	self.rot = z;
end

function Lua_CenterCell:checkInBound(posX, posY)
	local minX = self.centerPosX - self.w / 2 * self.scaleX;
	local maxX = self.centerPosX + self.w / 2 * self.scaleX;
	local minY = self.centerPosY - self.h / 2 * self.scaleY;
	local maxY = self.centerPosY + self.h / 2 * self.scaleY;
	posX, posY = self:getPointRotXY(posX, posY);
	if posX >= minX and posX <= maxX and posY >= minY and posY <= maxY then
		return true;
	end
	return false
end

--获取一个点旋转后的x，y
function Lua_CenterCell:getPointRotXY(x, y)
	local rad = -self.rot * Mathf.Deg2Rad;
	local new_X = (x - self.centerPosX) * Mathf.Cos(rad) - (y - self.centerPosY) * Mathf.Sin(rad) + self.centerPosX;
	local new_Y = (y - self.centerPosY) * Mathf.Cos(rad) + (x - self.centerPosX) * Mathf.Sin(rad) + self.centerPosY;
	return new_X, new_Y;
end

function Lua_CenterCell:setData(data)
	
end

function Lua_CenterCell:setCellIndex(value)
	self.cellIndex = value
	self.content.name = self.classname..self.cellIndex
end

function Lua_CenterCell:onPressClick()

end

function Lua_CenterCell:onMouseDownHandle()

end

function Lua_CenterCell:onMouseUpHandle()

end