-- 循环列表
Lua_CirculateList = class("Lua_CirculateList", Lua_DisplayObject)
function Lua_CirculateList:ctor(p, width, height, showMode)
	self.rectWidth = width
	self.rectHeight = height
	self.showMode = showMode

	self.minX = -width / 2
	self.maxX = width / 2
	self.minY = -height / 2
	self.maxY = height / 2

	self.cellSpacingX = 0
	self.cellSpacingY = 0

	self.dataList = Array()
	self.showList = Array()
	self.hideList = Array()
	self.tempList = Array()

	self.showCount = 1 -- 显示个数
	self.offsetPos = 0 -- 偏移位置 大于0生效
	-- self.moveSpeed = 500
	self.needRegulate = false -- 是否校正
	self.regulateTime = 0.3

	self:superCall("ctor", p)
end

function Lua_CirculateList:buildShowItem()
	self:superCall("buildShowItem")

	self:setRectSize(self.rectWidth, self.rectHeight)

	self.maskContainer = UICreate.createUIMaskContainer(self.content, nil, nil, 0, 0, 0, 0)
	self.maskContainer:setAnchorMin(0, 0)
	self.maskContainer:setAnchorMax(1, 1)
	self.maskContainer:setSubRectPivot(0.5, 0.5)
	self.maskContainer:setColor(0, 0, 0, 1)
	self.maskContainer.clickScale = false
	self.maskContainer:addMousDownFuntion(self, self.onMouseDownFun)
	self.maskContainer:addMouseUpFuntion(self, self.onMouseUpFun)
	self.maskContainer:addMask()

	self.sub = self.maskContainer.sub
end

function Lua_CirculateList:dispose()
	self:stopTimer()
	self:stopRegulate()

	if self.dataList then
		self.dataList:dispose()
		self.dataList:clear()
		self.dataList = nil
	end

	if self.showList then
		self.showList:dispose()
		self.showList:clear()
		self.showList = nil
	end

	if self.hideList then
		self.hideList:dispose()
		self.hideList:clear()
		self.hideList = nil
	end

	if self.tempList then
		self.tempList:dispose()
		self.tempList:clear()
		self.tempList = nil
	end

	self.sub = nil
	if self.maskContainer then
		self.maskContainer:dispose()
		self.maskContainer = nil
	end
	self:superCall("dispose")
end

function Lua_CirculateList:setCell(class, width, height, pivot)
	self.cellClass = class
	self.cellWidth = width
	self.cellHeight = height
	self.cellPivot = pivot or 4
	self.cellPivotX, self.cellPivotY = self:getPivotXY(self.cellPivot)
	self.blockSizeX = width + self.cellSpacingX
	self.blockSizeY = height + self.cellSpacingY
end

function Lua_CirculateList:setCellSpacing(spacingX, spacingY)
	self.cellSpacingX = spacingX
	self.cellSpacingY = spacingY
	self.blockSizeX = self.cellWidth + spacingX
	self.blockSizeY = self.cellHeight + spacingY
end

function Lua_CirculateList:getPivotXY(value)
	if value == 0 then
		return 0, 1
	elseif value == 1 then
		return 0.5, 1
	elseif value == 2 then
		return 1, 1
	elseif value == 3 then
		return 0, 0.5
	elseif value == 4 then
		return 0.5, 0.5
	elseif value == 5 then
		return 1, 0.5
	elseif value == 6 then
		return 0, 0
	elseif value == 7 then
		return 0.5, 0
	elseif value == 8 then
		return 1, 0
	end
	return 0.5, 0.5
end

function Lua_CirculateList:setData(data)
	self.data = data
	self.dataLen = 0
	if data == nil then
		return
	end

	if data.typeof and data:typeof() == "Array" then
		self.dataLen = data:getSize()
	else
		self.dataLen = #data
	end
	if self.dataLen < self.showCount * 2 then
		self.cellCount = self.dataLen + (self.showCount - (self.dataLen % self.showCount))
	else
		self.cellCount = self.dataLen
	end

	local _data
	local num = self.dataList:getSize() - self.cellCount
	while num > 0 do
		_data = self.dataList:remove()
		_data:dispose()
		num = num - 1
	end
	while num < 0 do
		_data = Lua_CirculateData.new(self)
		self.dataList:add(_data)
		num = num + 1
	end

	self:updateChildPos()
end

function Lua_CirculateList:updateChildPos()
	local data
	local index = 0
	local posX = 0
	local posY = 0
	if self.showMode == Lua_AxisDefine.Horizontal then

	else
		local startY = self.rectHeight / 2 - (1 - self.cellPivotY) * self.cellHeight
		for i = 1, self.cellCount do
			data = self.dataList[i]
			index = index + 1
			if index > self.dataLen then
				index = 1
			end
			data:setData(i, self.data[index])
			
			posY = startY - (i - 1) * self.blockSizeY
			data:setLocation(posX, posY)
		end
	end

	self:updateDragItem(0)
end

function Lua_CirculateList:updateDragItem(value)
	local data
	local index = 0
	self.tempList:clear()
	if self.showMode == Lua_AxisDefine.Horizontal then

	else
		if value > 0 then
			while value > self.maxY do
				value = value - self.maxY
			end
		elseif value < 0 then
			while value < self.minY do
				value = value - self.minY
			end
		end

		for i = 1, self.cellCount do
			data = self.dataList[i]
			data:setLocation(data.posX, data.posY + value)
			if self:checkItemInBound(data.posX, data.posY) then
				data:setSiblingIndex(index)
				data:createItemFromPool()
				index = index + 1
				self.tempList:add(data)
			else
				data:returnItemToPool()
			end
		end

		if self.offsetPos ~= 0 then
			local len = self.tempList:getSize()
			local theta = 180 / len
			local angle = 0
			for i = 1, len do
				data = self.tempList[i]
				radian = (angle / 180) * Mathf.PI
				posX = Mathf.Cos(radian) * self.offsetPos
				data:setLocation(posX, data.posY)
				if i > len / 2 then
					angle = angle - theta
				else
					angle = angle + theta
				end
			end
		end

		if value > 0 then
			data = self.dataList[1]
			if not self:checkItemInBound(data.posX, data.posY) then
				local temp = self.dataList:getItem(self.cellCount)
				data:setLocation(temp.posX, temp.posY - self.blockSizeY)
				self.dataList:remove(1)
				self.dataList:add(data)
			end
		elseif value < 0 then
			data = self.dataList[self.cellCount]
			if not self:checkItemInBound(data.posX, data.posY) then
				local temp = self.dataList:getItem(1)
				data:setLocation(temp.posX, temp.posY + self.blockSizeY)
				self.dataList:remove(self.cellCount)
				self.dataList:insert(data, 1)
			end
		end
	end
end

function Lua_CirculateList:checkItemInBound(posX, posY)
	if self.showMode == Lua_AxisDefine.Horizontal then
		if posX > self.minX - self.blockSizeX and posX < self.maxX + self.blockSizeX then
			return true
		end
	else
		if posY > self.minY - self.blockSizeY and posY < self.maxY + self.blockSizeY then
			return true
		end
	end
	return false
end

function Lua_CirculateList:createItem()
	local item
	if self.hideList:getSize() > 0 then
		item = self.hideList:remove(1)
		item:setEnable(true)
	else
		item = self.cellClass.new(self.sub)
		item:setRectSize(self.cellWidth, self.cellHeight)
	end

	self.showList:add(item)
	return item
end

function Lua_CirculateList:returnItem(item)
	if item then
		local len = self.showList:getSize()
		for i = 1, len do
			if self.showList[i] == item then
				item:setEnable(false)
				self.showList:remove(i)
				self.hideList:add(item)
				break
			end
		end
	end
end

function Lua_CirculateList:moveToIndex(index, offsetIdx)
	offsetIdx = offsetIdx or 0
	index = index + offsetIdx
	index = index < 0 and index + self.cellCount or index
	index = index > self.cellCount and index - self.cellCount or index
	if self.showMode == Lua_AxisDefine.Horizontal then

	else
		for i = 1, index - 1 do
			self:updateDragItem(self.blockSizeY)
		end
	end
end

function Lua_CirculateList:onMouseDownFun(go)
	self:stopRegulate()
	self.startClickPos = self.mousePosition
	self.startClickTime = getTime()
	self.moveDis = 0
	self.curMousePos = self.startClickPos
	self:startTimer()
end

function Lua_CirculateList:onMouseUpFun(go)
	self:stopTimer()

	local dis = self.mousePosition - self.startClickPos
	if getTime() - self.startClickTime <= 0.3 and Mathf.Abs(dis) < 2 then
		self.maskContainer:refreshTouchPosInverseSubTf()
		local posX = self.maskContainer.touchSubRectPx
		local posY = self.maskContainer.touchSubRectPy
		local len = self.showList:getSize()
		for i = 1, len do
			if self.showList[i]:checkInBound(posX, posY) then
				self.showList[i]:onPressClick()
			end
		end
	else
		if self.needRegulate == true then
			self.showList:sort("siblingIndex", false)
			local posY
			local targetY
			if self.showMode == Lua_AxisDefine.Horizontal then

			else
				if dis > 0 then
					posY = self.showList[1].posY
					targetY = self.rectHeight / 2 - (1 - self.cellPivotY) * self.cellHeight + self.blockSizeY
					if posY > targetY then
						posY = self.showList[2].posY
					end
					for i = 1, self.cellCount do
						data = self.dataList[i]
						data.targetPosY = data.posY + (targetY - posY)
					end
				elseif dis < 0 then
					local len = self.showList:getSize()
					posY = self.showList[len].posY
					targetY = -(self.rectHeight / 2 - (1 - self.cellPivotY) * self.cellHeight + self.blockSizeY)
					if posY < targetY then
						posY = self.showList[len - 1].posY
					end
					for i = 1, self.cellCount do
						data = self.dataList[i]
						data.targetPosY = data.posY + (targetY - posY)
					end
				end
			end
			if posY and targetY then
				self:startRegulate()
			end
		end
	end
end

function Lua_CirculateList.Get:mousePosition()
	if self.showMode == Lua_AxisDefine.Horizontal then
		return CommonUtil.mousePositionX
	end
	return CommonUtil.mousePositionY
end

function Lua_CirculateList:startTimer()
	if self.bTimer ~= true then
		self.bTimer = true
		self:addTimer("Lua_CirculateList", 0.01, self, self.onTimer)
	end
end

function Lua_CirculateList:stopTimer()
	if self.bTimer == true then
		self.bTimer = false
		self:removeTimer(self, self.onTimer)
	end
end

function Lua_CirculateList:onTimer()
	-- local t = getTime()
	-- local dt = t - self.startClickTime -- 时差
	-- self.startClickTime = t

	-- local moveDis = self.mousePosition - self.startClickPos
	-- if moveDis ~= self.moveDis then
	-- 	if moveDis > self.moveDis then
	-- 		self:updateDragItem(self.moveSpeed * dt)
	-- 	else
	-- 		self:updateDragItem(-self.moveSpeed * dt)
	-- 	end
	-- end
	-- self.moveDis = moveDis

	self:updateDragItem(self.mousePosition - self.curMousePos)
	self.curMousePos = self.mousePosition
end

function Lua_CirculateList:startRegulate()
	if self.bRegulate ~= true then
		self.bRegulate = true
		self.startRegulateTime = getTime()
		self:addTimer("Lua_CirculateList_Regulate", 0.01, self, self.onRegulate)
	end
end

function Lua_CirculateList:stopRegulate()
	if self.bRegulate == true then
		self.bRegulate = false
		self:removeTimer(self, self.onRegulate)
	end
end

function Lua_CirculateList:onRegulate()
	local dt = getTime() - self.startRegulateTime
	if dt >= self.regulateTime then
		self:stopRegulate()
	else
		local data
		local index = 0
		self.tempList:clear()
		if self.showMode == Lua_AxisDefine.Horizontal then
			
		else
			for i = 1, self.cellCount do
				data = self.dataList[i]
				data.posY = Mathf.Lerp(data.posY, data.targetPosY, dt / self.regulateTime)
				if self:checkItemInBound(data.posX, data.posY) then
					data:setSiblingIndex(index)
					data:createItemFromPool()
					index = index + 1
					self.tempList:add(data)
				else
					data:returnItemToPool()
				end
			end

			if self.offsetPos ~= 0 then
				local len = self.tempList:getSize()
				local theta = 180 / len
				local angle = 0
				for i = 1, len do
					data = self.tempList[i]
					radian = (angle / 180) * Mathf.PI
					posX = Mathf.Cos(radian) * self.offsetPos
					data:setLocation(posX, data.posY)
					if i > len / 2 then
						angle = angle - theta
					else
						angle = angle + theta
					end
				end
			end
		end
	end
end




















Lua_CirculateData = class("Lua_CirculateData")
function Lua_CirculateData:ctor(ownerList)
	self.ownerList = ownerList
end

function Lua_CirculateData:dispose()
	self:returnItemToPool()
	self.ownerList = nil
end

function Lua_CirculateData:setData(index, data)
	self.index = index
	self.data = data
end

function Lua_CirculateData:setLocation(posX, posY, posZ)
	self.posX = posX or 0
	self.posY = posY or 0
	self.posZ = posZ or 0
	if self.item then
		self.item:setLocation(self.posX, self.posY, self.posZ)
	end
end

function Lua_CirculateData:setSiblingIndex(value)
	self.siblingIndex = value or 0
end

function Lua_CirculateData:createItemFromPool()
	if self.item == nil then
		self.item = self.ownerList:createItem()
	end
	self.item:setSiblingIndex(self.siblingIndex)
	self.item:setLocation(self.posX, self.posY, self.posZ)
	self.item:setCellIndex(self.index)
	self.item:setData(self.data)
end

function Lua_CirculateData:returnItemToPool()
	if self.item then
		self.ownerList:returnItem(self.item)
		self.item = nil
	end
end




















Lua_CirculateItem = class("Lua_CirculateItem", Lua_DisplayObject)
function Lua_CirculateItem:ctor(p, skin, frameBuild)
	self:superCall("ctor", p, skin, frameBuild)
end

function Lua_CirculateItem:checkInBound(posX, posY)
	local minX = self.centerPosX - self.w / 2;
	local maxX = self.centerPosX + self.w / 2;
	local minY = self.centerPosY - self.h / 2;
	local maxY = self.centerPosY + self.h / 2;
	if posX >= minX and posX <= maxX and posY >= minY and posY <= maxY then
		return true;
	end
	return false
end

function Lua_CirculateItem:setData(data)
	
end

function Lua_CirculateItem:setCellIndex(value)
	self.cellIndex = value
	self.content.name = self.classname..self.cellIndex
end

function Lua_CirculateItem:onPressClick()

end