-- 消耗物品Item
Lua_ConsumeIconItem = class("Lua_ConsumeIconItem", Lua_DisplayObject);
function Lua_ConsumeIconItem:ctor(p, bChcekUpdate)
	self.bChcekUpdate = bChcekUpdate;
	self:superCall("ctor", p);
end

function Lua_ConsumeIconItem:buildShowItem()
	self:superCall("buildShowItem");
	if self.bChcekUpdate == true then
		self:addLuaEvent("PACK_ITEMS_UPDATE", self, self.updateData);
		self:addLuaEvent("MONEY_UPDATE", self, self.updateData);
	end

	self.joinerStr = "/";
	self.pivot = 0;
	self.spaceWidth = 0;
	self:setStyle();
	self:setColorStr();
	self.icon = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
end

function Lua_ConsumeIconItem:dispose()
	self.bg = nil;
	self.icon = nil;
	self._leftText = nil;
	self._rightText = nil;
	self.data = nil;
	self:superCall("dispose");
end

-- 1:从左往右 0:居中 -1:从右往左
function Lua_ConsumeIconItem.Set:pivot(val)
    self._pivot = val;
end

-- function Lua_ConsumeIconItem.Get:pivot()
--     return self._pivot;
-- end

function Lua_ConsumeIconItem:setBg(bgAtlas, bgSprite, bgWidth, bgHeight)
	self.bgWidth = bgWidth == nil and 0 or bgWidth;
	self.bgHeight = bgHeight == nil and 0 or bgHeight;
	self.bg = UICreate.createUIAtlasImage(self.content, bgAtlas, bgSprite, 0, 0, 0, 0);
	self.bg.siblingIndex = 0;
end

function Lua_ConsumeIconItem:setStyle(fontName, fontSize, colorStr, effectStyle, effectColor, effectDistanceX, effectDistanceY)
	self.fontName = fontName == nil and Lua_MyUISettings.defaultFontName or fontName;
	self.fontSize = fontSize == nil and 22 or fontSize;
	self.colorStr = colorStr == nil and "FFFFFF" or colorStr;
	self.effectStyle = effectStyle == nil and 0 or effectStyle;
	self.effectColor = effectColor == nil and "000000" or effectColor;
	self.effectDistanceX = effectDistanceX == nil and 1 or effectDistanceX;
	self.effectDistanceY = effectDistanceY == nil and 1 or effectDistanceY;
end

function Lua_ConsumeIconItem:setColorStr(enoughtColor, unEnoughtColor)
	self.enoughtColor = enoughtColor == nil and Lua_ColorDefine.GREEN_STR or enoughtColor;
	self.unEnoughtColor = unEnoughtColor == nil and Lua_ColorDefine.RED_STR or unEnoughtColor;
end

function Lua_ConsumeIconItem:setId(id, count, leftStr, rightStr)
	if self.data == nil then
		self.data = Lua_ToolVo.createToolVo(id, count);
	else
		self.data:setData(id, count);
	end
	self:setData(self.data, leftStr, rightStr);
end

function Lua_ConsumeIconItem:updateData(tab)
	if self.data == nil or tab == nil or tab[self.data.id] == nil then
		return;
	end
	self:setData(self.data, self.leftStr, self.rightStr);
end

function Lua_ConsumeIconItem:setData(data, leftStr, rightStr)
	self.leftStr = leftStr
	self.rightStr = rightStr
	if data.classname == nil then
		if self.data == nil then
			if #data > 0 then -- 配置表数据
				self.data = Lua_ToolVo.createToolVo(data[1], data[2]);
			else -- 服务端数据
				self.data = Lua_ToolVo.createToolVo(data.id);
				self.data:fromServer(data);
			end
		else
			if #data > 0 then -- 配置表数据
				self.data:setData(data[1], data[2]);
			else -- 服务端数据
				self.data:setData(data.id);
				self.data:fromServer(data);
			end
		end
	else
		self.data = data;
	end
	self:updateShow(self.data.iconAtlas, self.data.iconSprite, self.data.iconWidth, self.data.iconHeight, leftStr, rightStr);
end

function Lua_ConsumeIconItem:setIcon(atlas, sprite, width, height, leftStr, rightStr)
	self:updateShow(atlas, sprite, width, height, leftStr, rightStr);
end

function Lua_ConsumeIconItem:updateShow(atlas, sprite, width, height, leftStr, rightStr)
	self.icon:setAltas(atlas, sprite);
	self.icon:SetDimensions(width, height);
	self.icon:setScale(1, 1, 1);

	if not strIsNullOrEmpty(leftStr) then
		self.leftText.text = leftStr;
		self.leftText:resetSize();
	end
	
	if strIsNullOrEmpty(rightStr) then
		local costCount = self.data.count;
		local haveCount = DataCenter.packData:getItemCountById(self.data.id);
		if haveCount >= costCount then
			self.rightText.text = spliceStr("<color=#", self.enoughtColor, ">", numberTrans(haveCount), "</color>", self.joinerStr, numberTrans(costCount));
		else
			self.rightText.text = spliceStr("<color=#", self.unEnoughtColor, ">", numberTrans(haveCount), "</color>", self.joinerStr, numberTrans(costCount));
		end
	else
		self.rightText.text = rightStr;
	end
	self.rightText:resetSize();
	
	local iconWidth = self.icon.width;
	local leftWidth = self._leftText == nil and 0 or self.leftText.width;
	local rightWidth = self.rightText.width;
	width = iconWidth + leftWidth + rightWidth;
	if self.bg == nil then
		if self._pivot == 1 then
			if self._leftText ~= nil then
				self.leftText.pivotValue = 3;
				self.leftText.alignmentValue = 3;
				self.leftText:setLocation(0, 0, 0);
			end

			self.icon.pivotValue = 3;
			self.icon:setLocation(leftWidth + self.spaceWidth, 0, 0);

			self.rightText.pivotValue = 3;
			self.rightText.alignmentValue = 3;
			self.rightText:setLocation(self.icon.posX + iconWidth + self.spaceWidth, 0, 0);
		elseif self._pivot == -1 then
			self.rightText.pivotValue = 5;
			self.rightText.alignmentValue = 5;
			self.rightText:setLocation(0, 0, 0);

			self.icon.pivotValue = 5;
			self.icon:setLocation(-rightWidth - self.spaceWidth, 0, 0);

			if self._leftText ~= nil then
				self.leftText.pivotValue = 5;
				self.leftText.alignmentValue = 5;
				self.leftText:setLocation(self.icon.posX - iconWidth - self.spaceWidth, 0, 0);
			end
		else
			if self._leftText ~= nil then
				self.leftText.pivotValue = 5;
				self.leftText.alignmentValue = 5;
			end

			self.rightText.pivotValue = 3;
			self.rightText.alignmentValue = 3;

			width = width / 2;
			if leftWidth == rightWidth then
				self.icon.pivotValue = 4;
				self.icon:setLocation(0, 0, 0);
				if self._leftText ~= nil then
					self.leftText:setLocation(-iconWidth / 2 - self.spaceWidth, 0, 0);
				end
				self.rightText:setLocation(iconWidth / 2 + self.spaceWidth, 0, 0);
			elseif leftWidth > rightWidth then
				self.icon.pivotValue = 5;
				self.icon:setLocation(width - rightWidth, 0, 0);
				if self._leftText ~= nil then
					self.leftText:setLocation(self.icon.posX - iconWidth - self.spaceWidth, 0, 0);
				end
				self.rightText:setLocation(self.icon.posX + self.spaceWidth, 0, 0);
			else
				self.icon.pivotValue = 3;
				self.icon:setLocation(-(width - leftWidth), 0, 0);
				if self._leftText ~= nil then
					self.leftText:setLocation(self.icon.posX - self.spaceWidth, 0, 0);
				end
				self.rightText:setLocation(self.icon.posX + iconWidth + self.spaceWidth, 0, 0);
			end
		end
	else
		width = self.bgWidth + iconWidth / 2;

		self.bg.pivotValue = 4;
		self.bg:setLocation((width - self.bgWidth) / 2, 0, 0);
		self.bg:SetDimensions(self.bgWidth, self.bgHeight);

		self.icon.pivotValue = 4;
		self.icon:setLocation(-self.bgWidth / 2 + self.bg.posX, 0, 0);

		local w = self.bgWidth - iconWidth / 2;
		self.rightText.pivotValue = 4;
		self.rightText.alignmentValue = 4;
		self.rightText:setLocation((width - w) / 2 - 5.5, 0, 0);
	end
end

function Lua_ConsumeIconItem.Get:leftText()
	if self._leftText == nil then
		self._leftText = UICreate.createLabel(self.content, nil, 0, 0, 100, 31, self.fontSize);
		self._leftText.fontName = self.fontName;
		self._leftText.pivotValue = 5;
		self._leftText.alignmentValue = 5;
		self._leftText.horizontalOverflowValue = 1;
		self._leftText.colorStr = Lua_ColorDefine.transColorStr(self.colorStr);
		self._leftText.effectStyle = self.effectStyle;
		self._leftText.effectColorStr = Lua_ColorDefine.transColorStr(self.effectColor);
		self._leftText:setEffectDistance(self.effectDistanceX, self.effectDistanceY);
	end
	return self._leftText;
end

function Lua_ConsumeIconItem.Get:rightText()
	if self._rightText == nil then
		self._rightText = UICreate.createLabel(self.content, nil, 0, 0, 100, 31, self.fontSize);
		self._rightText.fontName = self.fontName;
		self._rightText.pivotValue = 3;
		self._rightText.alignmentValue = 3;
		self._rightText.horizontalOverflowValue = 1;
		self._rightText.colorStr = Lua_ColorDefine.transColorStr(self.colorStr);
		self._rightText.effectStyle = self.effectStyle;
		self._rightText.effectColorStr = Lua_ColorDefine.transColorStr(self.effectColor);
		self._rightText:setEffectDistance(self.effectDistanceX, self.effectDistanceY);
	end
	return self._rightText;
end

function Lua_ConsumeIconItem:setCanClick(bCanClick)
	if bCanClick then
		self.icon.mouseEnable = true;
		self.icon:addClickFuntion(self, self.onIconClick);
	else
		self.icon.mouseEnable = false;
		self.icon:removeClickFuntion(self, self.onIconClick);
	end
end

function Lua_ConsumeIconItem:onIconClick(go)
	if self.data == nil then
		return;
	end
	DataCenter.packUtil:openPathWindow(self.data, false);
end