Lua_DragListContainerData = class("Lua_DragListContainerData")
function Lua_DragListContainerData:ctor(dragType)
	self.dragType = dragType
	self.dataLen = 0
	self.cellSpacingX = 0
	self.cellSpacingY = 0
	self.itemVec = Array()
	self.itemDic = {}
end

function Lua_DragListContainerData:setCellSpacing(spacingX, spacingY)
	self.cellSpacingX = spacingX
	self.cellSpacingY = spacingY
end

function Lua_DragListContainerData:setData(data, cellClass, cellWidth, cellHeight, perLine, offsetVal)
	self.data = data
	if data == nil then
		return
	end

	if type(data) == type_str_table then
		self.isArray = true
		self.perLine = perLine
		if data.typeof and data:typeof() == type_str_array then
			self.dataLen = data:getSize()
		else
			self.dataLen = #data
		end
	else
		self.isArray = false
		self.perLine = 1
		self.dataLen = 1
	end

	self.cellClass = cellClass
	self.cellWidth = cellWidth
	self.cellHeight = cellHeight
    self.blockSizeX = cellWidth + self.cellSpacingX
    self.blockSizeY = cellHeight + self.cellSpacingY
	self.offsetVal = offsetVal or 0

	if self.dragType == Lua_AxisDefine.Horizontal then -- 水平
		self.showWidth = (self.dataLen > perLine) and (perLine * self.blockSizeX + self.offsetVal) or (self.dataLen * self.blockSizeX + self.offsetVal)
		self.showHight = Mathf.Ceil(self.dataLen / perLine) * self.blockSizeY
	else
		self.showWidth = (self.dataLen > perLine) and (perLine * self.blockSizeX) or (self.dataLen * self.blockSizeX)
		self.showHight = Mathf.Ceil(self.dataLen / perLine) * self.blockSizeY + self.offsetVal
	end
end

function Lua_DragListContainerData:setInfo(ownerList, index, posX, posY)
	self.ownerList = ownerList
	self.index = index
	self.posX = posX
	self.posY = posY

	local wIdx
	local hIdx
	local startX = -self.showWidth / 2 + ownerList.cellPivotX * self.blockSizeX + posX
	local startY = self.showHight / 2 - (1 - ownerList.cellPivotY) * self.blockSizeY + posY
	local itemX
	local itemY
	local vec
	for i = 1, self.dataLen do
		wIdx = Mathf.Floor((i - 1) % self.perLine)
		hIdx = Mathf.Floor((i - 1) / self.perLine)

		itemX = wIdx * self.blockSizeX + startX
		itemY = -hIdx * self.blockSizeY + startY
		if self.dragType == Lua_AxisDefine.Horizontal then
			itemX = itemX + self.offsetVal
		else
			itemY = itemY - self.offsetVal
		end

		vec = self.itemVec[i]
		if vec == nil then
			vec = Vector2(itemX, itemY)
			self.itemVec:add(vec)
		else
			vec:Set(itemX, itemY)
		end
	end
end

function Lua_DragListContainerData:getCellFromPool(index, siblingIndex, subRectX, subRectY)
	local centerOffX = (self.ownerList.cellPivotX - 0.5) * self.blockSizeX
	local centerOffY = (self.ownerList.cellPivotY - 0.5) * self.blockSizeY
	local vec
	local item
	local posX
	local posY
	local sellSeqIndex
	for i = 1, self.dataLen do
		vec = self.itemVec[i]
		item = self.itemDic[i]
		posX = vec.x - centerOffX + subRectX
		posY = vec.y - centerOffY + subRectY
		if self.ownerList:checkItemInBound(posX, posY, self.blockSizeX / 2, self.blockSizeY / 2) then
			sellSeqIndex = self.index * 1000 + index
			if item == nil then
				item = self.ownerList:createCellItem(self.cellClass)
				self.itemDic[i] = item
			end
			item:setLocation(vec.x, vec.y, 0)
			item:setScale(1, 1, 1)
			item:setRectSize(self.cellWidth, self.cellHeight)
			item:setCellSeqIndex(sellSeqIndex)
			item:setSiblingIndex(siblingIndex)
			item:onSelectionChange(sellSeqIndex == self.ownerList.curSelectedIdx)
			siblingIndex = siblingIndex + 1

			item:setCustomParam(self.ownerList.customParam)
			if self.isArray == true then
				item:setData(self.data[i])
			else
				item:setData(self.data)
			end
		else
			if item then
				self.ownerList:returnCell(item)
				self.itemDic[i] = nil
			end
		end
		index = index + 1
	end
	return index, siblingIndex
end

function Lua_DragListContainerData:returnCellToPool()
	if self.itemDic and self.ownerList then
		for k, v in pairs(self.itemDic) do
			self.ownerList:returnCell(v)
			self.itemDic[k] = nil
		end
	end
end

function Lua_DragListContainerData:getPosByIdx(idx)
	if self.dragType == Lua_AxisDefine.Horizontal then
		local count = self.dragType == Lua_AxisDefine.Horizontal and Mathf.Ceil(self.dataLen / self.perLine) or self.perLine
		local line = Mathf.Ceil(idx / count)
		if line > self.perLine then
			idx = count
		else
			idx = idx - (line - 1) * count
		end
		return (idx - 1) * self.blockSizeX
	else
		idx = Mathf.Ceil(idx / self.perLine)
		return (idx - 1) * self.blockSizeY
	end
end

function Lua_DragListContainerData:getItemByIdx(idx)
	return self.itemDic[idx]
end

function Lua_DragListContainerData:clean()
	self:returnCellToPool()
	if self.itemVec then
		self.itemVec:clear()
	end
end

function Lua_DragListContainerData:dispose()
	self.index = 0
	self.posX = 0
	self.posY = 0
	self.dataLen = 0
	self.data = nil
	self.cellClass = nil
	self.ownerList = nil
	self.itemDic = nil
	if self.itemVec then
		self.itemVec:clear()
	end
	self.itemVec = nil
end