-- 不可拖动道具列表
Lua_ItemIconList = class("Lua_ItemIconList", Lua_DisplayObject)
function Lua_ItemIconList:ctor(p)
	self:superCall("ctor", p);
end

function Lua_ItemIconList:buildShowItem()
	self:superCall("buildShowItem");

    self.rowSpace = 130;
    self.columnSpace = 130;
    self.rowLimit = 999;
    self.pivot = 0;
    self.canClick = false;
    self.scaleVal = 1;
    self.showNumLimit = false;
    self.showObtainPath = false;
	self.showArr = Array();
    self.hideArr = Array();
end

function Lua_ItemIconList:dispose()
    if self.showArr then
        self.showArr:dispose();
        self.showArr:clear();
        self.showArr = nil;
    end

    if self.hideArr then
        self.hideArr:dispose();
        self.hideArr:clear();
        self.hideArr = nil;
    end

    self.owner = nil;
    self.callBack = nil;

	self:superCall("dispose");
end

-- 1:从左往右 0:居中 -1:从右往左
function Lua_ItemIconList.Set:pivot(val)
    self._pivot = val;
end

-- function Lua_ItemIconList.Get:pivot()
--     return self._pivot;
-- end

function Lua_ItemIconList.Set:canClick(val)
    self._canClick = val;
end

-- function Lua_ItemIconList.Get:canClick()
--     return self._canClick;
-- end

function Lua_ItemIconList.Set:scaleVal(val)
    self._scaleVal = val;
    self.rowSpace = self.rowSpace * val;
    self.columnSpace = self.columnSpace * val;
end

-- function Lua_ItemIconList.Get:scaleVal()
--     return self._scaleVal;
-- end

function Lua_ItemIconList.Set:showNumLimit(val)
    self._showNumLimit = val;
end

-- function Lua_ItemIconList.Get:showNumLimit()
--     return self._showNumLimit;
-- end

-- 显示特效
function Lua_ItemIconList:showEffect(bShow)
    for i = 1, self.showArr:getSize() do
        self.showArr[i]:showEffect(bShow)
    end
end

-- 行间距，列间距，每行限制显示多少个
function Lua_ItemIconList:setLayout(rowSpace, columnSpace, rowLimit)
    self.rowSpace = rowSpace * self._scaleVal;
    self.columnSpace = columnSpace * self._scaleVal;
    self.rowLimit = rowLimit;
end

function Lua_ItemIconList:setItemDrawMaskState(bState, atlas, sprite)
    for i = 1, self.showArr:getSize() do
        self.showArr[i]:setDrawMask(bState, atlas, sprite);
    end
end

function Lua_ItemIconList:setItemMaskState(bState)
    for i = 1, self.showArr:getSize() do
        self.showArr[i]:setMask(bState);
    end
end

function Lua_ItemIconList:setClick(owner, callBack)
    self.owner = owner;
    self.callBack = callBack;
end

function Lua_ItemIconList:onBoundClick()
    if self.owner ~= nil and self.callBack ~= nil then
        self.callBack(self.owner);
        return;
    end

    local winPos = LuaUtil.getTouchPosInverseObj(self.mainContent);
    for i = 1, self.showArr:getSize() do
        if self.showArr[i].isActive and self.showArr[i]:checkInBound(winPos.x, winPos.y) then
            self.showArr[i]:onItemClick(nil);
        end
    end
end

function Lua_ItemIconList:setArrayData(data)
    self:returntItem();

	if data == nil then
		return;
	end

    local dataType;
    local len;
    if data.typeof ~= nil and data:typeof() == "Array" then
        dataType = 1;
        len = data:getSize();
    else
        dataType = 2;
        len = #data;
    end

	local posX;
	local posY;
    local w;
    if self._pivot == 1 then
        w = 0;
    elseif self._pivot == -1 then
        w = self.rowSpace * Mathf.Min((len - 1), self.rowLimit);
        w = -w;
    else
        w = self.rowSpace * Mathf.Min(len, self.rowLimit);
        w = - w / 2 + self.rowSpace / 2;
    end
        
	local item;
    local vo;
	for i = 1, len do
		posX = ((i - 1) % self.rowLimit) * self.rowSpace + w;
        posY = -1 * Mathf.Floor((i - 1) / self.rowLimit) * self.columnSpace;

		item = self:createItem();
        if dataType == 1 then
            if type(data[i]) == "table" then
                if data[i].classname ~= nil then
                    vo = data[i];
                else
                    if #data[i] > 1 then -- 配置表数据
                        vo = Lua_ToolVo.createToolVo(data[i][1], data[i][2]);
                    else -- 服务端数据
                        vo = Lua_ToolVo.createToolVo(data[i].id);
                        vo:fromServer(data[i]);
                    end
                end
            else
                vo = Lua_ToolVo.createToolVo(data[i]);
            end
        else
            if type(data[i]) == "table" then
                if #data[i] > 1 then -- 配置表数据
                    vo = Lua_ToolVo.createToolVo(data[i][1], data[i][2]);
                else -- 服务端数据
                    vo = Lua_ToolVo.createToolVo(data[i].id);
                    vo:fromServer(data[i]);
                end
            else
                vo = Lua_ToolVo.createToolVo(data[i]);
            end
        end
        item:setData(vo);

        if self._showNumLimit == true then
            item:showNumLimit(vo.count, DataCenter.packData:getItemCount(vo.id), 1);
        end

        item.isShowBtns = self.showObtainPath;
		item:setCanClick(self._canClick);
		item:setLocation(posX, posY, 0);
		item:setScale(self._scaleVal, self._scaleVal);
	end
end

function Lua_ItemIconList:createItem()
    local len = self.hideArr:getSize();
    local item = nil;
    if len > 0 then
        item = self.hideArr:remove();
        item:setEnable(true);
    end

    if item == nil then
        item = Lua_ItemIcon.new(self.content);
    end
    self.showArr:add(item);
    return item;
end

function Lua_ItemIconList:returntItem()
    local len = self.showArr:getSize();
    local item;
    while len > 0 do
        item = self.showArr:remove(len);
        item:setEnable(false);
        self.hideArr:add(item)
        len = len - 1;
    end
end