Lua_RoundList = class("Lua_RoundList", Lua_DisplayObject)
function Lua_RoundList:ctor(p, width, height, showMode)
	self.rectWidth = width
	self.rectHeight = height
	self.showMode = showMode

	self.minX = -width / 2
	self.maxX = width / 2
	self.minY = -height / 2
	self.maxY = height / 2

	self.cellMinX = 0
	self.cellMaxX = 0
	self.cellMinY = 0
	self.cellMaxY = 0

	self.cellSpacingX = 0
	self.cellSpacingY = 0

	self.dataArr = Array()
	self.showArr = Array()
	self.hideArr = Array()
	self.showDataArr = Array()

	self.nowDistance = 0; --定位(或校正)已经移动的距离
	self.moveDistance = 0; -- 定位(或校正)需要移动的距离
	self.moveTime = 0.2; --移动时间

	self.showTween = true; -- 是否显示动画，播放一次后设置false，重新播放需要手动设置true
    self.tweenId = 0;
    self.tweenTime = 0;
    self.delayTween = false; -- 延后表现动画，moveToIndex定位后再播放(immediately=true)

	self:superCall("ctor", p)
end

function Lua_RoundList:buildShowItem()
	self:superCall("buildShowItem");

	self:setRectSize(self.rectWidth, self.rectHeight)

	self.maskContainer = UICreate.createUIMaskContainer(self.content, nil, nil, 0, 0, 0, 0)
	self.maskContainer:setAnchorMin(0, 0)
	self.maskContainer:setAnchorMax(1, 1)
	self.maskContainer:setSubRectPivot(0.5, 0.5)
	self.maskContainer:setColor(0, 0, 0, 1)
	self.maskContainer.clickScale = false
	self.maskContainer:addMousDownFuntion(self, self.onMouseDownFun)
	self.maskContainer:addMouseUpFuntion(self, self.onMouseUpFun)
	self.maskContainer:addMask()

	self.sub = self.maskContainer.sub
end

function Lua_RoundList:dispose()
	self:stopTimer();
	self:stopMoveToPosTimer();
	self:removeTweenTimer();

	if self.dataArr then
		self.dataArr:dispose()
		self.dataArr:clear()
		self.dataArr = nil
	end

	if self.showArr then
		self.showArr:dispose()
		self.showArr:clear()
		self.showArr = nil
	end

	if self.hideArr then
		self.hideArr:dispose()
		self.hideArr:clear()
		self.hideArr = nil
	end

	if self.showDataArr then
		self.showDataArr:dispose()
		self.showDataArr:clear()
		self.showDataArr = nil;
	end

	if self.maskContainer then
		self.maskContainer:dispose()
		self.maskContainer = nil
	end

	if self.content ~= nil then
        TweenUtil.getInstance():stop(self.content);
    end

    self.pressItem = nil;
	self.sub = nil

	self:superCall("dispose")
end

--设置item
function Lua_RoundList:setCell(class, width, height, pivot)
	self.cellClass = class
	self.cellWidth = width
	self.cellHeight = height
	self.cellPivot = pivot or 4
	self.cellPivotX, self.cellPivotY = self:getPivotXY(self.cellPivot)
	self.blockSizeX = width + self.cellSpacingX
	self.blockSizeY = height + self.cellSpacingY
end

--设置item间隔
function Lua_RoundList:setCellSpacing(spacingX, spacingY)
	self.cellSpacingX = spacingX
	self.cellSpacingY = spacingY
	self.blockSizeX = self.cellWidth + spacingX
	self.blockSizeY = self.cellHeight + spacingY
end

--设置显示数量，即间接设置item间隔
function Lua_RoundList:setShowCount(count)
	self.cellSpacingY = (self.rectHeight - count * self.cellHeight) / (count - 1);
	self.cellSpacingX = (self.rectWidth - count * self.cellWidth) / (count - 1);
	self.blockSizeX = self.cellWidth + self.cellSpacingX
	self.blockSizeY = self.cellHeight + self.cellSpacingY
end

--设置，参数1：半径，参数2：true需要校正，false不校正，参数3：整体位置偏移
function Lua_RoundList:setRound(radius, isRegulate, offset)
	self.needRegulate = isRegulate or true;
	self.radius = radius or 0;
	self.offset = offset or 0;
end

function Lua_RoundList:getPivotXY(value)
	if value == 0 then
		return 0, 1
	elseif value == 1 then
		return 0.5, 1
	elseif value == 2 then
		return 1, 1
	elseif value == 3 then
		return 0, 0.5
	elseif value == 4 then
		return 0.5, 0.5
	elseif value == 5 then
		return 1, 0.5
	elseif value == 6 then
		return 0, 0
	elseif value == 7 then
		return 0.5, 0
	elseif value == 8 then
		return 1, 0
	end
	return 0.5, 0.5
end

--设置数据
function Lua_RoundList:setData(data, bUpdatePos)
	if data == nil then
		return;
	end

	if data.typeof and data:typeof() == "Array" then
		self.cellCount = data:getSize();
	else
		self.cellCount = #data;
	end

	local _data
	local num = self.dataArr:getSize() - self.cellCount
	while num > 0 do
		_data = self.dataArr:remove()
		_data:dispose()
		num = num - 1
	end
	while num < 0 do
		_data = Lua_RoundData.new(self)
		self.dataArr:add(_data)
		num = num + 1
	end
	for i = 1, self.cellCount do
		_data = self.dataArr[i]
		_data:setData(i, data[i]);
	end

    if bUpdatePos then
		self:updateChildPos()
	end
end

--更新item位置
function Lua_RoundList:updateChildPos()
	local data
	if self.showMode == Lua_AxisDefine.Horizontal then
		self.startPosX = self.minX + self.cellPivotX * self.cellWidth;
		self.cellMaxX = self.startPosX + (self.cellCount - 1) * (self.cellWidth + self.cellSpacingX);
		self.cellMinX = -self.cellMaxX;

		for i = 1, self.cellCount do
			data = self.dataArr[i];
			data.posX = self.startPosX + (i - 1) * (self.cellWidth + self.cellSpacingX);
			data.posY = 0;
		end
	else
		self.startPosY = self.maxY - (1 - self.cellPivotY) * self.cellHeight;
		self.cellMinY = self.startPosY - (self.cellCount - 1) * (self.cellHeight + self.cellSpacingY);
		self.cellMaxY = -self.cellMinY;
		
		for i = 1, self.cellCount do
			data = self.dataArr[i];
			data.posX = 0;
			data.posY = self.startPosY - (i - 1) * (self.cellHeight + self.cellSpacingY);
		end;
	end

	self:updateDragItem(0)
	if not self.delayTween then
        self:playTween();
    end
end

--拖拽计时执行
function Lua_RoundList:updateDragItem(value)
	local data;
	local interval = 0;
	local val;
	local isReturn = false;
	local itemY;
	local itemX;
	local siblingIndex = 1;

	if self.showMode == Lua_AxisDefine.Horizontal then
		if self.dataArr[self.cellCount].posX + value >= self.cellMaxX and value > 0 then
			value = self.cellMaxX - self.dataArr[self.cellCount].posX
		elseif self.dataArr[1].posX + value <= self.cellMinX and value < 0 then
			value = self.cellMinX - self.dataArr[1].posX
		end

		for i = 1, self.cellCount do
			data = self.dataArr[i];
			data.posX = data.posX + value;
			if self:checkItemInBound(data.posX, data.posY) then
				interval = 0
	            if data.posX >= self.radius then
					interval = data.posX - self.radius;
					itemX = self.radius - interval;
					isReturn = true;
				elseif data.posX <= -self.radius then
					interval = data.posX + self.radius;
					itemX = -self.radius - interval;
					isReturn = true;
				else
					itemX = data.posX;
					isReturn = false;
				end
				val = self.radius ^ 2 - itemX ^ 2;
				if isReturn == true then
					itemY = -(val ^ 0.5);
				else
					itemY = val ^ 0.5;
				end
				itemY = itemY - self.rectHeight / 2 + self.cellHeight / 2 + self.offset;

				if self:checkItemInBound(itemX, itemY) then
					data:setSiblingIndex(siblingIndex)
					data:createItemFromPool()
					data:setLocation(itemX, itemY);
					siblingIndex = siblingIndex + 1

					if value > 0 and self.showDataArr:indexOf(data) == -1 then
						self.showDataArr:add(data, 1);
					elseif value <= 0 and self.showDataArr:indexOf(data) == -1 then
						self.showDataArr:add(data);
					end
				else
					if self.showDataArr:indexOf(data) ~= -1 then
						self.showDataArr:removeItem(data);
					end
					data:setLocation(itemX, itemY);
					data:returnItemToPool();
				end
			else
				if self.showDataArr:indexOf(data) ~= -1 then
					self.showDataArr:removeItem(data);
				end
				data:returnItemToPool();
			end
		end
	else
		if self.dataArr[1].posY + value >= self.cellMaxY and value > 0 then
			value = self.cellMaxY - self.dataArr[1].posY
		elseif self.dataArr[self.cellCount].posY + value <= self.cellMinY and value < 0 then
			value = self.cellMinY - self.dataArr[self.cellCount].posY
		end

		for i = 1, self.cellCount do
			data = self.dataArr[i];
			data.posY = data.posY + value;
			if self:checkItemInBound(data.posX, data.posY) then
				interval = 0
	            if data.posY >= self.radius then
					interval = data.posY - self.radius;
					itemY = self.radius - interval;
					isReturn = true;
				elseif data.posY <= -self.radius then
					interval = data.posY + self.radius;
					itemY = -self.radius - interval;
					isReturn = true;
				else
					itemY = data.posY;
					isReturn = false;
				end
				val = self.radius ^ 2 - itemY ^ 2;
				if isReturn == true then
					itemX = -(val ^ 0.5);
				else
					itemX = val ^ 0.5;
				end
				itemX = itemX - self.rectWidth / 2 + self.cellWidth / 2 + self.offset;

				if self:checkItemInBound(itemX, itemY) then
					data:setSiblingIndex(siblingIndex)
					data:createItemFromPool()
					data:setLocation(itemX, itemY);
					siblingIndex = siblingIndex + 1

					if value >= 0 and self.showDataArr:indexOf(data) == -1 then
						self.showDataArr:add(data);
					elseif value < 0 and self.showDataArr:indexOf(data) == -1 then
						self.showDataArr:add(data, 1);
					end
				else
					if self.showDataArr:indexOf(data) ~= -1 then
						self.showDataArr:removeItem(data);
					end
					data:returnItemToPool();
				end
			else
				if self.showDataArr:indexOf(data) ~= -1 then
					self.showDataArr:removeItem(data);
				end
				data:returnItemToPool();
			end
		end
    end
end

--检查item是否在list范围内
function Lua_RoundList:checkItemInBound(itemX, itemY)
	if itemX == nil or itemY == nil then
		return;
	end
	if itemX > self.minX - (1 - self.cellPivotX) * self.cellWidth and 
	itemX < self.maxX + self.cellPivotX * self.cellWidth and
	itemY > self.minY - (1 - self.cellPivotY) * self.cellHeight and 
	itemY < self.maxY + self.cellPivotY * self.cellHeight then
		return true
	end
	return false
end

--创建item
function Lua_RoundList:createItem()
	local item;
	if self.hideArr:getSize() > 0 then
		item = self.hideArr:remove(self.hideArr:getSize());
		item:setEnable(true);
		self.showArr:add(item);
		return item;
	else
		item = self.cellClass.new(self.sub);
		item:setRectSize(self.cellWidth, self.cellHeight);
		self.showArr:add(item);
		return item;
	end
end

--回收item
function Lua_RoundList:returnItem(item)
	if self.showArr:indexOf(item) ~= -1 then
		item:setEnable(false);
		self.showArr:removeItem(item);
		self.hideArr:add(item);
		item:setEnable(false);
	end
end

--移动到某个索引，index索引，immediately立刻移动到索引，isBottom是否底端
function Lua_RoundList:moveToIndex(index, immediately, isBottom)
	immediately = immediately ~= nil and immediately or false;
	isBottom = isBottom ~= nil and isBottom or false;
    if not self.delayTween and self.bTweenTimer then
        self.moveIndex = idx;
        self.immediately = immediately;
        self.isBottom = isBottom;
        return;
    end

	if index < 1 then
		index = 1;
	elseif index > self.cellCount then
		index = self.cellCount;
	end

	if self.showMode == Lua_AxisDefine.Horizontal then
		self.moveDistance = self.startPosX - self.dataArr[index].posX;
		if isBottom then
			self.moveDistance = self.moveDistance + self.cellWidth - self.rectWidth;
		end
		self.moveIndex = index;
		if self.moveDistance ~= 0 then
			if immediately then
				self:updateDragItem(self.moveDistance);
				if self.delayTween then
		            self:playTween();
	            end
			else
				self.moveTime = 0.5;
				self:stopMoveToPosTimer();
				self:startMoveToPosTimer();
			end
		end
	else
		self.moveDistance = self.startPosY - self.dataArr[index].posY;
		if isBottom then
			self.moveDistance = self.moveDistance + self.cellHeight - self.rectHeight;
		end
		self.moveIndex = index;
		if self.moveDistance ~= 0 then
			if immediately then
				self:updateDragItem(self.moveDistance);
				if self.delayTween then
		            self:playTween();
	            end
			else
				self.moveTime = 0.5;
				self:stopMoveToPosTimer();
				self:startMoveToPosTimer();
			end
		end
	end
end

--检查点击中了哪个item
function Lua_RoundList:checkClick()
	self.maskContainer:refreshTouchPosInverseSubTf()
	local posX = self.maskContainer.touchSubRectPx
	local posY = self.maskContainer.touchSubRectPy
	local len = self.showArr:getSize()
	for i = 1, len do
		if self.showArr[i]:checkInBound(posX, posY) then
			return self.showArr[i];
		end
	end
	return nil;
end

--鼠标按下回调
function Lua_RoundList:onMouseDownFun(go)
	self.startClickPos = self.mousePosition;
	self.curMousePos = self.mousePosition;
	self.mouseDownTime = getTime();
	self:stopTimer();
	self:startTimer();

	self.pressItem = self:checkClick();
	if self.pressItem ~= nil then
		self.pressItem:onMouseDownHandle();
	end
end

--鼠标抬起回调
function Lua_RoundList:onMouseUpFun(go)
	self:stopTimer()
    if self.pressItem ~= nil then
    	self.pressItem:onMouseUpHandle();
    end
	local dis = self.mousePosition - self.startClickPos
	if Mathf.Abs(dis) < 2 then
		if self.pressItem ~= nil then
			self.pressItem:onPressClick();
		end
	else
		local showDataArr = self.showDataArr;
		if self.needRegulate == true then
			if self.showMode == Lua_AxisDefine.Horizontal then
				showDataArr:sort("siblingIndex", false);
				if dis > 0 then
					local len = showDataArr:getSize();
					if showDataArr[len].posX >= -self.startPosX then
						self.moveDistance = -self.startPosX + self.cellWidth + self.cellSpacingX - showDataArr[len].posX;
					else
						self.moveDistance = -self.startPosX - showDataArr[len].posX;
					end
				elseif dis < 0 then
					if self.showDataArr[1].posX <= self.startPosX then
						self.moveDistance = self.startPosX - self.cellWidth - self.cellSpacingX - showDataArr[1].posX;
					else
						self.moveDistance = self.startPosX - showDataArr[1].posX;
					end
				end
				if self.moveDistance ~= 0 then
					self.moveTime = 0.2;
					self:stopMoveToPosTimer();
					self:startMoveToPosTimer();
				end
			else
				self.showDataArr:sort("siblingIndex", false);
				if dis > 0 then
					if showDataArr[1].posY >= self.startPosY then
						self.moveDistance = self.startPosY + self.cellHeight + self.cellSpacingY - showDataArr[1].posY;
					else
						self.moveDistance = self.startPosY - showDataArr[1].posY;
					end
				elseif dis < 0 then
					local len = self.showDataArr:getSize();
					if showDataArr[len].posY <= -self.startPosY then
						self.moveDistance = -self.startPosY - self.cellHeight - self.cellSpacingY - showDataArr[len].posY;
					else
						self.moveDistance = -self.startPosY - showDataArr[len].posY;
					end
				end
				if self.moveDistance ~= 0 then
					self.moveTime = 0.2;
					self:stopMoveToPosTimer();
					self:startMoveToPosTimer();
				end
			end	
		end
	end
end

--鼠标位置
function Lua_RoundList.Get:mousePosition()
	if self.showMode == Lua_AxisDefine.Horizontal then
		return CommonUtil.mousePositionX
	end
	return CommonUtil.mousePositionY
end

--鼠标按下开始计时
function Lua_RoundList:startTimer()
	if self.bStartTimer ~= true then
		self.bStartTimer = true;
		self:addTimer("onTimer", 0.01, self, self.onTimer);
	end
end

--鼠标抬起停止计时
function Lua_RoundList:stopTimer()
	if self.bStartTimer == true then
		self.bStartTimer = false;
		Lua_CommonUtil.removeTimer(self, self.onTimer);
	end
end

--鼠标按下计时执行方法
function Lua_RoundList:onTimer()
	self:updateDragItem(self.mousePosition - self.curMousePos)
	self.curMousePos = self.mousePosition
end

--移动到某个位置计时（开始）
function Lua_RoundList:startMoveToPosTimer()
	if self.bStartMoveToPosTimer ~= true then
		self.bStartMoveToPosTimer = true;
		self.startTime = getTime();
		self.nowDistance = 0;
		self:addTimer("MoveToPos", 0.01, self, self.onMoveToPos);
	end
end

--移动到某个位置计时（结束）
function Lua_RoundList:stopMoveToPosTimer()
	if self.bStartMoveToPosTimer == true then
		self.bStartMoveToPosTimer = false;
		Lua_CommonUtil.removeTimer(self, self.onMoveToPos);
	end

	self.moveIndex = nil;
    self.immediately = nil;
    self.isBottom = nil;
end

--移动到某个位置执行方法
function Lua_RoundList:onMoveToPos()
	if getTime() - self.startTime < self.moveTime then
		local lastVal = self.nowDistance;
		self.nowDistance = Mathf.Lerp(self.nowDistance, self.moveDistance, (getTime() - self.startTime) / self.moveTime);
		self:updateDragItem(self.nowDistance - lastVal);
	else
		self:stopMoveToPosTimer();
	end
end

--播放动画
function Lua_RoundList:playTween()
	self:removeTweenTimer();
	if self.showTween == true then
        self.showTween = false;

        local tweenData = Lua_UITweenConfigData.getConfigData(self.tweenId);
        if tweenData == nil then
            return;
        end

        local len = self.showArr:getSize();
        local item;
        local startPosX;
        local startPosY;
        for i = 1, len do
            item = self.showArr[i];
            TweenUtil.getInstance():stop(item.content);

            if tweenData.startRot ~= nil then
                item:setRot(tweenData.startRot.x, tweenData.startRot.y, tweenData.startRot.z);
            end

            startPosX = nil;
            startPosY = nil;
            if tweenData.startPos ~= nil then
                if tweenData.startPos.x ~= 0 then
                    startPosX = tweenData.startPos.x * self.rectWidth;
                    startPosY = self.dataArr[item.cellIndex].itemY;
                elseif tweenData.startPos.y ~= 0 then
                    startPosX = self.dataArr[item.cellIndex].itemX;
                    startPosY = tweenData.startPos.y * self.rectHeight;
                end
            end
            startPosX = startPosX == nil and self.dataArr[item.cellIndex].itemX or startPosX;
            startPosY = startPosY == nil and self.dataArr[item.cellIndex].itemY or startPosY;
            if tweenData.offsetPosTab ~= nil then
                startPosX = startPosX + tweenData.offsetPosTab[1];
                startPosY = startPosY + tweenData.offsetPosTab[2];
            end
            LuaUtil.setObjPos(item.content, startPosX, startPosY, 0);

            if tweenData.startScale ~= nil then
                LuaUtil.setObjScale(item.content, tweenData.startScale.x, tweenData.startScale.y, tweenData.startScale.z);
            end

            local delay = (i - 1) * tweenData.delayTime + tweenData.firstDelayTime;
            local offect = tweenData.offect;
            local targetPosX = self.dataArr[item.cellIndex].itemX;
            local targetPosY = self.dataArr[item.cellIndex].itemY;
            local targetRot = tweenData.targetRotTab;
            local targetScale = tweenData.targetScaleTab;
            item:setEnable(not tweenData.delayShow);

            if tweenData.bFade then
                TweenUtil.getInstance():addTweenTargetAlpha(item.content, tweenData.keepTime, tweenData.delayTime, tweenData.fadeValue, tweenData.isFadeIn);
            end

            TweenUtil.getInstance():addTween(item.content, tweenData.keepTime, delay, 
                targetPosX, targetPosY, 0, 
                targetRot[1], targetRot[2], targetRot[3], 
                targetScale[1], targetScale[2], targetScale[3], 
                nil, nil, offect, tweenData.delayShow);

            if self.tweenTime < (tweenData.keepTime + delay) then
                self.tweenTime = tweenData.keepTime + delay;
            end
        end

        self.bTweenTimer = true;
        Lua_CommonUtil.delayCall(self.tweenTime, self, self.onTweenCompleted);
    end
end

--动画播放完成回调
function Lua_RoundList:onTweenCompleted()
    self:removeTweenTimer();
    if self.moveIndex ~= nil then
        self:moveToIndex(self.moveIndex, self.immediately, self.isBottom);
        self.moveIndex = nil;
        self.immediately = nil;
        self.isBottom = nil;
    end
end

--移除动画
function Lua_RoundList:removeTweenTimer()
    if self.bTweenTimer == true then
        self.bTweenTimer = false;
        Lua_CommonUtil.removeTimer(self, self.onTweenCompleted);
    end
    self.tweenTime = 0;
end








Lua_RoundData = class("Lua_RoundData")
function Lua_RoundData:ctor(ownerList)
	self.ownerList = ownerList
	self.posX = 0;
	self.posY = 0;
end

function Lua_RoundData:dispose()
	self:returnItemToPool()
	self.ownerList = nil
end

function Lua_RoundData:setData(index, data)
	self.index = index
	self.data = data
end

function Lua_RoundData:setLocation(itemX, itemY, itemZ)
	itemX = Mathf.Floor(itemX + 0.5);
	itemY = Mathf.Floor(itemY + 0.5);
	self.itemX = itemX or 0
	self.itemY = itemY or 0
	self.itemZ = itemZ or 0
	if self.item ~= nil then
		self.item:setLocation(self.itemX, self.itemY, self.itemZ)
	end
end

function Lua_RoundData:setSiblingIndex(value)
	self.siblingIndex = value or 0
end

function Lua_RoundData:createItemFromPool()
	if self.item == nil then
		self.item = self.ownerList:createItem()
	end

	self.item:setCellIndex(self.index)
	self.item:setSiblingIndex(self.siblingIndex)
	self.item:setLocation(self.itemX, self.itemY, self.itemZ)
	self.item:setData(self.data)
end

function Lua_RoundData:returnItemToPool()
	if self.item then
		self.ownerList:returnItem(self.item)
		self.item = nil
	end
end










Lua_RoundItem = class("Lua_RoundItem", Lua_DisplayObject)
function Lua_RoundItem:ctor(p, skin, frameBuild)
	self:superCall("ctor", p, skin, frameBuild)
	self.rot = 0;
end

function Lua_RoundItem:dispose()
	self:superCall("dispose");
end

function Lua_RoundItem:setRot(x, y, z)
	self:superCall("setRot", x, y, z);
	self.rot = z;
end

function Lua_RoundItem:checkInBound(posX, posY)
	local minX = self.centerPosX - self.w / 2 * self.scaleX;
	local maxX = self.centerPosX + self.w / 2 * self.scaleX;
	local minY = self.centerPosY - self.h / 2 * self.scaleY;
	local maxY = self.centerPosY + self.h / 2 * self.scaleY;
	if posX >= minX and posX <= maxX and posY >= minY and posY <= maxY then
		return true;
	end
	return false
end

function Lua_RoundItem:setData(data)
	
end

function Lua_RoundItem:setCellIndex(value)
	self.cellIndex = value
	self.content.name = self.classname..self.cellIndex
end

function Lua_RoundItem:onPressClick()

end

function Lua_RoundItem:onMouseDownHandle()

end

function Lua_RoundItem:onMouseUpHandle()

end