--[[
local arr = Array();
local num = 0;
local len = string.len(num);
for i = 1, len do
    local digit = num % 10;
    arr:add(digit + 1);
    num = Mathf.Floor(num / 10);
end
self.textScroll = Lua_ScrollList.new(self.content, 1, Vector4(0, 0, 13 * len, 22), 13, len);
self.textScroll:setAllListCell(Lua_ScrollAreaTestItem, 22, 22);
self.textScroll:setAllCellDirection(1);
self.textScroll:setLocation(0, 0, 0);

local numArr = Array();
for i = 0, 9 do
    numArr:add(i);
end

self.textScroll:setRect(Vector4(0, 0, 13 * len, 22));
self.textScroll.setNum = len;
self.textScroll:setAllArrayData(numArr);
self.textScroll:setGroupMode(Lua_ScrollList.meanWhileMode);
self.textScroll:runModeGroup(Lua_ScrollList.meanWhileMode, arr, 1, 20, false);





Lua_ScrollAreaTestItem = class("Lua_ScrollAreaTestItem", Lua_ScrollAreaItem);
function Lua_ScrollAreaTestItem:ctor(p)
    self:superCall("ctor", p);
end

function Lua_ScrollAreaTestItem:buildShowItem()
    self:superCall("buildShowItem");

    self.numText = UICreate.createLabel(self.content, "", 0, 0, 100, 31, 22);
    self.numText.pivot = Lua_WindowUtil.getPivot(4);
    self.numText.alignmentValue = 4;
    self.numText.horizontalOverflowValue = 1;
end

function Lua_ScrollAreaTestItem:dispose()
    if self.numText then
        self.numText:dispose();
        self.numText = nil;
    end
    self:superCall("dispose");
end

function Lua_ScrollAreaTestItem:setData(data)
    self.numText.text = data;
end
--]]

Lua_ScrollList = class("Lua_ScrollList", Lua_DisplayObject);
Lua_ScrollList.meanWhileMode = 1;
Lua_ScrollList.carryDigitMode = 2;
Lua_ScrollList.slotMachineMode = 3;
function Lua_ScrollList:ctor(p, dType, vsize, distance, num)
    self.dType = type(dType) == "number" and dType or 1;
    self.vsize = vsize;
    self.num = num;
    self.distance = distance;
    self.isActive = true;
    self.showList = Array();
    self.modeGroup = {};
    self.modeOrderDic = {};

    self:superCall("ctor", p);
end

function Lua_ScrollList:buildShowItem()
    self:superCall("buildShowItem");
    self:setRectSize(self.vsize.z, self.vsize.w);

    self.mask = UICreate.createUIMaskContainer(self.content, nil, nil, 0, 0, 0, 0);
    self.mask.pivot = Lua_WindowUtil.getPivot(4);
    self.mask:setAnchorMin(0, 0);
    self.mask:setAnchorMax(1, 1);
    self.mask:SetDimensions(0, 0);
    self.mask:setSubRectPivot(0.5, 0.5);
    self.mask:setColor(255, 255, 255, 1);
    self.mask.clickScale = false;
    self.mask:addMask();
    self.sub = self.mask.sub;
    
    LuaUtil.setRectSize(self.mask.subRect, self.vsize.z, self.vsize.w);

    self:initShowList();
    self:addLuaEvent("scrollGroupModeWork", self, self.groupModeAction);
end

function Lua_ScrollList:dispose()
    for k, arr in pairs(self.modeGroup) do
        if arr ~= nil then
            arr:dispose();
            arr:clear();
        end
    end
    self.modeGroup = nil;

    if self.showList ~= nil then
        local item;
        local len = self.showList:getSize();
        while self.showList:getSize() > 0 do
            item = self.showList:remove();
            item:dispose();
        end
        self.showList:dispose();
        self.showList:clear();
        self.showList = nil;
        item = nil;
    end

    self.cell = nil;
    self.sub = nil;
    self.mask = nil;
    self:superCall("dispose");
end

function Lua_ScrollList:setRect(vsize)
    self.vsize = vsize;
    self:setRectSize(self.vsize.z, self.vsize.w);
    self:initShowList();
end

function Lua_ScrollList.Set:setNum(val)
    self.num = val;
    self:initShowList();
    if self.cell ~= nil then
        self:setAllListCell(self.cell, self.cellHeight, self.cellWidth);
    end

    if self.direction ~= nil then
        self:setAllCellDirection(self.direction);
    end
end

function Lua_ScrollList:initShowList()
    if self.showList:getSize() > 0 then
        self.showList:dispose();
        self.showList:clear();
    end
    for i = 1, self.num do
        local cell = Lua_ScrollCell.new(self.sub, self.dType);
        self.showList:add(cell);
    end

    for i=1, self.showList:getSize() do
        if self.dType == 1 then
            self.showList[i]:setLocation(self.distance * ((self.num / 2) - i + 0.5), 0, 0);
        else
            self.showList[i]:setLocation(0, self.distance * (i - (self.num / 2) - 0.5), 0);
        end
    end
end

function Lua_ScrollList:groupModeAction(tab)
    if tab.mode == nil then
        return;
    end

    if self.modeGroup[tab.mode] == nil then
        return;
    end
    
    if tab.mode == Lua_ScrollList.carryDigitMode then
        if self.modeOrderDic[tab.mode].order == 1 then
            if self.modeGroup[tab.mode][tab.id + 1] ~= nil then
                self.modeGroup[tab.mode][tab.id + 1]:callModeRun(tab.curIdx, tab.order, tab.speed, self.modeOrderDic[tab.mode].immediatly);
            end
        else
            if self.modeGroup[tab.mode][tab.id - 1] ~= nil then
                self.modeGroup[tab.mode][tab.id - 1]:callModeRun(tab.curIdx, tab.order, tab.speed, self.modeOrderDic[tab.mode].immediatly);
            end
        end
    elseif tab.mode == Lua_ScrollList.meanWhileMode then
    elseif tab.mode == Lua_ScrollList.slotMachineMode then
    end
end

function Lua_ScrollList:setSingalListCell(index, cell, cellHeight, cellWidth)
    if self.showList[index] == nil then
        return;
    end
    if self.dType == 1 then
        self.showList[index]:setListCell(cell, cellHeight, cellWidth, self.vsize.y + self.vsize.w, self.distance);
    else
        self.showList[index]:setListCell(cell, cellHeight, cellWidth, self.distance, self.vsize.x + self.vsize.z);
    end
end

function Lua_ScrollList:setAllListCell(cell, cellHeight, cellWidth)
    self.cell = cell;
    self.cellHeight = cellHeight;
    self.cellWidth = cellWidth;
    for i=1, self.showList:getSize() do
        self:setSingalListCell(i, cell, cellHeight, cellWidth);
    end
end

function Lua_ScrollList:setSingalArrayData(index, data)
    if self.showList[index] == nil then
        return;
    end
    self.showList[index]:setLuaArrayData(data);
end

function Lua_ScrollList:setAllArrayData(data)
    self.data = data;
    for i=1, self.showList:getSize() do
        self:setSingalArrayData(i, data);
    end
end

function Lua_ScrollList:setSingalCellDirection(index, direction)
    if self.showList[index] == nil then
        return;
    end
    self.showList[index]:setScrollDirection(direction);
end

function Lua_ScrollList:setAllCellDirection(direction)
    self.direction = direction;
    if self.showList == nil then
        return;
    end

    for i=1, self.showList:getSize() do
        self:setSingalCellDirection(i, direction);
    end
end

function Lua_ScrollList:setGroupMode(mode)
    if self.showList == nil then
        return;
    end

    if mode ~= Lua_ScrollList.meanWhileMode and mode ~= Lua_ScrollList.carryDigitMode and mode ~= Lua_ScrollList.slotMachineMode then
        return;
    end

    self.allListCanDrag = false;

    if mode ~= nil then
        if self.modeGroup[mode] == nil then
            self.modeGroup[mode] = Array();
        else
            for i=1, self.modeGroup[mode]:getSize() do
                self.modeGroup[mode]:remove();
            end
        end
        for i=1, self.showList:getSize() do
            self.showList[i]:setScrollMode(i, mode);
            self.modeGroup[mode]:add(self.showList[i]);
        end
    else
        for i=1, self.showList:getSize() do
            self.showList[i]:setScrollMode(i, mode);
        end

        for k,v in pairs(self.modeGroup) do
            if v ~= nil then
                v:clear();
            end
        end
    end
end

function Lua_ScrollList:runModeGroup(mode, param, order, speed, immediatly)
    if mode ~= Lua_ScrollList.meanWhileMode and mode ~= Lua_ScrollList.carryDigitMode and mode ~= Lua_ScrollList.slotMachineMode or mode == nil then
        return;
    end

    if param == nil then
        return;
    end

    if self.modeGroup[mode] == nil then
        return;
    end
    
    if self.modeOrderDic[mode] == nil then
        self.modeOrderDic[mode] = {};
    end
    self.modeOrderDic[mode].order = order;
    self.modeOrderDic[mode].immediatly = immediatly;

    if self.modeGroup[mode][1].direction == nil then
        self.modeGroup[mode][1]:setScrollDirection(1);
    end

    if type(param) == "string" then
        param = tonumber(param);
    end

    if type(param) == "number" then

        param = Mathf.Floor(Mathf.Abs(param));

        if mode == Lua_ScrollList.carryDigitMode then

            if param > Mathf.Pow(10, self.num) then
                param = Mathf.Pow(10, self.num);
            end

            if order == 1 then
                self.modeGroup[mode][1]:callModeRun(param, order, speed, immediatly);
            else
                self.modeGroup[mode][self.modeGroup[mode]:getSize()]:callModeRun(param, order, speed, immediatly);
            end
        elseif mode == Lua_ScrollList.meanWhileMode then
            --local p = param - 1;
            param = tostring(param);
            local digit;
            if order == 1 then
                for i=1, self.modeGroup[mode]:getSize() do
                    digit = string.sub(param,  - i,  - i);
                    if digit == "" then
                        digit = 1;
                    end
                    digit = tonumber(digit);
                    self.modeGroup[mode][i]:callModeRun(digit, order, speed, immediatly);
                    -- self.modeGroup[mode][i].curMoveNum = p;
                    -- p = Mathf.Floor(p / 10);
                end
            else
                for i = self.modeGroup[mode]:getSize(), 1, -1 do
                    digit = string.sub(param, i - self.modeGroup[mode]:getSize() - 1, i - self.modeGroup[mode]:getSize() - 1);
                    if digit == "" then
                        digit = 1;
                    end
                    digit = tonumber(digit);
                    self.modeGroup[mode][i]:callModeRun(digit, order, speed, immediatly);
                    -- self.modeGroup[mode][i].curMoveNum = p;
                    -- p = Mathf.Floor(p / 10);
                end
            end
        end
    elseif type(param) == "table" then
        local digit = 1;
        local num = 0;
        if param.typeof ~= nil and param:typeof() == "Array" then
            for i=1, param:getSize() do
                if type(param[i]) ~= "number" then
                    return;
                end
                if mode == Lua_ScrollList.meanWhileMode then
                    local val;
                    if order == 1 then
                        val = param[i];
                        if val == nil then
                            val = 1;
                        end
                        self.modeGroup[mode][i]:callModeRun(val, order, speed, immediatly);
                    else
                        val = param[param:getSize() - i + 1];
                        if val == nil then
                            val = 1;
                        end
                        self.modeGroup[mode][i]:callModeRun(val, order, speed, immediatly);
                    end
                end
                num = num + param[i] * digit;
                digit = digit * 10;
            end
        else
            for i=1, #param do
                if type(param[i]) ~= "number" then
                    return;
                end
                if mode == Lua_ScrollList.meanWhileMode then
                    local val;
                    if order == 1 then
                        val = param[i];
                        if val == nil then
                            val = 1;
                        end
                        self.modeGroup[mode][i]:callModeRun(val, order, speed, immediatly);
                    else
                        val = param[param:getSize() - i + 1];
                        if val == nil then
                            val = 1;
                        end
                        self.modeGroup[mode][i]:callModeRun(val, order, speed, immediatly);
                    end
                end
                num = num + param[i] * digit;
                digit = digit * 10;
            end
        end
        if mode == Lua_ScrollList.carryDigitMode then
            self:runModeGroup(mode, num, order, speed, immediatly);
        end
    else
        return;
    end
end

function Lua_ScrollList.Set:allListCanDrag(val)
    for i=1, self.showList:getSize() do
        self.showList[i].canDrag = val;
    end
end





















Lua_ScrollCell = class("Lua_ScrollCell", Lua_DisplayObject);
function Lua_ScrollCell:ctor(p, ltype)
    self.ltype = ltype;
    self.isActive = true;
    self.periodLen = 0;
    self.startMouse = -1;
    self.selectIdx = 1;
    self._isClick = false;
    self.showList = Array();
    self.canDrag = true;
    self.cycleDisplace = 0;
    self.scrollBackSpeed = 10;
    self.cellHeight = 0;
    self.cellWidth = 0;
    self.cycleNum = 0;
    self.bRefresh = false;
    self.setDispose = false;
    self.curMoveNum = 0;

    self:superCall("ctor", p);
end

function Lua_ScrollCell:buildShowItem()
    self:superCall("buildShowItem");
    -- self:setRectAnchor(0);
    -- self:setRectPivot(0);
end

function Lua_ScrollCell:setScrollDirection(val)
    self.direction = val;
end

function Lua_ScrollCell:setScrollMode(index, mode)
    if mode ~= Lua_ScrollList.meanWhileMode and mode ~= Lua_ScrollList.carryDigitMode and mode ~= Lua_ScrollList.slotMachineMode or mode == nil then
        if self.mode ~= nil then
            self.mode = nil;
        end
        self.groupID = nil;
        --self:removeLuaEvent(self, self.groupModeAction);
    else
        if self.mode ~= nil then
            if self.mode ~= Lua_ScrollList.carryDigitMode and mode == Lua_ScrollList.carryDigitMode then
                if self.ltype == 1 then
                    self:setCyclePos(self.curMoveNum * self.cellHeight);
                else
                    self:setCyclePos(self.curMoveNum * self.cellWidth);
                end
            elseif self.mode ~= Lua_ScrollList.meanWhileMode and mode == Lua_ScrollList.meanWhileMode then
                self:setCyclePos(self.cycleDisplace % self.periodLen);
            end
        end
        self.mode = mode;
        self.groupID = index;
        --self:addLuaEvent("scrollCellWork"..tostring(self.groupID), self, self.groupModeAction);
    end
end

function Lua_ScrollCell:callModeRun(param, order, speed, immediatly)
    self.order = order;
    if self.mode == Lua_ScrollList.carryDigitMode then
        self:moveToDataIndex(param, immediatly, speed / 2);--
    elseif self.mode == Lua_ScrollList.meanWhileMode then
        self:moveToDataIndex(param, immediatly, speed);
    end
end

-- function Lua_ScrollCell:groupModeAction(tab)
--     if tab.mode ~= self.mode then
--         return;
--     else
--         if self.mode == Lua_ScrollList.carryDigitMode then
--             self:callModeRun(1, tab.num, tab.order, tab.speed);
--         elseif tab.mode == Lua_ScrollList.meanWhileMode then
--         elseif tab.mode == Lua_ScrollList.slotMachineMode then
--         end
--     end
-- end

function Lua_ScrollCell:checkModeChange(mode, order, num)
    -- body
end

function Lua_ScrollCell:setCyclePos(num)
    self.targetPos = num;
    self.cycleDisplace = num;
end

function Lua_ScrollCell:setListCell(cell, cellHeight, cellWidth, viewHeight, viewWidth)
    self.cell = cell;
    self.cellWidth = cellWidth;
    self.cellHeight = cellHeight;

    self.showMinX = - viewWidth / 2 - cellWidth;
    self.showMinY = - viewHeight / 2 - cellHeight;
    self.showMaxX = viewWidth / 2 + cellWidth;
    self.showMaxY = viewHeight / 2 + cellHeight;

    if self.bg == nil then
        self.bg = UICreate.createUIAtlasImage(self.content,  "common_altas_1", "xui_common_tips_001", 0, 0, 0, 0);
    end
    self.bg:SetDimensions(viewWidth, viewHeight);
    self.bg.alpha = 0.01;
    self.bg:addMousDownFuntion(self, self.onMouseDownHandle);
    self.bg:addMouseUpFuntion(self, self.onMouseUpHandle);
end

function Lua_ScrollCell:moveToDataIndex(val, immediatly, speed)
    if self.data == nil or self.data:getSize() == 0 then
        return;
    end

    if self.setDispose == true then
        return;
    end

    if self.mode == nil then
        if val <= 0 or val > self.data:getSize() then
            val = self.data:getSize();
        end
    end

    local startPos = self.cycleDisplace;
    local offsetDisplace = Mathf.Abs(startPos) % self.periodLen;
    if startPos < 0 then
        offsetDisplace = self.periodLen - offsetDisplace;
    end
    local distance;
    local dataId;
    if self.ltype == 1 then
        distance = self.cellHeight;
    else
        distance = self.cellWidth;
    end
    dataId = Mathf.Floor(offsetDisplace / distance) + 1;
    offsetDisplace = offsetDisplace % distance;

    if speed ~= nil and speed > distance * 0.46 then
        speed = distance * 0.46;
    end
    self.scrollSpeed = speed;

    while(dataId > self.data:getSize()) do
        dataId = dataId - self.data:getSize();
    end

    if val == dataId and offsetDisplace == 0 then
        return;
    end

    if self.mode == nil then
        if self.direction == 1 then   --从下往上移动，从右往左移动
            self.targetPos = self.cycleDisplace + (val - dataId + self.data:getSize()) % self.data:getSize() * distance - offsetDisplace;
        elseif self.direction == 2 then   --从上往下移动，从左往右移动
            self.targetPos = self.cycleDisplace + (dataId - val + self.data:getSize()) % self.data:getSize() * distance - offsetDisplace;
        else    --最短距离移动
            val = val - dataId;
            if Mathf.Ceil(self.data:getSize() / 2) <= Mathf.Abs(val) then   --距离超过总数据距离一半反向移动
                self.targetPos = self.cycleDisplace - (self.data:getSize() - Mathf.Abs(val)) * (Mathf.Abs(val) / val) * distance - offsetDisplace;
            else
                self.targetPos = self.cycleDisplace + val * distance - offsetDisplace;
            end
        end
    else
        if self.mode == Lua_ScrollList.carryDigitMode then
            if self.direction == 1 then
                self.targetPos = (val - 1) * distance - offsetDisplace;
            else
                self.targetPos = - (val - 1) * distance + offsetDisplace;
            end
        elseif self.mode == Lua_ScrollList.meanWhileMode then
            if val <= 0 or val > self.data:getSize() then
                val = self.data:getSize();
            end
            if self.direction == 1 then   --从下往上移动，从右往左移动
                self.targetPos = self.cycleDisplace + (val - dataId + self.data:getSize()) % self.data:getSize() * distance - offsetDisplace;
            elseif self.direction == 2 then   --从上往下移动，从左往右移动
                self.targetPos = self.cycleDisplace - ((dataId - val + self.data:getSize()) % self.data:getSize()) * distance - offsetDisplace;
                -- if offsetDisplace ~= 0 then
                --     print("targetPos", self.targetPos);
                -- end
            else    --最短距离移动
                val = val - dataId;
                if Mathf.Ceil(self.data:getSize() / 2) <= Mathf.Abs(val) then   --距离超过总数据距离一半反向移动
                    self.targetPos = self.cycleDisplace - (self.data:getSize() - Mathf.Abs(val)) * (Mathf.Abs(val) / val) * distance - offsetDisplace;
                else
                    self.targetPos = self.cycleDisplace + val * distance - offsetDisplace;
                end
            end
        end
    end

    if immediatly then
        if self.ltype == 1 then
            self:updateCycleDis(0, self.targetPos, 0);
        else
            self:updateCycleDis(self.targetPos, 0, 0);
        end
    else
        if self.targetPos ~= self.cycleDisplace then
            Lua_CommonUtil.addTimer("onScrollBack", 0.01, self, self.onScrollBack, speed);
        else
            Lua_CommonUtil.removeTimer(self, self.onScrollBack);
        end
    end
end

function Lua_ScrollCell.Get:dataMidIdx()
    local offsetDisplace = Mathf.Abs(self.cycleDisplace) % self.periodLen;
    if self.cycleDisplace < 0 then
        offsetDisplace = self.periodLen - offsetDisplace;
    end
    if self.ltype == 1 then
        return Mathf.Floor(offsetDisplace / self.cellHeight) + 1;
    else
        return Mathf.Floor(offsetDisplace / self.cellWidth) + 1;
    end
end

function Lua_ScrollCell:setLuaArrayData(data)
    self.data = data;
    self.periodLen = 0;
    if data ~= nil and data:getSize() > 0 then
        self.bRefresh = true;
        if self.ltype == 1 then
            self.periodLen = data:getSize() * self.cellHeight;
        else
            self.periodLen = data:getSize() * self.cellWidth;
        end
    end

    self:initCellList();
    self:checkPosFix();
end

function Lua_ScrollCell:initCellList()
    if self.setDispose == true then
        return;
    end

    if self.data == nil then
        return;
    end

    local dataLen = self.showList:getSize();

    local item;
    if (dataLen > 0) then
        self.showList:dispose();
        self.showList:clear();
    end

    local up_left_Idx = 1;
    local down_right_Idx = 1;
    local upLeft_State = true;
    local downRight_State = true;
    local upLeft_Pos = 0;
    local downRight_Pos = 0;
    while(upLeft_State or downRight_State) do

        if self.ltype == 1 then
            upLeft_State = self:checkInBound(Vector2(0, upLeft_Pos));
        else
            upLeft_State = self:checkInBound(Vector2(upLeft_Pos, 0));
        end
        if upLeft_State then
            local item = self.cell.new(self.content);
            item:setData(self.data[up_left_Idx]);
            if self.ltype == 1 then
                item:setLocation(0, upLeft_Pos, 0);
                item.sortIdx = upLeft_Pos / self.cellHeight;
            else
                item:setLocation(upLeft_Pos, 0, 0);
                item.sortIdx = upLeft_Pos / self.cellWidth;
            end
            self.showList:add(item);
        end

        if downRight_State and upLeft_Pos ~= downRight_Pos then
            local item = self.cell.new(self.content);
            item:setData(self.data[down_right_Idx]);
            if self.ltype == 1 then
                item:setLocation(0, downRight_Pos, 0);
                item.sortIdx = downRight_Pos / self.cellHeight;
                downRight_State = self:checkInBound(Vector2(0, downRight_Pos));
            else
                item:setLocation(downRight_Pos, 0, 0);
                item.sortIdx = downRight_Pos / self.cellWidth;
                downRight_State = self:checkInBound(Vector2(downRight_Pos, 0));
            end
            self.showList:add(item);
        end

        up_left_Idx = up_left_Idx - 1;
        if up_left_Idx <= 0 then
            up_left_Idx = self.data:getSize();
        end

        down_right_Idx = down_right_Idx + 1;
        if down_right_Idx > self.data:getSize() then
            down_right_Idx = 1;
        end

        if self.ltype == 1 then
            upLeft_Pos = upLeft_Pos + self.cellHeight;
            downRight_Pos = downRight_Pos - self.cellHeight;
        else
            upLeft_Pos = upLeft_Pos + self.cellWidth;
            downRight_Pos = downRight_Pos - self.cellWidth;
        end
    end
    self.showList:sort("sortIdx", true);
end

function Lua_ScrollCell:onMouseDownHandle(go)
    self:onPress(true);
end

function Lua_ScrollCell:onMouseUpHandle(go)
    self:onPress(false);
end

function Lua_ScrollCell:onPress(state)
    if self.data == nil then
        return;
    end

    if self.setDispose == true then
        return;
    end

    if self.canDrag == false then
        return;
    end

    if state then
        if self.startMouse == -1 then
            self.startContainer = self.cycleDisplace;
            if self.ltype == 1 then
                self.startMouse = CommonUtil.mousePositionY;
            else
                self.startMouse = CommonUtil.mousePositionX;
            end
        end

        Lua_CommonUtil.removeTimer(self, self.onScrollBack);
        if self._isClick == false then
            self._isClick = true;
            Lua_CommonUtil.addTimer("ttt", 0.01, self, self.onPressFrameEnter);
        end
    else
        self:checkPosFix();
        self.startMouse = -1;
        self.startContainer = -1;
        if self._isClick == true then
            self._isClick = false;
            Lua_CommonUtil.removeTimer(self, self.onPressFrameEnter);
        end
    end
end

function Lua_ScrollCell:onPressFrameEnter()
    local posy = CommonUtil.mousePositionY;
    local posx = CommonUtil.mousePositionX;

    local de;
    if self.ltype == 1 then
        de = posy - self.startMouse;
    else
        de = posx - self.startMouse;
    end
    local rs = self.startContainer + de;
    if self.ltype == 1 then
        self:updateCycleDis(0, rs);
    else
        self:updateCycleDis(rs, 0);
    end
end

function Lua_ScrollCell:updateCycleDis(px, py)
    if self.setDispose == true then
        return;
    end

    if self.ltype == 1 then
        if self.cycleDisplace == py then
            return;
        end
        self.cycleDisplace = py;
    else
        if self.cycleDisplace == px then
            return;
        end
        self.cycleDisplace = px;
    end

    if self.mode == Lua_ScrollList.carryDigitMode then
        if self.cycleNum ~= Mathf.Floor(Mathf.Abs(self.cycleDisplace) / self.periodLen) then
            self.cycleNum = Mathf.Floor(Mathf.Abs(self.cycleDisplace) / self.periodLen);
            -- if self.groupID == 1 then
            --     print("num",self.cycleNum)
            -- end
            -- if self.order == 1 then
            Lua_MsgUtil.callFunc("scrollGroupModeWork", {curIdx = self.cycleNum + 1, id = self.groupID, speed = self.scrollSpeed, mode = self.mode, order = self.order});
            -- else
            --     Lua_MsgUtil.callFunc("scrollCellWork", {num = self.cycleNum + 1, id = self.groupID, speed = self.scrollSpeed, mode = self.mode, order = self.order});
            -- end
        end
    elseif self.mode == Lua_ScrollList.slotMachineMode then
    elseif self.mode == Lua_ScrollList.meanWhileMode then
    end


    local periodDisplace;
    local midId;
    local pos;
    if self.ltype == 1 then
        periodDisplace = Mathf.Abs(py) % self.periodLen;
        if py < 0 then
            periodDisplace = self.periodLen - periodDisplace;
        end
        midId = Mathf.Floor(periodDisplace / self.cellHeight) + 1;
        pos = py % self.cellHeight;
    else
        periodDisplace = Mathf.Abs(px) % self.periodLen;
        if px < 0 then
            periodDisplace = self.periodLen - periodDisplace;
        end
        midId = Mathf.Floor(periodDisplace / self.cellWidth) + 1;
        pos = px % self.cellWidth;
    end

    local item;
    local sortId;
    local sortPos;
    sortId = midId - self.showList[1].sortIdx;
    if sortId <= 0 then
        sortId = self.data:getSize() + sortId;
    end
    if self.ltype == 1 then
        sortPos = pos + self.showList[1].sortIdx * self.cellHeight;
    else
        sortPos = pos + self.showList[1].sortIdx * self.cellWidth;
    end

    for i=1, self.showList:getSize() do
        if self.ltype == 1 then
            self.showList[i]:setLocation(0, sortPos, 0);
        else
            self.showList[i]:setLocation(sortPos, 0, 0);
        end
        self.showList[i]:setData(self.data[sortId]);
        sortId = sortId + 1;
        if sortId > self.data:getSize() then
            sortId = 1;
        end
        if self.ltype == 1 then
            sortPos = sortPos - self.cellHeight;
        else
            sortPos = sortPos - self.cellWidth;
        end
    end
end

function Lua_ScrollCell:checkInBound(cellPos)
   local cellLeftX = cellPos.x - self.cellWidth / 2;
    local cellRightX = cellPos.x + self.cellWidth / 2;
    local cellUpY = cellPos.y + self.cellHeight / 2;
    local cellDownY = cellPos.y - self.cellHeight / 2;
    if cellLeftX >= self.showMinX and cellRightX <= self.showMaxX and cellUpY <= self.showMaxY and cellDownY >= self.showMinY then
        return true;
    else
        return false;
    end
end

function Lua_ScrollCell:checkPosFix()
    if self.setDispose == true then
        return;
    end

    self.targetPos = self.cycleDisplace;
    local differentDisplace = Mathf.Abs(self.cycleDisplace) % self.periodLen;
    if self.cycleDisplace < 0 then
        differentDisplace = self.periodLen - differentDisplace;
    end
    local dpos;
    if self.ltype == 1 then
        dpos = differentDisplace % self.cellHeight;
        if dpos > self.cellHeight / 2 then
            self.targetPos = self.targetPos + self.cellHeight - dpos;
        else
            self.targetPos = self.targetPos - dpos;
        end
    else
        dpos = differentDisplace % self.cellWidth;
        if dpos > self.cellWidth / 2 then
            self.targetPos = self.targetPos + self.cellWidth - dpos;
        else
            self.targetPos = self.targetPos - dpos;
        end
    end

    if self.targetPos ~= self.cycleDisplace then
        Lua_CommonUtil.addTimer("onScrollBack", 0.01, self, self.onScrollBack, self.scrollBackSpeed);
    else
        Lua_CommonUtil.removeTimer(self, self.onScrollBack);
        --self.cycleDisplace = self.cycleDisplace % self.periodLen;
    end
end

function Lua_ScrollCell:onScrollBack(speed)
    if self.targetPos == self.cycleDisplace then
        Lua_CommonUtil.removeTimer(self, self.onScrollBack);
    else
        speed = (speed == nil or speed <= 0 )and self.scrollBackSpeed or speed;
        local rs;
        if self.targetPos > self.cycleDisplace then
            rs = self.cycleDisplace + speed;
            if rs > self.targetPos then
                rs = self.targetPos;
            end
            if self.ltype == 1 then
                self:updateCycleDis(0, rs, 0);
            else
                self:updateCycleDis(rs, 0, 0);
            end
        else
            rs = self.cycleDisplace - speed;
            if rs < self.targetPos then
                rs = self.targetPos;
            end
            if self.ltype == 1 then
                self:updateCycleDis(0, rs, 0);
            else
                self:updateCycleDis(rs, 0, 0);
            end
        end
    end
end

function Lua_ScrollCell:dispose()
    self.cell = nil;

    if self.showList ~= nil then
        local dataLen = self.showList:getSize();
        local item;
        while(dataLen > 0) do
            item = self.showList:remove();
            item:dispose();
            dataLen = dataLen - 1;
        end
        self.showList:clear();
        self.showList = nil;
        item = nil;
    end

    if self.data ~= nil then
        self.data:clear();
        self.data = nil;
    end

    self.setDispose = true;
    Lua_CommonUtil.removeTimer(self, self.onScrollBack);
    Lua_CommonUtil.removeTimer(self, self.onPressFrameEnter);
    self:superCall("dispose");
end

















Lua_ScrollAreaItem = class("Lua_ScrollAreaItem", Lua_DisplayObject);
function Lua_ScrollAreaItem:ctor(p, skin, frameBuild)
    self:superCall("ctor", p, skin, frameBuild);
end

function Lua_ScrollAreaItem:buildShowItem()
    self:superCall("buildShowItem");
end

function Lua_ScrollAreaItem:dispose()
    self:superCall("dispose");
end

function Lua_ScrollAreaItem:setData(data)

end

function Lua_ScrollAreaItem.Set:selected(val)
    self._selected = val;
end

function Lua_ScrollAreaItem.Get:selected()
    return self._selected;
end