Lua_SliderProgressBar = class("Lua_SliderProgressBar", Lua_DisplayObject);
function Lua_SliderProgressBar:ctor(p, barAtlas, bgSprite, barSprite, sliderSprite, barWidth, barHeight, offsetWidth, offsetHeight)
	self.barAtlas = barAtlas;
	self.bgSprite = bgSprite;
	self.barSprite = barSprite;
	self.sliderSprite = sliderSprite;
	self.barWidth = barWidth == nil and 150 or barWidth;
	self.barHeight = barHeight == nil and 30 or barHeight;
	self.offsetWidth = offsetWidth == nil and 0 or offsetWidth;
	self.offsetHeight = offsetHeight == nil and 0 or offsetHeight;
	self:superCall("ctor", p);
end

function Lua_SliderProgressBar:buildShowItem()
	self:superCall("buildShowItem");

	self.bar = UICreate.createProgressBar(self.content, self.barAtlas, self.bgSprite, self.barAtlas, self.barSprite, 0, 0, self.barWidth, self.barHeight);
	self.bar:setBarOffset(self.offsetWidth, self.offsetHeight);
	if self.sliderSprite ~= nil then
		self.bar:createSlider(self.barAtlas, self.sliderSprite, 0, 0);
		self:addSlideFun();
	end

	self:setValue(0, 0, 100);
end

function Lua_SliderProgressBar:dispose()
	if self.bTimer ~= nil and self.bTimer == true then
		self.bTimer = false;
		self:removeTimer(self, self.onLongPress);
	end
	self.minTips = nil;
	self.maxTips = nil;
	self.owner = nil;
	self.callBack = nil;
	self.endCallBack = nil;
	self:superCall("dispose");
end

function Lua_SliderProgressBar:addSlideFun()
	if self.bar == nil then
		return;
	end
	self.bar:setMouseEnable(true);
	self.bar:addSlidingFun(self, self.onSlideingFun);
	self.bar:addSlideEndFun(self, self.onSlideEndFun);
end

function Lua_SliderProgressBar:removeSlideFun()
	if self.bar == nil then
		return;
	end
	self.bar:setMouseEnable(false);
	self.bar:removeSlidingFun(self, self.onSlideingFun);
	self.bar:removeSlideEndFun(self, self.onSlideEndFun);
end

function Lua_SliderProgressBar:createLessPlus(atlas, lessSprite, plusSprite, spacingX)
	spacingX = spacingX or 0;
	local posX = self.barWidth / 2;
	if not strIsNullOrEmpty(lessSprite) then
		self.btnLess = UICreate.createUIAtlasImage(self.content, atlas, lessSprite, 0, 0, 0, 0);
		self.btnLess.pivotValue = 5;
		self.btnLess:setLocation(-posX - spacingX, 0, 0);
		self.btnLess:addMousDownFuntion(self, self.onBtnLessClick);
		self.btnLess:addMouseUpFuntion(self, self.onBtnUpClick);
	end
	if not strIsNullOrEmpty(plusSprite) then
		self.btnPlus = UICreate.createUIAtlasImage(self.content, atlas, plusSprite, 0, 0, 0, 0);
		self.btnPlus.pivotValue = 3;
		self.btnPlus:setLocation(posX + spacingX, 0, 0);
		self.btnPlus:addMousDownFuntion(self, self.onBtnPlusClick);
		self.btnPlus:addMouseUpFuntion(self, self.onBtnUpClick);
	end
end

function Lua_SliderProgressBar:createMinMax(atlas, minSprite, maxSprite, spacingX)
	spacingX = spacingX or 0;
	local posX;
	if not strIsNullOrEmpty(minSprite) then
		self.btnMin = UICreate.createUIAtlasImage(self.content,  atlas, minSprite, 0, 0, 0, 0);
		self.btnMin.pivotValue = 5;
		posX = Mathf.Abs(self.btnLess.posX) + self.btnLess.width - spacingX;
		self.btnMin:setLocation(-posX, 0, 0);
		self.btnMin:addClickFuntion(self, self.onBtnMinClick);
	end
	if not strIsNullOrEmpty(maxSprite) then
		self.btnMax = UICreate.createUIAtlasImage(self.content,  atlas, maxSprite, 0, 0, 0, 0);
		self.btnMax.pivotValue = 3;
		posX = Mathf.Abs(self.btnPlus.posX) + self.btnPlus.width + spacingX;
		self.btnMax:setLocation(posX, 0, 0);
		self.btnMax:addClickFuntion(self, self.onBtnMaxClick);
	end
end

function Lua_SliderProgressBar:createFollowSpr(atlas, sprite, offsetX, offsetY)
	spacingX = spacingX or 0;
	local posX = self.barWidth / 2;
	if not strIsNullOrEmpty(sprite) then
		self.followSpr = UICreate.createUIAtlasImage(self.content, atlas, lessSprite, 0, 0, 0, 0);
		self.btnLess.pivotValue = 5;
		self.btnLess:setLocation(-posX - spacingX, 0, 0);

		self.followTxt = UICreate.createLabel(self.content, "", -280, 402, 600, 40, 28);
		self.followTxt.pivot = UIUtils.Pivot.TopLeft;
		self.followTxt.fontName = "huakang_font";
		self.followTxt.alignmentValue = 0;
		self.followTxt.horizontalOverflowValue = 0;
		self.followTxt.verticalOverflowValue = 1;
	end
end

function Lua_SliderProgressBar:createFollowTxt(fontName, colorStr, offsetX, offsetY)
	if not strIsNullOrEmpty(sprite) then
		self.followTxt = UICreate.createLabel(self.content, "", -280, 402, 600, 40, 28);
		self.followTxt.pivot = UIUtils.Pivot.TopLeft;
		self.followTxt.fontName = "huakang_font";
		self.followTxt.alignmentValue = 0;
		self.followTxt.horizontalOverflowValue = 0;
		self.followTxt.verticalOverflowValue = 1;
	end
end

function Lua_SliderProgressBar:setText(str)
	self.bar:setText(str);
end

function Lua_SliderProgressBar:setTips(minTips, maxTips)
	self.minTips = minTips;
	self.maxTips = maxTips;
end

function Lua_SliderProgressBar:setCallBack(owner, callBack, endCallBack)
	self.owner = owner;
	self.callBack = callBack;
	self.endCallBack = endCallBack;
end

function Lua_SliderProgressBar:setValue(curNum, minNum, maxNum)
	self.curNum = curNum;
	self.minNum = minNum;
	self.maxNum = maxNum;
	self.bar:setMinValue(minNum);
	self.bar:setMaxValue(maxNum);
	self.bar:setValue(curNum);
	self:updateNum();
end

function Lua_SliderProgressBar:onSlideingFun(num)
	if self.curNum ~= num then
		self.curNum = num;
		self:updateNum();
	end
end

function Lua_SliderProgressBar:onSlideEndFun(num)
	if self.curNum ~= num then
		self.curNum = num;
		self:updateNum();
	end
end

function Lua_SliderProgressBar:onBtnLessClick(go)
	self.bLess = true;
	self:startTimer();
end

function Lua_SliderProgressBar:onBtnPlusClick(go)
	self.bLess = false;
	self:startTimer();
end

function Lua_SliderProgressBar:onBtnUpClick(go)
	self:cleanTimer();
end

function Lua_SliderProgressBar:startTimer()
	if self.bTimer ~= true then
		self.bTimer = true;
		self.clickTime = getTime();
		self:addTimer("Lua_SliderProgressBar", 0.01, self, self.onLongPress, nil, -1);
	end
end

function Lua_SliderProgressBar:cleanTimer()
	if self.bTimer == true then
		self.bTimer = false;
		self:removeTimer(self, self.onLongPress);
		if not self.bLongPress then
			self:onLessPlusNum();
		end
	end
end

function Lua_SliderProgressBar:onLongPress()
	if getTime() - self.clickTime > 0.3 then
		self.bLongPress = true;
		self:onLessPlusNum();
	else
		self.bLongPress = false;
	end
end

function Lua_SliderProgressBar:onLessPlusNum()
	if self.bLess == true then
		self.curNum = self.curNum - 1;
	else
		self.curNum = self.curNum + 1;
	end
	if self.curNum < self.minNum or self.curNum > self.maxNum then
		self:cleanTimer();
	end
	self:checkValue();
end

function Lua_SliderProgressBar:onBtnMinClick(go)
	self.curNum = self.minNum;
	self:checkValue();
end

function Lua_SliderProgressBar:onBtnMaxClick(go)
	self.curNum = self.maxNum;
	self:checkValue();
end

function Lua_SliderProgressBar:checkValue()
	if self.curNum > self.maxNum then
		self.curNum = self.maxNum;
		if not strIsNullOrEmpty(self.maxTips) then
			OperateAlert.show(self.maxTips);
		end
	elseif self.curNum < self.minNum then
		self.curNum = self.minNum;
		if not strIsNullOrEmpty(self.minTips) then
			OperateAlert.show(self.minTips);
		end
	end
	self.bar:setValue(self.curNum);
	self:updateNum();
end

function Lua_SliderProgressBar:updateNum()
	if self.sliderSprite == nil then
		self.bar:setText(tostring(self.curNum));
	end

	if self.owner ~= nil and self.callBack ~= nil then
		self.callBack(self.owner, self.curNum);
	end

	if self.owner ~= nil and self.endCallBack ~= nil then
		self.endCallBack(self.owner, self.curNum);
	end
end
