Lua_BroadcastVo = class("Lua_BroadcastVo")
function Lua_BroadcastVo:ctor(config)
	self:superCall("ctor")
	self._config = config;
end

function Lua_BroadcastVo:dispose()
	self:clean();
	self.itemInfo = nil;
end

function Lua_BroadcastVo:clean()
	self.addTime = nil; -- 添加到广播面板时间戳
	self._config = nil;
	self.broad_args = nil;
	self.id = nil;
	self._type = nil;
	self._content = nil;
	self._broadArgsStr = nil;
	if self.itemInfo ~= nil then
		table.clear(self.itemInfo);
	end
end

function Lua_BroadcastVo:fromServer(msg)
	self.id = msg.shout_id
	self.broad_args = msg.broad_args
	self._broadArgsStr = nil;
end

function Lua_BroadcastVo.Set:config(val)
	self._config = val;
end

function Lua_BroadcastVo.Get:config()
	if self._config then
		return self._config
	end
	if self.id == nil then
		return nil
	end
	return getConfig("broadcast", self.id)
end

function Lua_BroadcastVo.Set:type(value)
	self._type = value
end

function Lua_BroadcastVo.Get:type()
	if self._type ~= nil then
		return self._type
	end
	if self.config == nil then
		return 1
	end
	return self.config.type or 1
end

function Lua_BroadcastVo.Set:content(value)
	self._content = value
end

function Lua_BroadcastVo.Get:content()
	if self._content ~= nil then
		return self._content
	end
	if self.config == nil or self.config.board_content == nil then
		return nil
	end
	return self:analysisArgs(self.config.board_content)
end

function Lua_BroadcastVo.Get:chatContent()
	if self._content ~= nil then
		return self._content
	end
	if self.config == nil or self.config.content == nil then
		return nil
	end
	return self:analysisArgs(self.config.content)
end

function Lua_BroadcastVo:analysisArgs(str)
	if self._broadArgsStr ~= nil then
		return self._broadArgsStr;
	end

	if self.broad_args then
		local info
		for i = 1, #self.broad_args do
			info = self.broad_args[i]
			local reStr = nil
			if info.arg_type == 1 then -- 整型
				reStr = tonumber(tostring(info.ivalue))
			elseif info.arg_type == 2 then -- 浮点型
				reStr = tostring(info.dvalue)
			elseif info.arg_type == 3 then -- 字符串
				reStr = info.strvalue
			elseif info.arg_type == 4 then -- 角色信息
				reStr = info.role_info.role_name
			elseif info.arg_type == 5 then -- 物品信息
				local tvo = DataCenter.packData:getRewardVo(info.item_info)
				if DataCenter.moneyData:isMoney(tvo.id) then
					reStr = tvo.count..tvo.name
				else
					if self.itemInfo == nil then
						self.itemInfo = {}
					end
					self.itemInfo[i] = tvo
					if tvo.count > 1 then
						reStr = tvo.broadcastName.."X"..tvo.count
					else
						reStr = tvo.broadcastName
					end
					reStr = "<url=itemClick_" .. i .. ">"..reStr.."</url>"
				end
			end
			if reStr == nil then
				reStr = ""
			end
			str = LanUtil.replaceStr(str, "{"..(i - 1).."}", reStr)
		end
	end

	self._broadArgsStr = str;
	return str
end

function Lua_BroadcastVo:getItemInfo(index)
	if self.itemInfo == nil then
		return nil
	end
	return self.itemInfo[index]
end

function Lua_BroadcastVo.Get:priority()
	if self.config == nil then
		return 0
	end
	return self.config.priority or 0
end

function Lua_BroadcastVo.Get:showSec()
	if self.config == nil then
		return 3
	end
	return self.config.show_sec or 3
end

function Lua_BroadcastVo.Get:broadType()
	if self.config == nil then
		return 3
	end
	return self.config.broadcast_type or 3
end

function Lua_BroadcastVo.Get:playType()
	if self.config == nil then
		return 2
	end
	return self.config.play_type or 2
end

function Lua_BroadcastVo.Get:playNum()
	if self.config == nil then
		return 1
	end
	return self.config.play_num or 1
end

function Lua_BroadcastVo.Get:channel()
	if self.config == nil then
		return nil
	end
	return self.config.channel
end

function Lua_BroadcastVo.Get:isFighting()
	if self.config == nil then
		return false
	end
	
	local list = self.config.fight_id
	if list then
		for i = 1, #list do
			if DataCenter.fightData:checkFightingByTemplateId(list[i]) then
				return true
			end
		end
	end
	return false
end