checkRequire("Lua_CollectionVo");
Lua_CollectionData = class("Lua_CollectionData");
function Lua_CollectionData:ctor()
	self._collectionArr = Array();
	self.popupInfoArr = Array();
	self.showItemArr = Array();
	self.collectionDic = {};
end

function Lua_CollectionData:dispose()
	if self._collectionArr ~= nil then
		self._collectionArr:dispose();
		self._collectionArr:clear();
	end
	if self.popupInfoArr ~= nil then
		self.popupInfoArr:dispose();
		self.popupInfoArr:clear();
	end
	if self.showItemArr ~= nil then
		self.showItemArr:dispose();
		self.showItemArr:clear();
	end
	self.collectionDic = {};
end

function Lua_CollectionData:fromServerCollection(msg)
	if msg == nil then
		return;
	end

	local obj;
	local itemId;
	local vo;
	for i = 1, #msg.collection_article_list do
		obj = msg.collection_article_list[i];

		itemId = obj.collection_item_id;
		vo = self.collectionDic[itemId];
		if vo == nil then
			local config = getConfig("item", itemId);
			if config ~= nil and config.main_type == Lua_PackDefine.COLLECTION then
				vo = Lua_ToolVo.createToolVo(itemId);
				self.collectionDic[itemId] = vo;
				self._collectionArr:add(itemId);
			end
		end
		if vo ~= nil then
			-- type 1.获取 2.激活 3.成长变化 4.分享状态更新
			if msg.type == 2 then
				if self.showItemArr:indexOf(itemId) == -1 then
					self.showItemArr:add(itemId);
				end
			elseif msg.type == 3 then -- 弹窗显示
				if obj.is_active == 1 then
					-- print("Lua_CollectionData", obj.grow_progress, vo.grow_progress);
					local change_progress = obj.grow_progress - vo.grow_progress; -- 改变的进度值
					if change_progress > 0 then
						vo.change_progress = change_progress;
						if self.popupInfoArr:indexOf(itemId) == -1 then
							self.popupInfoArr:add(itemId);
						end
					end
				end
			elseif msg.type == 4 then
				OperateAlert.showLangValue(460012);
			end
			vo:fromServer(obj);
		end
	end

	self.bSortCollections = true;
	self:showCollectionDetail();
	self:checkPopupInfo();
	self:updateCollectionRedShow();
end

function Lua_CollectionData:getCollectionVo(id)
	return self.collectionDic[id];
end

function Lua_CollectionData.Get:collectionArr()
	if self.bSortCollections then
		self.bSortCollections = false;
		self._collectionArr:sort(nil, false);
	end
	return self._collectionArr;
end

function Lua_CollectionData:showCollectionDetail()
	if self.showItemArr:getSize() > 0 and not Lua_WindowUtil.isWindowShowing("Lua_CollectionDetailWindow") then
		local itemId = self.showItemArr:shift();
		local vo = self:getCollectionVo(itemId);
		OperateAlert.show(LanUtil.formateStr(460013, vo.name));
		Lua_WindowUtil.openWin("Lua_CollectionDetailWindow", Lua_WinParam.ACTION_OPEN, itemId);
	end
end

function Lua_CollectionData:checkPopupInfo()
	if self.popupInfoArr:getSize() > 0 and not Lua_WindowUtil.isWindowShowing("Lua_CollectionPopupWindow") then
		Lua_WindowUtil.openWin("Lua_CollectionPopupWindow", Lua_WinParam.ACTION_OPEN, self.popupInfoArr:shift());
	end
end

function Lua_CollectionData:updateCollectionRedShow()
	local vo;
	local redShowState = false;
	for i = 1, self.collectionArr:getSize() do
		vo = self:getCollectionVo(self.collectionArr[i]);
		if not redShowState then
			if vo.is_have == 1 and vo.is_active == 0 then -- 已拥有未激活
				redShowState = true;
				break;
			elseif vo.is_active == 1 and vo.is_draw_active_reward == 0 then -- 已激活未领取
				redShowState = true;
				break;
			end
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Home_CollectionCabinet, redShowState);
end

-- 藏品柜上限
function Lua_CollectionData.Get:totalCollectionCount()
	if self._totalCollectionCount == nil then
		local config = getConfig("collection_cabinet_const", "collection_list","jsonvalue");
		self._totalCollectionCount = config ~= nil and #config or 0;
	end
	return self._totalCollectionCount;
end








-- 引导用
function Lua_CollectionData:getCollectionId(collectionNum, collectionState)
	if self.collectionDic ~= nil then
		local resultCount = 0;
		local resultCollectId = nil;
		local totalCount = self._collectionArr:getSize();
		local collectionId;
		for i = 1, totalCount do
			collectionId = self._collectionArr[i];
			if self:getCollectionVoStateById(collectionId) == collectionState then
				resultCount = resultCount + 1;
				if resultCount == collectionNum then
					resultCollectId = collectionId;
					break;
				end
			end
		end
		return resultCollectId;
	end
end

-- 根据藏品id获取该藏品的状态
function Lua_CollectionData:getCollectionVoStateById(collectionId)
	local collectionVo = self:getCollectionVo(collectionId);
	if collectionVo ~= nil then
		local activeState;
		if collectionVo.is_have == 1 and collectionVo.is_active == 0 then-- 可激活但未激活
			activeState = 1;
		elseif collectionVo.is_active == 1 then -- 已激活
			activeState = 2;
		end
		return activeState;
	end
end