Lua_CustomGiftVo = class("Lua_CustomGiftVo");
function Lua_CustomGiftVo:ctor()
	self:dispose();
end

function Lua_CustomGiftVo:dispose()
	self.showArr = Array();
end

function Lua_CustomGiftVo:init(msg)
	self.id = msg.id; -- 礼包ID
	self.name = msg.name; -- 礼包名字
	self.fixed_rewards = self:fixedRewards(msg.fixed_rewards); -- 固定奖励
	self.custom_rewards = self:customRewards(msg.custom_rewards); -- 可选奖励
	self.recharge_id = msg.recharge_id; -- 充值id（充值档次）
	self.recharge_money = msg.recharge_money; -- 充值金额
	self.fixed_num = #self.fixed_rewards; -- 固定奖励个数
	self.choose_num = msg.choose_num; -- 可选择奖励个数
end

function Lua_CustomGiftVo:fixedRewards(fixedReward)
	local fixedDic = {};
	for i = 1, #fixedReward do
		fixedDic[i] = Lua_ToolVo.createToolVo(fixedReward[i].key, fixedReward[i].value);
	end
	return fixedDic;
end

function Lua_CustomGiftVo:customRewards(customReward)
	local vo;
	local index;
	local customDic = {};
	for i = 1, #customReward do
		vo = Lua_ToolVo.createToolVo(customReward[i].key, customReward[i].value);
		index = customReward[i].ex_value; -- 格子位置
		if customDic[index] == nil then
			customDic[index] = Array();
		end
		customDic[index]:add(vo);
	end
	return customDic;
end

function Lua_CustomGiftVo:updateServerInfo(buy_times, buy_limit, cur_rewards)
	self.buy_times = buy_times; -- 已购买次数
	self.buy_limit = buy_limit; -- 购买次数上限
	self.custom_num = #cur_rewards;
	self.cur_rewards = self:curRewards(cur_rewards); -- 当前已定义礼包奖励
	self.all_rewards = self:integrateRewardInfo(self.fixed_rewards, self.cur_rewards, self.choose_num); -- 整合后的全部奖励
	self:setShowArr();
end

function Lua_CustomGiftVo:curRewards(curReward)
	local hasCustomDic = {};
	for i = 1, #curReward do
		hasCustomDic[curReward[i].ex_value] = Lua_ToolVo.createToolVo(curReward[i].key, curReward[i].value);
	end
	return hasCustomDic;
end

-- 固定和定制奖励合起来的全部奖励
function Lua_CustomGiftVo:integrateRewardInfo(fixed, customed, chooseNum)
	local allDic = {};
	for i = 1, #fixed + chooseNum do
		if i <= #fixed then
			allDic[i] = fixed[i];
		else
			allDic[i] = customed[i - #fixed] or 0;
		end
	end
	return allDic;
end

function Lua_CustomGiftVo:setShowArr()
	local vo;
	for i = 1, #self.all_rewards do
		vo = self.showArr[i];
		if vo == nil then
			vo = {
				id = self.id,
				index = i, -- 分辨是固定还是可选奖励
				fixed_num = self.fixed_num,
				choose_num = self.choose_num,
				custom_rewards = self.custom_rewards
			};
			self.showArr:add(vo);
		end
		vo.cur_count = self.buy_limit - self.buy_times; -- 当前可购买数量
		vo.cur_rewards = self.cur_rewards;
		vo.all_rewards = self.all_rewards;
	end
end