-- 战斗
checkRequire("Lua_FightRecordVo");
checkRequire("Lua_SingleFightPlayer");
Lua_FightData = class("Lua_FightData");
function Lua_FightData:ctor()
	self:superCall("ctor");
    self.bSaveFightData = Lua_PlatformUtil.IsNoSdk() and (Lua_PlatformUtil.IsWindowsEditor() or Lua_PlatformUtil.IsWindowsPlayer());
	self:dispose();
end

function Lua_FightData:dispose()
    self:stopTimer()

    if self.fightServerMsgList ~= nil then
        self.fightServerMsgList:clear();
        self.fightServerMsgList = nil;
    end

    self.curPlayFightRecordId = 0; -- 当前播放战斗id
    self.curShowResultRecordId = 0; -- 当前显示结算战斗id

    -- 战斗记录
	if self.recordIdArr ~= nil then
        local len = self.recordIdArr:getSize();
        while (len > 0) do
            local rid = self.recordIdArr:remove(len);
            if self.recordMsgDic[rid] ~= nil then
                self.recordMsgDic[rid]:dispose();
                self.recordMsgDic[rid] = nil;
            end

            if self.showFightPlayerDic[rid] ~= nil then
                self.showFightPlayerDic[rid]:dispose();
                self.showFightPlayerDic[rid] = nil;
            end

            len = len - 1;
        end
    end

    self.recordMsgDic = {}; -- 战斗记录数据
    self.recordIdArr = Array(); -- 战斗记录id集合
    self.recordTeamDic = {}; -- 战斗队伍
    self.recordGroupDic = {}; -- 战斗组
    self.showFightPlayerDic = {}; -- 战斗播放器集合
    -- self.autoSwitchIdArr = Array(); -- 自动切换战斗id集合 --影响组队，暂注释
    self.bEquipFightTips = nil;
    
    if self.fightResultList then
        self.fightResultList:clear()
        self.fightResultList = nil
    end
    self.fightResultDic = nil

    self.sendFightMsg = nil; -- 记录发起战斗的参数

    self.testConfigPath = ""; -- 用于技能测试
    self.testTargetsArr = Array(); -- 用于技能测试
    self.testAttackVo = nil; -- 用于技能调试
    self:clearRecordFightResult();
end

function Lua_FightData:startTimer()
    if self.bTimer ~= true then
        self.bTimer = true;
        self.startTime = getTime();
        Lua_CommonUtil.addTimer("Lua_FightData", 0.1, self, self.onTimer, nil, -1);
    end
end

function Lua_FightData:stopTimer()
    if self.bTimer == true then
        self.bTimer = false;
        Lua_CommonUtil.removeTimer(self, self.onTimer);
    end
end

function Lua_FightData:onTimer()
    local t = getTime();
    local dt = t - self.startTime;
    self.startTime = t;

    if self.delayHandleTime and self.delayHandleTime > 0 then
        self.delayHandleTime = self.delayHandleTime - dt;
        if self.delayHandleTime <= 0 then
            self.delayHandleTime = 0;
            self:onDelayHandle();
        end
    end

    if self.bInfinityHallBuffTimer == true then
        self.infinityHallBuffSec = self.infinityHallBuffSec - dt;
        if self.infinityHallBuffSec <= 0 then
            self:onBuffChooseComplete(self.infinityHallBuffRId)
        end
    end

    if self.bParseMsg == true then -- 解析后续战斗记录数据
        local len = self.fightServerMsgList:getSize();
        local msg;
        if len > 0 then
            msg = self.fightServerMsgList:remove(1);
            self:parseFightRecordMsg(msg);
            len = len - 1;
        end

        if len <= 0 then
            self.bParseMsg = false;
        end
    end
end

-- 播放速度索引
function Lua_FightData:getFightSpeedIdx(templateId)
    local speedIdx
    local config = Lua_FightTemplateConfigData.getConfigData(templateId);
    speedIdx = DataCenter.roleData:getOperationRecord(config.speedKey, 1);
    speedIdx = speedIdx == nil and 1 or speedIdx;
    speedIdx = speedIdx <= 0 and 1 or speedIdx;
    return speedIdx
end

function Lua_FightData.Get:curPlayFightRecordId()
    if self._curPlayFightRecordId and self._curPlayFightRecordId > 0 then
        local player = self:getFightPlayerById(self._curPlayFightRecordId);
        if player and not player.isPlayEnd then
            return self._curPlayFightRecordId;
        end
    end
    return 0
end

function Lua_FightData.Set:curPlayFightRecordId(value)
    if self._curPlayFightRecordId ~= value then
        self._curPlayFightRecordId = value;
        DataCenter.loginData:checkFpsState();
    end
end

function Lua_FightData.Get:isFighting()
    if self._curPlayFightRecordId and self._curPlayFightRecordId > 0 then
        local player = self:getFightPlayerById(self._curPlayFightRecordId)
        if player and not player.isPlayEnd then
            return true
        end
    end
    return false
end

function Lua_FightData.Get:curPlayRecordVo()
    return self.recordMsgDic[self.curPlayFightRecordId];
end

function Lua_FightData:isCurPlayFight(rid)
    rid = tonumber(rid);
    return self._curPlayFightRecordId and self._curPlayFightRecordId > 0 and self._curPlayFightRecordId == rid;
end

function Lua_FightData.Get:isShowResult()
    return self.curShowResultRecordId and self.curShowResultRecordId > 0;
end

function Lua_FightData:isCurShowResult(rid)
    rid = tonumber(rid);
    return self.curShowResultRecordId and self.curShowResultRecordId > 0 and self.curShowResultRecordId == rid;
end

function Lua_FightData:getRecordVoById(rid)
    if rid ~= nil then
        rid = tonumber(rid);
        return self.recordMsgDic[rid];
    end
    return nil;
end

function Lua_FightData:getFightPlayerById(rid)
    if rid ~= nil then
        rid = tonumber(rid);
        return self.showFightPlayerDic[rid];
    end
    return nil;
end

function Lua_FightData:getTeamArrById(teamId)
    if teamId ~= nil then
        teamId = tonumber(teamId);
        return self.recordTeamDic[teamId];
    end
    return nil;
end

function Lua_FightData:getGroupArrById(groupId)
    if groupId ~= nil then
        groupId = tonumber(groupId);
        return self.recordGroupDic[groupId];
    end
    return nil;
end

--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
-- 战斗解析
function Lua_FightData:addFightRecord(msg)
    if msg == nil then
        return;
    end
    
    local rid = tonumber(msg.record_id);
    if rid <= 0 then return; end

    local vo = self:getRecordVoById(rid);
    if vo == nil then
        self.recordIdArr:add(rid);
        vo = Lua_FightRecordVo.new(rid);
        self.recordMsgDic[rid] = vo;
    elseif msg.is_replay == 1 then
        vo:clear();
    end
    vo:updateFightInfo(msg);

    if vo.templateConfig == nil then
        self.recordIdArr:removeItem(rid);
        self.recordMsgDic[rid]:dispose();
        self.recordMsgDic[rid] = nil;
        return;
    end

    print("通知战斗记录 ===== Proto80102001 --- ", msg.record_id, "templateId = ", msg.template_id);
    LogManager.showLog("通知战斗记录 80102001 "..msg.record_id.." templateId = "..msg.template_id);
    
    if vo.groupId > 0 then
        -- 组队
        if self.recordGroupDic[vo.groupId] == nil then
            self.recordGroupDic[vo.groupId] = Array();
        end
        self.recordGroupDic[vo.groupId]:add(rid);
    elseif vo.teamId > 0 then
        -- 分队
        if self.recordTeamDic[vo.teamId] == nil then
            self.recordTeamDic[vo.teamId] = Array();
        end
        self.recordTeamDic[vo.teamId]:add(rid);        
    end

    -- 协助处理
    if vo.isAssist then
        -- 协助者
        local _vo = self:getRecordVoById(vo.last_record_id);
        _vo:setAssistInfo(vo.last_record_id, vo.help_member_record_id, rid);
            
        -- 被协助者
        _vo = self:getRecordVoById(vo.help_member_record_id);
        _vo:setAssistInfo(vo.last_record_id, 0, rid);
        return;
    end

    -- 重登续播延迟处理
    if vo.skipTime > 0 then
        self.delayHandleTime = 2;
         self:startTimer();
        return;
    end

    local bRunBack = false; -- 后台播放
    -- 对应功能未打开
    local winName = vo.templateConfig.run_back_cond;
    if not strIsNullOrEmpty(winName) and not Lua_WindowUtil.isWindowShowing(winName) then
        bRunBack = true;
        
        -- 处理通天塔继续下一层挑战
        local curFight = self:getRecordVoById(self._curPlayFightRecordId);
        if curFight and curFight.templateId == msg.template_id and vo.ownType == 1 then
            bRunBack = false;
        end
    end

    -- 组队不是本人
    if vo.groupId > 0 and vo.ownType == -1 then
        bRunBack = true;
    end

    -- 其他分队（两队同时播放）
    if vo.index and vo.index > 1 then
        bRunBack = true;
    end

    -- 录像馆/配置不进入后台播放
    if msg.is_replay == 1 and (vo.flag == Lua_FightDefine.REPLAY_VIDEO or vo.templateConfig.not_run_back == 1) then
        bRunBack = false;
    end

    local fightPlayer = self:getFightPlayerById(rid);
    if fightPlayer == nil then
        if not bRunBack then
        --     -- 插入新的战斗播放，保存上一个战斗，等待新的播放完成后，继续上一个战斗播放。
        --     if self.curPlayFightRecordId > 0 then
        --         self.autoSwitchIdArr:add(self.curPlayFightRecordId);
        --     end
            self.curPlayFightRecordId = rid;
        end

        fightPlayer = Lua_SingleFightPlayer.new();
        if bRunBack then
            vo.bPlayedStartEff = true;
            fightPlayer:hideToBackStage();
        end
        fightPlayer:setData(vo);
        self.showFightPlayerDic[rid] = fightPlayer;
    else
        if vo.index and vo.index > 1 then
            fightPlayer:setNextPlay(vo.index); -- 继续下一队播放
        else
            fightPlayer:setContinuePlay();
        end
    end

    if vo.isReplay ~= true then
        if bRunBack and vo.templateId == Lua_FightDefine.TEMPLATE_ID_COPY_MORE and vo.ownType ~= -1 then -- 寂灭岛组队
            if self.bEquipFightTips == true then
                OperateAlert.showLangValue(420119);
            end
            self.bEquipFightTips = true;
        end
        Lua_MsgUtil.callFunc("FIGHTPLAY_ENTER", vo.templateId);

        if vo.templateId == Lua_FightDefine.TEMPLATE_ID_COPY_MORE then
            self:removeReplayFight(vo.templateId);
        end
    end
end

function Lua_FightData:parseFightRecordSegentMsg(mp)
	if self.fightServerMsgList == nil then
        self.fightServerMsgList = Array();
    end
    self.fightServerMsgList:add(mp.data);
    self.bParseMsg = true;
    self:startTimer()
end

function Lua_FightData:parseFightRecordMsg(data)
    if data ~= nil then
        local msg = ProtoPublic_pb.Proto80102002();
        msg:ParseFromString(data);
        local rid = tonumber(msg.record_id);
        if rid > 0 and self.recordMsgDic[rid] ~= nil then
            self.recordMsgDic[rid]:updateFightSegmentInfo(msg);
        end
    end
end

-- 重登续播延迟处理
function Lua_FightData:onDelayHandle()
    if self.recordIdArr == nil then
        return;
    end

    local len = self.recordIdArr:getSize();
    local vo;
    local player;
    for i = 1, len do
        vo = self:getRecordVoById(self.recordIdArr[i]);
        if vo.skipTime > 0 and not vo.isAssist then
            player = self:getFightPlayerById(self.recordIdArr[i]);
            if player == nil then
                vo.bPlayedStartEff = true;
                player = Lua_SingleFightPlayer.new();
                player:hideToBackStage();
                player:setData(vo);
                self.showFightPlayerDic[vo.recordId] = player;
                Lua_MsgUtil.callFunc("FIGHTPLAY_ENTER", vo.templateId);
            end
        end
    end
end

-- 移除回放战斗处理 暂时只处理魔灵岛组队
function Lua_FightData:removeReplayFight(templateId)
    if self.recordIdArr == nil then
        return;
    end

    local len = self.recordIdArr:getSize();
    local vo;
    local arr;
    while len > 0 do
        vo = self:getRecordVoById(self.recordIdArr[len]);
        if vo ~= nil and vo.isReplay and vo.templateId == templateId then
            arr = self:getGroupArrById(vo.groupId);
            if arr ~= nil then
                arr:removeItem(vo.recordId);
                if arr:getSize() <= 0 then
                    self.recordGroupDic[vo.recordId] = nil;
                end
            end
            self:removeSingleFight(vo.recordId);
        end
        len = len - 1;
    end
end










--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
-- 战斗结果
function Lua_FightData:updateFightResult(msg)
    if msg == nil then
        return;
    end
    if self.fightResultList == nil then
        self.fightResultList = Array()
        self.fightResultDic = {}
    end

    local id
    local vo
    if msg.team_id and msg.team_id > 0 then -- 80102004 多分队战斗分开下发处理
        local bPlaying = false
        for i = 1, #msg.fight_info do
            id = msg.fight_info[i].record_id
            self:addFightResult(id, msg)
            vo = self:getRecordVoById(id)
            if vo then
                vo:setResult(msg.fight_info[i])
                if self:checkIsPlaying(vo.recordId) then
                    bPlaying = true
                end
            end
        end
        if not bPlaying then
            self:showResult(msg)
        end
    else
        if msg.template_id == Lua_FightDefine.TEMPLATE_ID_COPY_MORE then -- 魔灵岛组队更新新的结算，先移除旧的结算
            local len = self.fightResultList:getSize()
            while len > 0 do
                id = self.fightResultList[len]
                local _msg = self.fightResultDic[id]
                if _msg == nil or _msg.template_id == Lua_FightDefine.TEMPLATE_ID_COPY_MORE then
                    self.fightResultList:remove(len)
                end
                len = len - 1
            end
        end
        self:addFightResult(msg.record_id, msg)

        vo = self:getRecordVoById(msg.record_id)
        if vo then
            vo:setResult(msg)
            if not self:checkIsPlaying(vo.recordId) then
                self:showResult(msg)
            end
        else
            self:showResult(msg)
        end
    end
end

function Lua_FightData:addFightResult(id, msg)
    id = tonumber(id)
    if self.fightResultDic[id] == nil then
        self.fightResultList:add(id)
        self.fightResultDic[id] = msg
    end
end

function Lua_FightData:showResultById(id)
    local msg = self.fightResultDic[tonumber(id)]
    return self:showResult(msg)
end

function Lua_FightData:showResult(msg)
    if msg == nil then
        return 0
    end

    local isCurPlayFight = false
    if msg.fight_info then -- 分队信息
        for i = 1, #msg.fight_info do
            if self:isCurPlayFight(msg.fight_info[i].record_id) then
                isCurPlayFight = true
            end
        end
    else
        isCurPlayFight = self:isCurPlayFight(id)
    end

    local config
    -- 当前有结算打开中，保存下一奖励结算界面 
    if self.isShowResult then
        local curMsg = self.fightResultDic[self.curShowResultRecordId]
        if curMsg then
            config = Lua_FightTemplateConfigData.getConfigData(curMsg.template_id)
            local winName = config.failOpenWin
            if curMsg.result == 1 then
                winName = config.winOpenWin
            end
            if not strIsNullOrEmpty(winName) then
                if not Lua_WindowUtil.isWindowShowing(winName) then
                    Lua_WindowUtil.openWin(winName, Lua_WinParam.ACTION_OPEN, curMsg)
                end
                return 0
            end
        end
    end

    -- 非rid的战斗界面显示中，保存下一奖励结算界面
    if self.isFighting and not isCurPlayFight then
        return 0
    end

    config = Lua_FightTemplateConfigData.getConfigData(msg.template_id)
    if strIsNullOrEmpty(config.show_result_cond) or Lua_WindowUtil.isWindowShowing(config.show_result_cond) then
        local winName = config.failOpenWin
        if msg.result == 1 then -- 1赢2输
            winName = config.winOpenWin
        end
        if not strIsNullOrEmpty(winName) then
            Lua_WindowUtil.openWin(winName, Lua_WinParam.ACTION_OPEN, msg)
            return config.needResultWinExitFight
        end
    end
    return 0
end

function Lua_FightData:showNextResult()
    if self.isFighting or self.isShowResult then -- 正在播放战斗，停止显示下一奖励结算界面
        return nil
    end

    if self.fightResultList == nil then
        return nil
    end

    local id = self.fightResultList[1]
    if self:checkIsPlaying(id) then
        return nil
    end

    local msg = self.fightResultDic[id]
    if msg ~= nil and self:showResult(msg) ~= 0 then
        local config = Lua_FightTemplateConfigData.getConfigData(msg.template_id)
        if msg.result == 1 then
            return config.winOpenWin
        else
            return config.failOpenWin
        end
    end
    return nil
end

function Lua_FightData:removeFightResult(rid)
    local vo = self:getRecordVoById(rid);
    if vo ~= nil then
        if vo.groupId > 0 then
            local arr = self:getGroupArrById(vo.groupId);
            if arr ~= nil then
                for i = 1, arr:getSize() do
                    self:removeSingleResult(arr[i]);
                end
                return;
            end
        elseif vo.teamId > 0 then
            local arr = self:getTeamArrById(vo.teamId);
            if arr ~= nil then
                for i = 1, arr:getSize() do
                    self:removeSingleResult(arr[i]);
                end
                return;
            end
        end
    end
    self:removeSingleResult(rid)
end

function Lua_FightData:removeSingleResult(rid)
    if self.curShowResultRecordId == rid then
        self.curShowResultRecordId = 0
    end

    if rid == nil or self.fightResultList == nil or self.fightResultDic == nil then
        return
    end
    rid = tonumber(rid)
    if self.fightResultDic[rid] then
        self.fightResultList:removeItem(rid)
        self.fightResultDic[rid] = nil
        print("removeFightResult : ", rid);
    end
end

-- 检查是否播放中
function Lua_FightData:checkIsPlaying(rid)
    local vo = self:getRecordVoById(rid)
    if vo == nil then
        return false;
    end

    local player
    if vo.groupId > 0 then
        local arr = self:getGroupArrById(vo.groupId);
        if arr ~= nil then
            for i = 1, arr:getSize() do
                player = self:getFightPlayerById(arr[i])
                if player and player.isPlayEnd ~= true then
                    return true;
                end
            end
        end
    elseif vo.teamId > 0 then
        local arr = self:getTeamArrById(vo.teamId);
        if arr ~= nil then
            for i = 1, arr:getSize() do
                player = self:getFightPlayerById(arr[i])
                if player and player.isPlayEnd ~= true then
                    return true;
                end
            end
        end
    end

    player = self:getFightPlayerById(rid)
    if player and player.isPlayEnd ~= true then
        return true;
    end

    return false;
end

function Lua_FightData:showFightResult(rid)
    if self.fightResultList == nil then
        return 0
    end

    local vo = self:getRecordVoById(rid);
    if vo ~= nil and (vo.teamId > 0 or vo.groupId > 0) then
        if not self:checkIsPlaying(rid) then
            if vo.groupId > 0 then
                local arr = self:getGroupArrById(vo.groupId);
                if arr ~= nil then
                    for i = 1, arr:getSize() do
                        vo = self:getRecordVoById(arr[i]);
                        if vo.ownType ~= -1 then
                            return self:showResultById(arr[i]);
                        end
                    end
                end
            else
                -- 分队
                if vo.index and vo.index > 0 then
                    return self:showResultById(rid);
                end
            end
        end
        return 3;
    else
        local msg = self.fightResultDic[rid]
        if msg ~= nil then
            if msg.index and msg.index > 0 then -- 开始下一分队战斗
                Lua_ProxyManager.getProxy("Lua_FightProxy"):sStartFight(msg.template_id, msg.stage_id)
                self:removeFightResult(rid)
                return 3
            end

            if msg.skill_id_list ~= nil and #msg.skill_id_list > 0 then -- buff选择
                self.infinityHallBuffSec = 30
                local player = self:getFightPlayerById(rid)
                if player ~= nil and player.bRunBack then
                    print("BuffChoose start 1")
                    self:setBuffChooseInfo(vo.recordId, self.infinityHallBuffSec, msg.skill_id_list[1])
                else
                    print("BuffChoose start 2")
                    Lua_WindowUtil.openWin("Lua_FightBuffChooseWindow", Lua_WinParam.ACTION_OPEN, msg)
                end
                return 2
            end
            return self:showResult(msg)
        end
    end

    return 0
end

function Lua_FightData:checkBuffChoose(recordId)
    if self.infinityHallBuffRId and self.infinityHallBuffRId == recordId then
        local msg = self.fightResultDic[recordId]
        Lua_WindowUtil.openWin("Lua_FightBuffChooseWindow", Lua_WinParam.ACTION_OPEN, msg)
        self.infinityHallBuffRId = nil
    end
end

function Lua_FightData:setBuffChooseInfo(recordId, sec, skillId)
    Lua_MsgUtil.callFunc("INFINITYHALL_BUFF_CHOOSE", recordId)
    self.infinityHallBuffRId = recordId
    self.infinityHallBuffSec = sec
    self.infinityHallBuffId = skillId
    self.bInfinityHallBuffTimer = true
    self:startTimer()
end

function Lua_FightData:onBuffChooseComplete(recordId, skillId)
    self.bInfinityHallBuffTimer = false
    Lua_MsgUtil.callFunc("INFINITYHALL_BUFF_CHOOSE")
    
    local msg = self.fightResultDic[recordId]
    if msg == nil then
        self:removeFight(recordId)
    else
        if skillId == nil then
            skillId = self.infinityHallBuffId
        end
        if skillId == nil then
            skillId = msg.skill_id_list[1]
        end
        Lua_ProxyManager.getProxy("Lua_FightProxy"):sStartFight(msg.template_id, msg.stage_id, 0, skillId)
    end
    self:removeFightResult(recordId)
    self.infinityHallBuffRId = nil
    self.infinityHallBuffId = nil
    Lua_WindowUtil.openWin("Lua_FightBuffChooseWindow", Lua_WinParam.ACTION_CLOSE)
end










--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
function Lua_FightData:removeFight(rid)
    if rid == nil then
        return;
    end

    local player = self:getFightPlayerById(rid);
    if player == nil then
        return;
    end

    local vo = self:getRecordVoById(rid);
    if vo == nil then
        self:removeSingleFight(rid);
    else
        local templateId = vo.templateId;
        local bRemove = true;
        if vo.groupId > 0 then
            local arr = self:getGroupArrById(vo.groupId);
            if arr ~= nil then
                local len = arr:getSize();
                for i = 1, len do
                    player = self:getFightPlayerById(arr[i])
                    if player ~= nil and player.isPlayEnd ~= true then
                        bRemove = false;
                    end
                end

                if bRemove then
                    local groupId = vo.groupId;
                    local ownRid = rid; --自己的rid
                    while len > 0 do
                        rid = arr:remove(len);
                        vo = self:getRecordVoById(rid);
                        if vo.ownType ~= -1 then
                            ownRid = rid
                        end
                        self:removeSingleFight(rid, templateId);
                        len = len - 1;
                    end
                    if templateId == Lua_FightDefine.TEMPLATE_ID_COPY_MORE then
                        Lua_ProxyManager.getProxy("Lua_EquipCopyProxy"):sEquipCopyFightEnd(ownRid);
                    end
                    self.recordGroupDic[groupId] = nil;
                    Lua_MsgUtil.callFunc("FIGHTPLAY_COMPLETE", templateId);
                end
            end
        elseif vo.teamId > 0 then
            local arr = self:getTeamArrById(vo.teamId);
            if arr ~= nil then
                local len = arr:getSize();
                for i = 1, len do
                    player = self:getFightPlayerById(arr[i])
                    if player ~= nil and player.isPlayEnd ~= true then
                        bRemove = false;
                    end
                end

                if bRemove then
                    local teamId = vo.teamId;
                    while len > 0 do
                        rid = arr:remove(len);
                        self:removeSingleFight(rid, templateId);
                        len = len - 1;
                    end
                    self.recordTeamDic[teamId] = nil;
                    Lua_MsgUtil.callFunc("FIGHTPLAY_COMPLETE", templateId);
                end
            end
        else
            self:removeSingleFight(rid, templateId);
            Lua_MsgUtil.callFunc("FIGHTPLAY_COMPLETE", templateId);
        end
    end
end

function Lua_FightData:removeSingleFight(rid, templateId)
    -- self.autoSwitchIdArr:removeItem(rid);

    if self.recordMsgDic[rid] then
        if not self.bSaveFightData then
            self.recordMsgDic[rid]:dispose();
            self.recordMsgDic[rid] = nil;
            self.recordIdArr:removeItem(rid);
        else
            self.recordMsgDic[rid]:init();
        end
    end

    if self.showFightPlayerDic[rid] ~= nil then
        self.showFightPlayerDic[rid]:dispose();
        self.showFightPlayerDic[rid] = nil;
        print("removeFight : ", rid, templateId);
    end
    
    if self.curPlayFightRecordId == rid then
        self.curPlayFightRecordId = 0;
        -- if self.autoSwitchIdArr:getSize() > 0 then
        --     rid = self.autoSwitchIdArr:remove();
        --     self:showFightToStageByRid(rid);
        -- end
    end
end

-- 暂停、继续
function Lua_FightData:controllFightPlay(rid, bPlay)
    local player = self:getFightPlayerById(rid);
    if player ~= nil then
        player:setPlayState(not bPlay);
    end
end

-- 重播
function Lua_FightData:replayFight(rid)
    local fightPlayer = self:getFightPlayerById(rid);
    if fightPlayer ~= nil then
        fightPlayer:replay();
    end
end










--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
-- 除战斗id外其他隐藏到后台
function Lua_FightData:hideOtherFightToBackStage(recordId)
    local rid;
    local player;
    local len = self.recordIdArr:getSize();
    for i = 1, len do
        rid = self.recordIdArr[i];
        player = self:getFightPlayerById(rid);
        if player ~= nil and rid ~= recordId then
            if self.curPlayFightRecordId == rid then
                self.curPlayFightRecordId = 0;
            end
            player:hideToBackStage();
        end
    end
end

-- 根据战斗id隐藏到后台
function Lua_FightData:hideFightToBackStageByRecordId(recordId)
    if self:isCurPlayFight(recordId) == false then
        -- LogManager.showLog("Lua_FightMainWindow:onHang   "..recordId.."  -  "..self._curPlayFightRecordId)
        return;
    end

    -- if self.autoSwitchIdArr:getSize() > 0 then
    --     local nextId = self.autoSwitchIdArr:remove();
    --     self:showFightToStageByRid(nextId);
    -- else
        local player = self:getFightPlayerById(recordId);
        if player ~= nil then
            self.curPlayFightRecordId = 0;

            local vo = self:getRecordVoById(recordId);
            local list = vo.templateConfig.hang_close_win;
            if list ~= nil and #list > 0 then
                DataCenter.mainMenuData:resetNavigationBar();
                for i = 1, #list do
                    Lua_WindowUtil.openWin(list[i], Lua_WinParam.ACTION_CLOSE);
                end
            end

            Lua_WindowUtil.openWin("Lua_FightMainWindow", Lua_WinParam.ACTION_CLOSE);
            player:hideToBackStage();
        end
    -- end
end

-- 根据模板隐藏战斗到后台
function Lua_FightData:hideFightToBackStage(templateId)
    local rid;
    local player;
    local len = self.recordIdArr:getSize();
    for i = 1, len do
        rid = self.recordIdArr[i];
        player = self:getFightPlayerById(rid);
        if player ~= nil and player.templateId == templateId then
            if self.curPlayFightRecordId == rid then
                self.curPlayFightRecordId = 0;
            end
            player:hideToBackStage();
        end
    end
end

-- 根据模板显示战斗
function Lua_FightData:showFightToStage(templateId)
    local rid;
    local player;
    local len = self.recordIdArr:getSize();
    for i = 1, len do
        rid = self.recordIdArr[i];
        player = self:getFightPlayerById(rid);
        if player ~= nil then
            if player.templateId == templateId then
                self.curPlayFightRecordId = rid;
                -- self.autoSwitchIdArr:removeItem(rid);
                player:showToStage();
                break;
            end
        end
    end
end

-- 根据战斗id显示战斗
function Lua_FightData:showFightToStageByRid(rid)
    LogManager.showLog("start showFightToStageByRid")
    if self:isCurShowResult(rid) then
        -- LogManager.showLog("error 当前结算显示中： "..rid)
        return;
    end

    local vo = self:getRecordVoById(rid)
    if vo == nil then
        return;
    end

    local player = self:getFightPlayerById(rid);
    if self:isCurPlayFight(rid) and player ~= nil then
        self.curPlayFightRecordId = rid;
        player:showToStage();
        -- LogManager.showLog("success 当前战斗显示中： "..rid)
        return;
    end
    
    -- 魔灵岛组队 若当前玩家已结束战斗，直接切换至顺位成员战斗
    if vo.templateId == Lua_FightDefine.TEMPLATE_ID_COPY_MORE and (player == nil or player.isPlayEnd == true) then
        local arr = self:getGroupArrById(vo.groupId);
        if arr ~= nil then
            local len = arr:getSize()
            local count = len;
            local index = arr:indexOf(rid)
            rid = nil;
            player = nil;
            while len > 0 do
                index = index + 1
                index = (index < 1 or index > count) and 1 or index
                player = DataCenter.fightData:getFightPlayerById(arr[index])
                if player ~= nil and player.isPlayEnd ~= true then
                    rid = arr[index]
                    break
                end
                len = len - 1
            end
            if rid ~= nil then
                player = self:getFightPlayerById(rid);
            end
        end
    end

    if player ~= nil then
        -- self.autoSwitchIdArr:removeItem(rid);
        self.curPlayFightRecordId = rid;
        player:showToStage();
        -- LogManager.showLog("success 进入战斗显示： "..rid)
    end
end

-- 根据模板获取还在播放的战斗id
function Lua_FightData:getRecordIdByTemplateId(templateId)
    local len = self.recordIdArr:getSize();
    local rid;
    local vo;
    local player;
    for i = 1, len do
        rid = self.recordIdArr[i];
        vo = self:getRecordVoById(rid);
        if vo and vo.isReplay ~= true and vo.templateId == templateId then
            if vo.groupId > 0 then
                local arr = self:getGroupArrById(vo.groupId);
                if arr ~= nil then
                    for j = 1, arr:getSize() do
                        vo = self:getRecordVoById(arr[j]);
                         -- 返回自己的
                        if vo.ownType ~= -1 then
                            return arr[j]
                        end
                    end
                end
            elseif vo.teamId > 0 then
                local arr = self:getTeamArrById(vo.teamId);
                if arr ~= nil then
                    for j = 1, arr:getSize() do
                        player = self:getFightPlayerById(arr[j]);
                        if player and player.isPlayEnd ~= true then
                            return arr[j]
                        end
                    end
                end
            end

            player = self:getFightPlayerById(rid);
            if player and player.isPlayEnd ~= true then
                return rid;
            end
        end
    end
    return 0;
end

function Lua_FightData:checkFightingByTemplateId(templateId)
    return self:getRecordIdByTemplateId(templateId) > 0;
end

function Lua_FightData:playRecordByFightId(rid, bRunBack)
    if rid == nil or rid <= 0 then
        return;
    end

    local vo = self:getRecordVoById(rid);
    if vo == nil then
        return;
    end

    -- self.autoSwitchIdArr:removeItem(rid);
    local player = self:getFightPlayerById(rid);
    if player == nil then
        player = Lua_SingleFightPlayer.new();
        if bRunBack == true then
            vo.bPlayedStartEff = true;
            player:hideToBackStage();
        else
            self.curPlayFightRecordId = rid;
        end
        player:setData(vo);
        self.showFightPlayerDic[rid] = player;
    else
        self:showFightToStageByRid(rid);
    end
end










--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
function Lua_FightData:recordSendFightMsg(msg)
    if self.sendFightMsg == nil then
        self.sendFightMsg = {}
    end
    if msg then
        self.sendFightMsg[msg.template_id] = msg;
    end
end

function Lua_FightData:getSendFightMsg(template_id)
    if self.sendFightMsg == nil then
        return nil
    end
    return self.sendFightMsg[template_id]
end













------引导记录战斗结算界面是否已弹出------------
function Lua_FightData:recordFightResult(fightType, stage_id)
    if fightType == nil or stage_id == nil then
        return;
    end

    if self.fightResultDir == nil then
        self.fightResultDir = {};
    end
    if self.fightResultDir[fightType] == nil then
        self.fightResultDir[fightType] = {};
    end
    self.fightResultDir[fightType][stage_id] = true;
end

function Lua_FightData:clearRecordFightResult()
    self.fightResultDir = nil;
end

function Lua_FightData:getRecordFightResult(fightType, stage_id)
    if fightType == nil or stage_id == nil then--or self.fightResultDir == nil or self.fightResultDir[fightType] == nil then
        return false;
    end

    if DataCenter.mainLineData.levelIdWhenFistEnter > stage_id or 
        (DataCenter.mainLineData.levelIdWhenFistEnter == stage_id and DataCenter.mainLineData.levelStateWhenFistEnter == 1)
        then
        return true;
    end
    if self.fightResultDir == nil or self.fightResultDir[fightType] == nil then
        return false;
    end

    return self.fightResultDir[fightType][stage_id];
end

function Lua_FightData:getRecordFightResultWin(fightType, stage_id)
    if fightType == nil or stage_id == nil then
        return false;
    end

    if self.fightResultDir == nil or self.fightResultDir[fightType] == nil then
        return false;
    end

    return self.fightResultDir[fightType][stage_id];
end