-- 战斗播放器
checkRequire("Lua_SkillConfigData");
checkRequire("Lua_PlayerHangerConfigData");
Lua_SingleFightPlayer = class("Lua_SingleFightPlayer");
function Lua_SingleFightPlayer:ctor()
	self:superCall("ctor");

    self.bRunBack = false;
    self.isPlayEnd = false;
    self.bOpenResultWin = false;
    self.intervalTime = 0;
    self.skillAdditionalTime = 0; -- 技能额外时间
    self._endDelayTime = 0; -- 战斗结算等待时间
    self.approachUnitCount = 0;
    self.startEffectTime = -1;
    self.skillHurtCount = 0;
    self.attackerTeamIdx = 1; -- 攻方分队索引
    self.defenderTeamIdx = 1; -- 守方分队索引
    self.attackFormationEffsArr = nil; -- 攻方场景buff特效集合
    self.defendFormationEffsArr = nil; -- 守方场景buff特效集合
    self.bShowAttackThing = false; -- 是否已显示攻方单位
    self.showThingIntervalTime = 0; -- 显示单位间隔时间
    self.artifactSkillDic = {}
    Lua_MsgUtil.addEventListener("thingPlayerLoadComplete", self, self.onThingLoaded);
end

function Lua_SingleFightPlayer:dispose()
    if self.isDispose == true then
        return;
    end
    self.isDispose = true;

    self:endTimer();
    self._recordVo = nil;
    self:clean();
    self.artifactSkillDic = nil;
    
    if self.bRunBack ~= true then
        Lua_WindowUtil.openWin("Lua_FightMainWindow", Lua_WinParam.ACTION_CLOSE);
    end
    Scene3DManager.instance:destroyScene(self._sceneId, self._recordId);
    Lua_MsgUtil.removeEventListener("thingPlayerLoadComplete", self, self.onThingLoaded);
end

function Lua_SingleFightPlayer:clean()
    self.curActionInfo = nil;
    self:cleanArtifact()

    if self.showThingsArr ~= nil then
        self.showThingsArr:clear();
        self.showThingsArr = nil;
    end

    if self.attackVo ~= nil then
        self.attackVo:dispose();
        self.attackVo = nil;
    end

    if self.attackFormationEffsArr ~= nil then
        self.attackFormationEffsArr:clear();
        self.attackFormationEffsArr:dispose();
        self.attackFormationEffsArr = nil;
    end

    if self.defendFormationEffsArr ~= nil then
        self.defendFormationEffsArr:clear();
        self.defendFormationEffsArr:dispose();
        self.defendFormationEffsArr = nil;
    end

    if self.attackerCampList ~= nil then
        self.attackerCampList:clear();
        self.attackerCampList = nil;
    end

    if self.defenderCampList ~= nil then
        self.defenderCampList:clear();
        self.defenderCampList = nil;
    end

     self.fighter_list = nil;
end

function Lua_SingleFightPlayer:cleanArtifact()
    if self.artifactSkillDic then
        for k, _ in pairs(self.artifactSkillDic) do
            self.artifactSkillDic[k] = nil
        end
    end
end

function Lua_SingleFightPlayer.Get:bSkipShow()
    return self.bRunBack or self.bSkipPlay;
end

function Lua_SingleFightPlayer.Get:playSpeedRate()
    return self._playSpeedRate;
end

function Lua_SingleFightPlayer.Get:playSpeed()
    return self._playSpeed;
end

function Lua_SingleFightPlayer:setPlaySpdAndSpdRate(spd, rate)
    if self._playSpeed == spd and self._playSpeedRate == rate then
        return;
    end
    
    self._playSpeed = spd;
    self._playSpeedRate = rate;
    local list = DataCenter.thingData:getThingIds(self._sceneId, self._recordId);
    if list ~= nil then
        for i = 1, list:getSize() do
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, list[i]);
            if thing ~= nil then
                thing.player:setAniSpeed(spd, rate);
            end
        end
    end

    AudioManager.instance:updateAudioPlaySpeed(self._recordId, spd, rate);
    if self.attackVo ~= nil then
        self.attackVo.speed = spd;
        self.attackVo.speedRate = rate;
    end
end

function Lua_SingleFightPlayer.Get:sceneId()
    return self._sceneId;
end

function Lua_SingleFightPlayer.Get:recordId()
    return self._recordId;
end

function Lua_SingleFightPlayer.Get:templateId()
    if self._recordVo ~= nil then
        return self._recordVo.templateId;
    end
    return -1;
end

function Lua_SingleFightPlayer.Get:curRotation()
    return self._curRotation;
end

function Lua_SingleFightPlayer.Set:curRotation(value)
    self._recordVo.curRotation = value;
    if value > 1 and self._curRotation ~= value and not self.bSkipShow then
        self:cleanArtifact()
        if self.templateId ~= 1005 and self.templateId ~= 1006 then -- 无尽火域
            self.startEffectTime = -1;
            Lua_MsgUtil.callFunc("FIGHTPLAY_ROTATION_CHANGE", value);
        end
    end
    self._curRotation = value;
end

function Lua_SingleFightPlayer.Get:curRound()
    return self._curRound;
end

function Lua_SingleFightPlayer.Set:curRound(value)
    if self._curRound ~= value then
        self._curRound = value;
        self._recordVo:checkRound(value);
    end
end

function Lua_SingleFightPlayer.Get:curPlayActionInfo()
    if self.curActionInfo == nil then
        self.curActionInfo = self._recordVo:getActionInfo(self.curRotation, self.curSession, self.curRound, self.curAction);
    end
    return self.curActionInfo;
end

function Lua_SingleFightPlayer:onThingLoaded(obj)
    if obj == nil or self.showThingsArr == nil then
        return;
    end

    if obj.sceneId ~= self._sceneId then
        return;
    end

    if self.bSkipPlay then
        return;
    end

    -- 模型加载完成
    local playerId = obj.playerId;
    local idx = self.showThingsArr:indexOf(playerId);
    if idx > 0 then
        if self.approachUnitCount > 0 then -- 初始化阵型，全部初始化ok再显示
            self.approachUnitCount = self.approachUnitCount - 1;
            if self.approachUnitCount <= 0 then
                self:showBornEffect();
            end
        elseif self.bSceneLoad == true or self.bRunBack == true then -- 其他，如复活
            if self.bRunBack ~= true then
                local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, playerId);
                if thing ~= nil then
                    thing.player:setActive(true);
                    local pos = thing.player.currentPoint;
                    ThingUtil.instance:addSceneEffect(self._sceneId, self._recordId, "show_light", pos.x, pos.y, pos.z, playerId);
                end
            end

            if self.intervalTime < Lua_FightDefine.BORN_ACTION_TIME then
                self.intervalTime = Lua_FightDefine.BORN_ACTION_TIME;
            end

            self.showThingsArr:remove(idx);
            if self.showThingsArr:getSize() <= 0 then
                self.showThingsArr = nil;
            end
        end
    end
end

function Lua_SingleFightPlayer:showBornEffect()
    if self.bSkipPlay then
        return false;
    end

    if self.bSceneLoad ~= true or self.approachUnitCount > 0 or self.showThingsArr == nil or self.startEffectTime > 0 then
        return false;
    end

    local len = self.showThingsArr:getSize();
    if self.bShowAttackThing == true then
        while (len > 0) do
            local roleId = self.showThingsArr:remove(len);
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing ~= nil then
                thing.player:setActive(true);
                if self.bRunBack ~= true then
                    if thing.configData and thing.configData.hpBarType == 1 then
                        Lua_MsgUtil.callFunc("FIGHT_BOSS_INFO_UPDATE", thing.data);
                    end
                    local pos = thing.player.currentPoint;
                    ThingUtil.instance:addSceneEffect(self._sceneId, self._recordId, "show_light", pos.x, pos.y, pos.z, roleId);
                end
            end
            len = len - 1;
        end
        self.showThingsArr = nil;
    else
        while (len > 0) do
            local roleId = self.showThingsArr[len];
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing ~= nil and thing.formationType == Lua_FightDefine.FIGHT_ATTACKER then
                self.showThingsArr:remove(len);
                thing.player:setActive(true);
                if self.bRunBack ~= true then
                    local pos = thing.player.currentPoint;
                    ThingUtil.instance:addSceneEffect(self._sceneId, self._recordId, "show_light", pos.x, pos.y, pos.z, roleId);
                end
            end
            len = len - 1;
        end
        self.bShowAttackThing = true;
        self.showThingIntervalTime = 0.5;
    end

    if self.intervalTime < DataCenter.fightUtil.animalBornEffTime then
        self.intervalTime = DataCenter.fightUtil.animalBornEffTime;
    end
    return true;
end

-- 客户端本地设置重播
function Lua_SingleFightPlayer:replay()
    if self._recordVo == nil then
        return;
    end

    self.isReplay = true;
    if self.intervalTime > 0 or self.skillAdditionalTime > 0 then
        self.needReplay = true;
    else
        self:endTimer();
        self:clean();
        self:endBuffGuard();

        -- 双方退场
        self:removeThingsByFormationType(Lua_FightDefine.FIGHT_ATTACKER);
        self:removeThingsByFormationType(Lua_FightDefine.FIGHT_DEFENDER);
        -- ThingUtil.instance:removeThingsByFormationType(self._sceneId, self._recordId, -1);

        -- 重置参数
        self._recordVo.bPlayedStartEff = true; -- 重播不显示开场动画
        self.approachUnitCount = 0;
        self.needReplay = false;
        self.bSceneLoad = false;
        self.loadSceneId = 0;
        self.curActionInfo = nil;
        self.bCheckNextAction = false;
        self.bPlaySkill = false;
        self.skillAdditionalTime = 0;
        self.bShowAttackThing = false;
        self:setData(self._recordVo);
    end
end

function Lua_SingleFightPlayer:setData(rVo)
    self._recordVo = rVo;
    if rVo.bPlayedStartEff == true then
        self.startEffectTime = 0;
    else
        self.startEffectTime = -1;
    end
    
    if rVo.skipTime > 0 and not rVo.isAssist then
        self.totalTime = 0;
        if self.bSkipPlay then
            rVo.bPlayedStartEff = true;
        end
    else
        self.bSkipPlay = false;
    end
    
    self._recordId = rVo.recordId;
    self._sceneId = rVo.sceneId;
    self._playSpeed = rVo.speed;
    self._playSpeedRate = rVo.speedRate;
    self.curRotation = 1; -- 轮次
    self.curSession = 1; -- 场次
    self.curRound = 1; -- 回合
    self.curAction = 1; -- 行动

    self.bStartEffLoading = false;
    if self.bRunBack ~= true then
        self.bStartEffLoading = true;
        self.intervalTime = 0.5;
        local url = UrlManager.fixFileSuffix("/effects/scsEffects/zhandou_jianchang_01");
        LoadManager.getInstance():Load(Lua_LoadFileType.UNITY3D, url, self, self.onStartEffLoaded, self.onStartEffLoaded, nil, 999, 30);
    end
    if self.bTimer ~= true then
        self.bTimer = true;
        self.curFrameScript = getTime();
        Lua_CommonUtil.addTimer("frameScript", 0.01, self, self.frameScript);
    end
end

function Lua_SingleFightPlayer:onStartEffLoaded()
    self.bStartEffLoading = false;
    self.intervalTime = 0;
end

function Lua_SingleFightPlayer:endTimer()
    if self.bTimer == true then
        self.bTimer = false;
        Lua_CommonUtil.removeTimer(self, self.frameScript);
    end
end

function Lua_SingleFightPlayer:frameScript()
    if self._recordVo == nil then
        self:dispose();
        return;
    end

    local t = getTime();
    local dt = (t - self.curFrameScript) * self.playSpeedRate; -- 等比加速
    self.curFrameScript = t;
    self:checkScene();
    self:executeCurAction(dt);
end

function Lua_SingleFightPlayer:checkScene()
    if self.bSceneLoad == true then
        return;
    end

    if self.loadSceneId == self._sceneId then
        return;
    end
    self.loadSceneId = self._sceneId;

    if self.bRunBack == true then -- 后台
        -- LogManager.showLog("播放器 checkScene 1 "..self._recordId.." 场景ID "..self._sceneId)
        Scene3DManager.instance:createSceneRoot(self._sceneId, self._recordId); -- 创建模型放置的root节点
    else
        -- LogManager.showLog("播放器 checkScene 2 "..self._recordId.." 场景ID "..self._sceneId)
        Scene3DManager.instance:OpenSceneBySceneId(self._sceneId, self._recordId, self, self.onSceneShow, nil); -- 加载场景
    end
end

function Lua_SingleFightPlayer:onSceneShow()
    if DataCenter.fightData:isCurPlayFight(self._recordId) ~= true then
        -- LogManager.showLog("播放器 onSceneShow 1 ")
        return;
    end

    -- 布阵开始战斗后立即关闭，这时战斗场景被关闭，因此设置该战斗后台播放
    if not Scene3DManager.instance:isShowingScene(self._sceneId, self._recordId) then
        -- LogManager.showLog("播放器 onSceneShow 2 ")
        self._recordVo.bPlayedStartEff = true;
        DataCenter.fightData:hideFightToBackStageByRecordId(self._recordId);
        return;
    end

    self.bStopPlay = false;
    self.bSceneLoad = true;
    if self:showBornEffect() ~= true and self.showThingsArr ~= nil and self.bRunBack ~= true then
        local len = self.showThingsArr:getSize();
        for i = 1, len do
            local roleId = self.showThingsArr[i];
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing ~= nil then
                thing.player:setActive(false);
            end
        end
    end

    if self.bRunBack ~= true then
        DataCenter.fightData:hideOtherFightToBackStage(self._recordId);
        Lua_WindowUtil.openWin("Lua_FightMainWindow", Lua_WinParam.ACTION_OPEN, self._recordId);
    end
    Lua_MsgUtil.callFunc("fight_enter_scene", self._recordId);
end

-- 执行当前行动
function Lua_SingleFightPlayer:executeCurAction(dt)
    if self.bSkipPlay then
        self:onSkipHandle();
        return;
    end

    if self.bOpenResultWin == true and self.intervalTime > 0 then -- 移除播放器容错处理
        self.intervalTime = self.intervalTime - dt;
        if self.intervalTime <= 0 then
            self:endTimer();
            self:removeFightPlay();
            return;
        end
    end

    if self.isPlayEnd == true then
        return;
    end

    if self.bStartEffLoading == true and self.intervalTime > 0 then -- 加载进场特效中
        self.intervalTime = self.intervalTime - dt;
        if self.intervalTime <= 0 then
            self.bStartEffLoading = false;
            return;
        end
    end

    if self.showThingIntervalTime > 0 then
        self.showThingIntervalTime = self.showThingIntervalTime - dt;
        if self.showThingIntervalTime <= 0 then
            self.showThingIntervalTime = 0;
            self:showBornEffect();
        end
    end

    if self.skillAdditionalTime > 0 then
        self.skillAdditionalTime = self.skillAdditionalTime - dt;
    end

    if self.intervalTime > 0 then -- 等待
        self.intervalTime = self.intervalTime - dt;
        return;
    end

    if self.startEffectTime > 0 then -- 开始动画播放时间
        self.startEffectTime = self.startEffectTime - dt;
        if self.startEffectTime <= 0 then
            self.startEffectTime = 0;
            self:showBornEffect();
        end
        return;
    end

    if self.approachUnitCount > 0 then -- 初始化阵型
        if self.bRunBack == true then
            self.approachUnitCount = 0;
            self:showBornEffect();
        end
        return;
    end

    if self.bPlaySkill == true then -- 技能释放期间
        if self.intervalTime <= 0 then -- 走完配置的技能最长时间
            self:attackComplete();
        elseif self.skillAdditionalTime <= 0 then
            self:completeSkill();
        end
        return;
    end

    if self.needReplay == true then
        self:replay();
        return;
    end

    if self.bOpenResultWin == true or self.bStopPlay == true then -- 结束界面退出
        return;
    end

    if self._endDelayTime > 0 then
        self._endDelayTime = self._endDelayTime - dt;
        if self._endDelayTime <= 0 then -- 移除
            self:playEnd();
        end
        return;
    end

    if self.curActionInfo == nil then
        if self.bCheckNextAction == true then
            self.curActionInfo, self.curAction = self._recordVo:getNextActionInfo(self.curRotation, self.curSession, self.curRound, self.curAction);
            if self.curActionInfo ~= nil then
                self.curSession = self.curActionInfo.session; -- 场次
                self.curRotation = self.curActionInfo.rotation; -- 轮次
                self.curRound = self.curActionInfo.round; -- 回合
            end
        else
            self.curActionInfo = self._recordVo:getActionInfo(self.curRotation, self.curSession, self.curRound, self.curAction);
        end
    end

    if self.curActionInfo == nil then
        if self._recordVo.isComplete == true then -- 下发完成
            self._recordVo:checkResult();
            self._endDelayTime = DataCenter.fightUtil.playOverIntervalTime / self.playSpeed;
            if self._endDelayTime <= 0 then
                self:playEnd();
            end
        end
        return;
    end

    -- print("self.curRotation", self.curRotation, "self.curSession", self.curSession, "self.curRound", self.curRound, "self.curAction", self.curAction)
    self.bCheckNextAction = false;
    if self.bRunBack ~= true then
        Lua_MsgUtil.callFunc("fight_execute_action", self.curActionInfo);
        if self.bStopPlay then -- 剧情播放控制暂停了
            return;
        end
    end

    self.bDoFlash = false;
    self.skillHurtTimes = 0;
    local curType = self.curActionInfo.type;
    if curType == 1 then -- 单位进场
        self.approachUnitCount = 0; -- 进场数量
        local list = self.curActionInfo.fighter_list
        table.sort( list, function(a, b) return a.aod_type < b.aod_type end )
        self:initFormation(list);
        self:checkInitRoleSkill();
        if self.startEffectTime == -1 then
            if self.templateId == Lua_FightDefine.TEMPLATE_ID_KUNXU then
                self.startEffectTime = DataCenter.kunXuHolyLandUtil.startFightTime;
            else
                self.startEffectTime = DataCenter.fightUtil.startEffectTime;
            end
        end
        self:setCurActionCompleted();
    elseif curType == 2 then -- 释放技能（对目标或友军造成的伤害或回血），伤害值和技能目标在hurt_list字段
        -- self:calculateMaxHundun();
        self:releaseSkillEff();
    elseif curType == 3 then -- buff变化（新增或移除），新增或者移除的buff在buff_list字段
        self:refreshBuffAndHurt();
    elseif curType == 4 then -- buff伤害（buff扣血或回血），伤害值和目标在hurt_list字段(buff复活用)
        self:refreshBuffAndHurt();
    elseif curType == 5 then -- 不可行动（受buff控制效果导致的不可行动，buff_id为对应的控制效果）
        if self.bRunBack ~= true then
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, tonumber(tostring(self.curActionInfo.fighter_id)));
            if thing ~= nil then
                thing.player:showStateText(0, nil, "number", "xui_wfxd_word_0", 0, 6); -- 飘字不可行动
                self.intervalTime = 1.2;
            end
        end
        self:refreshBuffAndHurt();
    elseif curType == 6 then -- 攻方退场
        self:removeThingsByFormationType(Lua_FightDefine.FIGHT_ATTACKER);
    elseif curType == 7 then -- 守方退场
        self:removeThingsByFormationType(Lua_FightDefine.FIGHT_DEFENDER);
    elseif curType == 8 then -- 更新主角技能冷却
        local id = tonumber(tostring(self.curActionInfo.fighter_id))
        local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, id);
        if thing and thing.formationType == Lua_FightDefine.FIGHT_ATTACKER then
            self.artifactEnergy = self.curActionInfo.energy;
        end
        self.artifactSkillDic[id] = self.curActionInfo.skill_list;
        if self.bRunBack ~= true then
            Lua_MsgUtil.callFunc("fight_roleSkill_update");
        end
        self:setCurActionCompleted();
    elseif curType == 9 then -- 站位变化（数据在fighter_list）
        self:updatePosition();
    elseif curType == 10 then -- 销毁场地效果，数据在site_effect
        if self:updateFormationEff(self.curActionInfo.site_effect, false) then
            self.intervalTime = 0.5;
        end
        self:refreshBuff(self.curActionInfo.buff_list);
        self:setCurActionCompleted();
    elseif curType == 11 then -- 召唤单位进场
        self:reliveUnits(self.curActionInfo.fighter_list);
        self:setCurActionCompleted();
    elseif curType == 12 then -- 触发的剧情事件，下一节点是事件效果
        if self.bRunBack ~= true then
            Lua_MsgUtil.callFunc("fight_plotEvent_update", self.curActionInfo.event_id);
        end
        self:setCurActionCompleted();
    else
        self:refreshBuffAndHurt();
    end
end

function Lua_SingleFightPlayer:setCurActionCompleted()
    self.bCheckNextAction = true;
    self.curActionInfo = nil;
end

function Lua_SingleFightPlayer:checkInitRoleSkill()
    if not self.bRunBack then
        local info;
        local rot = self.curRotation;
        local session = self.curSession;
        local round = self.curRound;
        local action = self.curAction;
        local bUpdate = false;
        while true do
            info, action = self._recordVo:getNextActionInfo(rot, session, round, action);
            if info == nil or info.rotation > self.curRotation then
                break;
            elseif info.type == 8 then
                local id = tonumber(tostring(info.fighter_id));
                if self.artifactSkillDic[id] == nil then
                    local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, id);
                    if thing and thing.formationType == Lua_FightDefine.FIGHT_ATTACKER then
                        self.artifactEnergy = info.energy;
                    end
                    self.artifactSkillDic[id] = info.skill_list;
                end
                bUpdate = true;
            end
            rot = info.rotation; -- 轮次
            session = info.session; -- 场次
            round = info.round; -- 回合
        end
        if bUpdate then
            Lua_MsgUtil.callFunc("fight_roleSkill_update");
        end
    end
end

function Lua_SingleFightPlayer:initFormation(list)
    if list == nil or #list <= 0 then
        return;
    end

    self.fighter_list = list
    local attackerTeamIdx;
    local defenderTeamIdx;
    for i = 1, #list do
        local obj = list[i];
        local roleId = tonumber(tostring(obj.id));
        local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
        local data = self:generateAnimalData(obj);
        if thing == nil then -- 创建
            ThingUtil.instance:addAnimal(data, false);
            if not self.bRunBack then
                if tonumber(tostring(data.hp)) > 0 then
                    self.approachUnitCount = self.approachUnitCount + 1;
                    if self.showThingsArr == nil then 
                        self.showThingsArr = Array(); 
                    end
                    self.showThingsArr:add(roleId);
                end
            end

            self._recordVo:setMaxHp(obj.aod_type, tonumber(tostring(data.maxHp)));
            if obj.aod_type == Lua_FightDefine.FIGHT_ATTACKER and attackerTeamIdx == nil then
                attackerTeamIdx = obj.rotation_index;
            elseif obj.aod_type == Lua_FightDefine.FIGHT_DEFENDER and defenderTeamIdx == nil then
                defenderTeamIdx = obj.rotation_index;
            end
        else
            if obj.type == 3 then
                local artifactId = tonumber(tostring(obj.artifact_id))
                if thing.player.data.artifact_id ~= artifactId then
                    thing.player.data = data;

                    thing.player:resetBodyUrl();
                    thing.player:loadBody();

                    local pos = thing.player.currentPoint;
                    ThingUtil.instance:addSceneEffect(self._sceneId, self._recordId, "hx_jnsf_01", pos.x, pos.y, pos.z, thing.playerId);
                else
                    thing.player.data = data;
                end
            else
                thing.player.data = data;
            end
        end
    end

    if attackerTeamIdx then
        self.attackerTeamIdx = attackerTeamIdx;
    end
    if defenderTeamIdx then
        self.defenderTeamIdx = defenderTeamIdx;
    end
    self._recordVo:checkHealth();
    self._recordVo:checkDeath();
    self._recordVo:checkConnect();
    if not self.bRunBack then
        Lua_MsgUtil.callFunc("fight_formation_change");
    end
end

function Lua_SingleFightPlayer:generateAnimalData(obj)
    if obj == nil then
        return nil;
    end

    local data;
    if obj.type == 0 then -- 英雄
        data = HeroData();
        data.kind = Lua_ThingDefine.HERO;
        data.isShowName = true;
        data.isShowHpBar = true;
        data.isShowBuffIcon = true;
        data.shenbingId = tonumber(tostring(obj.weapon_id));
        local configData = Lua_HeroConfigData.getConfig(tonumber(tostring(obj.sort_id)));
        if configData ~= nil then
            data.lysisCountTime = configData.lysisCountTime;
        end
    elseif obj.type == 1 or obj.type == 4 then -- 1怪物 4召唤物
        data = MonsterData();
        data.kind = Lua_ThingDefine.MONSTER;
        data.isShowName = true;
        data.isShowHpBar = true;
        data.isShowBuffIcon = true;
        data.isSummoned = obj.type == 4; -- 召唤物
        data.shenbingId = tonumber(tostring(obj.weapon_id));
        local configData = Lua_MonsterConfigData.getMonsterConfig(tonumber(tostring(obj.sort_id)));
        if configData ~= nil then
            configData = Lua_HeroConfigData.getConfig(configData.heroId);
            if configData ~= nil then
                data.lysisCountTime = configData.lysisCountTime;
            end
        end
    elseif obj.type == 2 then -- boss 
        data = MonsterData();
        data.kind = Lua_ThingDefine.MONSTER;
        data.isShowName = true;
        data.isShowHpBar = true;
        local configData = Lua_MonsterConfigData.getMonsterConfig(tonumber(tostring(obj.sort_id)));
        if configData ~= nil then
            configData = Lua_HeroConfigData.getConfig(configData.heroId);
            if configData ~= nil then
                data.lysisCountTime = configData.lysisCountTime;
            end
        end
    elseif obj.type == 3 then -- zhujue
        data = UsData();
        data.kind = Lua_ThingDefine.PLAYER;
        data.isShowName = false;
        data.artifact_id = tonumber(tostring(obj.artifact_id));
    elseif obj.type == 5 or obj.type == 6 then -- 木桩boss
        data = MonsterData();
        data.kind = Lua_ThingDefine.MONSTER;
        data.isShowName = true;
        data.isShowHpBar = false;
        data.isShowBuffIcon = false;
        local configData = Lua_MonsterConfigData.getMonsterConfig(tonumber(tostring(obj.sort_id)));
        if configData ~= nil then
            configData = Lua_HeroConfigData.getConfig(configData.heroId);
            if configData ~= nil then
                data.lysisCountTime = configData.lysisCountTime;
            end
        end
    end
    
    data.unitType = obj.type;
    data.roleId = tonumber(tostring(obj.id));
    data.objId = tonumber(tostring(obj.sort_id));
    data.fightRecordId = self._recordId;
    data.formationType = obj.aod_type;
    data.formationPos = obj.pos;
    data.lvl = obj.level;
    if obj.type ~= 3 then
        data.star = DataCenter.heroUtil:getHeroAwakenStarId(obj.quality, obj.quality_star);
    end
    local hp = tonumber(tostring(obj.health));
    local maxHp = tonumber(tostring(obj.max_health));
    data.maxHp = (maxHp == nil or maxHp < hp) and hp or maxHp;
    data.hp = hp;
    data.sceneId = self._sceneId;
    data.aniSpeed = self.playSpeed;
    data.aniSpdRate = self.playSpeedRate;
    data.fashion_id = tonumber(tostring(obj.fashion_id));
    -- local hudun = 0;
    data:resetBuffDataList();
    for i = 1, #obj.buff_list do -- buff
        data:addTemporaryBuffId(obj.buff_list[i].sort_id, obj.buff_list[i].finish_round);
        -- hudun = hudun + tonumber(tostring(obj.buff_list[i].shield_value));
    end
    -- data.hudun = hudun;
    -- data.maxHudun = hudun;
    data.showShadow = false;
    data.bPlanarShadow = false;
    data.campPosId = self._recordVo:getCampPosByType(obj.aod_type);
    data.dir = self._recordVo:getDirByType(obj.aod_type);
    data.px = self._recordVo:getPosxByIdx(obj.aod_type, obj.pos);
    data.py = self._recordVo:getPosyByIdx(obj.aod_type, obj.pos);
    data.pz = self._recordVo:getPoszByIdx(obj.aod_type, obj.pos);
    data:refreshShowPos();
    return data;
end

function Lua_SingleFightPlayer:releaseSkillEff()
    if self.curActionInfo == nil or self.curActionInfo.type ~= 2 then
        return;
    end

    self.bPlaySkill = true;
    self.skillHurtStep = 0;
    self.skillHurtTimes = 1;
    self.skillAdditionalTime = 0; -- 技能额外时间
    local skillId = self.curActionInfo.skill_id;
    local skillCfg = Lua_SkillConfigData.getConfigData(skillId);
    if skillCfg == nil then
        self:attackComplete();
        return;
    end
    
    if skillCfg.artifactId > 0 then
        local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, self.curActionInfo.fighter_id)
        if thing ~= nil then
            local artifactId = thing.data.artifact_id
            if artifactId ~= skillCfg.artifactId then
                self.changeArtifactId = thing.data.artifact_id;
                thing.data.artifact_id = skillCfg.artifactId;
                if not self.bRunBack then
                    thing.player:resetBodyUrl();
                    thing.player:loadBody();
                end

                local pos = thing.player.currentPoint;
                ThingUtil.instance:addSceneEffect(self._sceneId, self._recordId, "hx_jnsf_01", pos.x, pos.y, pos.z, thing.playerId);
            end

            if not self.bRunBack and thing.kind == Lua_ThingDefine.PLAYER then
                if thing.formationType == Lua_FightDefine.FIGHT_ATTACKER then
                    Lua_MsgUtil.callFunc("FIGHT_ARTIFACT_RELEASE_A", skillId);
                -- else
                --     Lua_MsgUtil.callFunc("FIGHT_ARTIFACT_RELEASE_D", skillId);
                end
            end
        else
            LogManager.showLog("fight thing = nil:"..self.curActionInfo.fighter_id.." , "..self._recordId.." , "..skillId);
        end
    end
    
    self.skillHurtCount = 0; -- 技能伤害
    self.skillHurtTimes = skillCfg.showHurtTimes; -- 播放伤害总次数
    self.intervalTime = skillCfg.countTime;
    self.bDoFlash = skillCfg.bShowFlash;
    if self.bRunBack ~= true then
        self.intervalTime = self.intervalTime + 2; -- 非后台则增加2s容错时间
        self.attackVo = AttackVO(self._sceneId, self._recordId);
        self.attackVo:addLuaCallBack(self, self.attackedCallBack, self.attackComplete, self.attackPointCallBack, self.shutterCallBack);
        self.attackVo.attackerId = tonumber(tostring(self.curActionInfo.fighter_id));
        self.attackVo.skillId = skillId;
        self.attackVo.speed = self.playSpeed;
        self.attackVo.speedRate = self.playSpeedRate;
        self:ansysisSkillTargets(self.curActionInfo.multi_attack, skillCfg);
        self.attackVo:start();
        self:startGuard(self.curActionInfo.guard_pos);
        self:startBuffGuard(self.curActionInfo.multi_attack);
    end

    -- 刷新技能播放前buff
    self:refreshPreBuff(self.curActionInfo.buff_list);
end

-- 伤害点
function Lua_SingleFightPlayer:attackedCallBack(rid, sectionIdx, hurtIdx, isTarget)
    if self.curActionInfo == nil or self.curActionInfo.multi_attack == nil then
        return; 
    end

    sectionIdx = sectionIdx + 1; --C#是0开始
    if sectionIdx <= 0 or sectionIdx > #self.curActionInfo.multi_attack then
        return;
    end

    rid = tonumber(tostring(rid));
    hurtIdx = hurtIdx + 1; --C#是0开始
    local changeVal = self.skillHurtCount;
    local attackerId = tonumber(tostring(self.curActionInfo.fighter_id));

    -- 伤害飘字
    local attack_list = self.curActionInfo.multi_attack[sectionIdx].attack_list;
    if attack_list ~= nil and #attack_list > 0 then
        for i = 1, #attack_list do
            local hurtObj = attack_list[i];
            local roleId = tonumber(tostring(hurtObj.id));
            if rid == roleId and isTarget == hurtObj.is_target then
                local thing;
                local health = tonumber(tostring(hurtObj.health));
                local hurtList = hurtObj.hurt_list;
                hurtObj.showhurtidx = hurtIdx; -- 存储已表现的伤害索引
                -- if hurtIdx == 1 then -- 初始化护盾  -- 暂时屏蔽护盾进度条，就不刷新护盾值
                --     thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                --     if thing ~= nil then
                --         thing.player:updateHudun(tonumber(tostring(hurtObj.shield_value)));
                --     end
                -- end

                if hurtIdx == 1 and hurtObj.buff_sort > 0 then -- 伤害来源buff
                    thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                    if thing ~= nil then
                        local buffCfg = Lua_BuffConfigData.getConfigData(hurtObj.buff_sort);
                        if buffCfg.buffType ~= 7 and not strIsNullOrEmpty(buffCfg.hangerId) then -- 场地特效不显示buff效果
                            -- print("showBuffEff ",buffCfg.hangerId)
                            thing.player:showBuffEff(buffCfg.hangerId);
                        end
                    end
                end

                if #hurtList >= hurtIdx then
                    if thing == nil then
                        thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                    end

                    if thing ~= nil then
                        -- 每段伤害的buff列表
                        local buff_list = hurtList[hurtIdx].buff_list;
                        self:refreshPreBuff(buff_list);
                        self:refreshBuff(buff_list);

                        local hurt = hurtList[hurtIdx].hurt;
                        -- print("伤害点 attackedCallBack", rid, sectionIdx, hurtIdx, #hurt);
                        for k = 1, #hurt do
                            local obj = hurt[k];
                            local val = tonumber(tostring(obj.value));
                            if thing.data.unitType == 5 or thing.data.unitType == 6 then
                                self._recordVo.bossHurt = val
                            end
                            
                            -- print("lua attackedCallBack", rid, sectionIdx, hurtIdx, isTarget, obj.type, val);
                            local dt = thing.player:updateHealthChange(obj.type, val, health, self.bDoFlash, not self.bRunBack);
                            if obj.type ~= 6 and changeVal ~= nil and roleId ~= attackerId then
                                changeVal = changeVal + Mathf.Abs(val)
                            end

                            if dt > self.intervalTime then
                                self.intervalTime = dt;
                            end
                        end
                    end
                end
                -- break;
            end
        end
    end

    -- 溅射伤害
    local sputter_list = self.curActionInfo.multi_attack[sectionIdx].sputter_list;
    if sputter_list ~= nil and #sputter_list > 0 then
        for i = 1, #sputter_list do
            local fromId = tonumber(tostring(sputter_list[i].from_id));
            if rid == fromId then
                local sputterHurtList = sputter_list[i].sputter_hurt;
                for j = 1, #sputterHurtList do
                    local hurtObj = sputterHurtList[j];
                    local hurt_list = hurtObj.hurt_list;
                    if hurt_list ~= nil and #hurt_list >= hurtIdx then
                        local thing;
                        local roleId = tonumber(tostring(hurtObj.id));
                        local health = tonumber(tostring(hurtObj.health));
                        hurtObj.showhurtidx = hurtIdx; -- 存储已表现的伤害索引
                        -- if hurtIdx == 1 then -- 初始化护盾  -- 暂时屏蔽护盾进度条，就不刷新护盾值
                        --     thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                        --     if thing ~= nil then
                        --         thing.player:updateHudun(tonumber(tostring(hurtObj.shield_value)));
                        --     end
                        -- end

                        local bShowHurt = self.attackVo:showShutterHurtEff(fromId, roleId, sectionIdx, hurtIdx); -- 播放溅射特效
                        if bShowHurt then
                            local isTarget = hurtObj.is_target == 0 or hurtObj.is_target == 1;
                            if thing == nil then
                                thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                            end

                            if thing ~= nil then
                                local hurt = hurt_list[hurtIdx].hurt;
                                for k = 1, #hurt do
                                    local obj = hurt[k];
                                    local val = tonumber(tostring(obj.value));
                                    if thing.data.unitType == 5 or thing.data.unitType == 6 then
                                        self._recordVo.bossHurt = val
                                    end

                                    local dt = 0;
                                    if isTarget then -- 还是技能伤害  
                                        dt = thing.player:updateHealthChange(obj.type, val, health, self.bDoFlash, not self.bRunBack);
                                        if obj.type ~= 6 and changeVal ~= nil and roleId ~= attackerId then
                                            changeVal = changeVal + Mathf.Abs(val)
                                        end
                                    else 
                                        dt = thing.player:updateHealthChange(obj.type, val, health, false, not self.bRunBack);
                                    end

                                    if dt > self.intervalTime then
                                        self.intervalTime = dt;
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end

    if not self.bRunBack and changeVal ~= self.skillHurtCount then
        self.skillHurtCount = changeVal;
        Lua_MsgUtil.callFunc("skillAttackHurtUpdate", changeVal);
    end
end

-- 伤害点回调
function Lua_SingleFightPlayer:attackPointCallBack(sectionIdx, hurtIdx, bLast) 
    if self.curActionInfo == nil or self.curActionInfo.multi_attack == nil then
        return; 
    end
    
    self.skillHurtStep = self.skillHurtStep + 1;
    if self.skillHurtStep == 1 then -- 首段伤害先判断复活，用于复活技能
        self:reliveUnits(self.curActionInfo.fighter_list);
    end

    if not bLast then
        return;
    end

    sectionIdx = sectionIdx + 1;
    if sectionIdx > #self.curActionInfo.multi_attack then
        return;
    end

    -- 最后一段时候刷新该段伤害列表其他飘字
    local changeVal = self.skillHurtCount;
    local attack_list = self.curActionInfo.multi_attack[sectionIdx].attack_list;
    if attack_list ~= nil and #attack_list >= 0 then
        for i = 1, #attack_list do
            local thing;
            local hurtObj = attack_list[i];
            local health = tonumber(tostring(hurtObj.health));
            local roleId = tonumber(tostring(hurtObj.id));
            local hurtList = hurtObj.hurt_list;
            local isTarget = hurtObj.is_target == 0 or hurtObj.is_target == 1;
            local showHurtIdx = hurtObj.showhurtidx; -- 已表现的伤害索引
            hurtObj.showhurtidx = #hurtList;
            if showHurtIdx == nil then 
                showHurtIdx = 0; 
            end

            if showHurtIdx == 0 and hurtObj.buff_sort > 0 then -- 伤害来源buff
                thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                if thing ~= nil then
                    local buffCfg = Lua_BuffConfigData.getConfigData(hurtObj.buff_sort);
                    if buffCfg.buffType ~= 7 and not strIsNullOrEmpty(buffCfg.hangerId) then -- 场地特效不显示buff效果
                        -- print("showBuffEff ",buffCfg.hangerId)
                        thing.player:showBuffEff(buffCfg.hangerId);
                    end
                end
            end

            -- print("attackPointCallBack", showHurtIdx, #hurtList)
            for j = showHurtIdx + 1, #hurtList do
                -- if j == 1 then -- 暂时屏蔽护盾进度条，就不刷新护盾值
                --     thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                --     if thing ~= nil then
                --         thing.player:updateHudun(tonumber(tostring(hurtObj.shield_value)));
                --     end
                -- end

                if thing == nil then
                    thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                end

                if thing ~= nil then
                    local hurt = hurtList[j].hurt;
                    -- print("最后一段时候刷新该段伤害列表其他飘字 attackPointCallBack ", j, #hurt);
                    for k = 1, #hurt do
                        local obj = hurt[k];
                        local val = tonumber(tostring(obj.value));
                        if thing.data.unitType == 5 or thing.data.unitType == 6 then
                            self._recordVo.bossHurt = val
                        end

                        local dt = 0;
                        if isTarget then -- 还是技能伤害
                            dt = thing.player:updateHealthChange(obj.type, val, health, self.bDoFlash, not self.bRunBack);
                            if obj.type ~= 6 and changeVal then
                                changeVal = changeVal + Mathf.Abs(val)
                            end
                        else 
                            dt = thing.player:updateHealthChange(obj.type, val, health, false, not self.bRunBack);
                        end

                        if dt > self.intervalTime then
                            self.intervalTime = dt;
                        end
                    end
                end
            end
        end
    end

    -- 溅射伤害
    local sputter_list = self.curActionInfo.multi_attack[sectionIdx].sputter_list;
    if sputter_list ~= nil and #sputter_list > 0 then
        for i = 1, #sputter_list do
            local fromId = tonumber(tostring(sputter_list[i].from_id));
            local sputter_hurt = sputter_list[i].sputter_hurt;
            if sputter_hurt ~= nil and #sputter_hurt > 0 then
                for j = 1, #sputter_hurt do
                    local thing;
                    local hurtObj = sputter_hurt[i];
                    local health = tonumber(tostring(hurtObj.health));
                    local roleId = tonumber(tostring(hurtObj.id));
                    local hurtList = hurtObj.hurt_list;
                    local isTarget = hurtObj.is_target == 0 or hurtObj.is_target == 1;
                    local showHurtIdx = hurtObj.showhurtidx; -- 已表现的伤害索引
                    hurtObj.showhurtidx = #hurtList;
                    if showHurtIdx == nil then 
                        showHurtIdx = 0; 
                    end

                    for j = showHurtIdx + 1, #hurtList do
                        -- if j == 1 then -- 暂时屏蔽护盾进度条，就不刷新护盾值
                        --     thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                        --     if thing ~= nil then
                        --         thing.player:updateHudun(tonumber(tostring(hurtObj.shield_value)));
                        --     end
                        -- end

                        if thing == nil then
                            thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                        end

                        if thing ~= nil then
                            local hurt = hurtList[j].hurt;
                            for k = 1, #hurt do
                                local obj = hurt[k];
                                local val = tonumber(tostring(obj.value));
                                if thing.data.unitType == 5 or thing.data.unitType == 6 then
                                    self._recordVo.bossHurt = val
                                end

                                local dt = 0;
                                if isTarget then -- 还是技能伤害
                                    dt = thing.player:updateHealthChange(obj.type, val, health, self.bDoFlash, not self.bRunBack);
                                    if obj.type ~= 6 and changeVal then
                                        changeVal = changeVal + Mathf.Abs(val)
                                    end
                                else 
                                    dt = thing.player:updateHealthChange(obj.type, val, health, false, not self.bRunBack);
                                end

                                if dt > self.intervalTime then
                                    self.intervalTime = dt;
                                end
                            end
                        end
                    end
                end
            end
        end
    end

    if not self.bRunBack and changeVal ~= self.skillHurtCount then
        self.skillHurtCount = changeVal;
        Lua_MsgUtil.callFunc("skillAttackHurtUpdate", changeVal);
    end

    -- 完成
    if sectionIdx >= #self.curActionInfo.multi_attack then
        self:refreshActionHurt();
    end
end

-- 溅射延迟回调触发伤害
function Lua_SingleFightPlayer:shutterCallBack(fromId, rid, sectionIdx, hurtIdx)
    if self.curActionInfo == nil or sectionIdx > #self.curActionInfo.multi_attack then
        return;
    end

    local sputter_list = self.curActionInfo.multi_attack[sectionIdx].sputter_list;
    if sputter_list ~= nil and #sputter_list > 0 then
        rid = tonumber(tostring(rid));
        fromId = tonumber(tostring(fromId));
        for i = 1, #sputter_list do
            local fId = tonumber(tostring(sputter_list[i].from_id));
            if fromId == fId then
                local sputterHurtList = sputter_list[i].sputter_hurt;
                for j = 1, #sputterHurtList do
                    local hurtObj = sputterHurtList[j];
                    local roleId = tonumber(tostring(hurtObj.id));
                    if roleId == rid then
                        local changeVal = self.skillHurtCount;
                        local hurt_list = hurtObj.hurt_list;
                        if hurt_list ~= nil and #hurt_list >= hurtIdx then
                            local isTarget = hurtObj.is_target == 0 or hurtObj.is_target == 1;
                            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                            if thing ~= nil then
                                local health = tonumber(tostring(hurtObj.health));
                                local hurt = hurt_list[hurtIdx].hurt;
                                for k = 1, #hurt do
                                    local obj = hurt[k];
                                    local val = tonumber(tostring(obj.value));
                                    if thing.data.unitType == 5 or thing.data.unitType == 6 then
                                        self._recordVo.bossHurt = val
                                    end

                                    local dt = 0;
                                    if isTarget then -- 还是技能伤害
                                        dt = thing.player:updateHealthChange(obj.type, val, health, self.bDoFlash, not self.bRunBack);
                                        if obj.type ~= 6 and changeVal then
                                            changeVal = changeVal + Mathf.Abs(val)
                                        end
                                    else 
                                        dt = thing.player:updateHealthChange(obj.type, val, health, false, not self.bRunBack);
                                    end

                                    if dt > self.intervalTime then
                                        self.intervalTime = dt;
                                    end
                                end
                            end
                        end

                        if changeVal ~= self.skillHurtCount and not self.bRunBack then
                            self.skillHurtCount = changeVal;
                            Lua_MsgUtil.callFunc("skillAttackHurtUpdate", changeVal);
                        end
                        return;
                    end
                end
            end
        end
    end
end

-- 结束
function Lua_SingleFightPlayer:attackComplete()
    -- print("attackComplete ", self._recordId, self.curRotation, self.curRound, self.curAction);
    if self.isDispose == true or self._recordVo == nil or self.curActionInfo == nil then
        return;
    end
    
    local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, self.curActionInfo.fighter_id)
    if thing ~= nil then
        if self.changeArtifactId and self.changeArtifactId > 0 then
            -- thing.data.artifact_id = self.changeArtifactId;
            -- if not self.bRunBack then
            --     thing.player:resetBodyUrl();
            --     thing.player:loadBody();
            -- end
            thing.player:setAction(0, nil, -1, false, 0, nil, false);
            self.changeArtifactId = 0;

            -- local pos = thing.player.currentPoint;
            -- ThingUtil.instance:addSceneEffect(self._sceneId, self._recordId, "hx_jnsf_01", pos.x, pos.y, pos.z, thing.playerId);
        end

        if not self.bRunBack and thing.kind == Lua_ThingDefine.PLAYER then
            if thing.formationType == Lua_FightDefine.FIGHT_ATTACKER then
                Lua_MsgUtil.callFunc("FIGHT_ARTIFACT_COMPLETE_A", self.curActionInfo.skill_id);
            else
                Lua_MsgUtil.callFunc("FIGHT_ARTIFACT_COMPLETE_D", self.curActionInfo.skill_id);
            end
        end
    else
        LogManager.showLog("2fight thing = nil:"..self.curActionInfo.fighter_id.." , "..self._recordId.." , "..(self.changeArtifactId ~= nil and self.changeArtifactId or 0));
    end
    
    self:endGuard(self.curActionInfo.guard_pos);
    self:endBuffGuard(self.curActionInfo.multi_attack);

    -- 首段伤害先判断复活，用于复活技能
    if self.skillHurtStep == 0 then
        self:reliveUnits(self.curActionInfo.fighter_list);
    end

    -- print("attackComplete", self.skillHurtStep)
    if self.skillHurtStep < 999 then -- 防止特殊情况未刷新
        self:refreshActionHurt();
    end
    
    -- if not self.bRunBack then
    --     Lua_MsgUtil.callFunc("skillAttackHurtUpdate", self.skillHurtCount);
    -- end
    
    if self.attackVo ~= nil then
        self.attackVo:dispose();
        self.attackVo = nil;
    end

    if self.skillAdditionalTime <= 0 then
        self:completeSkill();
    end
end

function Lua_SingleFightPlayer:completeSkill()
    -- 下一行动
    self.bPlaySkill = false;
    self.intervalTime = 0;
    self.skillHurtCount = 0;
    self.skillHurtTimes = 0;
    self.skillAdditionalTime = 0;
    self:setCurActionCompleted();
end

-- 计算最大护盾值
-- function Lua_SingleFightPlayer:calculateMaxHundun()
--     if self.curActionInfo == nil then return; end
--     local list = self.curActionInfo.buff_list;
--     for i = 1, #list do
--         local roleId = tonumber(tostring(list[i].id));
--         local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
--         if thing ~= nil then
--             local hudun = 0;
--             local buffList = list[i].buff_list;
--             for j = 1, #buffList do
--                 hudun = hudun + tonumber(tostring(buffList[j].shield_upper));
--             end
--             thing.player:updateHudun(maxHudun);
--         end
--     end
-- end

function Lua_SingleFightPlayer:refreshPreBuff(bufList)
    if not self.bRunBack then
        if bufList ~= nil and #bufList > 0 then
            for i = 1, #bufList do
                local pre_buff = bufList[i].pre_buff;
                if pre_buff ~= nil and #pre_buff > 0 then
                    local roleId = tonumber(tostring(bufList[i].id));
                    local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                    if thing ~= nil then
                        for j = 1, #pre_buff do
                            local dt = thing.player:showBuffText(pre_buff[j]);
                            if dt > self.intervalTime then
                                self.intervalTime = dt;
                            end
                        end
                    end
                end
            end
        end
    end
end

function Lua_SingleFightPlayer:refreshBuff(list)
    if list ~= nil and #list > 0 then
        for i = 1, #list do
            local roleId = tonumber(tostring(list[i].id));
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing ~= nil then
                local buffId = nil;
                local hangerId = nil;
                if self.bBuffGuard then
                    if self.buffGuardSrcDic ~= nil and self.buffGuardSrcDic[roleId] ~= nil then
                        buffId = self.buffGuardSrcDic[roleId];
                    elseif self.buffGuardTrgetDic ~= nil and self.buffGuardTrgetDic[roleId] ~= nil then
                        hangerId = self.buffGuardTrgetDic[roleId];
                    end 
                end
                -- print("refreshBuff", roleId, buffId, hangerId);

                -- local hudun = 0;
                local bBuffGuardSrc = false;
                local buffList = list[i].buff_list;
                thing.player:resetBuffDataList();
                for j = 1, #buffList do
                    -- print("1buffGuardSrcDic ", roleId, buffId, buffList[j].sort_id);
                    if buffId ~= nil and buffList[j].sort_id == buffId then bBuffGuardSrc = true; end
                    thing.player:addTemporaryBuff(buffList[j].sort_id, buffList[j].finish_round, buffId == nil or buffId ~= buffList[j].sort_id);
                    -- hudun = hudun + tonumber(tostring(buffList[j].shield_upper));
                end
                thing.player:updateBuff(true);
                -- thing.player:updateHudun(hudun); -- 暂时屏蔽护盾进度条，就不刷新护盾值
                if hangerId ~= nil then
                    thing.player:showBuffEff(hangerId);
                end

                if self.bRunBack ~= true then
                    local addList = list[i].add_buff;
                    for j = 1, #addList do
                        local dt = thing.player:showBuffText(addList[j]);
                        if dt > self.intervalTime then
                            self.intervalTime = dt;
                        end

                        local buffCfg = Lua_BuffConfigData.getConfigData(addList[j]);
                        if buffCfg ~= nil and buffCfg.buffType ~= 7 and not strIsNullOrEmpty(buffCfg.hangerId) then -- 场地特效不显示buff效果
                            -- print("2buffGuardSrcDic ", roleId, buffId, addList[j]);
                            if buffId ~= nil and addList[j] == buffId then bBuffGuardSrc = true; end
                            thing.player:showBuffEff(buffCfg.hangerId);
                        end
                    end
                end

                if not bBuffGuardSrc and buffId ~= nil then
                    -- print("buffGuardSrcDic remove", roleId, buffId);
                    self.buffGuardSrcDic[roleId] = nil;
                end
            end
        end
        if not self.bRunBack then
            Lua_MsgUtil.callFunc("refresh_fight_buff");
        end
    end
end

function Lua_SingleFightPlayer:refreshActionHurt()
    if self.curActionInfo == nil then 
        return; 
    end

    self:refreshBuff(self.curActionInfo.buff_list);
    local changeVal = self.skillHurtCount;
    local multi_attack = self.curActionInfo.multi_attack;
    if multi_attack ~= nil then
        for i = 1, #multi_attack do
            -- 伤害
            local attack_list = multi_attack[i].attack_list;
            if attack_list ~= nil then
                for j = 1, #attack_list do
                    local thing;
                    local hurtObj = attack_list[j];
                    local health = tonumber(tostring(hurtObj.health));
                    local roleId = tonumber(tostring(hurtObj.id));
                    local hurtList = hurtObj.hurt_list;
                    local isTarget = hurtObj.is_target == 0 or hurtObj.is_target == 1;
                    local showHurtIdx = hurtObj.showhurtidx; -- 已表现的伤害索引
                    hurtObj.showhurtidx = 0;
                    if showHurtIdx == nil then 
                        showHurtIdx = 0;
                    end

                    for m = showHurtIdx + 1, #hurtList do
                        -- if m == 1 then -- 暂时屏蔽护盾进度条，就不刷新护盾值
                        --     thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                        --     if thing ~= nil then
                        --         thing.player:updateHudun(tonumber(tostring(hurtObj.shield_value)));
                        --     end
                        -- end

                        if thing == nil then
                            thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                        end
                        
                        if thing ~= nil then
                            local hurt = hurtList[m].hurt;
                            for k = 1, #hurt do
                                local obj = hurt[k];
                                local val = tonumber(tostring(obj.value));
                                if thing.data.unitType == 5 or thing.data.unitType == 6 then
                                    self._recordVo.bossHurt = val
                                end

                                local dt = 0;
                                if isTarget then -- 还是技能伤害
                                    dt = thing.player:updateHealthChange(obj.type, val, health, self.bDoFlash, not self.bRunBack);
                                    if obj.type ~= 6 and changeVal then
                                        changeVal = changeVal + Mathf.Abs(val)
                                    end
                                else 
                                    dt = thing.player:updateHealthChange(obj.type, val, health, false, not self.bRunBack);
                                end
                                if dt > self.intervalTime then
                                    self.intervalTime = dt;
                                end
                            end
                        end
                    end

                    if thing == nil then
                        thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                    end
                    if thing ~= nil then
                        thing.player:updateHealth(health);
                        if showHurtIdx == 0 and hurtObj.buff_sort > 0 then
                            local buffCfg = Lua_BuffConfigData.getConfigData(hurtObj.buff_sort);
                            if buffCfg ~= nil and buffCfg.buffType ~= 7 and not strIsNullOrEmpty(buffCfg.hangerId) then -- 场地特效不显示buff效果
                                -- print("showBuffEff ", hurtObj.buff_sort, buffCfg.hangerId)
                                thing.player:showBuffEff(buffCfg.hangerId);
                            end
                        end
                    end
                end
            end

            -- 溅射
            local sputter_list = multi_attack[i].sputter_list;
            if sputter_list ~= nil and #sputter_list > 0 then
                for j = 1, #sputter_list do
                    local sputter_hurt = sputter_list[j];
                    if sputter_hurt ~= nil and #sputter_hurt > 0 then
                        for n = 1, #sputter_hurt do
                            local thing;
                            local hurtObj = sputter_hurt[n];
                            local health = tonumber(tostring(hurtObj.health));
                            local roleId = tonumber(tostring(hurtObj.id));
                            local hurtList = hurtObj.hurt_list;
                            local isTarget = hurtObj.is_target == 0 or hurtObj.is_target == 1;
                            local showHurtIdx = hurtObj.showhurtidx; -- 已表现的伤害索引
                            hurtObj.showhurtidx = 0;
                            if showHurtIdx == nil then 
                                showHurtIdx = 0; 
                            end

                            for m = showHurtIdx + 1, #hurtList do
                                -- if m == 1 then  -- 暂时屏蔽护盾进度条，就不刷新护盾值
                                --     thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                                --     if thing ~= nil then
                                --         thing.player:updateHudun(tonumber(tostring(hurtObj.shield_value)));
                                --     end
                                -- end

                                if thing == nil then
                                    thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                                end

                                if thing ~= nil then
                                    local hurt = hurtList[m].hurt;
                                    for k = 1, #hurt do
                                        local obj = hurt[k];
                                        local val = tonumber(tostring(obj.value));
                                        if thing.data.unitType == 5 or thing.data.unitType == 6 then
                                            self._recordVo.bossHurt = val
                                        end

                                        local dt = 0;
                                        if isTarget then -- 还是技能伤害
                                            dt = thing.player:updateHealthChange(obj.type, val, health, self.bDoFlash, not self.bRunBack);
                                            if obj.type ~= 6 and changeVal then
                                                changeVal = changeVal + Mathf.Abs(val)
                                            end
                                        else 
                                            dt = thing.player:updateHealthChange(obj.type, val, health, false, not self.bRunBack);
                                        end
                                        if dt > self.intervalTime then
                                            self.intervalTime = dt;
                                        end
                                    end
                                end
                            end

                            if thing == nil then
                                thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
                            end
                            if thing ~= nil then
                                thing.player:updateHealth(health);
                            end
                        end
                    end
                end
            end
        end
    end

    if self.bRunBack ~= true and changeVal ~= self.skillHurtCount then
        self.skillHurtCount = changeVal;
        Lua_MsgUtil.callFunc("skillAttackHurtUpdate", changeVal);
    end 

    self:returnToOrginPos(self.curActionInfo.shift_pos); -- 位移还原
    self:updateFormationEff(self.curActionInfo.site_effect, true);
    self._recordVo:checkHealth();
    self._recordVo:checkDeath();
    self.skillHurtStep = 999;-- 防止再次计算伤害，下一行动会重置为0；
end

function Lua_SingleFightPlayer:returnToOrginPos(list)
    if list == nil or #list <= 0 then
        return;
    end

    local bMoveThing = false;
    local moveTime = DataCenter.fightUtil:getPosChangeMoveTime();
    for i = 1, #list do
        local roleId = tonumber(tostring(list[i].key));
        local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
        if thing ~= nil then
            local pos = tonumber(tostring(list[i].value));
            local data = thing.data;
            data.formationPos = pos;
            local x = self._recordVo:getPosxByIdx(thing.formationType, pos);
            local y = self._recordVo:getPosyByIdx(thing.formationType, pos);
            local z = self._recordVo:getPoszByIdx(thing.formationType, pos);
            if not thing.data.isDead then
                thing.data.spd = Vector3.DistanceTwoPoint(x, y, z, data.px, data.py, data.pz) / moveTime;
                thing.player:moveTo(x, y, z, nil);
            end
            data.px = x;
            data.py = y;
            data.pz = z;
            data:refreshShowPos();
            bMoveThing = true;
        end
    end

    if bMoveThing then
        -- print("returnToOrginPos", moveTime, self.intervalTime);
        if self.bPlaySkill then
            if moveTime > self.skillAdditionalTime then
                self.skillAdditionalTime = moveTime;
            end
        else
            if moveTime > self.intervalTime then
                self.intervalTime = moveTime;
            end
        end
    end
end

function Lua_SingleFightPlayer:refreshBuffAndHurt()
    -- 先判断复活
    self:reliveUnits(self.curActionInfo.fighter_list);

    -- 再刷新伤害buff飘字
    self.skillHurtStep = 0;
    self:refreshActionHurt();
    self:setCurActionCompleted();

    if not self.bRunBack then
        Lua_MsgUtil.callFunc("refresh_fight_buff");
    end
end

function Lua_SingleFightPlayer:reliveUnits(list, bActive, bShowBornEff)
    if list ~= nil and #list > 0 then
        if bShowBornEff == nil then bShowBornEff = true; end
        if bActive == nil or bShowBornEff then bActive = false; end
        for i = 1, #list do
            local obj = list[i];
            local roleId = tonumber(tostring(obj.id));
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing == nil then -- 创建
                local data = self:generateAnimalData(obj);
                ThingUtil.instance:addAnimal(data, bActive);
                if bShowBornEff and self.bRunBack ~= true then
                    self.approachUnitCount = self.approachUnitCount + 1;
                    if self.showThingsArr == nil then self.showThingsArr = Array(); end
                    self.showThingsArr:add(roleId);
                end
            else
                local buffId = nil;
                local hangerId = nil;
                if self.bBuffGuard then
                    if self.buffGuardSrcDic ~= nil and self.buffGuardSrcDic[roleId] ~= nil then
                        buffId = self.buffGuardSrcDic[roleId];
                    elseif self.buffGuardTrgetDic ~= nil and self.buffGuardTrgetDic[roleId] ~= nil then
                        hangerId = self.buffGuardTrgetDic[roleId];
                    end 
                end

                local player = thing.player;
                local dt = 0;
                player:updateHealth(obj.health); -- 更新最终血量
                -- dt = player:updateHealthChange(6, obj.health, obj.health); -- 复活
                player:resetBuffDataList();
                local bBuffGuardSrc = false;
                local buffList = obj.buff_list;
                for j = 1, #buffList do
                    -- print("4buffGuardSrcDic ", roleId, buffId, buffList[j].sort_id);
                    if buffId ~= nil and buffList[j].sort_id == buffId then bBuffGuardSrc = true; end
                    player:addTemporaryBuff(buffList[j].sort_id, buffList[j].finish_round, buffId == nil or buffId ~= buffList[j].sort_id);
                end
                player.data.aniSpdRate = self._playSpeedRate;
                player:setActive(true);
                player:updateBuff(true);
                if hangerId ~= nil then
                    player:showBuffEff(hangerId);
                end
                if buffId ~= nil and not bBuffGuardSrc then
                    -- print("5 buffGuardSrcDic remove ", roleId, buffId);
                    self.buffGuardSrcDic[roleId] = nil;
                end
                dt = dt + player:showStateText(obj.health, nil, nil, nil, 3, 4); -- 回血飘字
                dt = dt + player:showStateText(0, nil, "number", "xui_fh_word_0", 0, 6); -- 复活飘字
                if dt > self.intervalTime then
                    self.intervalTime = dt;
                end
                if bShowBornEff and self.bRunBack ~= true then
                    local pos = player.currentPoint;
                    ThingUtil.instance:addSceneEffect(self._sceneId, self._recordId, "show_light", pos.x, pos.y, pos.z, roleId);
                end
            end
        end
    end
end

function Lua_SingleFightPlayer:removeThingsByFormationType(t)
    local list = DataCenter.thingData:getThingIdsByFormationType(self._recordId, t);
    if list ~= nil and list:getSize() > 0 then
        local removeDeadTime = 0;
        local len = list:getSize();
        for i = 1, len do
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, list[i]);
            if thing ~= nil and thing.player ~= nil then
                local val = thing.player.removeDeadTime;
                if val > removeDeadTime then
                    removeDeadTime = val;
                end
            end
        end

        if removeDeadTime > 0 then -- 等待死亡动作完成再移除
            self.intervalTime = removeDeadTime;
            return;
        end

        self.intervalTime = DataCenter.fightUtil:getRotOverRemoveTime();
        ThingUtil.instance:removeThingsByFormationType(self._sceneId, self._recordId, t);
    end
    self:setCurActionCompleted();
end

function Lua_SingleFightPlayer:updatePosition()
    local list = self.curActionInfo.fighter_list;
    if list ~= nil and #list > 0 then
        local moveTime = DataCenter.fightUtil:getPosChangeMoveTime();
        self.intervalTime = moveTime;
        for i = 1, #list do
            local obj = list[i];
            local roleId = tonumber(tostring(obj.id));
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing ~= nil then
                local data = thing.data;
                -- print("roleId = ", roleId, "从", data.formationPos, "移回", obj.pos);
                if data.formationPos ~= obj.pos then
                    data.formationPos = obj.pos;
                    local x = self._recordVo:getPosxByIdx(thing.formationType, obj.pos);
                    local y = self._recordVo:getPosyByIdx(thing.formationType, obj.pos);
                    local z = self._recordVo:getPoszByIdx(thing.formationType, obj.pos);
                    thing.data.spd = Vector3.DistanceTwoPoint(x, y, z, data.px, data.py, data.pz) / moveTime;
                    thing.player:moveTo(x, y, z, nil);
                    data.px = x;
                    data.py = y;
                    data.pz = z;
                    data:refreshShowPos();
                    -- thing.player:setPosPoint(x, y, z, true);
                end
            end
        end
    end

    self:setCurActionCompleted();
end

-- 更新场地buff
function Lua_SingleFightPlayer:updateFormationEff(buffIds, isAdd)
    if buffIds == nil or #buffIds <= 0 then
        return false;
    end

    local bChnage = false;
    for i = 1, #buffIds do
        local buffId = buffIds[i].buff_id;
        local formationType = buffIds[i].aod_pos;
        local buffCfg = Lua_BuffConfigData.getConfigData(buffId);
        if buffCfg ~= nil and not strIsNullOrEmpty(buffCfg.hangerId) then
            if isAdd == true then -- 增加
                if formationType == Lua_FightDefine.FIGHT_ATTACKER then
                    if self.attackFormationEffsArr == nil then
                        self.attackFormationEffsArr = Array();
                    end
                    if self.attackFormationEffsArr:indexOf(buffId) == -1 then
                        self.attackFormationEffsArr:add(buffId);
                        self:createFormationEff(formationType, buffId);-- 添加攻方场景特效
                        bChnage = true;
                    end
                else
                    if self.defendFormationEffsArr == nil then
                        self.defendFormationEffsArr = Array();
                    end
                    if self.defendFormationEffsArr:indexOf(buffId) == -1 then
                        self.defendFormationEffsArr:add(buffId);
                        self:createFormationEff(formationType, buffId);-- 添加守方场景特效
                        bChnage = true;
                    end
                end
            else -- 移除
                if formationType == Lua_FightDefine.FIGHT_ATTACKER then
                    if self.attackFormationEffsArr ~= nil and self.attackFormationEffsArr:removeItem(buffId) then
                        local rid = tonumber((formationType * 10)..buffId);
                        ThingUtil.instance:removeThing(self._sceneId, self._recordId, rid);-- 移除攻方场景特效
                        bChnage = true;
                    end
                else
                    if self.defendFormationEffsArr ~= nil and self.defendFormationEffsArr:removeItem(buffId) then
                        local rid = tonumber((formationType * 10)..buffId);
                        ThingUtil.instance:removeThing(self._sceneId, self._recordId, rid);-- 移除守方场景特效
                        bChnage = true;
                    end
                end
            end
        end
    end 

    return bChnage;
end

-- 创建场地特效
function Lua_SingleFightPlayer:createFormationEff(ftype, buffId)
    local rid = tonumber((ftype * 10)..buffId);
    local buffCfg = Lua_BuffConfigData.getConfigData(buffId);
    local hangerId = buffCfg.hangerId;
    local hangerCfg = Lua_PlayerHangerConfigData.getConfig(hangerId);
    if hangerCfg == nil then
        return false;
    end

    local mirrorType = hangerCfg.mirrorType;
    local flip = (mirrorType == 2 and ftype == Lua_FightDefine.FIGHT_DEFENDER) and -1 or 1;
    local data = SceneEffectData();
    data.kind = Lua_ThingDefine.SCENE_EFFECT;
    data.roleId = rid;
    data.objId = buffId;
    local sca = hangerCfg.offsetSca;
    local offsetPos = hangerCfg.offsetPos;
    local pos = self._recordVo:getCenterPosByType(ftype);
    local px = (pos ~= nil and #pos > 0) and pos[1] or 0;
    local py = (pos ~= nil and #pos > 1) and pos[2] or 0;
    local pz = (pos ~= nil and #pos > 2) and pos[3] or 0;
    px = px + ((offsetPos ~= nil and #offsetPos > 0) and offsetPos[1] or 0);
    py = py + ((offsetPos ~= nil and #offsetPos > 1) and offsetPos[2] or 0);
    pz = pz + ((offsetPos ~= nil and #offsetPos > 2) and offsetPos[3] or 0);
    data.px = px;
    data.py = py;
    data.pz = pz;
    data.dir = (mirrorType == 3 and self._recordVo:getDirByType(ftype) or 0) - 90;
    data.scaX = (sca ~= nil and #sca > 0) and sca[1] or 1;
    data.scaY = (sca ~= nil and #sca > 1) and sca[2] or 1;
    data.scaZ = (sca ~= nil and #sca > 2) and (sca[3] * flip) or flip;
    data.canBeClick = false;
    data.showShadow = false;
    data.isShowName = false;
    data.effectFile = hangerId;
    data.effectName = hangerId;
    data.formationType = ftype;
    data.fightRecordId = self._recordId;
    data.sceneId = self._sceneId;
    ThingUtil.instance:addSceneEffect(data, not self.bRunBack);
    return true;
end 

function Lua_SingleFightPlayer:playEnd(bSkip)
    if self.isPlayEnd == true then
        return;
    end
    self.isPlayEnd = true;
    self._recordVo:checkHealth();
    self._recordVo:checkDeath();

    -- 多分队：一场数据多轮战斗，采用多轮战斗形式，排除无尽火域
    if self.templateId ~= 1005 and self.templateId ~= 1006 then -- 无尽火域
        local actionInfo = self._recordVo:getActionInfo(self.curRotation + 1, 1, 1, 1);
        if actionInfo ~= nil then
            self.intervalTime = DataCenter.fightUtil:getRotOverRemoveTime();
            ThingUtil.instance:removeThingsByFormationType(self._sceneId, self._recordId, Lua_FightDefine.FIGHT_ATTACKER);
            ThingUtil.instance:removeThingsByFormationType(self._sceneId, self._recordId, Lua_FightDefine.FIGHT_DEFENDER);
            self:clean();
            self:setNextPlay(self.curRotation + 1);
            return;
        end
    end

    if self._recordVo.isIgnoreResult == false then
        Lua_ProxyManager.getProxy("Lua_FightProxy"):sFightResult(self._recordId, 1, self.curRotation);
    end

    -- 多分队：多场数据 分开下发
    if bSkip == true and self._recordVo.teamId <= 0 and self._recordVo.index > 0 then
        self.intervalTime = DataCenter.fightUtil:getRotOverRemoveTime();
        ThingUtil.instance:removeThingsByFormationType(self._sceneId, self._recordId, Lua_FightDefine.FIGHT_ATTACKER);
        ThingUtil.instance:removeThingsByFormationType(self._sceneId, self._recordId, Lua_FightDefine.FIGHT_DEFENDER);
        self:clean();
    end
    
    if self.templateId == Lua_FightDefine.TEMPLATE_ID_COPY_MORE then
        local arr = DataCenter.fightData:getGroupArrById(self._recordVo.groupId);
        if arr ~= nil then
            local len = arr:getSize();
            local player;
            local helpId, nextRid = self._recordVo:checkHelpInfo();
            if helpId > 0 and nextRid > 0 then
                player = DataCenter.fightData:getFightPlayerById(nextRid);
                if player == nil then
                    print(self._recordId, " --1进入协助1-- ", helpId, nextRid)
                    Lua_MsgUtil.callFunc("FIGHT_HELP_UPDATE", { helpId = helpId, nextRid = nextRid });
                    DataCenter.fightData:playRecordByFightId(nextRid, self.bRunBack);
                    return;
                end
            end

            -- 切入顺位队员的播放
            if self.bRunBack ~= true then
                local count = len;
                local index = arr:indexOf(self._recordId);
                local vo;
                local recordId;
                while len > 0 do
                    index = index + 1;
                    index = (index < 1 or index > count) and 1 or index;
                    vo = DataCenter.fightData:getRecordVoById(arr[index]);
                    player = DataCenter.fightData:getFightPlayerById(arr[index]);
                    if not vo.isAssist and player ~= nil and player.isPlayEnd ~= true then
                        recordId = arr[index];
                        break;
                    end
                    len = len - 1;
                end
                if recordId ~= nil then
                    DataCenter.fightData:showFightToStageByRid(recordId);
                end
            end
        end
    elseif self.templateId == Lua_FightDefine.TEMPLATE_ID_KUNXU then
        local arr = DataCenter.fightData:getTeamArrById(self._recordVo.teamId);
        if arr ~= nil then
            local player;
            -- 战斗失败就提前结束
            if self._recordVo.result == 2 then
                for i = 1, arr:getSize() do
                    player = DataCenter.fightData:getFightPlayerById(arr[i])
                    if arr[i] ~= self._recordId and player and player.isPlayEnd ~= true then
                        player:playEnd();
                    end
                end
            end
        end
    end

    local needResultWinExitFight = DataCenter.fightData:showFightResult(self._recordId);
    if needResultWinExitFight ~= 0 then
        self.bOpenResultWin = true; -- 通过战斗结算移除
        if needResultWinExitFight == 1 then -- 结算
            self.intervalTime = DataCenter.fightUtil.resultWinCloseTime + 5; -- 容错处理
        elseif needResultWinExitFight == 2 then -- BUFF选择
            self.isPlayEnd = false; -- 入口显示战斗中标识，点击才可进入正在播放的战斗
        end
    else
        self:removeFightPlay();
    end
end

-- 移除播放器
function Lua_SingleFightPlayer:removeFightPlay()
    DataCenter.fightData:removeFight(self._recordId);
end

-- 设置继续播放
function Lua_SingleFightPlayer:setContinuePlay()
    self.curActionInfo = nil;
    self.bOpenResultWin = false;
    self.bCheckNextAction = true;
end

-- 设置下一队伍播放
function Lua_SingleFightPlayer:setNextPlay(index)
    self.approachUnitCount = 0;
    self.bShowAttackThing = false;
    self.bPlaySkill = false;
    self.skillAdditionalTime = 0;

    self.curSession = 1; -- 场次
    self.curRotation = index; -- 轮次
    self.curRound = 1; -- 回合
    self.curAction = 1; -- 行动

    self.isPlayEnd = false;
    self.bOpenResultWin = false;
    self.bCheckNextAction = false;
end

function Lua_SingleFightPlayer:setPlayState(state)
    self.bStopPlay = state;
end

function Lua_SingleFightPlayer:hideToBackStage()
    if self.bRunBack ~= true then
        self.bRunBack = true;
        if self.isPlayEnd ~= true then
            self.bStopPlay = false;
        end
        if self.attackVo ~= nil then
            self.attackVo:dispose();
            self.attackVo = nil;
        end
        self.approachUnitCount = 0;
        if self.showThingsArr ~= nil then
            self.showThingsArr:clear();
        end
        if self._sceneId and self._recordId then
            Scene3DManager.instance:hideScene(self._sceneId, self._recordId);
        end
    end
end

function Lua_SingleFightPlayer:showToStage()
    -- LogManager.showLog("播放器 showToStage： "..self._recordId.." bRunBack： "..tostring(self.bRunBack))
    if self.bRunBack == true then
        self.bRunBack = false;
        self.bStopPlay = true;
        self.bSceneLoad = false;
        self.loadSceneId = 0;
        DataCenter.fightData:checkBuffChoose(self._recordId);
    end
end

-- obj 守护目标：key：被守护目标，value：抵挡伤害的单位
function Lua_SingleFightPlayer:startGuard(obj)
    if self.bRunBack == true then
        return;
    end
    
    local roleId = tonumber(tostring(obj.key));
    local guardThing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
    if guardThing ~= nil then
        roleId = tonumber(tostring(obj.value));
        local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
        if thing ~= nil then
            self.bOperateGuard = true;
            thing.player:createGhost();
            
            local guardPos = guardThing.formationPos;
            local guardFormationType = guardThing.formationType;
            local flip = guardFormationType == Lua_FightDefine.FIGHT_ATTACKER and 1 or -1;
            local offsetVec = DataCenter.fightUtil.guardOffsetVec;
            local x = self._recordVo:getPosxByIdx(guardFormationType, guardPos) + offsetVec.x;
            local y = self._recordVo:getPosyByIdx(guardFormationType, guardPos) + offsetVec.y;
            local z = self._recordVo:getPoszByIdx(guardFormationType, guardPos) + offsetVec.z * flip;
            -- local moveTime = 0.2;--DataCenter.fightUtil:getPosChangeMoveTime();
            -- local data = thing.data;
            -- thing.data.spd = Vector3.DistanceTwoPoint(x, y, z, data.px, data.py, data.pz) / moveTime;
            -- thing.player:moveTo(x, y, z, nil);
            thing.player:setPosPoint(x, y, z, false);
        end
    end
end

-- obj 守护目标：key：被守护目标，value：抵挡伤害的单位
function Lua_SingleFightPlayer:endGuard(obj)
    if self.bOperateGuard ~= true then
        return;
    end

    self.bOperateGuard = false;
    local roleId = tonumber(tostring(obj.value));
    local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
    if thing ~= nil then -- 还原
        local pos = thing.formationPos;
        local formationType = thing.formationType;
        local x = self._recordVo:getPosxByIdx(formationType, pos);
        local y = self._recordVo:getPosyByIdx(formationType, pos);
        local z = self._recordVo:getPoszByIdx(formationType, pos);
        thing.player:setPosPoint(x, y, z, false);
    end
end

-- 针对技能分析
function Lua_SingleFightPlayer:ansysisSkillTargets(multi_attack, skillCfg)
    if self.bRunBack == true then -- 后台无需筛选
        return;
    end

    if skillCfg == nil or self.attackVo == nil or multi_attack == nil then
        return;
    end

    for i = 1, #multi_attack do
        local attack_list = multi_attack[i].attack_list;
        for j = 1, #attack_list do
            local obj = attack_list[j];
            local is_target = obj.is_target;
            local rid = tonumber(tostring(obj.id));
            -- print("ansysisSkillTargets", i, j, rid, is_target, #obj.hurt_list);
            if is_target == 0 or is_target == 1 then -- 技能目标，0不用挂目标特效，1需要挂目标特效
                self.attackVo:addTargetId(i - 1, rid, is_target); -- 目标单位
            end
        end
    end
end

-- buff守护
function Lua_SingleFightPlayer:startBuffGuard(multi_attack)
    if self.bRunBack == true then -- 后台无需筛选
        return;
    end

    if multi_attack == nil or #multi_attack <= 0 then
        return;
    end

    for i = 1, #multi_attack do
        local attack_list = multi_attack[i].attack_list;
        for j = 1, #attack_list do
            local obj = attack_list[j];
            if obj.guard_soul_pos ~= nil then
                local buffId = tonumber(tostring(obj.guard_soul_pos.value));
                local buffCfg = Lua_BuffConfigData.getConfigData(buffId);
                if buffCfg ~= nil and not strIsNullOrEmpty(buffCfg.hangerId) then
                    local srcRid = tonumber(tostring(obj.guard_soul_pos.key));
                    local srcThing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, srcRid);
                    if srcThing ~= nil then
                        self.bBuffGuard = true;
                        srcThing.player:removeBuffEff(buffCfg.hangerId);
                        -- print("startBuffGuard removeBuffEff", srcRid, buffCfg.hangerId)
                        local rid = tonumber(tostring(obj.id));
                        local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, rid);
                        if thing ~= nil then
                            thing.player:showBuffEff(buffCfg.hangerId);
                            -- print("startBuffGuard showBuffEff", rid, buffCfg.hangerId)
                        end

                        if self.buffGuardSrcDic == nil then
                            self.buffGuardSrcDic = {};
                        end
                        self.buffGuardSrcDic[srcRid] = buffId;

                        if self.buffGuardTrgetDic == nil then
                            self.buffGuardTrgetDic = {};
                        end
                        self.buffGuardTrgetDic[rid] = buffCfg.hangerId;
                    end
                end
            end
        end
    end
end

-- 结束buff守护
function Lua_SingleFightPlayer:endBuffGuard()
    if self.bBuffGuard ~= true then
        return;
    end

    self.bBuffGuard = false;
    if self.buffGuardTrgetDic ~= nil then
        for k, v in pairs(self.buffGuardTrgetDic) do
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, k);
            if thing ~= nil then
                thing.player:removeBuffEff(v);
                -- print("endBuffGuard removeBuffEff", k, v)
            end
            self.buffGuardTrgetDic[k] = nil
        end
    end

    if self.buffGuardSrcDic ~= nil then
        for k, v in pairs(self.buffGuardSrcDic) do
            if not self.bRunBack then
                local buffCfg = Lua_BuffConfigData.getConfigData(v);
                if buffCfg ~= nil and buffCfg.hangerId ~= nil then
                    local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, k);
                    if thing ~= nil then
                        thing.player:showBuffEff(buffCfg.hangerId);
                        -- print("endBuffGuard showBuffEff", k, buffCfg.hangerId)
                    end
                end
            end
            self.buffGuardSrcDic[k] = nil
        end
    end
end

--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
function Lua_SingleFightPlayer.Set:totalTime(value)
    if self.bSkipPlay == false then
        return;
    end

    if self._totalTime == nil then
        self._totalTime = value;
    else
        self._totalTime = self._totalTime + value;
    end
    self.bSkipPlay = self._totalTime <= self._recordVo.skipTime; -- 跳过播放表现

    if not self.bSkipShow and self.bSceneLoad and self.showThingsArr then
        local len = self.showThingsArr:getSize();
        while (len > 0) do
            self.approachUnitCount = self.approachUnitCount - 1;
            local roleId = self.showThingsArr:remove(len);
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing then
                thing.player:setActive(true);
                if self.bRunBack ~= true then
                    if thing.configData and thing.configData.hpBarType == 1 then
                        Lua_MsgUtil.callFunc("FIGHT_BOSS_INFO_UPDATE", thing.data);
                    end
                end
            end
            len = len - 1;
        end
        self.showThingsArr = nil;
    end
end

function Lua_SingleFightPlayer:onSkipHandle()
    if self.isPlayEnd == true then
        return;
    end

    if self.bOpenResultWin == true or self.bStopPlay == true then -- 结束界面退出
        return;
    end

    if self._endDelayTime > 0 then
        self._endDelayTime = self._endDelayTime - dt;
        if self._endDelayTime <= 0 then -- 移除
            self:playEnd();
        end
        return;
    end

    if self.curActionInfo == nil then
        if self.bCheckNextAction == true then
            self.curActionInfo, self.curAction = self._recordVo:getNextActionInfo(self.curRotation, self.curSession, self.curRound, self.curAction);
            if self.curActionInfo ~= nil then
                self.curSession = self.curActionInfo.session; -- 场次
                self.curRotation = self.curActionInfo.rotation; -- 轮次
                self.curRound = self.curActionInfo.round; -- 回合
            end
        else
            self.curActionInfo = self._recordVo:getActionInfo(self.curRotation, self.curSession, self.curRound, self.curAction);
        end
    end

    if self.curActionInfo == nil then
        if self._recordVo.isComplete == true then -- 下发完成
            self._recordVo:checkResult();
            self._endDelayTime = DataCenter.fightUtil.playOverIntervalTime / self.playSpeed;
            if self._endDelayTime <= 0 then
                self:playEnd();
            end
        end
        return;
    end

    self.bCheckNextAction = false;
    local curType = self.curActionInfo.type;
    if curType == 1 then -- 单位进场
        self.approachUnitCount = 0; -- 进场数量
        local list = self.curActionInfo.fighter_list
        table.sort( list, function(a, b) return a.aod_type < b.aod_type end )
        self:initFormation(list);
        self:checkInitRoleSkill();
        self.totalTime = DataCenter.fightUtil.startEffectTime;
        self:setCurActionCompleted();
    elseif curType == 2 then -- 释放技能（对目标或友军造成的伤害或回血），伤害值和技能目标在hurt_list字段
        local skillId = self.curActionInfo.skill_id;
        local skillCfg = Lua_SkillConfigData.getConfigData(skillId);
        if skillCfg == nil then
            self:setCurActionCompleted();
            return;
        end

        self:onSkipHandle_reliveUnits(self.curActionInfo.fighter_list);
        self:onSkipHandle_refreshActionHurt();

        self.totalTime = skillCfg.countTime;
        self:setCurActionCompleted();
    elseif curType == 3 then -- buff变化（新增或移除），新增或者移除的buff在buff_list字段
        self:onSkipHandle_refreshActionHurt();
        self:setCurActionCompleted();
    elseif curType == 4 then -- buff伤害（buff扣血或回血），伤害值和目标在hurt_list字段(buff复活用)
        self:onSkipHandle_refreshActionHurt();
        self:setCurActionCompleted();
    elseif curType == 5 then -- 不可行动（受buff控制效果导致的不可行动，buff_id为对应的控制效果）
        self:onSkipHandle_refreshActionHurt();
        self:setCurActionCompleted();
        self.totalTime = 1.2;
    elseif curType == 6 then -- 攻方退场
        self:onSkipHandle_removeThingsByFormationType(Lua_FightDefine.FIGHT_ATTACKER);
    elseif curType == 7 then -- 守方退场
        self:onSkipHandle_removeThingsByFormationType(Lua_FightDefine.FIGHT_DEFENDER);
    elseif curType == 8 then -- 更新主角技能冷却
        local id = tonumber(tostring(self.curActionInfo.fighter_id));
        local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, id);
        if thing and thing.formationType == Lua_FightDefine.FIGHT_ATTACKER then
            self.artifactEnergy = self.curActionInfo.energy;
        end
        self.artifactSkillDic[id] = self.curActionInfo.skill_list;
        if not self.bRunBack then
            Lua_MsgUtil.callFunc("fight_roleSkill_update");
        end
        self:setCurActionCompleted();
    elseif curType == 9 then -- 站位变化（数据在fighter_list）
        self:onSkipHandle_updatePosition();
    elseif curType == 10 then -- 销毁场地效果，数据在site_effect
        if self:updateFormationEff(self.curActionInfo.site_effect, false) then
            self.totalTime = 0.5;
        end
        self:onSkipHandle_refreshBuff(self.curActionInfo.buff_list);
        self:setCurActionCompleted();
    elseif curType == 11 then -- 召唤单位进场
        self:onSkipHandle_refreshBuff(self.curActionInfo.buff_list);
        self:setCurActionCompleted();
    elseif curType == 12 then -- 触发的剧情事件，下一节点是事件效果
        self:setCurActionCompleted();
    else
        self:onSkipHandle_refreshActionHurt();
    end
end

function Lua_SingleFightPlayer:onSkipHandle_reliveUnits(list)
    if list == nil then
        return;
    end
    for i = 1, #list do
        local obj = list[i];
        local roleId = tonumber(tostring(obj.id));
        local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
        if thing == nil then
            local data = self:generateAnimalData(obj);
            ThingUtil.instance:addAnimal(data, false);
            self.approachUnitCount = self.approachUnitCount + 1;
            if self.showThingsArr == nil then self.showThingsArr = Array(); end
            self.showThingsArr:add(roleId);
        else
            thing.data.hp = obj.health;
            thing.data:resetBuffDataList();
            local buff_list = obj.buff_list;
            for j = 1, #buff_list do
                thing.data:addTemporaryBuffId(buff_list[j].sort_id, buff_list[j].finish_round);
            end
        end
    end

    self._recordVo:checkHealth();
    self._recordVo:checkDeath();
end

function Lua_SingleFightPlayer:onSkipHandle_refreshBuff(list)
    if list == nil then
        return;
    end
    for i = 1, #list do
        local roleId = tonumber(tostring(list[i].id));
        local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
        if thing then
            thing.data:resetBuffDataList();
            local buff_list = list[i].buff_list;
            for j = 1, #buff_list do
                thing.data:addTemporaryBuffId(buff_list[j].sort_id, buff_list[j].finish_round);
            end
        end
    end
end

function Lua_SingleFightPlayer:onSkipHandle_refreshActionHurt()
    self:onSkipHandle_refreshBuff(self.curActionInfo.buff_list);
    local multi_attack = self.curActionInfo.multi_attack;
    if multi_attack == nil then
        return;
    end

    for i = 1, #multi_attack do
        local attack_list = multi_attack[i].attack_list;
        for j = 1, #attack_list do
            local hurtObj = attack_list[j];
            local roleId = tonumber(tostring(hurtObj.id));
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing then
                local hurtList = hurtObj.hurt_list;
                local showHurtIdx = hurtObj.showhurtidx;
                local health = tonumber(tostring(hurtObj.health));
                local hp = tonumber(tostring(thing.data.hp));
                local maxHp = tonumber(tostring(thing.data.maxHp));
                for m = showHurtIdx + 1, #hurtList do
                    local hurt = hurtList[m].hurt;
                    for k = 1, #hurt do
                        local obj = hurt[k];
                        local val = tonumber(tostring(obj.value));
                        if thing.data.unitType == 5 or thing.data.unitType == 6 then
                            self._recordVo.bossHurt = val;
                        end
                        val = hp + val;
                        val = val > maxHp and maxHp or val;
                        val = val < 0 and health or val;
                        thing.data.hp = val;
                    end
                end
            end
        end

        local sputter_list = multi_attack[i].sputter_list;
        for j = 1, #sputter_list do
            local hurtObj = sputter_list[j];
            local roleId = tonumber(tostring(hurtObj.id));
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing then
                local hurtList = hurtObj.hurt_list;
                local showHurtIdx = hurtObj.showhurtidx;
                local health = tonumber(tostring(hurtObj.health));
                local hp = tonumber(tostring(thing.data.hp));
                local maxHp = tonumber(tostring(thing.data.maxHp));
                for m = showHurtIdx + 1, #hurtList do
                    local hurt = hurtList[m].hurt;
                    for k = 1, #hurt do
                        local obj = hurt[k];
                        local val = tonumber(tostring(obj.value));
                        if thing.data.unitType == 5 or thing.data.unitType == 6 then
                            self._recordVo.bossHurt = val;
                        end
                        val = hp + val;
                        val = val > maxHp and maxHp or val;
                        val = val < 0 and health or val;
                        thing.data.hp = val;
                    end
                end
            end
        end
    end

    self.totalTime = 1;
    self._recordVo:checkHealth();
    self._recordVo:checkDeath();
end

function Lua_SingleFightPlayer:onSkipHandle_removeThingsByFormationType(t)
    ThingUtil.instance:removeThingsByFormationType(self._sceneId, self._recordId, t);
    self.totalTime = DataCenter.fightUtil:getRotOverRemoveTime();
    self:setCurActionCompleted();
end

function Lua_SingleFightPlayer:onSkipHandle_updatePosition()
    local list = self.curActionInfo.fighter_list;
    if list then
        for i = 1, #list do
            local obj = list[i];
            local roleId = tonumber(tostring(obj.id));
            local thing = DataCenter.thingData:getThingPlayer(self._sceneId, self._recordId, roleId);
            if thing then
                local data = thing.data;
                if data.formationPos ~= obj.pos then
                    data.formationPos = obj.pos;
                    local x = self._recordVo:getPosxByIdx(thing.formationType, obj.pos);
                    local y = self._recordVo:getPosyByIdx(thing.formationType, obj.pos);
                    local z = self._recordVo:getPoszByIdx(thing.formationType, obj.pos);
                    data.px = x;
                    data.py = y;
                    data.pz = z;
                    data:refreshShowPos();
                end
            end
        end
        self.totalTime = DataCenter.fightUtil:getPosChangeMoveTime();
    end

    self:setCurActionCompleted();
end