Lua_FriendData = class("Lua_FriendData");
Lua_FriendData.FRIEND_TYPE = 1; -- 好友
Lua_FriendData.FRIEND_BACKLIST_TYPE = 2; -- 黑名单
Lua_FriendData.FRIEND_APPLY_TYPE = 3; -- 好友申请列表
-- Lua_FriendData.FRIEND_DELETE_TYPE = 4; -- 删除好友 无效
Lua_FriendData.FRIEND_RECOMMEND_SEARCH_TYPE = 5; -- 推荐和搜索好友
function Lua_FriendData:ctor()
    self:dispose();
end

function Lua_FriendData:dispose()
    self.curOpenUIType = 0;
    self.friendList = {};
    self.rewardState = {};
end

function Lua_FriendData:fromServer50107001(msg)
    if msg == nil then
        return;
    end
    
    local friendData = self:getFriendType(msg.type); -- 列表类型 
    self.sendTotalCount = msg.totaol_send_gift_num; -- 今日赠送礼物总次数
    friendData:dispose();
    friendData:updateSingleTypeData(msg.friend_list); -- 更新单一类型数据

    if msg.type == Lua_FriendData.FRIEND_APPLY_TYPE then
        local list = self:getFriendType(Lua_FriendData.FRIEND_APPLY_TYPE):getFriendList();
        DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Friend_Manage_Apply, #list > 0);
    end

    self:checkGiftRed(msg.type);
end

function Lua_FriendData:checkGiftRedState()
    local friendData;
    local state = self.rewardRedState;
    if state == false then
        friendData = self:getFriendType(Lua_FriendData.FRIEND_TYPE);
        if #friendData:getFriendList() > 0 and self.sendTotalCount < friendData.sendMaxCount and friendData.sendGiftState then
            state = true;
        end
    end

    return state;
end

function Lua_FriendData:checkGiftRed(type)
    if type ~= Lua_FriendData.FRIEND_TYPE then
        return;
    end
    local state = self:checkGiftRedState();
    DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Friend_Reward, state);
end

-- 根据类型获取列表信息
function Lua_FriendData:getFriendType(type)
    if self.friendList[type] == nil then
        self.friendList[type] = Lua_FriendSingleTypeDate.new(type);
    end
    return self.friendList[type];
end

function Lua_FriendData:fromServer50107007(msg)
    if msg == nil then
        return;
    end

    local friendData = self:getFriendType(Lua_FriendData.FRIEND_RECOMMEND_SEARCH_TYPE);
    friendData:dispose();
    friendData:updateSerchTypeData(msg.friend); -- 更新推荐和搜索类型的信息
end

-- 是否好友已满
function Lua_FriendData:isFriendFull()
    local maxFriendLimitCount = getConfig("friend_const", "friend_limit", "value");
    local friendData = self:getFriendType(Lua_FriendData.FRIEND_TYPE);
    local curCount = #friendData:getFriendList();
    if curCount >= maxFriendLimitCount then
        return true;
    end
    return false;
end

-- 是否好友
function Lua_FriendData:isFriendById(friendId)
    local friendSingleData = self:getFriendType(Lua_FriendData.FRIEND_TYPE);
    local item = friendSingleData:getFriendItemById(friendId);
    if item ~= nil then
        return true
    end
    return false;
end

-- 是否黑名单
function Lua_FriendData:isBacklistById(friendId)
    local backData = self:getFriendType(Lua_FriendData.FRIEND_BACKLIST_TYPE);
    local item = backData:getFriendItemById(friendId);
    if item ~= nil then
        return true
    end
    return false;
end

function Lua_FriendData:fromServer80107014(msg)
    if msg == nil then
        return;
    end

    if msg.type == 1 then
        DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Friend_Manage_Apply, true); -- 好友申请红点 
    end

    if msg.type == 0 or msg.type == 2 then
        self.rewardRedState = msg.type == 2 and true or false;
        self:checkGiftRed(Lua_FriendData.FRIEND_TYPE);
    end
end

function Lua_FriendData:fromServer50107002(msg)
    if msg == nil then
        return;
    end
    
    local list = self:getFriendType(Lua_FriendData.FRIEND_RECOMMEND_SEARCH_TYPE);
    if list ~= nil then
        local friendInfo = list:getFriendItemById(tonumber(msg.friend_id));
        if friendInfo ~= nil then
            friendInfo.isApply = msg.is_apply;
        end
    end
    OperateAlert.showLangValue(161023);
end

-- 请求返回好友申请结果，是否存在对方好友栏已满或在对方黑名单上
function Lua_FriendData:fromServer50107003(msg)
    for k, v in ipairs(msg.apply_result) do
        if v.value == 0 then --  0：成功
            OperateAlert.show(LanUtil.formateStr(161036, v.key));
        elseif v.value == -6006 then -- 1：对方好友已满 
            OperateAlert.show(LanUtil.formateStr(161037, v.key));
        elseif v.value == -6003 then -- 2：在对方黑名单
            OperateAlert.show(LanUtil.formateStr(161038, v.key));
        end
    end
end

function Lua_FriendData:fromServer50107010(msg)
    local friendInfo = self:getFriendInfo(Lua_FriendData.FRIEND_TYPE, msg.friend_id);
    if friendInfo ~= nil then
        friendInfo.giftsentCount = msg.send_gift_num;
        OperateAlert.show(LanUtil.formateStr(161011, friendInfo.show.name, self.getSetRewardStr));
    end
    self.sendTotalCount = msg.total_send_gift_num; -- 今日赠送次数
    self:checkGiftRed(Lua_FriendData.FRIEND_TYPE);
    Lua_MsgUtil.callFunc("SET_FRIENDPANEL_INFO", Lua_FriendData.FRIEND_TYPE);
end

function Lua_FriendData:fromServer50107012(msg)
    local friendInfo = self:getFriendInfo(Lua_FriendData.FRIEND_TYPE, msg.friend_id);
    if friendInfo ~= nil then
        friendInfo.giftGetInfo = msg.gift_info; -- 0 没有礼物可领取 1有礼物可领取
        OperateAlert.show(LanUtil.formateStr(161041, friendInfo.show.name, self.getGiftRewardStr));
    end
    Lua_MsgUtil.callFunc("SET_FRIENDPANEL_INFO", Lua_FriendData.FRIEND_TYPE);
end

function Lua_FriendData:fromServer50107013(msg)
    local friendInfo;
    for k, v in ipairs(msg.draw_list) do
        friendInfo = self:getFriendInfo(Lua_FriendData.FRIEND_TYPE, v);
        if friendInfo ~= nil then
            OperateAlert.show(LanUtil.formateStr(161041, friendInfo.show.name, self.getGiftRewardStr));
        end
    end

    for k, v in ipairs(msg.send_list) do
        friendInfo = self:getFriendInfo(Lua_FriendData.FRIEND_TYPE, v);
        if friendInfo ~= nil then
            OperateAlert.show(LanUtil.formateStr(161011, friendInfo.show.name, self.getSetRewardStr));
        end
    end
end

function Lua_FriendData:getFriendInfo(type, friendId)
    local list = self:getFriendType(type);
    if list ~= nil then
        local friendInfo = list:getFriendItemById(tonumber(friendId));
        if friendInfo ~= nil then
            return friendInfo;
        end
    end
    return nil;
end

function Lua_FriendData.Get:getSetRewardStr()
    if self._getSetRewardStr == nil then
        self._getSetRewardStr = "";
        local str;
        local config = getConfig("friend_const", "send_gift_reward", "reward");
        for k, v in pairs(config) do
            str = Lua_ResourcesData.getConfigDataById(v[1]).name .. "*" .. v[2];
            if self._getSetRewardStr == "" then
                self._getSetRewardStr = str;
            else
                self._getSetRewardStr = self._getSetRewardStr .. "，" .. str;
            end
        end

        str = "";
        config = getConfig("friend_const", "send_gift_reward", "other_reward");
        for k, v in pairs(config) do
            str = Lua_ResourcesData.getConfigDataById(v[1]).name .. "*" .. v[2];
            if self._getSetRewardStr == "" then
                self._getSetRewardStr = str;
            else
                self._getSetRewardStr = self._getSetRewardStr .. "，" .. str;
            end
        end
    end
    return self._getSetRewardStr;
end

function Lua_FriendData.Get:getGiftRewardStr()
    if self._getGiftRewardStr == nil then
        self._getGiftRewardStr = "";
        local str;
        local config = getConfig("friend_const", "get_gift_reward", "reward");
        for k, v in pairs(config) do
            str = Lua_ResourcesData.getConfigDataById(v[1]).name .. "*" .. v[2];
            if self._getGiftRewardStr == "" then
                self._getGiftRewardStr = str;
            else
                self._getGiftRewardStr = self._getGiftRewardStr .. "，" .. str;
            end
        end

        str = "";
        config = getConfig("friend_const", "get_gift_reward", "other_reward");
        for k, v in pairs(config) do
            str = Lua_ResourcesData.getConfigDataById(v[1]).name .. "*" .. v[2];
            if self._getGiftRewardStr == "" then
                self._getGiftRewardStr = str;
            else
                self._getGiftRewardStr = self._getGiftRewardStr .. "，" .. str;
            end
        end
    end
    return self._getGiftRewardStr;
end

function Lua_FriendData:fromServer80107015(msg)
    DataCenter.chatData:removeFriendInfo(msg.friend_id);
end

function Lua_FriendData.Get:tweenId()
    if self._tweenId == nil then
        self._tweenId = getConfig("friend_const", "friend_tween_id", "value");
    end
    return self._tweenId;
end





























-------------------- 好友信息 --------------------

Lua_FriendSingleTypeDate = class("Lua_FriendSingleTypeDate")
function Lua_FriendSingleTypeDate:ctor(type)
    self.friendType = type;

    self:dispose();
end

function Lua_FriendSingleTypeDate:dispose()
    self.friendInfoDic = {};
    self.friendIndexList = {};
end

function Lua_FriendSingleTypeDate:updateSingleTypeData(friend_list)
    -- self.playerInfo = friend_list;
    self:initFriendDic(friend_list);
end

function Lua_FriendSingleTypeDate:initFriendDic(friend_list)
    if friend_list ~= nil then
        local obj;
        local count;
        local friendId;

        for i = 1, #friend_list do
            obj = friend_list[i];
            friendId = tonumber(obj.show.id); -- 角色ID
            if friendId ~= nil then
                self.friendInfoDic[friendId] = self:dealwithFriendInfo(obj); -- 玩家id对应玩家信息
                count = #self.friendIndexList;
                self.friendIndexList[count + 1] = friendId;
            end
        end

        -- 排序
        if self.friendType ~= Lua_FriendData.FRIEND_APPLY_TYPE then
            table.sort(self.friendIndexList,
                function(a, b) 
                    local aTick = self:getFriendItemById(a).offlineTick;
                    local bTick = self:getFriendItemById(b).offlineTick;
                    return aTick < bTick;
                end
            );
        end
    end
end

function Lua_FriendSingleTypeDate:updateSerchTypeData(friendInfo)
    local friendId = tonumber(friendInfo.show.id); -- 角色ID
    if friendId ~= nil then
        self.friendInfoDic[friendId] = self:dealwithFriendInfo(friendInfo) -- friendInfo; -- 玩家id对应玩家信息
        self.friendIndexList[1] = friendId;
    end
end

function Lua_FriendSingleTypeDate:dealwithFriendInfo(friendInfo)
    local vo = {
        friendId = tonumber(friendInfo.id); -- 角色ID 类型：string
        maxSentCount = getConfig("friend_const", "everyday_give_limit", "value"); -- 每日赠送礼物次数上限
        limitSentCount = getConfig("friend_const", "everyday_every_friend_limit", "value"); -- 该好友每日赠送上限
        giftsentCount = friendInfo.send_gift_num; -- 该好友今日赠送礼物次数
        giftGetInfo = friendInfo.gift_info; -- 0 没有礼物可领取，1 有礼物可领取
        show = friendInfo.show; -- 好友具体信息
        isApply = friendInfo.is_apply; -- 是否申请过 1:已申请  0：未申请
        offlineTick = friendInfo.online == 1 and 0 or DataCenter.roleData.serverTime - tonumber(friendInfo.offline_tick) -- offline_tick 离线时间戳
    };
    return vo;
end

function Lua_FriendSingleTypeDate:getFriendList()
    return self.friendIndexList;
end

function Lua_FriendSingleTypeDate:getFriendItemById(friendId)
    if self.friendInfoDic == nil or friendId == nil then
        return;
    end
    return self.friendInfoDic[tonumber(friendId)];
end

-- 赠送礼物状态
function Lua_FriendSingleTypeDate.Get:sendGiftState()
    local giftLimitCount = getConfig("friend_const", "everyday_every_friend_limit", "value");
    for k,v in pairs(self.friendInfoDic) do
        if v.giftsentCount ~= nil and giftLimitCount > v.giftsentCount  then
            return true;
        end
    end
    return false;
end

function Lua_FriendSingleTypeDate.Get:sendMaxCount()
    if self._sendMaxCount == nil then
        self._sendMaxCount = getConfig("friend_const", "everyday_give_limit", "value");
    end
    return self._sendMaxCount;
end

-- 领取礼物状态  true:有🎁 false:无礼物
function Lua_FriendSingleTypeDate.Get:receiveGiftState()
    for k,v in pairs(self.friendInfoDic) do
        if v.giftGetInfo ~= nil and v.giftGetInfo == 1 then
            return true;
        end
    end
    return false;
end

-- 今日领取是否达上限
function Lua_FriendSingleTypeDate.Get:receiveLimitState()
    local receiveGiftCount = 0;
    local receiveMaxCount = getConfig("friend_const", "everyday_draw_limit", "value");
    for k,v in pairs(self.friendInfoDic) do
        if v.giftsentCount ~= nil and v.giftGetInfo == 1 then
            receiveGiftCount = receiveGiftCount + 1;
        end
    end
    return receiveGiftCount >= receiveMaxCount;
end