-- 英雄Vo
checkRequire("Lua_HeroConfigData");
Lua_HeroVo = class("Lua_HeroVo", Lua_ToolVo)
function Lua_HeroVo:ctor(id, config)
    self.id = nil;
    self.isHeroSystem = true; -- 英雄标识
    self:setData(id, config);
end

function Lua_HeroVo:setData(id, config)
    id = tonumber(id);
    if self.id ~= id then
        self.id = id;

        self.config = config;
        if self.config == nil then
            self.config = Lua_HeroConfigData.getConfig(id);
        end

        if self.config == nil and id ~= 0 then
            LogManager.LogError("no hero-英雄表 id = " .. id, false);
        end
    end
end

function Lua_HeroVo:fromServer(msg)
    self:reset();
    -- self.star = msg.star;  -- 星级
    self.level = msg.level;-- 等级
    self.quality = msg.quality;--质量
    self.quality_star = msg.quality_star;--质量星级
    
    self.ago_level = msg.resonance;-- 放入守护槽前的等级
    self.heroForce = tonumber(msg.force);-- 战力
    self.unique_id = tonumber(msg.unique_id);-- 唯一id
    self.clothes_id = tonumber(msg.clothes_id);-- 穿戴时装id
    self.rebuild_tick = tonumber(msg.rebuild_tick);-- 重修的时间戳
    self.trial_tick = tonumber(msg.trial_end_tick);-- 英雄试用结束时间戳

    --tab集合
    self.skill = msg.skill;-- 技能
    self.prop_list = msg.prop_list;-- 属性
    self.jewelry_list = msg.jewelry_list;-- 星魂

    local lineup_list = msg.lineup_state_list;-- 上阵状态列表
    if lineup_list ~= nil then
        local len = #lineup_list
        self.lineupDic = {};
        local team_index;--阵容类型
        local lineup_type;--分队索引
        for i=1, len do
            team_index = tonumber(lineup_list[i].value);
            lineup_type = tonumber(lineup_list[i].key);
            if self.lineupDic[lineup_type] == nil then
                self.lineupDic[lineup_type] = {};
            end
            
            self.lineupDic[lineup_type][team_index] = 1;
        end
    end
    self.rune_info = msg.rune_info;--符石
    self.equip_info = msg.equip_info;--装备
    self.equip_reonance_id = msg.equip_reonance_id;--共鸣id
    self.talent_info = msg.talent_info;--战技
    self.awaken_id = msg.awaken_id; -- 觉醒id
    self.awaken_info = msg.awaken_column_info; -- 已解锁的觉醒技能栏信息

    self._weapon_id = tonumber(msg.weapon_id); -- 神兵id 0未解锁

    --状态
    self.is_resonance = msg.resonance ~= 0;-- 同调器状态
    self.lock_state = msg.lock_state == 1; -- 是否上锁
    self.dispatch_state = msg.dispatch_state == 1;-- 英雄转换状态（0：未转换，1：转换中未确认）
    self.send_info_state = msg.send_info_state == 1; -- 情报派遣状态：0未派遣，1：派遣中
    self.is_league_transaction = msg.is_league_transaction == 1; -- 是否在交易行 0：不在 1:在交易行
    
    -- 结构体名：ProtoShowHero 其他字段
    self.pos = msg.pos;--英雄站位 站位，1到9
    self.location = msg.location;--英雄站位索引 1-5
    self._blood = msg.blood; -- 当前血量
    self._max_blood = msg.max_blood; -- 最大血量
    self._assist = msg.assist; -- 0 不是援助，1 邀请援助
    self._lineup_id = msg.lineup_id; -- 未上阵:0, 已上阵：阵容id
    self.is_leader = msg.is_leader;--怪物是否队长（神陵秘境用，1：是 0：否）
    self.is_try = msg.is_try; -- 试用标识 0：非试用 1：试用
end

function Lua_HeroVo:clean()
    self.skill = nil;
    self.prop_list = nil;
    self.equip_info = nil;
    self.talent_info = nil;
    self.jewelry_list = nil;
    self.runesDic = nil;
    self.skillDic = nil;
    self.lineupDic = nil;

	self._runsInfo = nil;
    self._equipInfo = nil;
    self._talentInfo = nil;

    self.fashionInfo = nil;
    self._otherLineup = nil;
    if self._heroUpStarVo ~= nil then
        self._heroUpStarVo:clean();
        self._heroUpStarVo = nil;
    end
end

--重置字段
function Lua_HeroVo:reset()
    self._backSort = nil;
    self._upStarSort = nil;
    self._lifeTreeSort = nil;
    self._reshapeSort = nil;
    self._decomposeSort = nil;

    self._runsInfo = nil; -- 符文数据
    self._runsInfo = nil; -- 符文数据
    self._equipInfo = nil;-- 装备数据
    self._talentInfo = nil;-- 符石数据
    
    self._isUpLineup = false;--是否上阵
    -- self._bDownLineup = nil;--是否可以下阵
    self._upLineupName = "";--上阵中的阵容名字
    self._otherLineup = nil;
end




---------------------------------- 初始化数据 -----------------------------------------------------
--初始品质色
function Lua_HeroVo.Get:initial_color()
    return self.config.color;
end

-- 当前初始星级
function Lua_HeroVo.Get:initial_star()
    if self._initial_star == nil then
        self._initial_star = DataCenter.heroUtil:getHeroAwakenStar(self.initial_awaken_star_id);
    end
    return self._initial_star;
end

-- 当前初始觉醒星级id
function Lua_HeroVo.Get:initial_awaken_star_id()
    if self._initial_awaken_star_id == nil then
        -- print(self.config,"   当前初始觉醒星级id")
        self._initial_awaken_star_id = DataCenter.heroUtil:getHeroAwakenStarId(self.config.quality, self.config.quality_star);
    end
    return self._initial_awaken_star_id;
end

--品质色
function Lua_HeroVo.Get:color()
    if self._color == nil then
        self._color = DataCenter.heroUtil:getHeroAwakenStarConfigInfo(self.awaken_star_id).color;
    end
    return self._color;
end

--星级
function Lua_HeroVo.Get:star()
    if self._star == nil then
        self._star = DataCenter.heroUtil:getHeroAwakenStar(self.awaken_star_id);
    end
    return self._star;
end

--初始觉醒星级id
function Lua_HeroVo.Get:awaken_star_id()
    if self._awaken_star_id == nil then
        if self._quality == nil or self._quality_star == nil then
            self._awaken_star_id = self.initial_awaken_star_id;
        else
            self._awaken_star_id = DataCenter.heroUtil:getHeroAwakenStarId(self.quality, self.quality_star);
        end
    end
    return self._awaken_star_id;
end

--质量
function Lua_HeroVo.Get:quality()
    if self._quality == nil then
        self._quality = self.config.quality;
    end
    return self._quality;
end

function Lua_HeroVo.Set:quality(val)
    if self._quality ~= val then
        self._star = nil;
        self._color = nil;
        self._awaken_star_id = nil;

        self._quality = val;
    end
end

--质量星级
function Lua_HeroVo.Get:quality_star()
    if self._quality_star == nil then
        self._quality_star = self.config.quality_star;
    end
    return self._quality_star;
end

function Lua_HeroVo.Set:quality_star(val)
    if self._quality_star ~= val then
        self._star = nil;
        self._color = nil;
        self._awaken_star_id = nil;

        self._quality_star = val;
    end
end

--等级
function Lua_HeroVo.Set:level(value)
    self._level = value;
end

function Lua_HeroVo.Get:level()
    return self._level == nil and 0 or self._level;
end

-- 最大星级
function Lua_HeroVo.Get:max_star()
    if self._max_star == nil then
        self._max_star = DataCenter.heroUtil:getHeroAwakenStar(self.max_awaken_star_id);
    end
    return self._max_star;
end

-- 最大初始星级id
function Lua_HeroVo.Get:max_awaken_star_id()
    if self._max_awaken_star_id == nil then
        self._max_awaken_star_id = DataCenter.heroUtil:getHeroAwakenStarId(self.config.max_quality, self.config.max_quality_star);
    end
    return self._max_awaken_star_id;
end

function Lua_HeroVo.Get:blood()
    return self._blood == nil and 0 or self._blood;
end

function Lua_HeroVo.Get:max_blood()
    return self._max_blood == nil and 1 or self._max_blood;
end

function Lua_HeroVo.Get:assist()
    return self._assist == nil and 0 or self._assist;
end

-- 是否为援助英雄-- 0 不是援助，1 邀请援助
function Lua_HeroVo.Set:assist(value)
   self._assist = value;
end

function Lua_HeroVo.Get:lineup_id()
    return self._lineup_id == nil and 0 or self._lineup_id;
end

function Lua_HeroVo.Get:weapon_id()
    return self._weapon_id or 0;
end

--是否展示英雄说明tips
function Lua_HeroVo.Set:bExplainTips(value)
    self._bExplainTips = value;
end

function Lua_HeroVo.Get:bExplainTips()
    if self._bExplainTips == nil then
        return false; 
    end
    return self._bExplainTips;
end

--其他人的阵容信息（用于连携激活）
function Lua_HeroVo.Set:otherLineup(list)
    self._otherLineup = list;
end

function Lua_HeroVo.Get:otherLineup()
    return self._otherLineup;
end

-- 英雄试用状态  0：非试用 1：试用
function Lua_HeroVo.Get:trial_state()
    if self.trial_tick ~= nil and self.trial_tick > 0 then
        return true;
    end
    return self.is_try == 1;
end



---------------------------------- 配置表数据 -------------------------------------------
function Lua_HeroVo.Get:name()
    if self._name ~= nil then
        return self._name;
    end
    return self.config.name;
end

function Lua_HeroVo.Set:name(val)
    self._name = val;
end

-- 称号
function Lua_HeroVo.Get:title()
    return self.config.title;
end

-- 职业
function Lua_HeroVo.Get:career()
    return self.config.career;
end

-- 性别
function Lua_HeroVo.Get:gender()
    return self.config.gender;
end

function Lua_HeroVo.Get:campType()
    if self._campType == nil then
        return self.config.camp;
    end
    return self._campType;
end

function Lua_HeroVo.Set:campType(val)
    self._campType = val;
end

--稀有度
function Lua_HeroVo.Get:rarity()
    return self.config.rarity;
end

--是否是核心
function Lua_HeroVo.Get:bcore()
    if self.config.bcore == nil then
        return false;
    end
    return self.config.bcore == 1;
end

-- -- 最高质量
-- function Lua_HeroVo.Get:max_quality() 
--     return self.config.max_quality;
-- end

-- -- 最高质量星级
-- function Lua_HeroVo.Get:max_quality_star() 
--     return self.config.max_quality_star;
-- end

-- 可调度
function Lua_HeroVo.Get:canTrans()
    return self.config.trans~=nil and self.config.trans>0;
end

function Lua_HeroVo.Get:career_name()
    return DataCenter.heroUtil:getHeroCareer(self.career);
end

-- 职业介绍
function Lua_HeroVo.Get:career_des()
    return self.config.career_des
end

-- 是否屏蔽英雄
function Lua_HeroVo.Get:isShield()
    return self.config.isShield;
end

--是否是怪物
function Lua_HeroVo.Get:isMonster()
    return self.config.isMonster;
end

-- 排序
function Lua_HeroVo.Get:sort()
    return self.config.sort;
end

-- 显示特效星级限制
function Lua_HeroVo.Get:show_effect_limit()
    return self.config.show_effect_limit;
end

-- 英雄描述
function Lua_HeroVo.Get:des()
    return self.config.des;
end

function Lua_HeroVo.Get:avatar_atlas_1()
    return self.config.avatar_atlas_1;
end

function Lua_HeroVo.Get:avatar_sprite_1()
    return self.config.avatar_sprite_1;
end

function Lua_HeroVo.Get:avatar_atlas_2()
    return self.config.avatar_atlas_2;
end

function Lua_HeroVo.Get:avatar_sprite_2()
    return self.config.avatar_sprite_2;
end

function Lua_HeroVo.Get:atlas()
    if self.clothes_id ~= nil and self.clothes_id ~= 0 then
        return getConfig("hero_clothes",self.clothes_id,"atlas");
    end
    return self.config.atlas;
end

function Lua_HeroVo.Get:sprite()
    if self.clothes_id ~= nil and self.clothes_id ~= 0 then
        return getConfig("hero_clothes",self.clothes_id,"sprite");
    end
    local sprite = self.config.sprite;
    if sprite == nil then
        sprite = self.id;
    end
    return tostring(sprite);
end

--英雄item立绘
function Lua_HeroVo.Get:path()
    return self.config.path;
end

--小卖部招募item立绘
function Lua_HeroVo.Get:path1()
    return self.config.path1;
end

--传记奖励
function Lua_HeroVo.Get:biography()
    return self.config.biography;
end

--传记描述列表
function Lua_HeroVo.Get:des_list()
    return self.config.des_list;
end

--战技推荐列表
function Lua_HeroVo.Get:talent_list()
    return self.config.talent_list;
end

function Lua_HeroVo.Get:modelUrl()
    return self.config.modelUrl;
end

--低模
function Lua_HeroVo.Get:modelSrc()
    return self.config.modelSrc;
end

function Lua_HeroVo.Set:uiSceneId(value)
    self._uiSceneId = value
end

function Lua_HeroVo.Get:uiSceneId()
    return self._uiSceneId or self.config.uiSceneId;
end

function Lua_HeroVo.Get:uiModelUrl()
    return self.config.uiModelUrl;
end

function Lua_HeroVo.Get:uiModelAvatarName()
    return self.config.uiModelAvatarName;
end

function Lua_HeroVo.Get:uiModelCamController()
    return self.config.uiModelCamController;
end

function Lua_HeroVo.Get:uiModelScale()
    return self.config.uiModelScale or 1;
end

function Lua_HeroVo.Get:uiModelDir()
    return 0;
end

function Lua_HeroVo.Get:uiShowAct()
    return self.config.uiShowAct;
end

function Lua_HeroVo.Get:poseActEffs()
    if self.weaponId > 0 and self.weapon_id > 0 then -- 有神兵且激活
        local arr = Array();
        local list = self.config.poseActEffs;
        if list then
            for i = 1, #list do
                arr:add(list[i]);
            end
        end
        list = self.config.activatePoseActEffs;
        if list then
            for i = 1, #list do
                arr:add(list[i]);
            end
        end
        return arr;
    end
    return self.config.poseActEffs
end

function Lua_HeroVo.Get:showActEffs()
    return self.config.showActEffs;
end

function Lua_HeroVo.Get:itemId()
    return self.config.itemId;
end

-- 碎片id
function Lua_HeroVo.Get:chipId()
    return self.config.chip_id;
end

-- 昆墟圣地进入战斗界面时显示的立绘
function Lua_HeroVo.Get:heroRolePath()
    return self.config.heroRolePath;
end

-- -- 英雄羁绊显示的立绘
-- function Lua_HeroVo.Get:heroFetterPath()
--     return self.config.heroFetterPath;
-- end

-- 模型Data
function Lua_HeroVo.Get:heroData()
    if self._heroData == nil then
        self._heroData = HeroData();
        self._heroData.roleId = self.id;
        self._heroData.objId = self.id;
        self._heroData.kind = Lua_ThingDefine.HERO;
        self._heroData.isShowName = false;
        self._heroData.IsUI = true;
        self._heroData.dir = 0;
        self._heroData.maxHp = 1;
        self._heroData.hp = 1;
    end
    -- if self.clothes_id ~= nil and self.clothes_id ~= 0 then
    --     self._heroData.hero_fashion_id = self.clothes_id;
    -- end
    return self._heroData;
end

-- 模型Data
function Lua_HeroVo:generateHeroData()
    local hd = HeroData();
    hd.roleId = self.id;
    hd.objId = self.id;
    hd.kind = Lua_ThingDefine.HERO;
    hd.isShowName = false;
    hd.IsUI = true;
    hd.dir = 0;
    hd.maxHp = 1;
    hd.hp = 1;

    -- if self.clothes_id ~= nil and self.clothes_id ~= 0 then
    --     self._heroData.hero_fashion_id = self.clothes_id;
    -- end
    return hd;
end

--特殊技能列表
function Lua_HeroVo.Get:lvl_skill_list()
    return self.config.lvl_skill_list;
end

function Lua_HeroVo.Get:fight_skill()--战斗技能
    return self.config.fight_skill;
end

-- 推荐觉醒
function Lua_HeroVo.Get:recom_awake()
    return self.config.recom_awake;
end

function Lua_HeroVo.Get:weaponId()
    return self.config.weaponId;
end

--故事卡id
function Lua_HeroVo.Get:cardList()
    return self.config.cardList;
end

--英雄简介
function Lua_HeroVo.Get:hero_des()
    return self.config.hero_des;
end

--定位
function Lua_HeroVo.Get:posit_des()
    return self.config.posit_des;
end

--特长
function Lua_HeroVo.Get:expertise_des()
    return self.config.expertise_des;
end

function Lua_HeroVo.Get:lineSound()
    return self.config.line_sound;
end

function Lua_HeroVo.Get:connectId()
    return self.config.connect_id;
end

--连携技能id
function Lua_HeroVo.Get:connectSkillId()
    return getConfig("hero_connect", self.connectId,"skill_id")
end









---------------------------------- 接口 -------------------------------------------
-- 获取当前英雄是否在指定阵容分队中上阵
function Lua_HeroVo:isShowLineup(lineup_type, team_index)
    if self.lineupDic == nil then
        return false;
    end
    if self.lineupDic[lineup_type] == nil or self.lineupDic[lineup_type][team_index] == nil then
        return false;
    end
    return self.lineupDic[lineup_type][team_index] == 1;
end













---------------------------------- 数据处理 -------------------------------------------
-- 是否是英雄完整道具
function Lua_HeroVo.Get:isIntactHero()
    return true;
end

-- 是否培养过等级
function Lua_HeroVo.Get:isCultivateHero()
    return self.level > 1;
end

-- 是否培养过星级
function Lua_HeroVo.Get:isCultivateStar()
    return self.star > self.initial_star;
end

-- 是否是最大星级
function Lua_HeroVo.Get:isMaxStar()
    return self.star >= self.max_star;
end

-- 是否在主线出战
function Lua_HeroVo.Get:isShowFight()
    local bool = self:isShowLineup(Lua_HeroUtil.LINEUP_MAIN_TYPE,1)
    return bool;
end

--上阵阵容名字
function Lua_HeroVo.Get:upLineupName()
    if self.lineupDic == nil then
        return "";
    end

    if self._upLineupName == "" then
        local config;
        for lineup_type, v in pairs(self.lineupDic) do
            config = getConfig("hero_camp_lineup", lineup_type)
            if config ~= nil and config.can_cancel == 1 then
                if self._upLineupName ~= "" then--允许下阵 阵容名字
                    self._upLineupName = self._upLineupName..","..config.name;
                else
                    self._upLineupName = config.name;
                end
            end
        end
    end
    
    return self._upLineupName;
end

--是否上阵
function Lua_HeroVo.Get:isUpLineup()
    if self.lineupDic == nil then
        return false;
    end

    if self._isUpLineup == false then
        local config;
        for lineup_type, v in pairs(self.lineupDic) do
            config = getConfig("hero_camp_lineup", lineup_type)
            if config ~= nil and config.can_cancel == 1  then
                self._isUpLineup = true;
            end
        end
    end
    
    return self._isUpLineup;
end

-- 职业图标
function Lua_HeroVo.Get:career_sprite()
    return "Hero_icon_7"..self.config.career+4;
end

-- 重修间隔时间
function Lua_HeroVo.Get:rebuild_interval_time()
    if self.rebuild_tick == 0 then
        return 0;
    end
    local interval_time = DataCenter.roleData.serverTime - self.rebuild_tick;
    if interval_time >= DataCenter.heroUtil.rebuild_next_time then
        return 0;
    end
    return DataCenter.heroUtil.rebuild_next_time - interval_time;
end

function Lua_HeroVo.Set:isSpecialState(value)
    self._isSpecialState = value;
end

-- 是否存在特殊状态  注：使用前请注意下面条件
function Lua_HeroVo.Get:isSpecialState()
    if self._isSpecialState ~= nil then
        return self._isSpecialState;
    end

    if self.lock_state then-- 上锁
        return true;
    end
    
    if self.isUpLineup then-- 是否已上阵
        return true;
    end

    return false;
end

--是否装备战技
function Lua_HeroVo.Get:isTalent()
    if self.talent_info ~= nil then
        for i=1,#self.talent_info do
            if tonumber(self.talent_info[i].value) ~= 0 then
                return true;
            end
        end
    end
    return false;
end

-- 最大星级升级等级上限
function Lua_HeroVo.Get:max_lvl_limit()
    return DataCenter.heroUtil:getHeroAwakenStarConfigInfo(self.max_awaken_star_id).lvl_limit;
end










---------------------------------- 功能数据 -------------------------------------------

-- 属性 Vo
function Lua_HeroVo.Get:heroPropVo()
    if self._heroPropVo == nil then
        self._heroPropVo = Lua_HeroPropVo.new(self.id);
    end
    if self.prop_list ~= nil then
        self._heroPropVo:updateServerInfo(self.prop_list);
    end
    return self._heroPropVo;
end

-- 升级 Vo
function Lua_HeroVo.Get:heroUpgradeVo()
    if self._upgradeVo == nil then
        self._upgradeVo = Lua_HeroUpgradeVo.new(self.initial_awaken_star_id);
    end
    self._upgradeVo:updateServerInfo(self.level, self.awaken_star_id, self.is_resonance, self.trial_state);
    return self._upgradeVo;
end

-- 升星 Vo
function Lua_HeroVo.Get:heroUpStarVo()
    if self._heroUpStarVo == nil then
        self._heroUpStarVo = Lua_HeroUpStarVo.new(self.id);
    end
    self._heroUpStarVo:updateServerInfo(self.awaken_star_id, self.quality, self.quality_star);
    return self._heroUpStarVo;
end

-- 技能数据
function Lua_HeroVo.Get:heroSkillInfo()
    if self.skillDic == nil then
        self.skillDic = {};
        local svo;
        local fight_skill = self.config.fight_skill;
        for i=1,#fight_skill do
            svo = Lua_HeroSkillVo.new(i);
            svo.skill_id = fight_skill[i];
            svo.skill_level = 1;
            self.skillDic[i] = svo;
        end
    end
    if self.skill ~= nil then
        local svo;
        for i=1,#self.skill do
            svo = self.skillDic[tonumber(self.skill[i].key)];
            if svo ~= nil then
                svo:fromServerInfo(self.skill[i]);
            end
        end
    end
    return self.skillDic;
end

-- 星魂数据
function Lua_HeroVo.Get:heroJewelryInfo()
    local jewelryInfo = {};
    if self.jewelry_list ~= nil then
        local jvo;
        for i=1,#self.jewelry_list do
            jvo = Lua_HeroJewelryVo.new(tonumber(self.jewelry_list[i].id));
            jvo:fromServerInfo(self.jewelry_list[i]);
            jewelryInfo[self.jewelry_list[i].part] = jvo;
        end
    end
    return jewelryInfo;
end


-- 时装数据
function Lua_HeroVo.Get:heroFashionInfo()
    local arr = Array();
    local list = self.config.clothes_list;
    local fvo;
    for i=1,#list do
        fvo = DataCenter.heroData:getFashionVoById(self.id, list[i]);
        if fvo == nil then
            fvo = Lua_HeroFashionVo.new(list[i]);
        end
        fvo.index = i;
        fvo.clothes_id = self.clothes_id;
        arr:add(fvo);
    end
    return arr;
end

-- 装备数据
function Lua_HeroVo.Get:equipInfo()
    if self._equipInfo == nil then
        self._equipInfo = {};
        if self.equip_info ~= nil then
            local evo;
            for i=1,#self.equip_info do
                evo = DataCenter.equipData:getEquipVoByUid(self.equip_info[i].unique_id);
                -- print(evo,"    装备数据 = =",self.equip_info[i].unique_id)
                if evo ~= nil then
                    self._equipInfo[evo.part] = tonumber(evo.unique_id);
                end
            end
        end
    end
    return self._equipInfo;
end

-- 符石数据
function Lua_HeroVo.Get:runsInfo()
    if self._runsInfo == nil then
        self._runsInfo = {};
        if self.rune_info ~= nil then
            for i=1,#self.rune_info do
                -- print(self.rune_info[i].key,"     部位   符石数据   uid   ",self.rune_info[i].value)
                self._runsInfo[tonumber(self.rune_info[i].key)] = tonumber(self.rune_info[i].value);
            end
        end
    end
    return self._runsInfo;
end

-- 战技数据
function Lua_HeroVo.Get:talentInfo()
    if self._talentInfo == nil then
        self._talentInfo = {};
        if self.talent_info ~= nil then
            for i=1,#self.talent_info do
                self._talentInfo[tonumber(self.talent_info[i].key)] = tonumber(self.talent_info[i].value);
            end
        end
    end
    return self._talentInfo;
end

--是否查看自己的英雄信息
function Lua_HeroVo.Set:bSelfRole(val)
    self._bSelfRole = val;
end

-- Tips装备数据
function Lua_HeroVo.Get:equipTipsInfo()
    local equipInfo = {};
    if self.equip_info ~= nil then
        local evo;
        for i=1,#self.equip_info do
            evo = Lua_EquipVo.new(self.equip_info[i].id);
            evo:fromServer(self.equip_info[i]);
            evo.bSelfRole = self._bSelfRole;
            equipInfo[evo.part] = evo;
        end
    end
 
    return equipInfo;
end

-- tips符石数据
function Lua_HeroVo.Get:runsTipsInfo()
    local rvo;
    local runsInfo = {};
    if self.rune_info ~= nil then
        for i=1,#self.rune_info do
            -- print("符石数据 = =",self.rune_info[i])
            if self.rune_info[i].rune ~= nil then
                rvo = Lua_RunesVo.new(self.rune_info[i].rune.id);
                rvo:fromServer(self.rune_info[i].rune);
                runsInfo[self.rune_info[i].part] = rvo;
            else
                runsInfo[tonumber(self.rune_info[i].key)] = tonumber(self.rune_info[i].value);
            end
        end
    end
    return runsInfo;
end


---------------------------------- 红点 -------------------------------------------
-- 闪卡红点
function Lua_HeroVo.Get:bHeroCardRedDot()
    if self.cardList ~= nil and self.cardList[1] ~= nil then
        local bool = DataCenter.collectionBookData:getRewardStateByFlashId(self.cardList[1]);
        if bool == nil then
            bool = false;
        end
        return bool;
    end

    return false;
end





---------------------------------- 权重排序 -------------------------------------------
-- rarity稀有度 star星级 level等级

-- 升星选择排序 (降序) 上锁 ＞ 上阵 ＞ 初始星级 > 等级 > 是否是核心
function Lua_HeroVo.Get:upStarSort()
    if self._upStarSort == nil then
        local weight = 0;
        if self.lock_state == true then--上锁
            weight = 100000000000;
        elseif self.isUpLineup == true then--上阵
            weight = 10000000000;
        end
        if self.bcore then
            weight = weight + 10000000;
        end
        self._upStarSort = weight + self.max_star*100000000 + self.level*10000 + self.sort * -1;
    end
    return self._upStarSort;
end

-- 【主线阵容出战英雄】＞星级高低 > 等级高低 > 是否是核心 
-- 1、背包 2、生命神树格子选择英雄列表
function Lua_HeroVo.Get:backSort()
    if self._backSort == nil then
        local weight = 0;
        if self.isShowFight == true then--出阵型主线
            weight = 10000000000;
        end

        if self.bcore then
            weight = weight + 10000;
        end

        self._backSort = weight + self.star*100000000 + self.level*100000 + self.sort;
    end
    return self._backSort;
end

function Lua_HeroVo:lineupSort(lineup_type, team_index)
    local value = 0;
    -- 【已上阵】>战力高低
    if self:isShowLineup(lineup_type, team_index) == true then
        value = 10000000000;
    end
    return value + self.heroForce
end

-- 神树入驻排序
function Lua_HeroVo.Get:lifeTreeSort()
    if self._lifeTreeSort == nil then
        self._lifeTreeSort = self.level*1000000000000 + self.heroForce *100 + self.sort;
    end
    return self._lifeTreeSort;
end


-- 回退/遣散排序 (降序)
function Lua_HeroVo.Get:decomposeSort()
    if self._decomposeSort == nil then
        local weight = 0;
        if self.lock_state == true then
            weight = 1000000000;
        elseif self.isUpLineup == true then
            weight = 100000000;
        elseif self.is_resonance == true then
            weight = 10000000;
        end
        self._decomposeSort = weight + self.star * 10000000 + self.level * 10000 + self.sort * -1;
    end
    return self._decomposeSort;
end