checkRequire("Lua_HeroMaterialVo")
Lua_HeroMaterialData = class("Lua_HeroMaterialData")
function Lua_HeroMaterialData:ctor()
	self:superCall("ctor")
end

function Lua_HeroMaterialData:dispose()
	self.curHeroUnId = nil
	self.selectedDic = nil
	self.uniqueIdDic = nil
end

function Lua_HeroMaterialData:init(uniqueId)
	self.curHeroUnId = uniqueId
	self.selectedDic = {} -- 以材料索引选中的英雄列表
	self.uniqueIdDic = {} -- 已选中英雄列表
end

-- 记录
function Lua_HeroMaterialData:record()
	local selectedDic = {}
	for index, arr in pairs(self.selectedDic) do
		if selectedDic[index] == nil then
			selectedDic[index] = Array()
		end
		for i = 1, arr:getSize() do
			selectedDic[index]:add(arr[i])
		end
	end
	return selectedDic
end

-- 还原
function Lua_HeroMaterialData:revert(selectedDic)
	self.selectedDic = {}
	self.uniqueIdDic = {}
	for index, arr in pairs(selectedDic) do
		for i = 1, arr:getSize() do
			self:addHero(index, arr[i])
		end
	end
end

function Lua_HeroMaterialData:addHero(index, uniqueId)
	if self.selectedDic[index] == nil then
		self.selectedDic[index] = Array()
	end
	if self.selectedDic[index]:indexOf(uniqueId) == -1 then
		self.selectedDic[index]:add(uniqueId)
	end

	self.uniqueIdDic[uniqueId] = 1
end

function Lua_HeroMaterialData:removeHero(index, uniqueId)
	if self.selectedDic and self.selectedDic[index] then
		self.selectedDic[index]:removeItem(uniqueId)
	end
	if self.uniqueIdDic then
		self.uniqueIdDic[uniqueId] = nil
	end
end

function Lua_HeroMaterialData:isSelectedHero(uniqueId)
	if self.uniqueIdDic == nil then
		return false
	end
	return self.uniqueIdDic[uniqueId] == 1
end

function Lua_HeroMaterialData:isSelectedHeroByIdx(index, uniqueId)
	if self.selectedDic == nil or self.selectedDic[index] == nil then
		return false
	end
	return self.selectedDic[index]:indexOf(uniqueId) ~= -1
end

function Lua_HeroMaterialData:getSelectedNum(index)
	if self.selectedDic == nil or self.selectedDic[index] == nil then
		return 0
	end
	return self.selectedDic[index]:getSize()
end

function Lua_HeroMaterialData:getHeroList(materialVo, bScreen)
	local vo;
	local arr = Array();
	local uidArr = DataCenter.heroData:getHeroUidArrByStar(materialVo.star)
	for i = 1, uidArr:getSize() do
		if uidArr[i] ~= self.curHeroUnId then
			if self:isSelectedHeroByIdx(materialVo.index, uidArr[i]) then
				arr:add(uidArr[i])
			else
				vo = DataCenter.heroData:getItemByUniqueId(uidArr[i]);
				local bool = true;
				if bScreen then--需要条件筛选
					bool =  not vo.isUpLineup and not vo.lock_state and not vo.bcore;
				end
				if bool then
					if materialVo.id > 0 then
						if vo.id == materialVo.id and vo.campType == materialVo.camp and vo.star == materialVo.star and vo.is_league_transaction == false and not vo.trial_state then
							arr:add(uidArr[i])
						end
					elseif materialVo.camp > 0 then
						if vo.campType == materialVo.camp and vo.star == materialVo.star and vo.is_league_transaction == false and not vo.trial_state then
							arr:add(uidArr[i])
						end
					elseif star > 0 then
						if vo.star == materialVo.star and vo.is_league_transaction == false then
							arr:add(uidArr[i])
						end
					end
				end
			end
		end
	end
	return arr
end

function Lua_HeroMaterialData:checkEnoughByVo(materialVo)
	local selectedNum = self:getSelectedNum(materialVo.index)
	local needNum = materialVo.num - selectedNum
	local arr = DataCenter.heroData:getHeroUidArrByStar(materialVo.star)
	for i = 1, arr:getSize() do
		vo = DataCenter.heroData:getItemByUniqueId(arr[i])
		if arr[i] ~= self.curHeroUnId and not self:isSelectedHero(arr[i]) and not vo.isUpLineup and not vo.lock_state and vo.is_league_transaction == false and not vo.trial_state then
			if materialVo.id > 0 then
				if vo.id == materialVo.id and vo.campType == materialVo.camp and vo.star == materialVo.star then
					needNum = needNum - 1
					if needNum <= 0 then
						return true
					end
				end
			elseif materialVo.camp > 0 then
				if vo.campType == materialVo.camp and vo.star == materialVo.star then
					needNum = needNum - 1
					if needNum <= 0 then
						return true
					end
				end
			elseif materialVo.star > 0 then
				if vo.star == materialVo.star then
					needNum = needNum - 1
					if needNum <= 0 then
						return true
					end
				end
			end
		end
	end
	return false
end

function Lua_HeroMaterialData:checkEnough(material)
	if material then
		for i = 1, #material do
			if self:getSelectedNum(i) < material[i][3] then
				return false
			end
		end
	end
	return true
end

--记录已解除上阵状态的英雄Uid
function Lua_HeroMaterialData:initRemoveLineupStateDic(state)
    if state then
        if self.uniqueIdDic ~= nil then
            for uid, v in pairs(self.uniqueIdDic) do
                if v == 1 then
                    self.removeLineupStateDic[uid] = 1;
                end
            end
        end
    else
        self._removeLineupStateDic = nil;
    end
end

function Lua_HeroMaterialData.Get:removeLineupStateDic()
    if self._removeLineupStateDic == nil then
        self._removeLineupStateDic = {};
    end

    return self._removeLineupStateDic;
end

-- 选择的英雄所在的阵容和已选择的英雄所在的阵容数量 > 所在阵容的数量才可以下阵
function Lua_HeroMaterialData:checkLineupNumState(select_uid)
    if self.uniqueIdDic == nil or select_uid == nil then
        return true;
    end

    
    local hvo;
    local config;
    local lineupList = {};
    for uid, v in pairs(self.uniqueIdDic) do
        if v >= 1 then
            hvo = DataCenter.heroData:getItemByUniqueId(uid);
            if hvo.lineupDic ~= nil then
                for lineup_type, vul in pairs(hvo.lineupDic) do
                    config = getConfig("hero_camp_lineup", lineup_type);
                    if config ~= nil and config.can_cancel == 1 then
                        if lineupList[lineup_type] == nil then
                            lineupList[lineup_type] = 1;
                        else
                            lineupList[lineup_type] = lineupList[lineup_type] +1;
                        end
                    end
                end
            end
        end
    end

    hvo = DataCenter.heroData:getItemByUniqueId(select_uid);
    if hvo ~= nil and hvo.lineupDic ~= nil then
        for lineup_type, vul in pairs(hvo.lineupDic) do
            config = getConfig("hero_camp_lineup", lineup_type);
            if config ~= nil and config.can_cancel == 1 then
                if lineupList[lineup_type] == nil then
                    lineupList[lineup_type] = 1;
                else
                    lineupList[lineup_type] = lineupList[lineup_type] +1;
                end

                if lineupList[lineup_type] >= DataCenter.heroLineupData:getLineupHeroTotalNum(lineup_type) then
                    return false, config.name;
                end
            end
        end
    end

    return true;
end


-- 外部接口
------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- [[星级，阵营，数量，英雄ID]]
function Lua_HeroMaterialData:checkEnoughByList(material, ignoreUnId)
	if material then
		local id
		local star
		local camp
		local needNum
		local totalNum = 0
		local arr
		local vo
		local unique_id
		local selectedList = {}
		local selectedNum = 0
		for i = 1, #material do
			id = material[i][4]
			star = material[i][1]
			camp = material[i][2]
			needNum = material[i][3]
			totalNum = totalNum + needNum

			arr = DataCenter.heroData:getHeroUidArrByStar(star)
			for j = 1, arr:getSize() do
				unique_id = arr[j]
				vo = DataCenter.heroData:getItemByUniqueId(unique_id)
				if unique_id ~= ignoreUnId and selectedList[unique_id] == nil then
					if id > 0 then
						if vo.id == id and vo.campType == camp and vo.star == star and vo.is_league_transaction == false and not vo.trial_state then
							needNum = needNum - 1
							selectedList[unique_id] = 1
							selectedNum = selectedNum + 1
							if needNum <= 0 then
								break
							end
						end
					elseif camp > 0 then
						if vo.campType == camp and vo.star == star then
							needNum = needNum - 1
							selectedList[unique_id] = 1
							selectedNum = selectedNum + 1
							if needNum <= 0 then
								break
							end
						end
					elseif star > 0 then
						if vo.star == star then
							needNum = needNum - 1
							selectedList[unique_id] = 1
							selectedNum = selectedNum + 1
							if needNum <= 0 then
								break
							end
						end
					end
				end
			end
		end

		return selectedNum >= totalNum
	end
	return true
end