-- 功能开启
Lua_FuncOpenData = class("Lua_FuncOpenData")
function Lua_FuncOpenData:ctor()
	self:superCall("ctor");
	self:dispose();
end

function Lua_FuncOpenData:dispose()
	self.bInit = false;
	self.bInitServerInfo = false;
	self.bFirstEnter = true;
end

function Lua_FuncOpenData:fromServerData(function_open)
	if function_open == nil or #function_open <= 0 then
		return;
	end

	local bChangeIcon = false;
	local vo;
	local openDic = {};
	for i = 1, #function_open do
		vo = DataCenter.funcOpenUtil:getFuncOpenVoById(function_open[i].function_id);
		if vo ~= nil then
			openDic[vo.id] = function_open[i].function_flag;
			vo:fromServerInfo(function_open[i]);
			if vo.iconId > 0 then
				if self:setMenuIconState(vo.iconId, vo.isOpen) then
					bChangeIcon = true;
				end
			end
			if vo.pop_state == 1 and vo.is_preview == 1 then
				Lua_WindowUtil.openWin("Lua_FuncPreviewTipsWindow", Lua_WinParam.ACTION_OPEN, vo.id)
			end
		end
	end

	if bChangeIcon then
		Lua_MsgUtil.callFunc("MENUICON_ALL_UPDATE");
	end

	if self.bFirstEnter then
		self.bFirstEnter = false;
		self:initServerInfo();
		Lua_MsgUtil.callFunc("INIT_FUNCTION_OPEN", openDic);
	else
		Lua_MsgUtil.callFunc("FUNCTION_OPEN", openDic);
	end
	self:checkRedPoint()
	self.bInit = true;
end

function Lua_FuncOpenData:updateSingleFunc(function_open)
	local vo = DataCenter.funcOpenUtil:getFuncOpenVoById(function_open.function_id);
	vo:fromServerInfo(function_open);
	self:checkRedPoint()
end

function Lua_FuncOpenData:setMenuIconState(iconId, state)
	local vo = DataCenter.mainMenuData:getMenuIconVo(iconId);
	if vo == nil then
		return false;
	end
	
	local bChange = false;
	if not state then
		state = self:getMenuIconState(iconId)
	end
	if vo.bFunOpenState ~= state then
		vo:setFunOpenState(state);
		bChange = true;
	end
	return bChange;
end

function Lua_FuncOpenData:getMenuIconState(iconId)
	local arr = DataCenter.funcOpenUtil:getFuncOpenArrByIconId(iconId)
	if arr then
		for i = 1, arr:getSize() do
			vo = DataCenter.funcOpenUtil:getFuncOpenVoById(arr[i]);
			if vo.isOpen then
				return true;
			end
		end
	end
	return false;
end

-- bGuideType 引导用 : 1:已弹窗  2：奖励已领取   3： 已弹窗且奖励已领取  4:已弹窗且奖励可领取
function Lua_FuncOpenData:isOpened(funcId, bShowTips, bGuideType)
	if funcId == nil or funcId <= 0 then
		return true;
	end

	local vo = DataCenter.funcOpenUtil:getFuncOpenVoById(funcId);
	if vo == nil then
		return true;
	end
	
	if bShowTips == true and not vo.isOpen then
		if not strIsNullOrEmpty(vo.advanceMessage) then
			OperateAlert.show(vo.advanceMessage);
		else
			OperateAlert.showLangValue(107707);
		end
	end

	local isOpen = vo.isOpen;
	if isOpen and bGuideType ~= nil then
		if bGuideType == 1 then
			isOpen = vo.pop_state == 2;
		elseif bGuideType == 2 then
			isOpen = vo.reward_state == 2;
		elseif bGuideType == 3 then
			isOpen = vo.pop_state == 2 and vo.reward_state == 2;
		elseif bGuideType == 4 then
			isOpen = vo.pop_state == 2 and vo.reward_state == 1;
		end
	end
	
	return isOpen;
end

-- 功能开启数据、背包数据、货币数据下发完成才初始化服务端数据
function Lua_FuncOpenData:initServerInfo()
	if self.bInitServerInfo ~= true and self.bInit == true and DataCenter.packData.bInit == true and DataCenter.moneyData.bInit == true then
		self.bInitServerInfo = true;
		DataCenter.mainData:initServerInfo();
	end
end

--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
function Lua_FuncOpenData:checkRedPoint()
	local arr = DataCenter.funcOpenUtil:getPreviewArr()
	local vo
	local previewEnd = true
	local redPointState = false
	for i = 1, arr:getSize() do
		vo = DataCenter.funcOpenUtil:getFuncOpenVoById(arr[i])
		if not vo.isOpen then
			previewEnd = false
		end
		if vo.reward_state == 1 then
			redPointState = true
		end
	end
	if previewEnd and not redPointState then
		DataCenter.mainMenuData:setIconShowInfo(Lua_MenuViewDefine.ICON_FUNC_PREVIEW, false)
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.FuncPreview, redPointState)
end