Lua_ChatInfoMiniPanel = class("Lua_ChatInfoMiniPanel", Lua_DisplayObject);
function Lua_ChatInfoMiniPanel:ctor(p)
    self:superCall("ctor", p, "ChatInfoMiniPanel_skin");
end

function Lua_ChatInfoMiniPanel:buildShowItem()
    self:superCall("buildShowItem");

	self.mask = UICreate.createUIMaskContainer(self.content, nil, nil, 0, 132, 635, 41);
	self.mask.pivot = Lua_WindowUtil.getPivot(4);
	self.mask:setColor(1, 1, 1, 1);
	self.mask:addRectMask2D();

    self.chatText = UICreate.createLabel(self.mask.sub, nil, -290, 0, 635, 40, 22);
    self.chatText.pivotValue = 3
    self.chatText.alignmentValue = 3;
	self.chatText.horizontalOverflowValue = 1;

	self.redPoint = Lua_RedPointIcon.new(self.content, Lua_RedPointList.Chat_Mini);
	self.redPoint:setLocation(-301, 150, 0);
	self.redPoint:setRectAnchor(7);

    self.chatBg:addClickFuntion(self, self.onChatClick);

	self:addLuaEvent("ADD_CHAT_MSG", self, self.updateChatInfo);
	self:addLuaEvent("CHAT_RED_UPDATE", self, self.updateRedDot);
	self:addLuaEvent("REMOVE_FRIEND_UPDATE", self, self.updateChatInfo);
	self:updateChatInfo();
end

function Lua_ChatInfoMiniPanel:dispose()
	Lua_CommonUtil.removeTimer(self, self.showRedDot);
	if self.redPoint ~= nil then
		self.redPoint:dispose();
		self.redPoint = nil;
	end

	if self.mask ~= nil then
		self.mask:dispose();
		self.mask = nil;
	end

	if self.emjioEff ~= nil then
		self.emjioEff:dispose();
		self.emjioEff = nil;
	end
	
	self:superCall("dispose");
end

function Lua_ChatInfoMiniPanel:onChatClick(go)
	if DataCenter.funcOpenData:isOpened(10237, true) then
		Lua_WindowUtil.openWin("Lua_ChatWindow", Lua_WinParam.ACTION_OPEN, DataCenter.chatData:getAutoOpenChannel());
	end
end

function Lua_ChatInfoMiniPanel:updateChatInfo()
	local chatVo = DataCenter.chatData.latestChatVo;
	if chatVo == nil or chatVo.channel == nil then
		return;
	end
	
	if self.emjioEff ~= nil then
		self.emjioEff:setEnable(false);
	end

	local chatStr = LanUtil.getValue(171050 + chatVo.channel)
	local faceSize = 30; -- 表情显示大小
	if chatVo.chat_player then
		if DataCenter.roleData:isMyself(chatVo.chat_player.id) then
			chatStr = spliceStr(chatStr, LanUtil.formateStr(171041, LanUtil.getValue(19)));
		else
			chatStr = spliceStr(chatStr, LanUtil.formateStr(171041, chatVo.chat_player.name));
		end
	end

	if chatVo.content_type == Lua_ChatData.CONTENT_VOICE then
		local str = LanUtil.formateStr(171040, tostring(chatVo.voice_len))
		self.chatText.text = spliceStr(chatStr, spliceIconStr("common_altas_1", "Chat_widget_020", faceSize, faceSize), str);
	elseif chatVo.content_type == Lua_ChatData.CONTENT_EQUIP_COPY then
		if chatVo.recordInfo == nil then
			return;
		end
		local bossDifficulty = chatVo.recordInfo.boss_difficulty;
		local str = LanUtil.formateStr(171048, getConfig("equip_copy_more", bossDifficulty).boss_name );
		self.chatText.text = LanUtil.formateStr(171042, chatStr, str);
	elseif chatVo.content_type == Lua_ChatData.CONTENT_COLLECTIONBOOK_CARD then
		if chatVo.recordInfo == nil then
			return;
		end
		self.chatText.text = LanUtil.formateStr(171042, chatStr, LanUtil.getValue(410056));
	elseif chatVo.content_type == Lua_ChatData.CONTENT_COLLECTION_BOOK then
		if chatVo.recordInfo == nil then
			return;
		end
		self.chatText.text = LanUtil.formateStr(171042, chatStr, LanUtil.getValue(410057));
	elseif chatVo.content_type == Lua_ChatData.CONTENT_DYNANIC_EMOJI then -- 动态表情
		local emojiInfo = DataCenter.chatData:getConfigById(chatVo.content);
        if emojiInfo == nil then
            return;
        end
        self.chatText.text = LanUtil.formateStr(171042, chatStr, "");
        self.chatText:resetSize();
        local posX = -289 + self.chatText.width + 17; -- -289为文本左中顶点的初始位置
        if self.emjioEff == nil then
            self.emjioEff = UICreate.createUIEffect(self.content, nil, nil, -204, -173)
        end
        self.emjioEff:setUrl(emojiInfo.url, emojiInfo.matName);
        self.emjioEff:setLocation(posX, 113);
        self.emjioEff:setScale(0.3,0.3,0.3)
        self.emjioEff:setEnable(true);
	else
		arr = Lua_ChatUtil.genChatString(chatVo.content);
		len = arr:getSize();
		local tempStr = "";
		local iconId;
		for i = 1, len do
			if Lua_ChatUtil.isChatFace(arr[i]) == true then
				iconId = string.sub(arr[i], 2, 4);
				tempStr = spliceStr(tempStr, spliceEmojiStr("Chat_icon_"..iconId, faceSize, faceSize));
			else
				tempStr = spliceStr(tempStr, arr[i]);
			end
		end
		self.chatText.text = LanUtil.formateStr(171042, chatStr, tempStr);
	end

	self:updateRedDot();
end

function Lua_ChatInfoMiniPanel:updateRedDot()
	Lua_CommonUtil.delayCollectCall("show_reddot_chatMiniPanel", self, self.showRedDot, nil, 1);
end

function Lua_ChatInfoMiniPanel:showRedDot()
	local count = DataCenter.chatData:getAllChannelNewMsgNum();
	if count <= 0 then
		count = DataCenter.chatData:getChannelNewMsgNum(Lua_ChatData.CHANNEL_FRIEND);--私聊数量
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Chat_Mini, count > 0);
end