Lua_RoleInfoPanel = class("Lua_RoleInfoPanel", Lua_DisplayObject);
function Lua_RoleInfoPanel:ctor(p, moneyType1, moneyType2)
	self.moneyType1 = moneyType1 == nil and Lua_ResourcesDefine.GOLD or moneyType1;
	self.moneyType2 = moneyType2 == nil and Lua_ResourcesDefine.DIAMOND or moneyType2;
	self:superCall("ctor", p, "RoleInfoPanel_skin", true);
end

function Lua_RoleInfoPanel:buildShowItem()
	self:superCall("buildShowItem");

	self.exp.typeValue = 3;
	self.exp.fillMethodValue = 0;
	self.exp.fillOrigin = 0;

	self.avatarItem = Lua_AvatarItem.new(self.content);
	self.avatarItem:setLocation(-298, -53, 0);
	self.avatarItem:setAvatarClick(self, self.onAvatarClick);

	self.forceNum = Lua_NumberImage.new(self.content);
	self.forceNum:setLocation(-185, -79, 0);
	self.forceNum.pivot = 1;
	self.forceNum.numOffsetX = -2;
	self.forceNum.numAtlas = "number";
	self.forceNum.preStr = "xui_zl1_number_0";

	if Lua_PlatformUtil.IsNoSdk() and (Lua_PlatformUtil.IsAndroid() or Lua_PlatformUtil.IsIPhonePlayer()) then
		self.gmBtn = UICreate.createButton(self.content, "GM", 310, -78, 100, 40);
		self.gmBtn:addClickFuntion(self, self.onGMClick);
	end

	self.moneyItem1 = Lua_MoneyItem.new(self.content);
	self.moneyItem1:setLocation(59, -26, 0);
	self.moneyItem1:setCanClick(true);
	self.moneyItem1:setMoneyType(self.moneyType1);

	self.moneyItem2 = Lua_MoneyItem.new(self.content);
	self.moneyItem2:setLocation(249, -26, 0);
	self.moneyItem2:setCanClick(true);
	self.moneyItem2:setMoneyType(self.moneyType2);

	self.redPointIcon = Lua_RedPointIcon.new(self.content, Lua_RedPointList.DressUp);
	self.redPointIcon:setLocation(-265, -15, 0);
	
	self:addLuaEvent("UPDATE_ROLE_INFO", self, self.updateRoleInfo);
	self:addLuaEvent("UPDATE_ROLE_LEVEL", self, self.updateRoleLevel);
	self:addLuaEvent("UPDATE_ROLE_FORCE", self, self.updateRoleForce);
	self:addLuaEvent("UPDATE_DRESSUP_INFO", self, self.updateDressUpInfo);
	self:addLuaEvent("MONEY_UPDATE", self, self.onMoneyUpdate);
	self:addLuaEvent("VIP_LEVEL_UPDATE", self, self.updateRoleVip);
	
	self:updateRoleInfo();
	self:updateRoleLevel();
	self:updateRoleForce();
	self:updateDressUpInfo();
end

function Lua_RoleInfoPanel:dispose()
	if self.forceNum ~= nil then
		self.forceNum:dispose();
		self.forceNum = nil;
	end
	if self.avatarItem ~= nil then
		self.avatarItem:dispose();
		self.avatarItem = nil;
	end
	if self.moneyItem1 ~= nil then
		self.moneyItem1:dispose();
		self.moneyItem1 = nil;
	end
	if self.moneyItem2 ~= nil then
		self.moneyItem2:dispose();
		self.moneyItem2 = nil;
	end
	if self.redPointIcon ~= nil then
		self.redPointIcon:dispose();
		self.redPointIcon = nil;
	end
	self:superCall("dispose");
end

function Lua_RoleInfoPanel:updateRoleInfo()
	self.name.text = DataCenter.roleData.name;
end

function Lua_RoleInfoPanel:updateRoleLevel()
	self.avatarItem:setRoleLevel(DataCenter.roleData.level, -31.88, -30.88);
	self.avatarItem:setRoleLevelBg("common_altas_1", "xui_common_widget_016");
	self.avatarItem:setRoleLevelStyle(Lua_MyUISettings.defaultFontName, 16, "FEFEFE", 2, "000000");
	self:updateRoleExp();
end

function Lua_RoleInfoPanel:updateRoleForce()
	self.forceNum.num = DataCenter.roleData.force;
end

function Lua_RoleInfoPanel:updateDressUpInfo()
	local roleData = DataCenter.roleData;
	local avatarId = roleData:getOwnDressUpId(Lua_DressUpDefine.MAINTYPE_HEAD);
	local frameId = roleData:getOwnDressUpId(Lua_DressUpDefine.MAINTYPE_FRAME);
	self.avatarItem:setAvatarBg("common_altas_1", "xui_common_widget_003");
	self.avatarItem:setRoleAvatar(avatarId, 0.68);
	self.avatarItem:setRoleVip(roleData.vip);
	self.avatarItem:setRoleFrame(frameId, 0.7);
end

function Lua_RoleInfoPanel:onMoneyUpdate(dic)
	if dic == nil or dic[Lua_ResourcesDefine.ROLE_EXP] == nil then
		return;
	end
	self:updateRoleExp();
end

function Lua_RoleInfoPanel:updateRoleExp()
	if DataCenter.roleData.bMaxLv then
		self.exp.fillAmount = 1
	else
		local exp = DataCenter.moneyData:getMoney(Lua_ResourcesDefine.ROLE_EXP);
		if exp <= 0 then
			exp = DataCenter.roleData.exp;
		end
		local vo = DataCenter.roleUtil:getRoleVoByLv(DataCenter.roleData.level);
		self.exp.fillAmount = exp / vo.exp
	end
end

function Lua_RoleInfoPanel:updateRoleVip()
	self.avatarItem:setRoleVip(DataCenter.roleData.vip);
end

function Lua_RoleInfoPanel:onAvatarClick()
	Lua_WindowUtil.openWin("Lua_PersonalInfoWindow", Lua_WinParam.ACTION_OPEN);
end

function Lua_RoleInfoPanel:onGMClick()
	Lua_WindowUtil.openWin("Lua_DebugWindow");
end