checkRequire("Lua_MainLineTaskVo");
Lua_MainLineData = class("Lua_MainLineData")
function Lua_MainLineData:ctor()
	self:superCall("ctor");
	self:dispose();
end

function Lua_MainLineData:dispose()
	self.checkpointId = 10001;
	self.checkpointState = 0;
	self.passDic = {}; -- 已通关列表

	self:removeCheckTask();
	self.bTaskCanDispatch = false; -- 情报可派遣
	self.bTaskCanReceive = false; -- 情报可领取
	self.bTaskHighQuality = false; -- 情报有高品质
	self._task_free_times = 0
	self._task_temp_info = 0
	self._task_temp_tick = 0
	self.levelIdWhenFistEnter = -1; -- 引导判断结算界面是否已弹出用
	self.levelStateWhenFistEnter = -1; -- 引导判断结算界面是否已弹出用
end

function Lua_MainLineData:fromServer50140002(msg)
	if msg == nil then
		return;
	end

	local id
	for i = 1, #msg.game_level_id do
		id = msg.game_level_id[i]
		self.passDic[id] = 2
	end
	self:checkRewardRedPoint();
end


function Lua_MainLineData:fromServer50140004(msg)
	if msg == nil then
		return;
	end
	self:setLevelWhenFistEnter(msg.stage_id, msg.state);

	self.checkpointId = msg.stage_id; -- 当前关卡id
	self.checkpointState = msg.state; -- 关卡通关状态：0：未通关，1：已通关。每一章的最后一关通关时才会置1，其他关卡通关后直接通知下一关
	
	self.passDic = {};
	local obj;
	for i = 1, #msg.pass_list do
		obj = msg.pass_list[i];
		self.passDic[tonumber(obj.key)] = tonumber(obj.value); -- 0: 不可领取 1: 可领取 2:已领取
	end

	self:updateCheckpointHeadInfo();
	self:checkRewardRedPoint();
end

function Lua_MainLineData:fromServer80140001(msg)
	if msg == nil then
		return;
	end
	self:setLevelWhenFistEnter(msg.stage_id, msg.state);

	self.checkpointId = msg.stage_id;
	self.checkpointState = msg.state;
	if msg.reward_stage_id > 0 then
		self.passDic[msg.reward_stage_id] = 1; -- 可领取
	end
	self:updateCheckpointHeadInfo();
	self:checkRewardRedPoint();
end

-- 当前显示场景
function Lua_MainLineData.Get:curSceneId()
	if DataCenter.bConfigLoaded ~= true then
		return Lua_SceneDefine.MAINLINE;
	end
	
	local config = Lua_MainLineChapterConfigData.getConfig(self.curChapterId);
	if config == nil then
		return Lua_SceneDefine.MAINLINE;
	end
	return config.scene_id;
end

-- 当前战斗模板ID
function Lua_MainLineData.Get:curTemplateId()
	if DataCenter.bConfigLoaded ~= true then
		return Lua_FightDefine.TEMPLATE_ID_MAINLINE;
	end
	
	local config = Lua_MainLineChapterConfigData.getConfig(self.curChapterId);
	if config == nil then
		return Lua_FightDefine.TEMPLATE_ID_MAINLINE;
	end
	return config.templateId;
end

-- 当前篇章ID的Sort
function Lua_MainLineData.Get:curChapterIdSort()
	local config = Lua_MainLineChapterConfigData.getConfig(self.curChapterId);
	if config == nil or config.sort == nil then
		return 1;
	end
	return config.sort;
end

-- 当前篇章ID
function Lua_MainLineData.Get:curChapterId()
	local config = Lua_MainLineCheckpointConfigData.getConfig(self.checkpointId);
	if config == nil then
		return 1;
	end
	return config.chapter_id;
end

-- 当前关卡ID
function Lua_MainLineData.Set:checkpointId(val)
	self._checkpointId = val;
end

function Lua_MainLineData.Get:checkpointId()
	return self._checkpointId;
end

function Lua_MainLineData.Get:curCheckpointName()
	local config = Lua_MainLineCheckpointConfigData.getConfig(self.checkpointId);
	if self.checkpointState ~= 1 then
		config = Lua_MainLineCheckpointConfigData.getConfig(config.prev_game_level_id);
	end
	if config == nil then
		return "";
	end
	return config.name;
end

-- 根据ID获取关卡完成状态
function Lua_MainLineData:getCheckpointIsCompleted(id)
	if id == nil or id <= 0 then
		return true;
	end
	
	if id == self.checkpointId then
		return self.checkpointState == 1; -- 已通关
	end

	if id > self.checkpointId then
		return false;
	end
	return true;
end

-- 根据ID获取关卡奖励领取状态
function Lua_MainLineData:getCheckpointReceiveState(id)
	if self.passDic == nil or self.passDic[id] == nil then
		return 0;
	end
	return self.passDic[id];
end

function Lua_MainLineData:updateCheckpointHeadInfo()
	local arr = DataCenter.thingData:getThingIds(self.curSceneId, 0);
	if arr ~= nil then
		local thing;
		local config;
		local effectName
		for i = 1, arr:getSize() do
			thing = DataCenter.thingData:getThingPlayer(self.curSceneId, 0, arr[i]);
			if thing ~= nil then
				if thing.kind == Lua_ThingDefine.LEVEL_NODE then
					config = Lua_MainLineCheckpointConfigData.getConfig(thing.objId);
					if config.is_boss == 1 then
						if thing.objId > self.checkpointId then
							effectName = "guanqia_1"; -- 未通关
						else
							effectName = "guanqia_2"; -- 已通关
						end
					else
						if thing.objId > self.checkpointId then
							effectName = "guanqia_3"; -- 未通关
						elseif thing.objId < self.checkpointId then
							effectName = "guanqia_5"; -- 已通关
						else
							effectName = "guanqia_4"; -- 通关中
						end
					end
					if thing.player.data.effectName ~= effectName then
						thing.player.data.effectName = effectName;
						thing.player:resetBodyUrl();
						thing.player:loadBody();
					end
					thing:updateHeadInfo();
				elseif thing.kind == Lua_ThingDefine.SCENE_EFFECT then
					if thing.objId > self.checkpointId then
						effectName = "guanqia_guangxian1"; -- 未通关
					elseif thing.objId == self.checkpointId then
						if self:getCheckpointIsCompleted(self.checkpointId) then
							effectName = "guanqia_guangxian3"; -- 已通关
						else
							effectName = "guanqia_guangxian2"; -- 通关中
						end
					else
						effectName = "guanqia_guangxian3"; -- 已通关
					end
					if thing.player.data.effectName ~= effectName then
						thing.player.data.effectName = effectName;
						thing.player:resetBodyUrl();
						thing.player:loadBody();
					end
				end
			end
		end
	end
end

-- 点击了关卡
function Lua_MainLineData:onClickCopyLevel(levelId)
    
end

function Lua_MainLineData:setLevelWhenFistEnter(levelId, levelState)
	if self.levelIdWhenFistEnter == -1 then
		self.levelIdWhenFistEnter = levelId;
		self.levelStateWhenFistEnter = levelState;
	end
end

function Lua_MainLineData:getSpecialRewardState()
	local list = DataCenter.mainLineUtil.specialReward
	if list then
		for i = 1, #list do
			if self:getCheckpointReceiveState(list[i]) ~= 2 then
				return true
			end
		end
	end
	return false
end









--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
-- 每日免费刷新次数
function Lua_MainLineData.Get:task_free_times()
	return self._task_free_times or 0
end

-- 额外情报值
function Lua_MainLineData.Get:task_temp_info()
	return self._task_temp_info or 0
end

-- 额外情报值清空时间戳
function Lua_MainLineData.Get:task_temp_tick()
	return self._task_temp_tick or 0
end

-- 任务
function Lua_MainLineData:updateTaskData(msg)
	self:removeCheckTask();
	self.bTaskCanDispatch = false;
	self.bTaskCanReceive = false;
	self.bTaskHighQuality = false;
	local minTime = 9999999999;

	local len;
	local vo;
	if self.taskArr == nil then
		self.taskArr = Array();
	else
		len = self.taskArr:getSize();
		while len > 0 do
			vo = self.taskArr:remove(len);
			vo:dispose();
			self:setTaskVo(vo);
			len = len - 1;
		end
	end

	len = #msg.info_list;
	for i = 1, len do
		vo = self:getTaskVo();
		vo:fromServer(msg.info_list[i]);
		if vo.state == 2 then
			self.bTaskCanReceive = true;
		elseif vo.state == 1 then
			if vo.leftTime > 0 and vo.leftTime < minTime then
				minTime = vo.leftTime;
			end
		elseif vo.state == 0 then
			if DataCenter.packData:getItemCountById(vo.costVo.id) > vo.costVo.count then
				self.bTaskCanDispatch = true;
			end
			if vo.color >= Lua_ColorDefine.TYPE_PURPLE then
				self.bTaskHighQuality = true;
			end
		end
		self.taskArr:add(vo);
	end
	self.taskArr:sort("sortWeight", true);

	if minTime > 0 and minTime ~= 9999999999 then
		Lua_CommonUtil.delayCall(minTime, self, self.onCheckTask);
	end

	if msg.free_times then
		self._task_free_times = msg.free_times;
	end
	if msg.temp_info then
		self._task_temp_info = msg.temp_info;
	end
	if msg.temp_tick then
		self._task_temp_tick = msg.temp_tick;
	end

	self:checkDispatchRedPoint();
end

function Lua_MainLineData:removeCheckTask()
	Lua_CommonUtil.removeTimer(self, self.onCheckTask);
end

function Lua_MainLineData:onCheckTask()
	Lua_ProxyManager.getProxy("Lua_MainLineProxy"):sTaskInfo();
end

function Lua_MainLineData:updateSingleTaskData(msg)
	if self.taskArr == nil then
		return;
	end

	self:removeCheckTask();
	self.bTaskCanDispatch = false;
	self.bTaskCanReceive = false;
	self.bTaskHighQuality = false;
	local minTime = 9999999999;
	local vo;
	for i = 1, self.taskArr:getSize() do
		vo = self.taskArr[i];
		if vo.uid == tonumber(msg.info.uid) then
			vo:fromServer(msg.info);
		end

		if vo.state == 2 then
			self.bTaskCanReceive = true;
		elseif vo.state == 1 then
			if vo.leftTime > 0 and vo.leftTime < minTime then
				minTime = vo.leftTime;
			end
		else
			if DataCenter.packData:getItemCountById(vo.costVo.id) > vo.costVo.count then
				self.bTaskCanDispatch = true;
			end
			if vo.color >= Lua_ColorDefine.TYPE_PURPLE then
				self.bTaskHighQuality = true;
			end
		end
	end
	self.taskArr:sort("sortWeight", true);

	if minTime > 0 and minTime ~= 9999999999 then
		Lua_CommonUtil.delayCall(minTime, self, self.onCheckTask);
	end

	self:checkDispatchRedPoint();
end

function Lua_MainLineData:setTaskVo(vo)
	if self.taskVoArr == nil then
		self.taskVoArr = Array();
	end
	self.taskVoArr:add(vo);
end

function Lua_MainLineData:getTaskVo()
	local vo;
	if self.taskVoArr ~= nil then
		vo = self.taskVoArr:remove();
	end
	if vo == nil then
		vo = Lua_MainLineTaskVo.new();
	end
	return vo;
end

-- 获取对应阵营的英雄集合
function Lua_MainLineData:getTaskHeroArr(campType, cond)
	local uidArr = DataCenter.heroData.allHeroUidArr;
	local vo;
	local arr = Array();
	for i = 1, uidArr:getSize() do
		vo = DataCenter.heroData:getItemByUniqueId(uidArr[i]);
		if campType == Lua_HeroUtil.All_TYPE or vo.campType == campType then
			arr:add(vo);
		end
	end
	self:heroArrSort(arr, cond);
	return arr;
end

-- 由大到小排序
function Lua_MainLineData:heroArrSort(arr, cond)
	local len = arr:getSize();
	if len <= 1 then
		return;
	end
	
	local vo_a;
	local vo_b;
	local weight_a;
	local weight_b;
	local temp;
	for i = 1, len do
		for j = i + 1, len do
			vo_a = arr[i];
			vo_b = arr[j];
			if vo_a.send_info_state then
				weight_a = 0;
			else
				weight_a = 10000;
			end
			weight_a = weight_a + self:getCondEnoughNum(vo_a, cond) * 10;

			if vo_b.send_info_state then
				weight_b = 0;
			else
				weight_b = 10000;
			end
			weight_b = weight_b + self:getCondEnoughNum(vo_b, cond) * 10;

			if weight_a < weight_b then
				temp = arr[i];
				arr[i] = arr[j];
				arr[j] = temp;
			end
		end
	end
end

-- 满足条件的个数
function Lua_MainLineData:getCondEnoughNum(vo, cond)
	local num = 0;
	if cond ~= nil then
		if cond[1] ~= nil then
			if vo.quality >= cond[1][1] and vo.quality_star >= cond[1][2] then
				num = num + 1;
			end
		end
		if cond[2] ~= nil then
			for i = 1, #cond[2] do
				if vo.campType == cond[2][i] then
					num = num + 1;
				end
			end
		end
	end
	return num;
end

-- 检查条件是否满足
function Lua_MainLineData:checkCondIsSatisfy(heroList, cond)
	if heroList ~= nil and cond ~= nil then
		local heroVo;
		for i = 1, heroList:getSize() do
			heroVo = DataCenter.heroData:getItemByUniqueId(heroList[i]);
			if heroVo ~= nil then
				if cond.condType == 1 then
					if heroVo.quality >= cond.condValue[1] and heroVo.quality_star >= cond.condValue[2] then
						return true;
					end
				elseif cond.condType == 2 then
					if heroVo.campType == cond.condValue then
						return true;
					end
				end
			end
		end
	end
	return false;
end

--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
-- 红点
function Lua_MainLineData:checkRewardRedPoint()
	local state = 0;
	if self.passDic ~= nil then
		for _, v in pairs(self.passDic) do
			if v == 1 then
				state = 1;
				break;
			end
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.MainLine_Reward, state == 1);
end

function Lua_MainLineData:checkDispatchRedPoint()
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.MainLine_Dispatch, self.bTaskCanDispatch);
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.MainLine_DispatchReward, self.bTaskCanReceive);
end