-- 主线推图章节配置
Lua_MainLineChapterConfigData = class("Lua_MainLineChapterConfigData");
function Lua_MainLineChapterConfigData.getConfig(chapterId)
    if chapterId == nil or chapterId == nil then
        return nil;
    end

    if Lua_MainLineChapterConfigData.configs == nil then
        Lua_MainLineChapterConfigData.configs = {};
    end

    if Lua_MainLineChapterConfigData.configs[chapterId] == nil then
        local cfg = getConfig("mainline_chapter", chapterId);
        if cfg == nil then return nil; end
        Lua_MainLineChapterConfigData.configs[chapterId] = Lua_MainLineChapterConfigData.new(chapterId, cfg);
    end
    return Lua_MainLineChapterConfigData.configs[chapterId];
end

function Lua_MainLineChapterConfigData:ctor(chapterId, config)
    self:superCall("ctor");
    self.chapterId = chapterId;
    self.config = config;
    if self.config == nil then
        self.config = getConfig("mainline_chapter", chapterId);
    end
end

function Lua_MainLineChapterConfigData.Get:camStartAngle()
	if self._camStartAngle == nil then
		if self.config ~= nil and self.config.camStartAngle ~= nil then
			local px = #self.config.camStartAngle > 0 and self.config.camStartAngle[1] or 0;
			local py = #self.config.camStartAngle > 1 and self.config.camStartAngle[2] or 0;
			local pz = #self.config.camStartAngle > 2 and self.config.camStartAngle[3] or 0;
			self._camStartAngle = Vector3(px, py, pz);
		else
			self._camStartAngle = Vector3.zero;
		end
	end
	return self._camStartAngle;
end

function Lua_MainLineChapterConfigData.Get:camStartPos()
	if self._camStartPos == nil then
		if self.config ~= nil and self.config.camStartPos ~= nil then
			local px = #self.config.camStartPos > 0 and self.config.camStartPos[1] or 0;
			local py = #self.config.camStartPos > 1 and self.config.camStartPos[2] or 0;
			local pz = #self.config.camStartPos > 2 and self.config.camStartPos[3] or 0;
			self._camStartPos = Vector3(px, py, pz);
		else
			self._camStartPos = Vector3.zero;
		end
	end
	return self._camStartPos;
end

function Lua_MainLineChapterConfigData.Get:camEndAngle()
	if self._camEndAngle == nil then
		if self.config ~= nil and self.config.camEndAngle ~= nil then
			local px = #self.config.camEndAngle > 0 and self.config.camEndAngle[1] or 0;
			local py = #self.config.camEndAngle > 1 and self.config.camEndAngle[2] or 0;
			local pz = #self.config.camEndAngle > 2 and self.config.camEndAngle[3] or 0;
			self._camEndAngle = Vector3(px, py, pz);
		else
			self._camEndAngle = Vector3.zero;
		end
	end
	return self._camEndAngle;
end

function Lua_MainLineChapterConfigData.Get:camEndPos()
	if self._camEndPos == nil then
		if self.config ~= nil and self.config.camEndPos ~= nil then
			local px = #self.config.camEndPos > 0 and self.config.camEndPos[1] or 0;
			local py = #self.config.camEndPos > 1 and self.config.camEndPos[2] or 0;
			local pz = #self.config.camEndPos > 2 and self.config.camEndPos[3] or 0;
			self._camEndPos = Vector3(px, py, pz);
		else
			self._camEndPos = Vector3.zero;
		end
	end
	return self._camEndPos;
end

-- 拖拽中心点
function Lua_MainLineChapterConfigData.Get:dragCenterPos()
	if self._dragCenterPos == nil then
		if self.config ~= nil and self.config.dragCenterPos ~= nil then
			local px = #self.config.dragCenterPos > 0 and self.config.dragCenterPos[1] or 0;
			local py = #self.config.dragCenterPos > 1 and self.config.dragCenterPos[2] or 0;
			local pz = #self.config.dragCenterPos > 2 and self.config.dragCenterPos[3] or 0;
			self._dragCenterPos = Vector3(px * 0.01, py * 0.01, pz * 0.01);
		end
	end
	return self._dragCenterPos;
end

-- 右向量
function Lua_MainLineChapterConfigData.Get:rightVector()
	if self._rightVector == nil then
		if self.config ~= nil and self.config.rightVector ~= nil then
			local px = #self.config.rightVector > 0 and self.config.rightVector[1] or 0;
			local py = #self.config.rightVector > 1 and self.config.rightVector[2] or 0;
			local pz = #self.config.rightVector > 2 and self.config.rightVector[3] or 0;
			self._rightVector = Vector3(px, py, pz);
		end
	end
	return self._rightVector;
end

-- 拖拽范围
function Lua_MainLineChapterConfigData.Get:dragRange()
	if self._dragRange == nil then
		if self.config ~= nil and self.config.dragRange ~= nil then
			local px = #self.config.dragRange > 0 and self.config.dragRange[1] or 0;
			local py = #self.config.dragRange > 1 and self.config.dragRange[2] or 0;
			self._dragRange = Vector2(px, py);
		end
	end
	return self._dragRange;
end

-- 拖拽速率
function Lua_MainLineChapterConfigData.Get:dragRate()
	if self._dragRate == nil then
		if self.config ~= nil then
			self._dragRate = self.config.dragRate * 0.01;
		end
		if self._dragRate == nil then
			self._dragRate = 1;
		end
	end
	return self._dragRate;
end

-- 高点观察点位置
function Lua_MainLineChapterConfigData.Get:topLookAtPos()
	if self._topLookAtPos == nil then
		if self.config ~= nil and self.config.topLookAtPos ~= nil then
			local px = #self.config.topLookAtPos > 0 and self.config.topLookAtPos[1] or 0;
			local py = #self.config.topLookAtPos > 1 and self.config.topLookAtPos[2] or 0;
			local pz = #self.config.topLookAtPos > 2 and self.config.topLookAtPos[3] or 0;
			self._topLookAtPos = Vector3(px * 0.01, py * 0.01, pz * 0.01);
		else
			self._topLookAtPos = Vector3.zero;
		end
	end
	return self._topLookAtPos;
end

-- 低点观察点位置
function Lua_MainLineChapterConfigData.Get:bottomLookAtPos()
	if self._bottomLookAtPos == nil then
		if self.config ~= nil and self.config.bottomLookAtPos ~= nil then
			local px = #self.config.bottomLookAtPos > 0 and self.config.bottomLookAtPos[1] or 0;
			local py = #self.config.bottomLookAtPos > 1 and self.config.bottomLookAtPos[2] or 0;
			local pz = #self.config.bottomLookAtPos > 2 and self.config.bottomLookAtPos[3] or 0;
			self._bottomLookAtPos = Vector3(px * 0.01, py * 0.01, pz * 0.01);
		else
			self._bottomLookAtPos = Vector3.zero;
		end
	end
	return self._bottomLookAtPos;
end

-- 高点距离大小
function Lua_MainLineChapterConfigData.Get:topDis()
	if self._topDis == nil then
		if self.config ~= nil then
			self._topDis = self.config.topDis;
		end
		if self._topDis == nil then
			self._topDis = 0;
		end
	end
	return self._topDis;
end

-- 低点距离大小
function Lua_MainLineChapterConfigData.Get:bottomDis()
	if self._bottomDis == nil then
		if self.config ~= nil then
			self._bottomDis = self.config.bottomDis;
		end
		if self._bottomDis == nil then
			self._bottomDis = 0;
		end
	end
	return self._bottomDis;
end

-- 高点panangle
function Lua_MainLineChapterConfigData.Get:topPanAngle()
	if self._topPanAngle == nil then
		if self.config ~= nil then
			self._topPanAngle = self.config.topPanAngle;
		end
		if self._topPanAngle == nil then
			self._topPanAngle = 0;
		end
	end
	return self._topPanAngle;
end

-- 低点panangel
function Lua_MainLineChapterConfigData.Get:bottomPanAngle()
	if self._bottomPanAngle == nil then
		if self.config ~= nil then
			self._bottomPanAngle = self.config.bottomPanAngle;
		end
		if self._bottomPanAngle == nil then
			self._bottomPanAngle = 0;
		end
	end
	return self._bottomPanAngle;
end

-- 高点titleangle
function Lua_MainLineChapterConfigData.Get:topTitleAngle()
	if self._topTitleAngle == nil then
		if self.config ~= nil then
			self._topTitleAngle = self.config.topTitleAngle;
		end
		if self._topTitleAngle == nil then
			self._topTitleAngle = 0;
		end
	end
	return self._topTitleAngle;
end

-- 低点titleAngle
function Lua_MainLineChapterConfigData.Get:bottomTitleAngle()
	if self._bottomTitleAngle == nil then
		if self.config ~= nil then
			self._bottomTitleAngle = self.config.bottomTitleAngle;
		end
		if self._bottomTitleAngle == nil then
			self._bottomTitleAngle = 0;
		end
	end
	return self._bottomTitleAngle;
end

function Lua_MainLineChapterConfigData.Get:perMinDis()
	if self.config ~= nil and self.config.perMinDis ~= nil then
		return self.config.perMinDis;
	end
	return self.bottomDis;
end

function Lua_MainLineChapterConfigData.Get:perMaxDis()
	if self.config ~= nil and self.config.perMaxDis ~= nil and self.config.perMaxDis > self.perMinDis then
		return self.config.perMaxDis;
	end
	return self.perMinDis;
end

function Lua_MainLineChapterConfigData.Get:panMinAngle()
	if self.config ~= nil and self.config.panMinAngle ~= nil then
		return self.config.panMinAngle;
	end
	return self.bottomPanAngle;
end

function Lua_MainLineChapterConfigData.Get:panMaxAngle()
	if self.config ~= nil and self.config.panMaxAngle ~= nil and self.config.panMaxAngle > self.panMinAngle then
		return self.config.panMaxAngle;
	end
	return self.panMinAngle;
end

function Lua_MainLineChapterConfigData.Get:titleMinAngle()
	if self.config ~= nil and self.config.titleMinAngle ~= nil then
		return self.config.titleMinAngle;
	end
	return self.bottomTitleAngle;
end

function Lua_MainLineChapterConfigData.Get:titleMaxAngle()
	if self.config ~= nil and self.config.titleMaxAngle ~= nil and self.config.titleMaxAngle > self.titleMinAngle then
		return self.config.titleMaxAngle;
	end
	return self.titleMinAngle;
end

function Lua_MainLineChapterConfigData.Get:name()
    return self.config.name;
end

function Lua_MainLineChapterConfigData.Get:scene_id()
    return self.config.scene_id or Lua_SceneDefine.MAINLINE;
end

function Lua_MainLineChapterConfigData.Get:templateId()
    return self.config.template_id or Lua_FightDefine.TEMPLATE_ID_MAINLINE;
end

function Lua_MainLineChapterConfigData.Get:level()
    return self.config.level;
end

function Lua_MainLineChapterConfigData.Get:des()
    return self.config.des;
end

function Lua_MainLineChapterConfigData.Get:cameraEffUrl()
	if self._cameraEffUrl == nil then
		self._cameraEffUrl = LanUtil.StringProfile("/effects/scsEffects/dsjEffects/", self.cameraEff);
	end
	return self._cameraEffUrl;
end

function Lua_MainLineChapterConfigData.Get:cameraEff()
	return self.config.camera_eff;
end

function Lua_MainLineChapterConfigData.Get:sceneEffUrl()
	if self._sceneEffUrl == nil then
		self._sceneEffUrl = LanUtil.StringProfile("/effects/scsEffects/dsjEffects/", self.sceneEff);
	end
	return self._sceneEffUrl;
end

function Lua_MainLineChapterConfigData.Get:sceneEff()
	return self.config.scene_eff;
end