-- 主线推图关卡配置
Lua_MainLineCheckpointConfigData = class("Lua_MainLineCheckpointConfigData");
function Lua_MainLineCheckpointConfigData.getConfig(checkpointId)
    if checkpointId == nil or checkpointId <= 0 then
        return nil;
    end

    if Lua_MainLineCheckpointConfigData.configs == nil then
        Lua_MainLineCheckpointConfigData.configs = {};
    end

    if Lua_MainLineCheckpointConfigData.configs[checkpointId] == nil then
        local cfg = getConfig("mainline_game_level", checkpointId);
        if cfg == nil then
            return nil;
        end
        Lua_MainLineCheckpointConfigData.configs[checkpointId] = Lua_MainLineCheckpointConfigData.new(checkpointId, cfg);
    end
    return Lua_MainLineCheckpointConfigData.configs[checkpointId];
end

function Lua_MainLineCheckpointConfigData:ctor(checkpointId, config)
    self:superCall("ctor");
    self.checkpointId = checkpointId;
    self.config = config;
    if self.config == nil then
        self.config = getConfig("mainline_game_level", checkpointId);
    end
end

function Lua_MainLineCheckpointConfigData.Get:actionPos()
    if self._actionPos == nil then
        if self.config == nil or self.scenePos == nil then
            self._actionPos = Vector3.zero;
        else
            -- 校正处理
            local cam = GlobalVar.mainCamera;
            local px = #self.scenePos > 0 and self.scenePos[1] or 0;
            local py = #self.scenePos > 1 and self.scenePos[2] or 0;
            local pz = #self.scenePos > 2 and self.scenePos[3] or 0;
            local vec = Vector3(px * 0.01, py * 0.01, pz * 0.01);

            local charterCfg = Lua_MainLineChapterConfigData.getConfig(self.chapter_id);
            local r = charterCfg.rightVector == nil and cam.transform.right or charterCfg.rightVector;
            local f = Vector3.Cross(Vector3.up, r).normalized;
            local dragRate = charterCfg.dragRate * 0.01;
            local x = (charterCfg.dragCenterPos.x - vec.x) / dragRate;
            local y = (charterCfg.dragCenterPos.y - vec.y) / dragRate;
            local z = (charterCfg.dragCenterPos.z - vec.z) / dragRate;
            vec:Set(x, y, z);

            -- local sceneCfg = Lua_SceneConfigData.getSceneConfig(charterCfg.scene_id);
            -- local defaultVal = cam.orthographic and sceneCfg.orthographicSize or sceneCfg.perFieldOfView;
            -- local curVal = cam.orthographic and cam.orthographicSize or cam.fieldOfView;
            -- local _cameraDragRate = defaultVal / curVal;
            local _cameraDragRate = 1;
            local _areaWidth = charterCfg.dragRange.x * _cameraDragRate;
            local _areaHeight = charterCfg.dragRange.y * _cameraDragRate;
            local _areaWidthHalf = _areaWidth / 2;
            local _areaHeightHalf = _areaHeight / 2;

            x = _areaWidth <= 0 and 0 or (vec.x * f.z - vec.z * f.x) / (r.x * f.z - r.z * f.x);
            y = _areaHeight <= 0 and 0 or (vec.x * r.z - vec.z * r.x) / (f.x * r.z - f.z * r.x);
            -- if x >= -_areaWidthHalf and x <= _areaWidthHalf and y >= -_areaHeightHalf and y <= _areaHeightHalf then
            --     vec:Set(px * 0.01, charterCfg.dragCenterPos.y, pz * 0.01);
            --     print("px =", vec.x, "py =", vec.y, "pz =", vec.z, "x =", x, "y =", y, "_areaWidthHalf =", _areaWidthHalf, "_areaHeightHalf =", _areaHeightHalf);
            -- else
                x = x < -_areaWidthHalf and -_areaWidthHalf or x;
                x = x > _areaWidthHalf and _areaWidthHalf or x;
                y = y < -_areaHeightHalf and -_areaHeightHalf or y;
                y = y > _areaHeightHalf and _areaHeightHalf or y;
                x = x * dragRate;
                y = y * dragRate;
                r = Vector3(r.x * x, r.y * x, r.z * x);
                f = Vector3(f.x * y, f.y * y, f.z * y);
                vec:Set(charterCfg.dragCenterPos.x - r.x + f.x, charterCfg.dragCenterPos.y, charterCfg.dragCenterPos.z - r.z + f.z);
            -- end
            self._actionPos = vec;
        end
    end
    return self._actionPos;
end

function Lua_MainLineCheckpointConfigData.Get:name()
    return self.config.name;
end

function Lua_MainLineCheckpointConfigData.Get:number()
    return self.config.number or 0;
end

function Lua_MainLineCheckpointConfigData.Get:chapter_id()
    return self.config.chapter_id or 0;
end

function Lua_MainLineCheckpointConfigData.Get:level()
    return self.config.level or 0;
end

function Lua_MainLineCheckpointConfigData.Get:prev_game_level_id()
    return self.config.prev_game_level_id or 0;
end

function Lua_MainLineCheckpointConfigData.Get:next_game_level_id()
    return self.config.next_game_level_id or 0
end

function Lua_MainLineCheckpointConfigData.Get:is_boss()
    return self.config.is_boss;
end

function Lua_MainLineCheckpointConfigData.Get:open_level()
    return self.config.open_level;
end

function Lua_MainLineCheckpointConfigData.Get:scenePos()
    return self.config.scenePos;
end

function Lua_MainLineCheckpointConfigData.Get:scenePosVec()
    if self._scenePosVec == nil then
        if self.scenePos ~= nil then
            local px = #self.scenePos > 0 and self.scenePos[1] or 0;
            local py = #self.scenePos > 1 and self.scenePos[2] or 0;
            local pz = #self.scenePos > 2 and self.scenePos[3] or 0;
            self._scenePosVec = Vector3(px, py, pz);
        else
            self._scenePosVec = Vector3.zero;
        end
    end
    return self._scenePosVec;
end

function Lua_MainLineCheckpointConfigData.Get:sceneNodeRot()
    return self.config.sceneNodeRot;
end

function Lua_MainLineCheckpointConfigData.Get:sceneNodeRotVec()
    if self._sceneNodeRotVec == nil then
        if self.sceneNodeRot ~= nil then
            local px = #self.sceneNodeRot > 0 and self.sceneNodeRot[1] or 0;
            local py = #self.sceneNodeRot > 1 and self.sceneNodeRot[2] or 0;
            local pz = #self.sceneNodeRot > 2 and self.sceneNodeRot[3] or 0;
            self._sceneNodeRotVec = Vector3(px, py, pz);
        else
            self._sceneNodeRotVec = Vector3.zero;
        end
    end
    return self._sceneNodeRotVec;
end

function Lua_MainLineCheckpointConfigData.Get:head_atlas()
    return self.config.head_atlas;
end

function Lua_MainLineCheckpointConfigData.Get:head_sprite()
    return self.config.head_sprite;
end

function Lua_MainLineCheckpointConfigData.Get:robot_id()
    return self.config.robot_id;
end

function Lua_MainLineCheckpointConfigData.Get:reward_id()
    return self.config.reward_id;
end

function Lua_MainLineCheckpointConfigData.Get:normal_profit()
    return self.config.normal_profit;
end

function Lua_MainLineCheckpointConfigData.Get:bResultPlot()
    return self.config.result_plot == 1;
end