checkRequire("Lua_MainLineChapterConfigData");
checkRequire("Lua_MainLineCheckpointConfigData");
Lua_MainLineUtil = class("Lua_MainLineUtil");
function Lua_MainLineUtil:ctor()
	self:superCall("ctor");
end

function Lua_MainLineUtil:dispose()

end

function Lua_MainLineUtil:getChapterIdArr(chapterId)
	local arr = Array();
	local count = 4;
	if chapterId > 3 then
		count = chapterId + 1;
	end

	local max_chapter_id = getConfig("mainline_const", "max_chapter_id", "value") or 1;
	count = count > max_chapter_id and max_chapter_id or count;
	for i = 1, count do
		arr:add(i);
	end
	return arr;
end

function Lua_MainLineUtil:getLevelsByChapterId(chapterId)
	local dic = self.checkpointDic;
	if dic == nil then
		return nil;
	end
	return dic[chapterId];
end

function Lua_MainLineUtil.Get:checkpointDic()
	if self._checkpointDic == nil then
		self:initCheckpointInfo();
	end
	return self._checkpointDic;
end

function Lua_MainLineUtil.Get:rewardArr()
	if self._rewardArr == nil then
		self:initCheckpointInfo();
	end
	self._rewardArr:sortf(self, self.rewardSortFun);
	return self._rewardArr;
end

function Lua_MainLineUtil:initCheckpointInfo()
	if self._checkpointDic ~= nil and self._rewardArr ~= nil then
		return;
	end

	local temp = {}
	local info
	for i = 1, #self.specialReward do
		info = self.specialReward[i]
		temp[info] = 1
	end

	self._checkpointDic = {};
	self._rewardArr = Array();
	local config = getConfig("mainline_game_level");
	for k, v in pairs(config) do
		local chapter_id = v.chapter_id;
		if self._checkpointDic[chapter_id] == nil then
			self._checkpointDic[chapter_id] = Array();
		end
		self._checkpointDic[chapter_id]:add(k); 

		if v.reward_id > 0 and temp[k] == nil then
			self._rewardArr:add(k);
		end
	end
	-- for _, v in pairs(self._checkpointDic) do
	-- 	quickSort(v, nil, true)
	-- end
end

function Lua_MainLineUtil:rewardSortFun(a, b)
	local state_a = DataCenter.mainLineData:getCheckpointReceiveState(a);
	local state_b = DataCenter.mainLineData:getCheckpointReceiveState(b);
	if state_a == 0 and state_b == 1 then
		return false;
	elseif state_a == 0 and state_b == 2 then
		return true;
	elseif state_a == 1 and state_b ~= 1 then
		return true;
	elseif state_a == 2 and state_b ~= 2 then
		return false;
	else
		if a > b then
			return false;
		end
	end
	return true;
end

function Lua_MainLineUtil:getRewardsInfo()
	local list = self.specialReward
	local str, id = self:getCurReceiveInfo(list, #list)
	if str and id then
		return true, str, id
	end

	local arr = self.rewardArr;
	local len = arr:getSize();
	str, id = self:getCurReceiveInfo(arr, len)
	if str and id then
		return true, str, id
	end

	str, id = self:getNextReceiveInfo(list, #list)
	if str and id then
		return false, str, id
	end

	return false, self:getNextReceiveInfo(arr, len)
end

function Lua_MainLineUtil:getCurReceiveInfo(list, len)
	local state;
	local config;
	for i = 1, len do
		state = DataCenter.mainLineData:getCheckpointReceiveState(list[i]);
		if state == 1 then
			config = Lua_MainLineCheckpointConfigData.getConfig(list[i]);
			return LanUtil.getValue(307003), config.reward_id;
		end
	end
	return nil, nil
end

function Lua_MainLineUtil:getNextReceiveInfo(list, len)
	local curCfg = Lua_MainLineCheckpointConfigData.getConfig(DataCenter.mainLineData.checkpointId);
	if curCfg ~= nil then
		local state;
		local config;
		for i = 1, len do
			state = DataCenter.mainLineData:getCheckpointIsCompleted(list[i]);
			if not state then
				config = Lua_MainLineCheckpointConfigData.getConfig(list[i]);
				if DataCenter.mainLineData.checkpointState == 1 then
					return LanUtil.formateStr(307004, tostring(config.number - curCfg.number)), config.reward_id;
				else
					return LanUtil.formateStr(307004, tostring(config.number - curCfg.number + 1)), config.reward_id;
				end
			end
		end
	end
	return nil, nil
end

function Lua_MainLineUtil:getEffectByTime(time)
	if self.effList == nil then
		self.effList = getConfig("mainline_const", "hook_effect", "jsonvalue") or {};
	end
	local len = #self.effList;
	while len > 0 do
		if time >= self.effList[len][1] then
			return self.effList[len][2], self.effList[len][3];
		end
		len = len - 1;
	end
	return 0, nil;
end

function Lua_MainLineUtil.Get:startToTopUseTime()
	if self._startToTopUseTime == nil then
		self._startToTopUseTime = getConfig("mainline_const", "startToTopUseTime", "float_value") or 0.5;
	end
	return self._startToTopUseTime;
end

function Lua_MainLineUtil.Get:topWaitUseTime()
	if self._topWaitUseTime == nil then
		self._topWaitUseTime = getConfig("mainline_const", "topWaitUseTime", "float_value") or 0.2;
	end
	return self._topWaitUseTime;
end

function Lua_MainLineUtil.Get:topToEndUseTime()
	if self._topToEndUseTime == nil then
		self._topToEndUseTime = getConfig("mainline_const", "topToEndUseTime", "float_value") or 0.5;
	end
	return self._topToEndUseTime;
end

function Lua_MainLineUtil.Get:headPanelStartPos()
	if self._headPanelStartPos == nil then
		self._headPanelStartPos = getConfig("mainline_const", "levelNodeHeadPanelStartPos", "value") or 1500;
	end
	return self._headPanelStartPos;
end

function Lua_MainLineUtil.Get:headPanelTweenTime()
	if self._headPanelTweenTime == nil then
		self._headPanelTweenTime = getConfig("mainline_const", "levelNodeHeadPanelTweenTime", "float_value") or 0.5;
	end
	return self._headPanelTweenTime;
end

function Lua_MainLineUtil.Get:maxInfoValue()
	return getConfig("mainline_const", "max_info_value", "value") or 1;
end

function Lua_MainLineUtil.Get:one_hook_earnings_time()
	return getConfig("mainline_const", "one_hook_earnings_time", "value") or 1;
end

function Lua_MainLineUtil.Get:speed_publicity_exp()
	return getConfig("mainline_const", "speed_publicity_exp", "jsonvalue");
end

function Lua_MainLineUtil.Get:fast_open_id()
	return getConfig("mainline_const", "fast_open_id", "value");
end

function Lua_MainLineUtil.Get:free_publicity_times()
	return getConfig("mainline_const", "free_publicity_times", "value") or 0;
end

function Lua_MainLineUtil.Get:pay_publicity_times()
	return getConfig("mainline_const", "pay_publicity_times", "value") or 0;
end

function Lua_MainLineUtil.Get:hook_open_id()
	return getConfig("mainline_const", "hook_open_id", "value");
end

function Lua_MainLineUtil.Get:dispatch_open_id()
	return getConfig("mainline_const", "dispatch_open_id", "value");
end

function Lua_MainLineUtil.Get:refresh_info_item()
	return getConfig("mainline_const", "refresh_info_item", "jsonvalue");
end

function Lua_MainLineUtil.Get:refresh_info_gold()
	return getConfig("mainline_const", "refresh_info_gold", "jsonvalue");
end

function Lua_MainLineUtil.Get:specialReward()
	return getConfig("mainline_const", "special_reward", "jsonvalue");
end

function Lua_MainLineUtil.Get:autoNextFight()
	return getConfig("mainline_const", "auto_next_fight", "value") or 0;
end

function Lua_MainLineUtil:getRewardSortValue(value)
	if value.id == 201000032 then
		return 1000000
	elseif value.id == 201000037 then
		return 100000
	elseif value.id == 201000007 then
		return 10000
	elseif value.id == 201000002 then
		return 1000
	elseif value.id == 201000003 then
		return 100
	elseif value.id == 201000008 then
		return 10
	end
	return value.color
end