--开服狂欢
Lua_OpenServerCarnivalData = class("Lua_OpenServerCarnivalData")
function Lua_OpenServerCarnivalData:ctor()

	self.dayIdBaseCount = 100; --天数id = 100 + 开启天数
	self:dispose();
end

function Lua_OpenServerCarnivalData:dispose()
	self.taskDic = {};
	self.rewardDic = {};
	self.redPointInActDic = {};-- 每期活动中首个有红点的dayid
end

function Lua_OpenServerCarnivalData:fromServer50197023(msg)
	if msg == nil then
		return;
	end

	self:updateServerData(msg);
	self:showRedPoint();
end

function Lua_OpenServerCarnivalData:updateServerData(msg)
	local obj;
	local list;
	local key;
	local value;
	local other;
	local dataInfo;

	self.closeTick = tonumber(msg.close_tick);

	--开启天数限制
	local maxActDay = DataCenter.openServerCarnivalUtil.maxActDay;
	self.openDay = msg.open_day <= 0 and 1 or msg.open_day;
	self.openDay = self.openDay > maxActDay and maxActDay or self.openDay;

	local str;
	list = msg.total_reward_state;
	for i = 1, #list do
		obj = list[i];
		key = tonumber(obj.key);
		value = tonumber(obj.value);
		str = DataCenter.openServerCarnivalUtil:getActStrByActIndex(key);
		self.rewardDic[str] = value;
	end

	list = msg.day_reward_state;
	for i = 1, #list do
		obj = list[i];
		key = tonumber(obj.key);
		value = tonumber(obj.value);
		self.rewardDic[key] = value;
	end


	list = msg.task_info;
	for i = 1, #list do
		obj = list[i];
		key = tonumber(obj.key);
		value = tonumber(obj.value);
		other = tonumber(obj.other);
		dataInfo = self:getTaskInfoById(key);
		if dataInfo == nil then
			dataInfo = { rewardState = value, curProcess = other };
			self.taskDic[key] = dataInfo;
		else
			dataInfo.rewardState = value;
			dataInfo.curProcess = other;
		end
	end
end

-- 任务信息
function Lua_OpenServerCarnivalData:getTaskInfoById(id)
	if id == nil or self.taskDic[id] == nil then
		return;
	end
	return self.taskDic[id];
end

-- 每日奖励和每期奖励状态
function Lua_OpenServerCarnivalData:getRewardInfoById(id)
	if id == nil or self.rewardDic[id] == nil then
		return;
	end
	return self.rewardDic[id];
end

-- 更新红点
function Lua_OpenServerCarnivalData:showRedPoint()
	local activityIndex = DataCenter.openServerCarnivalUtil:getActivityIndex(self.openDay);
	if activityIndex == nil then
		return;
	end

	local canExistActivites = DataCenter.openServerCarnivalUtil.canExistActivites;
	if canExistActivites then
		for i = 1, activityIndex do
			self:updateHasRewardByActIndex(i);
		end
	else
		self:updateHasRewardByActIndex(activityIndex);
		local lastActivityIndex = activityIndex > 1 and activityIndex - 1 or nil;
		if lastActivityIndex ~= nil then
			self.redPointInActDic[lastActivityIndex] = nil;
			self:updateRedPoint(lastActivityIndex);
		end
	end
end

function Lua_OpenServerCarnivalData:updateHasRewardByActIndex(actIndex)
	local startDay, endDay = DataCenter.openServerCarnivalUtil:getActivityTimeByActIndex(actIndex);
	local targetDayId;
	local dayId;

	endDay = endDay > self.openDay and self.openDay or endDay;
	for i = startDay, endDay do
		dayId = self.dayIdBaseCount + i;
		if self:getRewardInfoById(dayId) == 1 or self:enableReceiveRewardByDay(dayId) then 
			targetDayId = dayId;
			break;
		end
	end

	if targetDayId == nil then
		local id = DataCenter.openServerCarnivalUtil:getActStrByActIndex(actIndex);
		local rewards = DataCenter.openServerCarnivalUtil:getOCRewardConfig(id, 1);
		if rewards ~= nil then
			local data = self:getRewardInfoById(id);
			if data == 1 then
				targetDayId = self.dayIdBaseCount + endDay;
			end
		end
	end
	self.redPointInActDic[actIndex] = targetDayId;
	self:updateRedPoint(actIndex);
end

function Lua_OpenServerCarnivalData:updateRedPoint(actIndex)
	local redPointStr;
	if actIndex == 1 then
		redPointStr = Lua_RedPointList.OpenServerCarnival_First;
	elseif actIndex == 2 then
		redPointStr = Lua_RedPointList.OpenServerCarnival_Second;
	end

	if redPointStr ~= nil then
		DataCenter.redPointData:updateRedPointState(redPointStr, self.redPointInActDic[actIndex] ~= nil);
	end
end

-- 根据天数id,判断该天是否有奖励未领取
function Lua_OpenServerCarnivalData:enableReceiveRewardByDay(day)
	return self:enableSatisfyCondition(day,1);
end

-- 根据天数id,判断该天是否有任务未完成
function Lua_OpenServerCarnivalData:haveUncompleteTaskByDay(day)
	return self:enableSatisfyCondition(day,0);
end

--day天是否满足条件condition：0：不可领   1:可领取   2：已领取
function Lua_OpenServerCarnivalData:enableSatisfyCondition(dayId, condition)
	if dayId == nil or condition == nil then
		return false;
	end

	local id;
	local data;
	local taskList = DataCenter.openServerCarnivalUtil:getOCRewardConfig(dayId, 2);
	for i = 1, #taskList do
		id = taskList[i];
		data = self:getTaskInfoById(id);
		if data ~= nil and data.rewardState == condition then
			return true;
		end
	end
	return false;
end

function Lua_OpenServerCarnivalData:getSelectedDayByTab(tabId)
	local targetDayId;
	local targetIndex;
	local activityIndex = DataCenter.openServerCarnivalUtil:getActivityIndex(self.openDay);
	if tabId == nil then
		local dayId;
		for i = 1, activityIndex do
			dayId = self.redPointInActDic[i];
			if dayId ~= nil then
				targetDayId = dayId;
				targetIndex = i;
				break;
			end
		end
	else
		targetIndex = tabId;
		targetDayId = self.redPointInActDic[tabId];
	end
	if targetDayId and targetIndex then
		return targetDayId, targetIndex;
	end

	local startDay, endDay;
	targetIndex = tabId;
	if tabId == nil then
		activityIndex, startDay, endDay = DataCenter.openServerCarnivalUtil:getActivityIndex(self.openDay, true);
		endDay = self.openDay;
	else
		startDay, endDay = DataCenter.openServerCarnivalUtil:getActivityTimeByActIndex(tabId); 
		endDay = endDay > self.openDay and self.openDay or endDay;
	end

	targetDayId = self.dayIdBaseCount + endDay;

	for i = startDay, endDay do
		dayId = self.dayIdBaseCount + i;
		if self:haveUncompleteTaskByDay(dayId) == true then
			targetDayId = dayId;
			break;
		end
	end

	return targetDayId, DataCenter.openServerCarnivalUtil:getActivityIndex(targetDayId - self.dayIdBaseCount);
end

-- 根据dayId 获取具体天数
function Lua_OpenServerCarnivalData:getDayTxtByDayId(dayId)
	local startDay, endDay = 0, 0;
	local dayNum = dayId - self.dayIdBaseCount;
	local actIndex = DataCenter.openServerCarnivalUtil:getActivityIndex(dayNum);
	if actIndex > 1 then
		actIndex = actIndex - 1;
		startDay, endDay = DataCenter.openServerCarnivalUtil:getActivityTimeByActIndex(actIndex);
	end
	return dayId - self.dayIdBaseCount - endDay;
end

-- 根据dayId,页签tab 获取具体列表中的index
function Lua_OpenServerCarnivalData:getDayIndex(tabId, dayId)
	local startDay, endDay = DataCenter.openServerCarnivalUtil:getActivityTimeByActIndex(tabId);
	local index = 1;

	dayId = dayId - self.dayIdBaseCount;
	index = dayId - startDay + 1;
	return index;
end

-- -- 是否关闭活动
-- function Lua_OpenServerCarnivalData:closeActivity()
-- 	local bCloseAct = self.closeTick - DataCenter.roleData.serverTime <= 0;

-- 	if bCloseAct then 
-- 		DataCenter.mainMenuData:setIconShowInfo(Lua_MenuViewDefine.ICON_OPENSERVER_CARNIVAL, false);
-- 	end
-- end

function Lua_OpenServerCarnivalData.Get:residueTime()
	return self.closeTick - DataCenter.roleData.serverTime;
end

function Lua_OpenServerCarnivalData:getCompletedCountByDayId(dayId)
	if dayId == nil then
		return;
	end

	local id;
	local completeCount = 0;
	local taskList = DataCenter.openServerCarnivalUtil:getOCRewardConfig(dayId, 2);
	for i = 1, #taskList do
		id = taskList[i];
		data = self:getTaskInfoById(id);

		if data ~= nil and data.rewardState ~= 0 then
			completeCount = completeCount + 1;
		end
	end
	return completeCount, #taskList;
end

function Lua_OpenServerCarnivalData:getCompletedCountByActIndex(actIndex)
	if actIndex == nil then
		return;
	end

	local dayId;
	local completeCount, totalCount = 0, 0;
	-- local configStr = DataCenter.openServerCarnivalUtil:getActStrByActIndex(actIndex);
	local dayIdList = DataCenter.openServerCarnivalUtil:getActDayIdList(actIndex, self.openDay);
	local targetDay = self.openDay + self.dayIdBaseCount;
	if dayIdList ~= nil then
		local param1, param2;
		for i = 1, #dayIdList do
			dayId = dayIdList[i];
			param1, param2 = self:getCompletedCountByDayId(dayId);
			completeCount = targetDay < dayId and completeCount or completeCount + param1;
			totalCount = totalCount + param2;
		end
	end
	return completeCount, totalCount;
end

