checkRequire("Lua_OpenServerHeroGiftVo");
Lua_OpenServerHeroGiftData = class("Lua_OpenServerHeroGiftData");
function Lua_OpenServerHeroGiftData:ctor()
	self:dispose();
end

function Lua_OpenServerHeroGiftData:dispose()
	self.isOpen = 0; -- 默认没有打开英雄赠礼界面
	self.end_time = nil;
	self.cell_list = nil;
	self.isPlayTween = false;
	self.heroGiftDic = nil;
	self.phaseDic = nil;
	self._drawItemId = nil;
	self._drawItemNum = nil;
	self._posList = nil;
	self._phaseMaxNum = nil;
	self._iconList = nil;
	self._entranceList = nil;
	self._redDotDic = nil;
end

function Lua_OpenServerHeroGiftData:fromServer50197046(msg)
	if msg == nil then
		return;
	end

	if self.cell_list and #msg.cell_list == 0 and self.round_num == msg.round_num then
		self.heroGiftDic = nil;
	end
	if self.round_num and self.round_num ~= msg.round_num then
		self._redDotDic = nil; -- 换期重新获取红点信息
		self._phaseMaxNum = nil; -- 换期重新获取显示的奖励数量
	end

	self.round_num = msg.round_num; -- 轮次
	self.draw_num = msg.draw_num; -- 当前抽奖次数
	self.end_time = msg.end_time; -- 结束时间
	self.heroInfo = msg.hero_item; -- 英雄是否第一次抽中信息
	self.cell_num = msg.cell_num; -- 抽中格子id，0表示没进行抽奖
	self.cell_list = msg.cell_list; -- 已抽中格子id

	self:initConfigInfo();
	local id;
	for i = 1, #msg.cell_list do
		id = msg.cell_list[i];
		self.heroGiftDic[id]:updateDrawInfo(true);
	end
	
	self:updateHeroGiftRedDot();

	local time = self.end_time - DataCenter.roleData.serverTime
	DataCenter.mainMenuData:setIconShowInfo(Lua_MenuViewDefine.ICON_OPENSERVERTRIAL, time > 0, time);
end

function Lua_OpenServerHeroGiftData:initConfigInfo()
	if self.heroGiftDic == nil then
		self.heroGiftDic = {};
		self.phaseDic = {};

		local config = getConfig("hero_gift");
		local vo;
		local phase;
		local pos;
		for k, v in pairs(config) do
			if k ~= "const" then
				vo = Lua_OpenServerHeroGiftVo.new(k, v);
				self.heroGiftDic[vo.id] = vo;

				phase = Mathf.Floor(k / 10000);
				if self.phaseDic[phase] == nil then
					self.phaseDic[phase] = Array();
				end
				self.phaseDic[phase]:add(vo);
			end
		end

		phase = getConfig("hero_gift", "const", "round_limit");
		for i = 1, phase do
			self.phaseDic[i]:sort("id", false);
		end
	end
end

function Lua_OpenServerHeroGiftData:updateHeroGiftRedDot(dic)
	if self.end_time then
		local hasCount = DataCenter.packData:getItemCountById(self.drawItemId);
		local arr = self:curPhaseShowArr();
		local state = hasCount >= self.drawItemNum and arr:getSize() > self.draw_num;
		if dic ~= nil then
			if dic[self.drawItemId] ~= nil then
				DataCenter.redPointData:updateRedPointState(Lua_RedPointList.OpenServerTrial_HeroGift, state);
			end
		else
			DataCenter.redPointData:updateRedPointState(Lua_RedPointList.OpenServerTrial_HeroGift, state);
		end
	end
end

function Lua_OpenServerHeroGiftData:updateEntranceRedDot(redPointId)
	if self.end_time and self.redDotDic[redPointId] then
		local vo = DataCenter.redPointData:getRedPointVoById(redPointId);
		DataCenter.redPointData:updateRedPointState(self.redDotDic[redPointId], vo.showState);
	end
end

function Lua_OpenServerHeroGiftData:curPhaseShowArr()
	return self.phaseDic[self.round_num];
end

-- 抽奖道具id
function Lua_OpenServerHeroGiftData.Get:drawItemId()
	if self._drawItemId == nil then
		self._drawItemId = getConfig("hero_gift", "const", "recover_item");
	end
	return self._drawItemId[self.round_num];
end

-- 抽奖道具数量
function Lua_OpenServerHeroGiftData.Get:drawItemNum()
	if self._drawItemNum == nil then
		self._drawItemNum = getConfig("hero_gift", "const", "draw_num_list");
	end
	local curPhaseNumDic = self._drawItemNum[self.round_num];
	local times = self.draw_num >= #curPhaseNumDic and #curPhaseNumDic or self.draw_num + 1;
	return curPhaseNumDic[times];
end

-- 每行道具数量
function Lua_OpenServerHeroGiftData.Get:posList()
	if self._posList == nil then
		self._posList = getConfig("hero_gift", "const", "pos_list");
	end
	return self._posList[self.round_num];
end

function Lua_OpenServerHeroGiftData.Get:phaseMaxNum()
	if self._phaseMaxNum == nil then
		self._phaseMaxNum = 0;

		for i = 1, #self.posList do
			self._phaseMaxNum = self._phaseMaxNum + self.posList[i];
		end
	end
	return self._phaseMaxNum;
end

-- 每行道具数量
function Lua_OpenServerHeroGiftData.Get:iconList()
	if self._iconList == nil then
		self._iconList = getConfig("hero_gift", "const", "icon_list");
	end
	return self._iconList[self.round_num];
end

-- 道具获取入口
function Lua_OpenServerHeroGiftData.Get:entranceDic()
	if self._entranceDic == nil then
		self._entranceDic = {};

		local round;
		local config = getConfig("hero_gift_entrance");
		for k, v in pairs(config) do
			round = Mathf.Floor(k / 10000);
			if self._entranceDic[round] == nil then
				self._entranceDic[round] = Array();
			end
			if v.is_show_entrance ~= 1 then
				self._entranceDic[round]:add(v);
			end
		end
	end
	self._entranceDic[self.round_num]:sort("id", false);
	return self._entranceDic[self.round_num];
end

function Lua_OpenServerHeroGiftData.Get:redDotDic()
	if self._redDotDic == nil then
		self._redDotDic = {};

		local monitorRedId;
		local mapRedId;
		for i = 1, self.entranceDic:getSize() do
			monitorRedId = self.entranceDic[i].monitor_red_id;
			mapRedId = self.entranceDic[i].map_red_id;
			self._redDotDic[monitorRedId] = mapRedId;
		end
	end
	return self._redDotDic;
end