checkRequire("Lua_BackstageActivityVo");
Lua_BackstageActivitiesData = class("Lua_BackstageActivitiesData")
function Lua_BackstageActivitiesData:ctor()
	self:superCall("ctor");
	self:dispose();
end

function Lua_BackstageActivitiesData:dispose()
	self.bInit = false;

	self.actIdxDic = nil; -- 活动唯一Id
	self.actTypeDic = nil; -- 活动类型
	if self.openIdxArr ~= nil then -- 已开启活动列表
		self.openIdxArr:clear();
		self.openIdxArr = nil;
	end
	self.mainIconDic = nil; -- 主图标显示活动列表
	self.subIconDic = nil; -- 子图标显示活动列表
	self.actWinDic = nil; -- 活动窗口列表
	if self._actIconArr ~= nil then -- 活动图标
		self._actIconArr:clear();
		self._actIconArr = nil;
	end

	self.actList = nil;
	self.bInitServer = false;
end

function Lua_BackstageActivitiesData.Get:actIconArr()
	return self._actIconArr;
end

function Lua_BackstageActivitiesData:init()
	if self.bInit ~= true then
		self.bInit = true;
		self.actIdxDic = {};
		self.actTypeDic = {};
		self.openIdxArr = Array();
		self.mainIconDic = {};
		self.subIconDic = {};
		self.actWinDic = {};
	end
end

function Lua_BackstageActivitiesData:fromServer80108001(msg)
	if msg == nil then
		self.actList = nil;
	else
		DataCenter.roleData.serverTime = msg.tick;
		self.actList = msg.backact_list;
	end

	if self.actList == nil or #self.actList <= 0 then
		self:removeActivities();
		self:updateActivitiesIcon();
	else
		self:init();
		self:updateActivities();
		-- self:getActInfoOnLogin();
	end
end

function Lua_BackstageActivitiesData:fromServer80108002(msg)
	if msg == nil or msg.act_red == nil or #msg.act_red <= 0 then
		if self.mainIconDic ~= nil then
			local vo;
			for _, arr in pairs(self.mainIconDic) do
				vo = self:getActVoByIdx(arr[i]);
				if vo ~= nil then
					DataCenter.redPointData:updateRedPointState(vo.redPointId, false);
				end
			end
		end
	else
		local vo;
		for i = 1, #msg.act_red do
			vo = self:getActVoByIdx(msg.act_red[i].key);
			if vo ~= nil then
				DataCenter.redPointData:updateRedPointState(vo.redPointId, msg.act_red[i].value == 1);-- 1：显示红点，0：隐藏红点
			end
		end
	end
end

function Lua_BackstageActivitiesData:updateActivities()
	local bChange = false;
	local actInfo;
	local vo;
	local iconVo;
	for i = 1, #self.actList do
		actInfo = self.actList[i];
		vo = self.actIdxDic[actInfo.act_index];
		if vo == nil then
			vo = Lua_BackstageActivityVo.new(actInfo.act_index);
		end
		vo:fromServerInfo(actInfo);

		self.actIdxDic[vo.actIdx] = vo;
		if self.actTypeDic[vo.actType] == nil then
			self.actTypeDic[vo.actType] = Array();
		end
		if self.actTypeDic[vo.actType]:indexOf(vo.actIdx) == -1 then
			self.actTypeDic[vo.actType]:add(vo.actIdx);
		end

		if vo.bOpened == true then
			if self.openIdxArr:indexOf(vo.actIdx) == -1 then
				self.openIdxArr:add(vo.actIdx);
			end

			-- 主图标
			if vo.bShowMainIcon == true then
				if vo.countdown_switch == 1 then
					iconVo = DataCenter.mainMenuData:getMenuIconVo(vo.mainIconId, vo.mainIconId);
					iconVo:setIconInfo(vo.residueTime);
				end
				if self.mainIconDic[vo.mainIconId] == nil then
					self.mainIconDic[vo.mainIconId] = Array();
					bChange = true;
				end
				if self.mainIconDic[vo.mainIconId]:indexOf(vo.actIdx) == -1 then
					self.mainIconDic[vo.mainIconId]:add(vo.actIdx);
					DataCenter.redPointData:updateMainIconRedId(vo.actIdx, vo.mainIconId, vo.mainIconId);
				end
			end

			-- 子图标
			if vo.bShowSubIcon == true then
				if self.subIconDic[vo.actIdx] == nil or self.subIconDic[vo.actIdx] ~= vo.sub_icon then
					self.subIconDic[vo.actIdx] = vo.sub_icon;
					bChange = true;
				end
			end

			if not strIsNullOrEmpty(vo.main_win) then
				if self.actWinDic[vo.main_win] == nil then
					self.actWinDic[vo.main_win] = Array();
				end
				if self.actWinDic[vo.main_win]:indexOf(vo.actIdx) == -1 then
					self.actWinDic[vo.main_win]:add(vo.actIdx);
				end
			end
		end
	end

	for _, arr in pairs(self.mainIconDic) do
		arr:sortf(self, self.sortActivities);
	end

	bChange = self:removeActivities(bChange);

	-- 增删主图标或子图标时才更新menuIcon
	if bChange == true then
		self:updateActivitiesIcon();
	end
end

function Lua_BackstageActivitiesData:removeActivities(bChange)
	if self.openIdxArr ~= nil then
		local actVo;
		local arr;
		local iconVo;
		local state;
		local len = self.openIdxArr:getSize();
		while len > 0 do
			actVo = self:getActVoByIdx(self.openIdxArr[len]);
			if actVo.bOpened == false then
				if self.actTypeDic ~= nil and self.actTypeDic[actVo.actType] ~= nil then
					self.actTypeDic[actVo.actType]:removeItem(actVo.actIdx);
				end

				-- 移除主图标
				arr = self.mainIconDic[actVo.mainIconId];
				if arr ~= nil then
					arr:removeItem(actVo.actIdx);
					if arr:getSize() <= 0 then
						iconVo = DataCenter.mainMenuData:getMenuIconVo(actVo.mainIconId);
						if iconVo ~= nil and iconVo.type == 2 then
							if iconVo.iconType == Lua_MenuViewDefine.TYPE_ACTIVITY_ICON then
								state = false;
							else
								state = DataCenter.funcOpenData:getMenuIconState(actVo.mainIconId);
							end
							iconVo:setFunOpenState(state);
							iconVo:setShowState(state);
							iconVo:setIconInfo(0);
						end
						self.mainIconDic[actVo.mainIconId] = nil;
						DataCenter.redPointData:updateRedPointState(actVo.redPointId, false);
						bChange = true;
					end
				end

				-- 移除子图标
				if self.subIconDic[actVo.actIdx] ~= nil then
					iconVo = DataCenter.mainMenuData:getMenuIconVo(actVo.subIconId);
					if iconVo ~= nil and iconVo.type == 2 then
						if iconVo.iconType == Lua_MenuViewDefine.TYPE_ACTIVITY_ICON then
							state = false;
						else
							state = DataCenter.funcOpenData:getMenuIconState(actVo.sub_icon);
						end
						iconVo:setFunOpenState(state);
						iconVo:setShowState(state);
						iconVo:setIconInfo(0);
					end
					self.subIconDic[actVo.actIdx] = nil;
					DataCenter.redPointData:updateRedPointState(actVo.redPointId, false);
					bChange = true;
				end

				if not strIsNullOrEmpty(actVo.main_win) then
					arr = self.actWinDic[actVo.main_win];
					if arr ~= nil then
						arr:removeItem(actVo.actIdx);
						if arr:getSize() <= 0 then
							self.actWinDic[actVo.main_win] = nil;
						end
					end
				end

				self.openIdxArr:remove(len);
			end
			len = len - 1;
		end
	end
	return bChange;
end

function Lua_BackstageActivitiesData:sortActivities(a, b)
	local vo_a = self:getActVoByIdx(a);
	local vo_b = self:getActVoByIdx(b);
	if vo_a.sort > vo_b.sort then
		return false;
	end
	return true;
end

function Lua_BackstageActivitiesData:updateActivitiesIcon()
	if self._actIconArr == nil then
		self._actIconArr = Array();
	else
		self._actIconArr:clear();
	end

	local iconVo;
	local mainIconVo
	local actVo;
	if self.mainIconDic then
		for iconId, arr in pairs(self.mainIconDic) do
			iconVo = DataCenter.mainMenuData:getMenuIconVo(iconId, iconId);
			if iconVo.type == 2 then
				iconVo:setFunOpenState(true);
				iconVo:setShowState(true);
				if iconVo.iconType == Lua_MenuViewDefine.TYPE_ACTIVITY_ICON then
					self._actIconArr:add(iconVo);
				end
			end
		end
	end

	if self.subIconDic then
		for actIdx, iconId in pairs(self.subIconDic) do
			actVo = self:getActVoByIdx(actIdx);

			iconVo = DataCenter.mainMenuData:getMenuIconVo(actVo.subIconId, iconId);
			iconVo.actIdx = actIdx;
			if actVo.mainIconId and actVo.mainIconId > 0 then
				mainIconVo = DataCenter.mainMenuData:getMenuIconVo(actVo.mainIconId);
				iconVo.redPointId = mainIconVo.redPointId;
			else
				iconVo.redPointId = actVo.redPointId;
			end

			if actVo.sub_pos ~= nil and actVo.sub_pos > 0 then
				iconVo.posType = actVo.sub_pos;
			end
			if actVo.sub_row ~= nil and actVo.sub_row > 0 then
				iconVo.pos = actVo.sub_row;
			end
			if actVo.sub_sort ~= nil and actVo.sub_sort > 0 then
				iconVo.sortWeight = actVo.sub_sort;
			end
			iconVo:setFunOpenState(true);
			iconVo:setShowState(true);
			self._actIconArr:add(iconVo);
		end
	end

	Lua_MsgUtil.callFunc("MENUICON_ALL_UPDATE");
end

-- 需要提前获取服务端数据的活动
function Lua_BackstageActivitiesData:getActInfoOnLogin()
	if self.actList == nil or #self.actList <= 0 or self.bInitServer == true then
		return;
	end
	self.bInitServer = true;

	local info
	for i = 1, #self.actList do
		repeat
			info = self.actList[i]
			if info.type == 10007 then -- BOSS挑战
				-- Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sActivityInfo(info.act_index)
			else
				break
			end
		until true
	end
end

-- 根据活动唯一索引获取活动Vo
function Lua_BackstageActivitiesData:getActVoByIdx(actIdx)
	if actIdx == nil or self.actIdxDic == nil then
		return nil;
	end
	return self.actIdxDic[actIdx];
end

-- 根据活动类型获取活动Array
function Lua_BackstageActivitiesData:getActIdxArrByType(actType)
	if actType == nil or self.actTypeDic == nil then
		return nil;
	end
	return self.actTypeDic[actType];
end

-- 设置活动延迟结束
function Lua_BackstageActivitiesData:setActDelayState(actIdx)
	local vo = self:getActVoByIdx(actIdx);
	if vo == nil then
		return;
	end
	vo.isDelay = 1;
end

-- 根据主图标Id获取所有活动
function Lua_BackstageActivitiesData:getActArrByMainIconId(mainIconId)
	if self.mainIconDic == nil or mainIconId == nil or mainIconId <= 0 then
		return nil;
	end
	return self.mainIconDic[mainIconId];
end

-- 是否有后台活动
function Lua_BackstageActivitiesData:checkBackstageActivities(winName, iconId)
	if strIsNullOrEmpty(winName) or self.actWinDic == nil or GlobalVar.luaDecodeType == 0 then
		return false;
	end

	local arr = self.actWinDic[winName];
	if arr == nil or arr:getSize() <= 0 then
		arr = self:getActArrByMainIconId(iconId);
		if arr == nil or arr:getSize() <= 0 then
			return false;
		end
	end
	return true;
end

-- 检查活动是否结束 参数：actVo 如果该活动是最后一个活动，服务端已经通知全部移除，getActVoByIdx方法拿到的是nil，无法强制关闭整个活动界面
function Lua_BackstageActivitiesData:checkActIsEnd(actVo)
	if actVo == nil then
		OperateAlert.showLangValue(107706);
		return true;
	end

	if not actVo.bOpened then
		if not strIsNullOrEmpty(actVo.main_win) then
			Lua_WindowUtil.openWin(actVo.main_win, Lua_WinParam.ACTION_CLOSE); -- 强制关闭整个活动界面
		end
		OperateAlert.showLangValue(107706);
		return true;
	end
	return false;
end

-- 开启一个测试活动
function Lua_BackstageActivitiesData:backstageActivityTest()
	self:init();

	local vo = Lua_BackstageActivityVo.new(tonumber(randomString(4, 9)));
	vo.is_open = 1;
	vo.start_tick = 1675994446;
	vo.end_tick = DataCenter.roleData.serverTime + 86400; -- 604800;
	vo.actType = 10007; -- BOSS挑战 10007
	vo.title = "BOSS挑战";
	vo.description = "这是活动类型为10007的活动";
	vo.main_icon = 30017;
	vo.main_win = "Lua_LimitedTimeActivityWindow";
	vo.theme = 0;
	vo.sort = 1;
	vo.help_des = "这是活动类型为10007的活动帮助";

	self.actIdxDic[vo.actIdx] = vo;
	if self.actTypeDic[vo.actType] == nil then
		self.actTypeDic[vo.actType] = Array();
	end
	if self.actTypeDic[vo.actType]:indexOf(vo.actIdx) == -1 then
		self.actTypeDic[vo.actType]:add(vo.actIdx);
	end

	if self.openIdxArr:indexOf(vo.actIdx) == -1 then
		self.openIdxArr:add(vo.actIdx);
	end

	if vo.bShowMainIcon == true then
		if self.mainIconDic[vo.mainIconId] == nil then
			self.mainIconDic[vo.mainIconId] = Array();
		end
		if self.mainIconDic[vo.mainIconId]:indexOf(vo.actIdx) == -1 then
			self.mainIconDic[vo.mainIconId]:add(vo.actIdx);
		end
	end

	self:updateActivitiesIcon();
end

--获取活动开启状态 actType:活动类型 theme：活动主题
function Lua_BackstageActivitiesData:getActivitiesOpenState(actType, theme)
	local arr = self:getActIdxArrByType(actType);
	if arr == nil then
		return false;
	end
	theme = theme == nil and 0 or theme;
	local avo;
	local bOpened = false;
	for i=1,arr:getSize() do
		avo = self:getActVoByIdx(arr[i]);
		if avo.theme == theme then
			bOpened = avo.bOpened;
			break;
		end
	end

	return bOpened ~= false;
end