-- 后台活动单个数据
Lua_BackstageActivityVo = class("Lua_BackstageActivityVo");
function Lua_BackstageActivityVo:ctor(actIdx)
	self.actIdx = actIdx;
end

function Lua_BackstageActivityVo:resetServerInfo()
	self.is_open = 0;
	self.start_tick = 0;
	self.end_tick = 0;
	self.disappear_tick = 0;

	self.actType = 0;
	self.title = nil;
	self.description = nil;

	self.main_icon = 0;
	self.main_win = nil;

	self.sub_icon = 0;
	self.sub_pos = 0;
	self.sub_row = 0;
	self.sub_sort = 0;
	self.show_sub_icon = 0;

	self.theme = 0;
	self.sort = 0;
	self.help_des = nil;
	self.countdown_switch = nil;

	self.isDelay = 0;
end

function Lua_BackstageActivityVo:fromServerInfo(msg)
	if msg == nil or msg.act_index ~= self.actIdx then
		self:resetServerInfo();
		return;
	end

	self.is_open = msg.is_open; -- 0 关闭，1 开放
	self.start_tick = tonumber(tostring(msg.start_tick)); -- 开启时间戳
	self.end_tick = tonumber(tostring(msg.end_tick)); -- 结束时间戳
	self.disappear_tick = tonumber(tostring(msg.disappear_tick)); -- 结束后消失时间戳

	self.actType = msg.type; -- 活动类型
	self.title = msg.title; -- 活动标题
	self.description = msg.content; -- 活动内容

	self.main_icon = msg.main_icon; -- 活动主图标
	self.main_win = msg.main_win; -- 主界面窗口

	self.sub_icon = msg.sub_icon; -- 活动子图标
	self.sub_pos = msg.sub_pos; -- 活动子图标位置
	self.sub_row = msg.sub_row; -- 活动子图标行数
	self.sub_sort = msg.sub_sort; -- 活动子图标排序
	self.show_sub_icon = msg.show_sub_icon; -- 是否独立主图标显示

	self.theme = msg.theme; -- 活动主题
	self.sort = msg.sort; -- 活动排序
	self.help_des = msg.help_des; -- 帮助说明
	self.countdown_switch = msg.countdown_switch; -- 倒计时开关：0关，1开
end

function Lua_BackstageActivityVo.Get:mainIconId()
	return self.main_icon;
end

function Lua_BackstageActivityVo.Get:subIconId()
	return self.actType * 1000 + self.actIdx;
end

function Lua_BackstageActivityVo.Get:redPointId()
	return tostring(self.actIdx);
end

function Lua_BackstageActivityVo.Get:redPointState()
	return DataCenter.redPointData:getRedPointStateById(self.redPointId);
end

function Lua_BackstageActivityVo.Get:bOpened()
	if self.is_open == nil or self.is_open == 0 then
		return false;
	end
	return DataCenter.roleData:checkInTime(self.start_tick, self.endTick);
end

function Lua_BackstageActivityVo.Get:endTick()
	if self.isDelay == 1 then
		return self.disappear_tick;
	end
	return self.end_tick;
end

function Lua_BackstageActivityVo.Get:bShowMainIcon()
	if self.main_icon == nil or self.main_icon <= 0 then
		return false;
	end
	return true;
end

function Lua_BackstageActivityVo.Get:bShowSubIcon()
	if self.show_sub_icon == 0 or self.sub_icon == nil or self.sub_icon <= 0 then
		return false;
	end
	return true;
end

function Lua_BackstageActivityVo.Get:residueTime()
	return self.endTick - DataCenter.roleData.serverTime;
end

function Lua_BackstageActivityVo.Get:config()
	return getConfig("backstage_activities", self.actType * 1000 + self.theme);
end

function Lua_BackstageActivityVo.Get:prompt()
	return self.config.prompt;
end

function Lua_BackstageActivityVo.Get:panelName()
	return self.config.panel_name;
end

function Lua_BackstageActivityVo.Get:skinName()
	return self.config.skin_name;
end

function Lua_BackstageActivityVo.Get:iconAtlas()
	return self.config.icon_atlas;
end

function Lua_BackstageActivityVo.Get:iconNormal()
	return self.config.icon_normal;
end

function Lua_BackstageActivityVo.Get:iconSelected()
	return self.config.icon_selected;
end

function Lua_BackstageActivityVo.Get:winBgUrl()
	return self.config.win_bg;
end

function Lua_BackstageActivityVo.Get:winBgAnchor()
	return self.config.win_bg_anchor or 1;
end

function Lua_BackstageActivityVo.Get:winBgPivot()
	return self.config.win_bg_pivot or 1;
end

function Lua_BackstageActivityVo.Get:winBgPosY()
	return self.config.win_bg_posy or 0;
end

function Lua_BackstageActivityVo.Get:money_list()
	return self.config.money_list;
end