-- 运营活动基类
Lua_OperationalActivitiesWindow = class("Lua_OperationalActivitiesWindow", Lua_Window);
function Lua_OperationalActivitiesWindow:ctor()
	self:superCall("ctor");

	self.layer = MainWinDefine.NORMAL_WINDOW_LAYER;
	self.windowName = "Lua_OperationalActivitiesWindow";
	self.isFullSizeWin = true;
	self.isSingleShowWin = true;
	-- self.showBlackMask = true;
	self.showWinBg = true;
	self.isFullSizeBg = true;
	self.winBgPath = "";
	self.mainItemW = 162;
	self.mainItemH = 168;

	local vo = DataCenter.operationalActUtil.preActVo
	if vo and not strIsNullOrEmpty(vo.winBgUrl) then
		self.winBgPath = vo.winBgUrl
		-- self:addPreLoadRes(vo.winBgUrl, Lua_LoadFileType.IMG)
	end
end

function Lua_OperationalActivitiesWindow:buildShowItem()
	self:superCall("buildShowItem");

	self:addLuaEvent("OPERATIONAL_ACTIVITY_SELECT", self, self.onActivitySelected);
	self:addLuaEvent("OPERATIONAL_ACTIVITIES_CLICKLIMIT", self, self.onSelectLimit);

	self.actList = Lua_DragList.new(self.content, self.mainItemW, self.mainItemH, Lua_AxisDefine.Horizontal);
	self.actList:setLocation(360, 92, 0);
	self.actList:setRectAnchor(7);
	self.actList:setRectPivot(5);
	self.actList.clickMoveOffset = 1;
	self.actList:setListCell(Lua_ActivityMainItem, self.mainItemW, self.mainItemH, 1);
	self.actList:setCellSpacing(0, 0);

	self:createBackBtn(58, 86.5, "common_altas_3", "xui_common_button_031");
end

function Lua_OperationalActivitiesWindow:dispose()
	self.topBg = nil;
	self.downBg = nil;
	self.limitImg = nil;
	if self.actList ~= nil then
		self.actList:dispose();
		self.actList = nil;
	end
	if self.btnList ~= nil then
		self.btnList:dispose();
		self.btnList = nil;
	end
	if self.moneyList ~= nil then
		self.moneyList:dispose();
		self.moneyList:clear();
		self.moneyList = nil;
	end
	if self.showPanel ~= nil then
		self.showPanel:dispose();
		self.showPanel = nil;
	end
	self.actDic = nil;
	self.actVo = nil;
	DataCenter.preLoadData:removeBySign(self.preSignStr, self, self.onPreLoaded)
	DataCenter.winTweenManager:removeWinTween(self.tweenName);
	self.tweenName = nil;
	DataCenter.operationalActUtil.selectedActVo = nil;
	self:superCall("dispose");
end

function Lua_OperationalActivitiesWindow:showWin(param)
	self:superCall("showWin", param);
	self:checkActList(nil);
end

function Lua_OperationalActivitiesWindow:resetActive(bActive)
	self:superCall("resetActive", bActive);
	if bActive then
		if self.showPanel ~= nil and self.showPanel.setModelShow ~= nil then
			self.showPanel:setModelShow();
		end
	else
		if self.showPanel ~= nil and self.showPanel.returnModel ~= nil then
			self.showPanel:returnModel();
		end
	end
end

function Lua_OperationalActivitiesWindow:checkActList(actVo)

end

function Lua_OperationalActivitiesWindow:checkActivityIsExist(actVo)
	if self.actDic == nil then
		return false;
	end
	if actVo.actType ~= nil and self.actDic[actVo.actType] ~= nil then
		return true;
	end
	if actVo.id ~= nil and self.actDic[actVo.id] ~= nil then
		return true;
	end
	return false;
end

function Lua_OperationalActivitiesWindow:onActivitySelected(actVo)

end

function Lua_OperationalActivitiesWindow:onSubActivitySelected(index, actVo)

end

function Lua_OperationalActivitiesWindow:updateShowPanel(actVo)
	DataCenter.operationalActUtil.preActVo = nil;
	if self.actVo ~= nil and not self.actVo.bOpened then
		self.mainActVo = nil;
		self.actVo = nil;
		self:checkActList(actVo); -- 移除过期活动并选中当前活动
		return;
	end

	self.actVo = actVo;
	self.actList:updateShowList();

	if actVo.actType == Lua_ActivityList.TYPE_BOSSChallenge then -- BOSS挑战(后台)
		self:addLuaEvent("UPDATE_BOSSCHALLENGE_INFO", self, self.onBossChallengeInfo);
		Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sActivityInfo(actVo.actIdx);
		return;
	elseif actVo.id == Lua_ActivityList.ID_HeroChallenge then -- 英雄挑战(本地)
		self:preShowScene(DataCenter.heroChallengeData.scene_id, DataCenter.heroChallengeData.monster_id);
		return;
	elseif actVo.actType == Lua_ActivityList.TYPE_NewHeroRecruit then -- -新英雄招募(后台)
		self:addLuaEvent("UPDATE_NEWHERO_RECRUIT", self, self.onNewHeroActRecruitInfo);
		Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sActivityInfo(actVo.actIdx);
		return;
	end

	if strIsNullOrEmpty(actVo.winBgUrl) then
		self.winBg:setEnable(false);
		self:onWinBgLoadCompleted();
	else
		self.winBg:setEnable(true);
		if self.winBg.url == actVo.winBgUrl then
			self:onWinBgLoadCompleted();
		else
			self.winBg:addLoadCompleteFun(self, self.onWinBgLoadCompleted) -- 先添加回调参数后修改url才会正常回调
			self.winBg:setUrl(actVo.winBgUrl);
		end
	end

	self:checkWinBgShow();
end

function Lua_OperationalActivitiesWindow:onWinBgLoadCompleted()
	self.winBg.anchorValue = self.actVo.winBgAnchor;
	self.winBg.pivotValue = self.actVo.winBgPivot;
	self.winBg:setLocation(self.winBgPosX, self.actVo.winBgPosY + GlobalVar.screenOffsetTopH, 0);

	if self.tweenName ~= nil then
		DataCenter.winTweenManager:removeWinTween(self.tweenName);
	end
	if self.showPanel ~= nil then
		self.showPanel:dispose();
		self.showPanel = nil;
	end

	local showPanel = checkRequire(self.actVo.panelName);
	if self.actVo.actIdx == nil then -- 内置活动
		self.showPanel = showPanel.new(self.content);
	else -- 后台活动
		self.showPanel = showPanel.new(self.content, self.actVo.skinName);
		if self.actVo.actType ~= 10007 and self.actVo.actType ~= 20004 then
			Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sActivityInfo(self.actVo.actIdx);
		end
	end
	self.showPanel:setSiblingIndex(1);
	self.showPanel:setRectAnchor(9);
	if self.showPanel.setData ~= nil then
		self.showPanel:setData(self.actVo);
	end
	self.tweenName = DataCenter.winTweenManager:createWinTween(self.showPanel.content.name, self.content.transform);

	self:createMoney(self.actVo.money_list);
end

function Lua_OperationalActivitiesWindow:createMoney(list)
	local num = list == nil and 0 or #list;
	if num == 0 then
		if self.topBg then
			self.topBg:setEnable(false);
		end
	else
		if self.topBg == nil then
			self.topBg = UICreate.createUIAtlasImage(self.content, "common_altas_3", "xui_common_widget_036", 0, 0, 720, 52);
			self.topBg.anchorValue = 1;
			self.topBg.pivotValue = 1;
		end
		self.topBg:setEnable(true);
	end

	if self.moneyList == nil then
		self.moneyList = Array();
	end
	local len = self.moneyList:getSize();
	local maxLen = num > len and num or len;
	local item;
	for i = 1, maxLen do
		item = self.moneyList[i];
		if i > num then
			if item ~= nil then
				item:setEnable(false);
			end
		else
			if item == nil then
				item = Lua_MoneyItem.new(self.content);
				self.moneyList:add(item);
			else
				item:setEnable(true);
			end
			item:setRectAnchor(1);
			item:setLocation(249 - (i - 1) * 190, -26, 0);
			item:setCanClick(true);
			item:setMoneyType(list[i]);
		end
	end
end

function Lua_OperationalActivitiesWindow:onSelectLimit(state)
	if state == true then
		if self.limitImg == nil then
			self.limitImg = UICreate.createUIAtlasImage(self.content, "common_altas_1", "ui_common_bg_002", 0, 55, 720, 110);
			self.limitImg.anchorValue = 7;
			self.limitImg:addClickFuntion(self, self.onLimitClick);
		end
		self.limitImg:setEnable(true);
	else
		if self.limitImg then
			self.limitImg:setEnable(false);
		end
	end
end

function Lua_OperationalActivitiesWindow:onLimitClick(go)
	-- OperateAlert.show();
end

function Lua_OperationalActivitiesWindow:onBossChallengeInfo()
	self:removeLuaEvent("UPDATE_BOSSCHALLENGE_INFO", self, self.onBossChallengeInfo);
	self:preShowScene(DataCenter.bossChallengeData.scene_id, DataCenter.bossChallengeData.monster_id);
end

function Lua_OperationalActivitiesWindow:onNewHeroActRecruitInfo()
	self:removeLuaEvent("UPDATE_NEWHERO_RECRUIT", self, self.onNewHeroActRecruitInfo);

	local heroVo = Lua_HeroVo.new(DataCenter.newHeroActData.heroId);
	heroVo.uiSceneId = DataCenter.newHeroActData.sceneId;
	DataCenter.preLoadData:preShowModelScene("onNewHeroActRecruitInfo", Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, nil, heroVo);
end

function Lua_OperationalActivitiesWindow:preShowScene(sceneId, monsterId)
	if tostring(sceneId) == self.preSignStr then
		return;
	end
	self.preSignStr = tostring(sceneId);

	local config = Lua_MonsterPlayerConfigData.getConfigData(monsterId);
	config = Lua_MonsterConfigData.getMonsterConfig(config.fighter_list[1][1][1]);
	if config == nil then
		self:onPreLoaded(nil, nil, true)
	else
		DataCenter.preLoadData:preShowModelScene(self.preSignStr, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, nil, config.heroVo);
	end
end

function Lua_OperationalActivitiesWindow:onPreLoaded(param, modelScenePanel, isSuccess)
	DataCenter.preLoadData:removeBySign(self.preSignStr, self, self.onPreLoaded)
	self.preSignStr = nil
	if isSuccess == true then
		self.winBg:setEnable(false);
		self:onWinBgLoadCompleted();
		self:checkWinBgShow();
		if self.showPanel.setModelShow ~= nil then
			self.showPanel:setModelShow(modelScenePanel);
		end
	end
end




















Lua_ActivityMainItem = class("Lua_ActivityMainItem", Lua_ListCell)
function Lua_ActivityMainItem:ctor(p, skin)
	self:superCall("ctor", p, skin);
end

function Lua_ActivityMainItem:buildShowItem()
	self:superCall("buildShowItem");
	self.icon = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
end

function Lua_ActivityMainItem:dispose()
	if self.redPoint ~= nil then
		self.redPoint:dispose();
		self.redPoint = nil;
	end
	self.data = nil;
	self:superCall("dispose");
end

function Lua_ActivityMainItem:setData(data)
	if type(data) == "table" then
		self.data = data;
	elseif type(data) == "number" then
		self.data = DataCenter.backstageActData:getActVoByIdx(data);
	end
	self:updateData();
	self:checkRedPoint();
end

function Lua_ActivityMainItem:updateData()
	if self.data == DataCenter.operationalActUtil.selectedActVo then
		self.icon:setAltas(self.data.iconAtlas, self.data.iconSelected);
	else
		self.icon:setAltas(self.data.iconAtlas, self.data.iconNormal);
	end
end

function Lua_ActivityMainItem:onPressClick()
	if self.data == nil or not self.data.bOpened then
		OperateAlert.show(self.data.prompt);
		return;
	end
	Lua_MsgUtil.callFunc("OPERATIONAL_ACTIVITY_SELECT", self.data);
end

function Lua_ActivityMainItem:checkRedPoint()
	if self.data ~= nil and self.data.redPointId ~= nil then
		if self.redPoint == nil then
			self.redPoint = Lua_RedPointIcon.new(self.content, self.data.redPointId);
			self.redPoint:setLocation(63, 0);
		end
		self.redPoint:changeRedPoint(self.data.redPointId);
	else
		if self.redPoint ~= nil then
			self.redPoint:setEnable(false);
		end
	end
end




















Lua_ActivitySubItem = class("Lua_ActivitySubItem", Lua_ButtonCell)
function Lua_ActivitySubItem:ctor(p)
	self:superCall("ctor", p);
end

function Lua_ActivitySubItem:buildShowItem()
	self:superCall("buildShowItem");
	self.btnImg = UICreate.createUIAtlasImage(self.content, "common_altas_3", "xui_common_toggle_004_b", 0, 0, 0, 0);
	self.btnImg.clickScale = false;

	self.btnText = UICreate.createLabel(self.content, nil, 0, 1, 100, 40, 28);
	self.btnText.pivotValue = 4;
	self.btnText.fontName = Lua_MyUISettings.huakangFontName;
	self.btnText.alignmentValue = 4;
	self.btnText.horizontalOverflowValue = 1;
end

function Lua_ActivitySubItem:dispose()
	if self.redPoint ~= nil then
		self.redPoint:dispose();
		self.redPoint = nil;
	end
	self.data = nil;
	self:superCall("dispose");
end

function Lua_ActivitySubItem:setData(data)
	self.data = data;
	self.btnText.text = data.name;
	self:checkRedPoint();
end

function Lua_ActivitySubItem:setSelected(index)
	if index == self.index then
		self.btnImg:setAltas("common_altas_3", "xui_common_toggle_004_f");
		self.btnText.colorStr = Lua_ColorDefine.transColorStr("7E4E2A");
	else
		self.btnImg:setAltas("common_altas_3", "xui_common_toggle_004_b");
		self.btnText.colorStr = Lua_ColorDefine.transColorStr("836D51");
	end
end

function Lua_ActivitySubItem:setCanClick(bCan)
	if bCan == true then
		self.btnImg.mouseEnable = true;
		self.btnImg:addClickFuntion(self, self.onPressClick);
	else
		self.btnImg.mouseEnable = false;
		self.btnImg:removeClickFuntion(self, self.onPressClick);
	end
end

function Lua_ActivitySubItem:onPressClick(go)
	if self.data == nil or not self.data.bOpened then
	OperateAlert.show(self.data.prompt);
		return;
	end
	self:superCall("onPressClick", go);
end

function Lua_ActivitySubItem:checkRedPoint()
	if self.data ~= nil and self.data.redPointId ~= nil then
		if self.redPoint == nil then
			self.redPoint = Lua_RedPointIcon.new(self.content, self.data.redPointId);
			self.redPoint:setLocation(65, 15);
		end
		self.redPoint:changeRedPoint(self.data.redPointId);
	else
		if self.redPoint then
			self.redPoint:setEnable(false);
		end
	end
end