checkRequire("Lua_RecruitTypeVo");
Lua_RecruitData = class("Lua_RecruitData");
Lua_RecruitData.FRIENDSHIP = 2; -- 友情召唤
Lua_RecruitData.SENIOR = 3; -- 高级召唤
Lua_RecruitData.SCOUT = 6; -- 先知召唤
Lua_RecruitData.Ten = 8; -- 十连自选
Lua_RecruitData.NEWHERO = 9; -- 新英雄招募

function Lua_RecruitData:ctor()
    self:dispose();
end

function Lua_RecruitData:dispose()
    self.selectType = 1;--当前招募类型
    self.selectTimes = 1;--当前招募次数
    self.scoutSelect = 0;--先知召换阵容选择

    --服务端数据处理
    self.score = 0;--积分
    self.scoreState = 0;--能否进行积分招募（0：不能，1：能）
    self.hasFreeTimes = false;--拥有免费招募次数

    --招募返回处理
    self.onBtnControl = false; -- 召唤中按钮点击控制

    self.itemList = Array();--获得的道具列表
    self.rewardList = Array();--召唤获得道具

    --心愿祝福
    self.blessHeroId = nil;--心愿祝福英雄id
    self.bFirstWishBlessRedPoint = nil;
    -- 欧气排行榜
    self:resetRankData();
end

function Lua_RecruitData:fromServer50131001(msg)--召唤信息（返回）
	if msg == nil then return end

    for i = 1, #msg.recruit_info do
        local vo = DataCenter.recruitUtil:getRecTotVoById(tonumber(msg.recruit_info[i].recruit_type));
        if vo ~= nil then
            vo:updateServerInfo(msg.recruit_info[i]);
        end
    end
    self.score = msg.score;--积分
    self.scoreState = msg.score_draw;--能否进行积分招募（0：不能，1：能）

    if self.bFirstWishBlessRedPoint == nil or DataCenter.recruitUtil:openWishBlessType(false) == 2 then
        self.bFirstWishBlessRedPoint = true;
        self:updateWishBlessRedPoint(true);
    end
end

function Lua_RecruitData:fromServer80131001(msg)--通知签约中心红点
    self.hasFreeTimes = msg.state == 1;--拥有免费次数
end

function Lua_RecruitData:updateRedPoint()
    if DataCenter.funcOpenData:isOpened(10076) then
        local arr = DataCenter.recruitUtil.recruitTotalArr;
        for i = 1, arr:getSize() do
            arr[i]:checkNeedRedPoint();
        end
    end
end








-------------------------------------- 招募返回 --------------------------------------

function Lua_RecruitData:fromServer50131005(msg)
    self.itemList:clear();
    self.rewardList:clear();

    self.selectType = msg.recruit_type;--召唤类型
    if msg.recruit_show.luck_num ~= nil then
        self.curLuckValue = msg.recruit_show.luck_num;
    end

    if self.selectType == Lua_RecruitData.FRIENDSHIP or self.selectType == Lua_RecruitData.SENIOR then
        self.recruitNum = msg.draw_times; -- 友情、高级招募单连抽次数
        self.recruitShow = msg.recruit_show;
        self.itemList = self:showFlashHeroItemList(msg.recruit_show);

        -- 小卖部使用钻石招募时，有特殊奖励（经验等）需与流放奖励共同展示
        if #msg.recruit_reward > 0 then
            DataCenter.heroAssemblyData:addRewardInAutoDecoItemDic(msg.recruit_reward);
        end
    else
        if msg.item_list ~= nil then
            local replaceCfg;
            local vo;
            for k, v in ipairs(msg.item_list) do 
                replaceCfg = DataCenter.heroUtil:getHeroReplaceStarCfg(v.id);
                if replaceCfg ~= nil then
                    vo = Lua_ToolVo.createToolVo(replaceCfg.hero_id, v.amount);
                    vo.quality = replaceCfg.quality;
                    vo.quality_star = replaceCfg.quality_star;
                else
                    vo = Lua_ToolVo.createToolVo(v.id, v.amount);
                end

                if vo ~= nil then
                    self.itemList:add(vo);
                end
            end
        end

        for k, v in ipairs(msg.recruit_reward) do--召唤获得道具
            local vo = Lua_ToolVo.createToolVo(v.id, v.amount);
            self.rewardList:add(vo);
        end
    end

    -- 打开过场、特写、结算界面控制
    self:showRecruitResult();
    -- 老客回顾红点
    self:updateWishBlessRedPoint(self.selectType == Lua_RecruitData.SENIOR);
end

function Lua_RecruitData:showFlashHeroItemList(recruitShow)
    local arr = Array();
    local flashList = {};
    if recruitShow ~= nil then
        local replaceCfg;
        local vo;
        local id;
        self.maxHeroQuality = 0;
        for k, v in ipairs(recruitShow.hero_list) do -- 获得的道具列表
            id = tonumber(v);
            replaceCfg = DataCenter.heroUtil:getHeroReplaceStarCfg(id);
            if replaceCfg ~= nil then -- 四星转五星
                vo = Lua_ToolVo.createToolVo(replaceCfg.hero_id);
                vo.quality = replaceCfg.quality;
                vo.quality_star = replaceCfg.quality_star;
            else
                vo = Lua_ToolVo.createToolVo(id);
            end
            if vo ~= nil then
                if self.maxHeroQuality < vo.quality then
                    self.maxHeroQuality = vo.quality;
                end
                arr:add(vo);
            end
        end
        
        local info = recruitShow.flash_maps;
        if info ~= nil and #info > 0 then  
            local obj;
            local _pos;
            local heroVo;
            local hasCard;
            for i = 1, #info do
                obj = info[i];
                id = tonumber(obj.value);
                _pos = tonumber(obj.key)
                hasCard = DataCenter.collectionBookUtil:getSingleCardConfigById(id) ~= nil;
                heroVo = arr[_pos];
                if hasCard and heroVo ~= nil then
                    heroVo.recruitFlashId = id;
                    heroVo.recruitType = #recruitShow.hero_list;
                    flashList[#flashList + 1] = {pos = _pos, flashId = id};
                end
            end 
        end
    end
    return arr, flashList;
end

function Lua_RecruitData:showRecruitResult()
    if self.selectType == Lua_RecruitData.FRIENDSHIP or self.selectType == Lua_RecruitData.SENIOR then
        local vo = {};
        local selectData = DataCenter.recruitUtil:getRecTotVoById(self.selectType);
        if selectData ~= nil then
            local needNum = self.selectTimes == 1 and selectData.oneCost or selectData.tenCost;
            local ticket_num = DataCenter.packData:getItemCountById(selectData.costId);
            if selectData.isUseGold and ticket_num < needNum then -- 使用钻石
                vo.costId = selectData.currencyId;
                local needGold = self.selectTimes == 1 and selectData.oneGold or selectData.tenGold;
                local curMoney = DataCenter.moneyData:getMoney(selectData.currencyId);
                if curMoney < needGold then
                    vo.costTxt = LanUtil.formateStr(140337, numberTrans(curMoney), needGold);
                else
                    vo.costTxt = LanUtil.formateStr(140338, numberTrans(curMoney), needGold);
                end
            else
                vo.costId = selectData.costId;
                if ticket_num < needNum then
                    vo.costTxt = LanUtil.formateStr(140337, numberTrans(ticket_num), needNum);
                else
                    vo.costTxt = LanUtil.formateStr(140338, numberTrans(ticket_num), needNum);
                end
            end
        end
        vo.maxQuality = self.maxHeroQuality;
        vo.itemList = self.itemList;
        Lua_WindowUtil.openWin("Lua_RecruitShowHeroWindow", Lua_WinParam.ACTION_OPEN, vo);    
    elseif self.selectType == Lua_RecruitData.SCOUT then -- 先知召唤
        Lua_MsgUtil.callFunc("UPDATE_RECRUITSCOUT_ITEM", { isUpdate = true});
        Lua_WindowUtil.openWin("Lua_RecruitScoutShowRewardWindow", Lua_WinParam.ACTION_OPEN, self.itemList);
    end
end










-------------------------------------- 心愿单 --------------------------------------

function Lua_RecruitData:fromServer50131009(msg)--请求心愿单界面信息（返回）
    self.wishListInfo = msg.wish_list;
    table.sort(self.wishListInfo, function(a, b) return a.key < b.key end);

    self.wishSelectList = {};
    for k, v in ipairs(self.wishListInfo) do
        self.wishSelectList[v.key] = {key = v.key, value = tonumber(v.value), other = v.other};
   end
end

function Lua_RecruitData:setSelectWishInfo(index, heroId)
    local wishData = self.wishSelectList[index];
    if wishData ~= nil then
        wishData.value = tonumber(heroId);
    end
end

function Lua_RecruitData.Get:isWishListFree()
    for k, v in ipairs(self.wishSelectList) do
        if tonumber(v.value) == 0 then
            return v.key;
        end
    end
    return nil;
end

function Lua_RecruitData:getWishInfo(id)
    for k, v in ipairs(self.wishSelectList) do
        if tonumber(v.value) == id then
            return v;
        end
    end
    return nil;
end

function Lua_RecruitData:isInWishCamp(camp)
    for k, v in ipairs(self.wishSelectList) do
        config = Lua_HeroConfigData.getConfig(tonumber(v.value));
        if config ~= nil and config.camp == camp then
            return true;
        end
    end
    return false;
end

function Lua_RecruitData:hasNoneWishHero(noneWishHeroNum)
    if self.wishSelectList ~= nil and noneWishHeroNum ~= nil then
        local curNum = 0;
        local result = false;
        for i = 1, #self.wishSelectList do
            local other = self.wishSelectList[i].other;
            if other == 0 then
                curNum = curNum + 1;
                if curNum >= noneWishHeroNum then
                    result = true;
                    break;
                end
            end
        end
        return result;
    end
end








-------------------------------------- 欧气排行榜 --------------------------------------

function Lua_RecruitData:fromServer50131016(msg) --请求欧气排行榜(返回)
    if msg == nil then
        return;
    end

    self:resetRankData();
    self.totalRankInfo = {};
    self.topRankIdList = {-1, -1, -1};
    self.otherRankIdList = {};
    self.myselfRankInfo = self:dealwithSingleLuckyInfo(msg.self_ranker);

    self.curWeekType = msg.rank_type == 1;
    if self.curWeekType then --排行榜类型 1每周 2历史
        self.weekTime = msg.reset_tick;
    end

    local obj;
    for i = 1, #msg.ranker_list do
        obj = msg.ranker_list[i];
        self.totalRankInfo[i] = self:dealwithSingleLuckyInfo(obj);
        if i <= 3 then
            self.topRankIdList[i] = i;
        else
            self.otherRankIdList[i - 3] = i;
        end
    end

end

function Lua_RecruitData:dealwithSingleLuckyInfo(singleData)
    if singleData == nil then
        return;
    end

    return {rankId = singleData.rank, playerShow = singleData.player_show, recruitShow = singleData.recruit_show};
end

function Lua_RecruitData:getRankInfoById(id, isMyself)
    if isMyself then
        return self.myselfRankInfo;
    end

    if id == nil or self.totalRankInfo[id] == nil then
        return nil;
    end

    return self.totalRankInfo[id];
end

function Lua_RecruitData:resetRankData()
    if self.myselfRankInfo ~= nil then
        self.myselfRankInfo = nil;
    end
    if self.totalRankInfo ~= nil then
        self.totalRankInfo = nil;
        self.topRankIdList = nil;
        self.otherRankIdList = nil;
    end
end








-------------------------------------- 老客回顾 --------------------------------------

function Lua_RecruitData:updateWishBlessHeroFromServer(msg)
    if msg == nil then return end
    self.blessHeroId = msg.hero_id;
end

function Lua_RecruitData:updateWishBlessRedPoint(bSerniorRecruit)
    local bOpenWishBless = DataCenter.recruitUtil:openWishBlessType(false) > 1;
    if bSerniorRecruit and bOpenWishBless then
        local heroConfig = Lua_HeroConfigData.getConfig(self.blessHeroId);
        DataCenter.redPointData:updateRedPointState(Lua_RedPointList.RecruitTabHighWishBless, heroConfig == nil);
    end
end