Lua_RoleData = class("Lua_RoleData");
function Lua_RoleData:ctor()
	self:dispose();
end

function Lua_RoleData:dispose()
	self.roleId = 0;
	self.name = "";
	self.sex = 1;
	self.career = 0;
	self._level = 1;
	self.vip = 0;
	self.force = 0;
	self.market = 0;
	self.agent_code = 0;
	self.server_flag = "";
	self.server_name = "";
	self.server_prev = "";
	self.league_id = 0;
	self.league_name = "";
	self.league_position = 0;
	self.league_level = 0;
	self.artifact_id = 0;
	self.artifact_clothes_id = 0;

	self.exp = 0;
	self.open_days = 0;
	self.create_days = 0;
	self.serverTime = 0;

	self.bInitRoleData = false;
	self.leaderForce = 0;
	self.prop_list = nil;
	
	self.bInitFateData = false;

	self.dressUpDic = nil;
	self.dressUpRedDic = nil;
	self.dressCheckItemDic = nil;

	self.backConfigDic = nil;
	self.createRoleTime = TimeTransform.GetUnix();
end

------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- 玩家信息
function Lua_RoleData:fromServer(msg, reportType)
	if msg == nil then
		return;
	end

	self.roleId = tonumber(tostring(msg.show.id)); -- 角色ID
	self.name = msg.show.name; -- 角色名
	self.sex = msg.show.sex; -- 性别 1男 2女
	self.career = msg.show.career; -- 职业
	self.level = msg.show.level; -- 等级
	self.vip = msg.show.vip; -- vip等级
	self.force = tonumber(msg.show.force); -- 战力值
	self.market = msg.show.market; -- 包号
	self.agent_code = msg.show.agent_code; -- 渠道编号
	self.server_flag = msg.show.server_flag; -- 服务器标识
	self.server_name = msg.show.server_name; -- 服务器名字
	self.server_prev = msg.show.server_prev; -- 服务器前缀
	self.league_id = tonumber(msg.show.league_id); -- 联盟id
	self.league_name = msg.show.league_name; -- 联盟名字
	self.league_position = msg.show.league_position; -- 联盟职位
	self.league_level = msg.show.league_level; -- 联盟等级
	self.artifact_id = msg.show.artifact_id; -- 已装备神器，0：未装备 非0：神器id
	self.artifact_clothes_id = msg.show.artifact_clothes_id; -- 神器时装id
	self.createRoleTime = tonumber(tostring(msg.show.create_tick));

	local list = msg.show.dress_info;
	if list ~= nil then
		for i = 1, #list do
			self:updateDressUpInfo(list[i].key, list[i].value);
		end
	end

	self.exp = msg.exp;
	self.open_days = msg.open_days; -- 开服天数
	self.create_days = msg.create_days; -- 创号天数
	self.serverTime = msg.time; -- 当前时间戳
	if reportType == nil then reportType = Lua_SDKMessageType.UpdatePlayerInfo_PlayerUpdate; end
	DataCenter.gameReported:UpdatePlayerInfo(reportType);
end

function Lua_RoleData.Set:level(value)
	if self._level ~= value then
		self.prevLevel = self._level;
		self._level = value;
		Lua_MsgUtil.callFunc("UPDATE_ROLE_LEVEL", { oldLv = self.prevLevel, newLv = value });
	end
end

function Lua_RoleData.Get:level()
	return self._level;
end

function Lua_RoleData.Set:worldLv(val)
	self._worldLv = val;
end

function Lua_RoleData.Get:worldLv()
	return self._worldLv;
end

function Lua_RoleData.Get:bMaxLv()
	return self:checkMaxLv(self.level);
end

function Lua_RoleData:checkMaxLv(level)
	return level >= Lua_CommonUtil.getCommonConfig("role_max_level", 1);
end

function Lua_RoleData.Set:serverTime(val)
	self._serverTime = tonumber(val);
	self.setTime = getTime();
end

function Lua_RoleData.Get:serverTime()
	return Mathf.Floor(self._serverTime + getTime() - self.setTime);
end

function Lua_RoleData:isMyself(id)
	if tonumber(tostring(id)) == self.roleId then
		return true;
	end
	return false;
end

function Lua_RoleData:isSameAlliance(id)
	if tonumber(tostring(id)) == self.league_id then
		return true;
	end
	return false;
end

function Lua_RoleData:isSameServer(server_prev)
	if strIsNullOrEmpty(server_prev) or server_prev == self.server_prev then
		return true;
	end
	return false;
end

function Lua_RoleData:isJoinTheLeague()
	if self.league_id == nil or self.league_id <= 0 then
		return false;
	end
	return true;
end

function Lua_RoleData:getGenderText(gender)
	if gender == 1 then
		return LanUtil.getValue(200);
	elseif gender == 2 then
		return LanUtil.getValue(201);
	end
	return "";
end

function Lua_RoleData:checkInTime(startTime, endTime)
	if startTime == nil or endTime == nil then
		return false;
	end

	if self.serverTime - tonumber(tostring(endTime)) >= 0 then -- 已结束
		return false;
	end
	if self.serverTime - tonumber(tostring(startTime)) < 0 then -- 未开始
		return false;
	end
	return true;
end

function Lua_RoleData:updateAllianceInfo(id, name, duty, level)
	self.league_id = tonumber(id); -- 联盟id
	self.league_name = name; -- 联盟名字
	self.league_position = duty; -- 联盟职位
	self.league_level = level; -- 联盟等级
	DataCenter.gameReported:UpdatePlayerInfo(Lua_SDKMessageType.UpdatePlayerInfo_PlayerUpdate);
end






------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- 角色命格、神器
function Lua_RoleData:fromServer50111003(msg)
	if msg == nil then
		return;
	end

	-- self.level = msg.showrole.level;
	self.artifact_id = msg.showrole.artifact_id;
	self.artifact_clothes_id = msg.showrole.artifact_clothes_id;
	self.leaderForce = tonumber(msg.leader.force); -- 主角战力
	self.prop_list = msg.leader.prop_list;
	DataCenter.artifactData.show_artifact = msg.leader.look_id
	self:updateFateData(msg.destiny);
	self.bInitRoleData = true;
end

function Lua_RoleData:updateFateData(list)
	self.curFaceId = nil;
	if list == nil or #list <= 0 then
		self.curFaceId = DataCenter.roleUtil:getFitstFateId();
	else
		table.sort( list, function(a, b) return a.destiny_id < b.destiny_id end );
		local len = #list;
		local obj;
		local vo;
		for i = 1, len do
			obj = list[i];
			vo = DataCenter.roleUtil:getFateVoById(obj.destiny_id);
			vo:fromServer(obj);
			if self.curFaceId == nil and vo.is_activate == 0 then
				self.curFaceId = obj.destiny_id;
			end
		end
		if self.curFaceId == nil then
			self.curFaceId = list[len].destiny_id;
		end

		self.bInitFateData = true;
	end
end

function Lua_RoleData:fromServer50180001(msg)
	if msg == nil then
		return;
	end

	self.destiny_prop = msg.destiny_prop; -- 命格属性
	self:updateFateData(msg.destiny_list);
	self:checkFateRedPoint();
end

function Lua_RoleData:checkFateRedPoint()
	if self.curFaceId == nil then
		self.curFaceId = DataCenter.roleUtil:getFitstFateId();
	end
	local vo = DataCenter.roleUtil:getFateVoById(self.curFaceId);
	local chipId;
	if vo.level >= #vo.chips then
		chipId = vo.id * 100 + #vo.chips;
	else
		chipId = vo.id * 100 + vo.level + 1;
	end
	local chipData = DataCenter.roleUtil:getFateChipConfig(chipId);
	local bActivate = DataCenter.roleUtil:checkFateChipIsActivate(chipData.destiny_id, chipData.index);
	local state = false;
	if not bActivate and self.level >= chipData.level then
		local haveCount;
		for i = 1, #chipData.cost do
			haveCount = DataCenter.packData:getItemCountById(chipData.cost[i][1]);
			if haveCount >= chipData.cost[i][2] then
				state = true;
				break;
			end
		end
	end
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Fate, state);
end





------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- 设置
function Lua_RoleData:fromServer80182001(msg)
	if msg == nil then
		return;
	end

	if msg.operator_list ~= nil then
		local obj;
		for i = 1, #msg.operator_list do
			obj = msg.operator_list[i];
			self:setOperationRecord(obj.key, obj.value);
		end
	end

	self:updateSetUp(msg.game_set_list);
end

function Lua_RoleData:fromServer50182001(msg)
	if msg == nil then
		return;
	end

	self:updateSetUp(msg.operator_list);
end

function Lua_RoleData:fromServer50182003(msg)
	if msg == nil then
		return;
	end

	self:setOperationRecord(msg.key, msg.value);
end

function Lua_RoleData:updateSetUp(list)
	if list == nil then
		return;
	end

	local bUpdateDeviceQuilty = false;
	local playerPrefsUtil = PlayerPrefsUtil.instance;
	for i = 1, #list do
		if list[i].key == Lua_SetUpDefine.quality_lv then
			local qualityLv = DataCenter.deviceQuality.qualityLevel;
			if qualityLv ~= list[i].value then
				bUpdateDeviceQuilty = true;
				playerPrefsUtil:SetUnInt(Lua_SetUpDefine.quality_lv, list[i].value);
			end
		elseif list[i].key == Lua_SetUpDefine.music_volume then
			playerPrefsUtil.musicVolume = list[i].value;
		elseif list[i].key == Lua_SetUpDefine.sound_volume then
			playerPrefsUtil.soundVolume = list[i].value;
		elseif list[i].key == Lua_SetUpDefine.quality_msaa then
			local msaa = playerPrefsUtil:GetUnInt(Lua_SetUpDefine.quality_msaa, -1);
			if msaa ~= list[i].value then
				bUpdateDeviceQuilty = true;
				playerPrefsUtil:SetUnInt(Lua_SetUpDefine.quality_msaa, list[i].value);
			end
		elseif list[i].key == Lua_SetUpDefine.quality_renderscale then
			local renderScaleIdx = playerPrefsUtil:GetUnInt(Lua_SetUpDefine.quality_renderscale, -1);
			if renderScaleIdx ~= list[i].value then
				bUpdateDeviceQuilty = true;
				playerPrefsUtil:SetUnInt(Lua_SetUpDefine.quality_renderscale, list[i].value);
			end
		end
	end

	-- 更新性能设置
	if bUpdateDeviceQuilty then
		DataCenter.deviceQuality:updateQuality();
	end
end

function Lua_RoleData:setOperationRecord(key, value)
	if self.operateDic == nil then
		self.operateDic = {};
	end
	self.operateDic[key] = value;
end

function Lua_RoleData:getOperationRecord(key, defaultVal)
	key = key ~= nil and string.lower(key);
	if self.operateDic == nil or self.operateDic[key] == nil then
		return defaultVal == nil and 0 or defaultVal;
	end
	return self.operateDic[key];
end





------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- 装扮
function Lua_RoleData:fromServer80111004(msg)
	if msg == nil or msg.dress_info == nil then
		return;
	end

	if self.dressUpDic then
		table.clear(self.dressUpDic);
	end
	for i = 1, #msg.dress_info do
		self:updateDressUpInfo(msg.dress_info[i].key, msg.dress_info[i].value);
	end
end

function Lua_RoleData:fromServer50190007(msg)
	if msg == nil then
		return;
	end

	local list = msg.dress_list;
	local vo;
	for i = 1, #list do
		vo = DataCenter.roleUtil:getDressUpVoById(list[i].dress_id);
		vo:fromServer(list[i]);
	end
	self:checkDressUpRedPoint(msg.dress_type);
end

function Lua_RoleData:fromServer50190008(msg)
	if msg == nil then
		return;
	end
	
	local vo = DataCenter.roleUtil:getDressUpVoById(msg.dress_info.dress_id);
	vo:fromServer(msg.dress_info);
	OperateAlert.show(LanUtil.formateStr(301022, vo.name));
	self:checkDressUpRedPoint(vo.main_type);
end

function Lua_RoleData:fromServer50190009(msg)
	if msg == nil then
		return;
	end

	local list = msg.dress_info;
	local vo;
	for i = 1, #list do
		vo = DataCenter.roleUtil:getDressUpVoById(list[i].dress_id);
		vo:fromServer(list[i]);
		if vo.is_wear == 1 then
			self:updateDressUpInfo(vo.main_type, vo.id);
		end
	end
end

function Lua_RoleData:fromServer80190010(msg)
	if self.dressUpRedDic == nil then
		self.dressUpRedDic = {};
	end
	local info
	for i = 1, #msg.red_point_state do
		info = msg.red_point_state[i];
		self.dressUpRedDic[info.key] = info.value;
		self:checkDressUpRedPoint(info.key);
	end
end

function Lua_RoleData:updateDressUpInfo(mainType, dressId)
	if self.dressUpDic == nil then
		self.dressUpDic = {};
	end
	self.dressUpDic[mainType] = dressId;
end

-- 自己的装扮信息
function Lua_RoleData:getOwnDressUpId(mainType)
	if self.dressUpDic == nil then
		return self:getDefaultDressUpId(mainType);
	end
	if mainType == nil or self.dressUpDic[mainType] == nil then
		return nil;
	end
	return self.dressUpDic[mainType];
end

-- 提供外部使用
function Lua_RoleData:getDressUpId(mainType, dress_info)
	if dress_info ~= nil then
		for i = 1, #dress_info do
			if dress_info[i].key == mainType then
				return dress_info[i].value;
			end
		end
	end
	return self:getDefaultDressUpId(mainType);
end

function Lua_RoleData:getDefaultDressUpId(mainType)
	local arr = DataCenter.roleUtil:getDressUpArr(mainType);
	local vo;
	for i = 1, arr:getSize() do
		vo = DataCenter.roleUtil:getDressUpVoById(arr[i]);
		if vo.default == 1 then
			return arr[i];
		end
	end
	return nil;
end

function Lua_RoleData:checkDressUpRedPoint(mainType)
	local state = false;
	if self.dressUpRedDic and self.dressUpRedDic[mainType] == 1 then
		state = true;
	else
		local arr = DataCenter.roleUtil:getDressUpArr(mainType);
		local vo
		for i = 1, arr:getSize() do
			vo = DataCenter.roleUtil:getDressUpVoById(arr[i]);
			if vo.is_new == 1 or vo.canActivate == 1 then
				state = true;
				break;
			end
		end
	end
	if mainType == Lua_DressUpDefine.MAINTYPE_HEAD then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.DressUp_Avatar, state);
	elseif mainType == Lua_DressUpDefine.MAINTYPE_FRAME then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.DressUp_Frame, state);
	-- elseif mainType == Lua_DressUpDefine.MAINTYPE_BUBBLE then
	-- 	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.DressUp_Bubble, state);
	elseif mainType == Lua_DressUpDefine.MAINTYPE_TITLE then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.DressUp_Title, state);
	end
end

function Lua_RoleData:setDressCheckItem(itemId, dressId)
	if self.dressCheckItemDic == nil then
		self.dressCheckItemDic = {};
	end
	self.dressCheckItemDic[itemId] = dressId;
end

function Lua_RoleData:checkDressUpRedByItem(tab)
	local dressId = nil;
	if self.dressCheckItemDic then
		for k, v in pairs(self.dressCheckItemDic) do
			if tab[k] then
				dressId = v;
				break;
			end
		end
	end
	if dressId then
		local vo = DataCenter.roleUtil:getDressUpVoById(dressId);
		self:checkDressUpRedPoint(vo.main_type);
	end
end





------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- 后台配置
function Lua_RoleData:fromServer80113002(msg)
	if msg == nil or msg.back_config == nil then
		return;
	end
	if strIsNullOrEmpty(msg.back_config.name) or strIsNullOrEmpty(msg.back_config.json_str) then
		return;
	end
	self:setBackConfig(msg.back_config.name, decodeJson(msg.back_config.json_str));
end

function Lua_RoleData:setBackConfig(key, config)
	if self.backConfigDic == nil then
		self.backConfigDic = {};
	end
	self.backConfigDic[key] = config;
end

function Lua_RoleData:getBackConfig(key, id)
	if self.backConfigDic == nil then
		return nil;
	end

	local config = self.backConfigDic[key];
	if id == nil then
		return config;
	end
	
	id = tostring(id);
	return config[id];
end