Lua_ArtifactVo = class("Lua_ArtifactVo")
function Lua_ArtifactVo:ctor(id, config)
	self._id = id;
	self._config = config;
end

function Lua_ArtifactVo.Get:id()
	return self._id
end

function Lua_ArtifactVo.Get:config()
	if self._config == nil then
		self._config = getConfig("artifact", self.id)
	end
	return self._config
end

function Lua_ArtifactVo:fromServer(msg)
	if msg == nil or msg.artifact_id ~= self.id then
		return;
	end
	
	self._is_activate = msg.is_activate; -- 0未激活 1已激活
	self._skill_id = msg.artifact_skill; -- 神器技能id
	self._level = msg.artifact_level; -- 等级
	self._clothes_id = msg.artifact_clothes_id; -- 时装id， 0：未幻化 非0：幻化神器id
	self._role_props = msg.role_props; -- 神器增加的主角属性
	-- self._hero_props = msg.hero_props; -- 神器增加的英雄阵容属性
	self._lvl_progress = msg.artifact_lvl_progress; -- 神器升级进度值
	self._star_id = msg.star_id; -- 神器星级id
end

function Lua_ArtifactVo.Get:is_activate()
	if self.unlock_way == 0 then
		return 1
	end
	return self._is_activate or 0
end

function Lua_ArtifactVo.Get:skill_id()
	if self._skill_id == nil or self._skill_id == 0 then
		return self.skill
	end
	return self._skill_id
end

function Lua_ArtifactVo.Get:level()
	return self._level or 1
end

function Lua_ArtifactVo.Get:clothes_id()
	return self._clothes_id or 0
end

function Lua_ArtifactVo.Get:role_props()
	return self._role_props
end

-- function Lua_ArtifactVo.Get:hero_props()
-- 	return self._hero_props
-- end

function Lua_ArtifactVo.Get:lvl_progress()
	return self._lvl_progress
end

function Lua_ArtifactVo.Get:star_id()
	if self._star_id == nil or self._star_id == 0 then
		return self.id * 100 + 1
	end
	return self._star_id
end

function Lua_ArtifactVo.Get:levelId()
	return self.id * 1000 + self.level
end

function Lua_ArtifactVo.Get:bMaxLv()
	return self.level >= self.max_level
end

function Lua_ArtifactVo.Get:starNum()
	return self.starConfig.star
end

function Lua_ArtifactVo.Get:bMaxStar()
	return self.starNum >= self.max_star
end

function Lua_ArtifactVo.Get:talentLevel()
	local level = 0
	local arr = DataCenter.artifactUtil:getTalentListById(self.id)
	for i = 1, arr:getSize() do
		if self.level >= arr[i].artifact_level then
			level = arr[i].level
		end
	end
	return level
end

function Lua_ArtifactVo.Get:levelConfig()
	return DataCenter.artifactUtil:getLevelConfig(self.levelId)
end

function Lua_ArtifactVo.Get:starConfig()
	return DataCenter.artifactUtil:getStarConfig(self.star_id)
end

function Lua_ArtifactVo.Get:canRecast()
	return self.lvl_progress > 0 or self.level > 1
end

function Lua_ArtifactVo.Get:canUp()
	return self.canUnlock or self.canUpgrade or self.canUpStar
end

function Lua_ArtifactVo.Get:canUnlock()
	if self.is_activate == 0 and self.unlock_way == 2 then
		local cost = self.unlock_cond
		local haveCount;
		for i = 1, #cost do
			haveCount = DataCenter.packData:getItemCountById(cost[i][1])
			if haveCount < cost[i][2] then
				return false
			end
		end
		return true
	end
	return false
end

function Lua_ArtifactVo.Get:canUpgrade()
	if self.is_activate == 1 and not self.bMaxLv then
		local cost = self.levelConfig.lvlup_cost;
		local haveCount;
		for i = 1, #cost do
			haveCount = DataCenter.packData:getItemCountById(cost[i][1]);
			if haveCount < cost[i][2] then
				return false;
			end
		end
		return true;
	end
	return false;
end

function Lua_ArtifactVo.Get:canUpStar()
	if self.is_activate == 1 and not self.bMaxStar then
		local cost = self.starConfig.star_up_cost
		local haveCount
		for i = 1, #cost do
			haveCount = DataCenter.packData:getItemCountById(cost[i][1])
			if haveCount < cost[i][2] then
				return false;
			end
		end
		return true;
	end
	return false;
end

function Lua_ArtifactVo:getCheckId()
	local cost
	if self.is_activate == 0 then
		cost = self.unlock_cond
	elseif not self.bMaxLv then
		cost = self.levelConfig.lvlup_cost
	end
	if cost then
		local haveCount
		for i = 1, #cost do
			haveCount = DataCenter.packData:getItemCountById(cost[i][1])
			if haveCount < cost[i][2] then
				return cost[i][1], cost[i][2]
			end
		end
	end
	return nil, nil
end

function Lua_ArtifactVo:getUpStarCheckId()
	if self.is_activate == 1 and not self.bMaxStar then
		local cost = self.starConfig.star_up_cost
		local haveCount
		for i = 1, #cost do
			haveCount = DataCenter.packData:getItemCountById(cost[i][1])
			if haveCount < cost[i][2] then
				return cost[i][1], cost[i][2]
			end
		end
	end
	return nil, nil
end

-------------------- 配置数据 --------------------

function Lua_ArtifactVo.Get:name()
	return self.config.name
end

function Lua_ArtifactVo.Get:sort()
	return self.config.sort or 0
end

function Lua_ArtifactVo.Get:skill()
	return self.config.skill
end

function Lua_ArtifactVo.Get:color()
	return self.config.color or 1
end

function Lua_ArtifactVo.Get:unlock_way()
	return self.config.unlock_way
end

function Lua_ArtifactVo.Get:unlock_cond()
	return self.config.unlock_cond
end

function Lua_ArtifactVo.Get:max_level()
	return self.config.max_level or 1
end

function Lua_ArtifactVo.Get:max_star()
	return self.config.max_star or 1
end

function Lua_ArtifactVo.Get:position()
	return self.config.position;
end

function Lua_ArtifactVo.Get:des()
	return self.config.des;
end

function Lua_ArtifactVo.Get:unlock_tips()
	return self.config.unlock_tips;
end

function Lua_ArtifactVo.Get:talent_name()
	return self.config.talent_name;
end

-- 武器图标
-- function Lua_ArtifactVo.Get:weapon_icon()
-- 	return self.config.weapon_icon;
-- end

function Lua_ArtifactVo.Get:talent_icon()
	return self.config.talent_icon;
end

-- 布阵显示图标
-- function Lua_ArtifactVo.Get:icon_1()
-- 	return self.config.icon_1;
-- end

-- 战斗显示图标
function Lua_ArtifactVo.Get:icon_2()
	return self.config.icon_2;
end

-- 布阵神器选择图
-- function Lua_ArtifactVo.Get:image_1()
-- 	return self.config.image_1;
-- end

-- 浣熊全身图
-- function Lua_ArtifactVo.Get:image_2()
-- 	return self.config.image_2;
-- end

-- 浣熊半身图
-- function Lua_ArtifactVo.Get:image_3()
-- 	return self.config.image_3;
-- end

-- jta代替ID
function Lua_ArtifactVo.Get:jid()
	return self.config.jid;
end

function Lua_ArtifactVo.Get:modelUrl()
	if self._modelUrl == nil then
		self._modelUrl = LanUtil.StringProfile("/model/role/", self.modelSrc);
	end
	return self._modelUrl;
end

-- 模型名
function Lua_ArtifactVo.Get:modelSrc()
	return self.config.modelSrc;
end

function Lua_ArtifactVo.Get:uiSceneId()
	return self.config.uiSceneId;
end

function Lua_ArtifactVo.Get:uiModelUrl()
	if self._uiModelUrl == nil then
		self._uiModelUrl = LanUtil.StringProfile("/model/role/", self.uiModelSrc);
	end
	return self._uiModelUrl;
end

function Lua_ArtifactVo.Get:uiModelSrc()
	return self.config.uiModelSrc;
end

function Lua_ArtifactVo.Get:uiModelAvatarName()
	return self.config.uiModelAvatarName;
end

function Lua_ArtifactVo.Get:uiModelCamController()
	return self.config.uiModelCamController;
end

function Lua_ArtifactVo.Get:uiModelScale()
    return self.config.uiModelScale or 1;
end

function Lua_ArtifactVo.Get:showActEffs()
	return self.config.showActEffs;
end

function Lua_ArtifactVo.Get:poseActEffs()
	return self.config.poseActEffs;
end

function Lua_ArtifactVo.Get:uiShowAct()
	if self.config.uiShowAct == nil then
		return Lua_ThingDefine.UIWAIT;
	end
	return self.config.uiShowAct;
end

function Lua_ArtifactVo.Get:uiModelDir()
	return 0;
end

function Lua_ArtifactVo.Get:story_title()
	return self.config.story_title;
end

function Lua_ArtifactVo.Get:story_message()
	return self.config.story_message;
end

function Lua_ArtifactVo.Get:story_content()
	return self.config.story_content;
end