Lua_DressUpVo = class("Lua_DressUpVo")
function Lua_DressUpVo:ctor(id, config)
	self.id = id;
	self._config = config;
end

function Lua_DressUpVo.Get:config()
	if self._config == nil then
		self._config = getConfig("dress", self.id);
	end
	return self._config;
end

function Lua_DressUpVo:fromServer(data)
	if data == nil or data.dress_id ~= self.id then
		return;
	end

	self._is_active = data.is_active; -- 是否激活 0:未激活 1:已激活
	self._is_wear = data.is_wear; -- 是否穿戴 0:未穿戴 1：已穿戴
	self._is_new = data.is_new; -- 是否是新装扮 1：是 0：否
	self._expired_tick = tonumber(data.expired_tick); -- 过期时间 0:永久时效

	if data.is_active == 1 and self.active_item then
		DataCenter.roleData:setDressCheckItem(self.active_item[1], nil);
	end
end

function Lua_DressUpVo.Get:is_active()
	if self._is_active == 1 and self.expired_tick > 0 and self.expired_tick < DataCenter.roleData.serverTime then
		return 0
	end
	return self._is_active or 0
end

function Lua_DressUpVo.Get:is_wear()
	return self._is_wear or 0
end

function Lua_DressUpVo.Get:is_new()
	return self._is_new or 0
end

function Lua_DressUpVo.Get:expired_tick()
	return self._expired_tick or 0
end

function Lua_DressUpVo.Get:sortWeights()
	local weights = 90000000000;
	if self.is_wear == 1 then
		weights = 10000000000;
	elseif self.canActivate == 1 then
		weights = 30000000000;
	elseif self.is_active == 1 then
		weights = 60000000000;
	end
	return weights + self.sort;
end

function Lua_DressUpVo.Get:canActivate()
	if self.is_active == 0 then
		if self.active_item ~= nil then
			local vo = DataCenter.packData:getItemVoById(self.active_item[1]);
			if vo ~= nil and vo.isExpired == false and vo.count >= self.active_item[2] then
				return 1;
			end
		end
	end
	return 0;
end

function Lua_DressUpVo.Get:attrStr()
	if self._attrStr == nil then
		self._attrStr = DataCenter.attributesUtil:getAttrStr(self.prop, "  ");
	end
	return self._attrStr;
end

-------------------- 配置数据 --------------------

function Lua_DressUpVo.Get:name()
    return self.config.name;
end

function Lua_DressUpVo.Get:main_type()
    if self.config.main_type == nil then
        return 1;
    end
    return self.config.main_type;
end

function Lua_DressUpVo.Get:sub_type()
    if self.config.sub_type == nil then
        return 1;
    end
    return self.config.sub_type;
end

function Lua_DressUpVo.Get:sex()
    if self.config.sex == nil then
        return 0;
    end
    return self.config.sex;
end

function Lua_DressUpVo.Get:default()
    return self.config.default;
end

function Lua_DressUpVo.Get:active_hero_id()
    return self.config.active_hero_id or 0;
end

function Lua_DressUpVo.Get:raccoon_id()
    return self.config.raccoon_id or 0;
end

function Lua_DressUpVo.Get:active_item()
    return self.config.active_item;
end

function Lua_DressUpVo.Get:prop()
    return self.config.prop;
end

function Lua_DressUpVo.Get:atlas()
    return self.config.atlas;
end

function Lua_DressUpVo.Get:sprite()
    local sprite = self.config.sprite;
    if strIsNullOrEmpty(sprite) then
        sprite = self.id;
    end
    return tostring(sprite);
end

function Lua_DressUpVo.Get:floder()
	return self.config.floder or "/model/";
end

function Lua_DressUpVo.Get:jid()
	return self.config.jid;
end

function Lua_DressUpVo.Get:modelUrl()
	if self.active_hero_id > 0 then
		local config = Lua_HeroConfigData.getConfig(self.active_hero_id);
		return config.modelUrl;
	end
	if self.raccoon_id > 0 then
		local vo = DataCenter.artifactUtil:getArtifactVoById(self.raccoon_id)
		return vo.modelUrl;
	end
	if self._modelUrl == nil then
		self._modelUrl = LanUtil.StringProfile(self.floder, self.jid);
	end
	return self._modelUrl;
end

function Lua_DressUpVo.Get:modelSrc()
	if self.active_hero_id > 0 then
		local config = Lua_HeroConfigData.getConfig(self.active_hero_id);
		return config.modelSrc;
	end
	if self.raccoon_id > 0 then
		local vo = DataCenter.artifactUtil:getArtifactVoById(self.raccoon_id)
		return vo.modelSrc;
	end
	return self.config.modelSrc;
end

function Lua_DressUpVo.Get:uiSceneId()
	if self.active_hero_id > 0 then
		local config = Lua_HeroConfigData.getConfig(self.active_hero_id);
		return config.uiSceneId;
	end
	if self.raccoon_id > 0 then
		local vo = DataCenter.artifactUtil:getArtifactVoById(self.raccoon_id)
		return vo.uiSceneId;
	end
    return self.config.uiSceneId;
end

function Lua_DressUpVo.Get:uiModelUrl()
	if self.active_hero_id > 0 then
		local config = Lua_HeroConfigData.getConfig(self.active_hero_id);
		return config.uiModelUrl;
	end
	if self.raccoon_id > 0 then
		local vo = DataCenter.artifactUtil:getArtifactVoById(self.raccoon_id)
		return vo.uiModelUrl;
	end
	if self._uiModelUrl == nil then
		self._uiModelUrl = LanUtil.StringProfile(self.floder, self.uiModelSrc);
	end
	return self._uiModelUrl;
end

function Lua_DressUpVo.Get:uiModelSrc()
	if self.active_hero_id > 0 then
		local config = Lua_HeroConfigData.getConfig(self.active_hero_id);
		return config.uiModelSrc;
	end
	if self.raccoon_id > 0 then
		local vo = DataCenter.artifactUtil:getArtifactVoById(self.raccoon_id)
		return vo.uiModelSrc;
	end
	return self.config.uiModelSrc;
end

function Lua_DressUpVo.Get:uiModelAvatarName()
	if self.active_hero_id > 0 then
		local config = Lua_HeroConfigData.getConfig(self.active_hero_id);
		return config.uiModelAvatarName;
	end
	if self.raccoon_id > 0 then
		local vo = DataCenter.artifactUtil:getArtifactVoById(self.raccoon_id)
		return vo.uiModelAvatarName;
	end
    return self.config.uiModelAvatarName;
end

function Lua_DressUpVo.Get:uiModelCamController()
	if self.active_hero_id > 0 then
		local config = Lua_HeroConfigData.getConfig(self.active_hero_id);
		return config.uiModelCamController;
	end
	if self.raccoon_id > 0 then
		local vo = DataCenter.artifactUtil:getArtifactVoById(self.raccoon_id)
		return vo.uiModelCamController;
	end
	return self.config.uiModelCamController;
end

function Lua_DressUpVo.Get:uiModelScale()
	if self.active_hero_id > 0 then
		local config = Lua_HeroConfigData.getConfig(self.active_hero_id);
		return config.uiModelScale;
	end
	if self.raccoon_id > 0 then
		local vo = DataCenter.artifactUtil:getArtifactVoById(self.raccoon_id)
		return vo.uiModelScale;
	end
	return self.config.uiModelScale;
end

function Lua_DressUpVo.Get:sort()
    return self.config.sort or self.id;
end

function Lua_DressUpVo.Get:halfPic()
	return self.config.half_pic;
end

function Lua_DressUpVo.Get:halfPicUrl()
	if self._halfPicUrl == nil then
		self._halfPicUrl = LanUtil.StringProfile("/images/role/", self.halfPic, ".png");
	end
	return self._halfPicUrl;
end

function Lua_DressUpVo.Get:acti_des()
    return self.config.acti_des;
end

function Lua_DressUpVo.Get:des()
    return self.config.des;
end