-- 场景装饰物生成器
Lua_SceneOrnamentGenerater = class("Lua_SceneOrnamentGenerater");
function Lua_SceneOrnamentGenerater:ctor(sId, gid)
	self:superCall("ctor");

	self:setGenerateInfo(sId, gid);
	self._generateMatsTab = {}; -- 生成物料
	self._generateMatsArr = Array(); -- 生成物料id集合

	self._generateThingsIds = Array(); -- 生成的roleId集合
	self._generateThingsIdsTab = {}; -- key为 generateid, value 为 rid集合
	self._generateRIdToGId = {}; -- key rid, value generateid
	self._behaviorVosTab = {};
end

function Lua_SceneOrnamentGenerater:dispose()
	self:clean();
	self._generateMatsTab = nil;
	self._generateMatsArr = nil;
	self._generateThingsIds = nil;
	self._generateThingsIdsTab = nil;
	self._generateRIdToGId = nil;
	self._behaviorVosTab = nil;
end

function Lua_SceneOrnamentGenerater:clean()
	self._generateThingsIds:clear();
	local len = self._generateMatsArr:getSize();
	while (len > 0) do
		local id = self._generateMatsArr[len];
		self._generateMatsArr:remove(len);
		DataCenter.sceneOrnamentsManager:returnGeneraterVo(self._generateMatsTab[id]);
		self._generateMatsTab[id] = nil;
		local arr = self._generateThingsIdsTab[id];
		if arr ~= nil then
			local count = arr:getSize();
			while (count > 0) do
				local rid = arr[count];
				arr:remove(count);
				local behaviorVo = self._behaviorVosTab[rid];
				DataCenter.sceneOrnamentsManager:returnBehaviorVo(behaviorVo);
				self._behaviorVosTab[rid] = nil;
				self._generateRIdToGId[rid] = nil;
				ThingUtil.instance:removeThing(self.sceneId, 0, rid);
				count = count - 1;
			end
		end
		len = len - 1;
	end

	self.sceneId = 0;
	self.groupId = -1;
end

function Lua_SceneOrnamentGenerater:setGenerateInfo(sId, gid)
	self.sceneId = sId; -- 场景id
	self.groupId = gid; -- 行为类型组ID
end

function Lua_SceneOrnamentGenerater:hasGenerateMat(id)
	return self._generateMatsTab[id] ~= nil;
end

function Lua_SceneOrnamentGenerater:addGenerateMat(vo)
	if vo ~= nil and vo.behaviorGroup == self.groupId then
		local id = vo.id;
		if self._generateMatsTab[id] == nil then
			self._generateMatsArr:add(id);
			self._generateMatsTab[id] = vo;
		end
	end
end

function Lua_SceneOrnamentGenerater:removeGenerateMat(id)
	if self._generateMatsTab[id] ~= nil then
		DataCenter.sceneOrnamentsManager:returnGeneraterVo(self._generateMatsTab[id]);
		self._generateMatsArr:removeItem(id);
		self._generateMatsTab[id] = nil;
		self:removeThingByGId(id);-- 移除生成的对象
	end
end

function Lua_SceneOrnamentGenerater:removeMatBySceneHide(sceneId)
	local len = self._generateMatsArr:getSize();
	while (len > 0) do
		local id = self._generateMatsArr[len];
		local cfg = self._generateMatsTab[id].config;
		if cfg == nil then
			self._generateMatsArr:remove(len);
			self._generateMatsTab[id] = nil;
		elseif cfg.sceneId == sceneId and (cfg.showBackground == nil or cfg.showBackground ~= 1) then
			DataCenter.sceneOrnamentsManager:returnGeneraterVo(self._generateMatsTab[id]);
			self._generateMatsArr:remove(len);
			self._generateMatsTab[id] = nil;
			self:removeThingByGId(id);-- 移除生成的对象
		end
		len = len - 1;
	end
	return self._generateMatsArr:getSize() > 0;
end

-- 根据generate id移除生成的对象
function Lua_SceneOrnamentGenerater:removeThingByGId(gid)
	local arr = self._generateThingsIdsTab[gid];
	if arr ~= nil then
		local count = arr:getSize();
		while (count > 0) do
			local rid = arr[count];
			arr:remove(count);
			local behaviorVo = self._behaviorVosTab[rid];
			self._behaviorVosTab[rid] = nil;
			DataCenter.sceneOrnamentsManager:returnBehaviorVo(behaviorVo);
			self._generateRIdToGId[rid] = nil;
			self._generateThingsIds:removeItem(rid);
			ThingUtil.instance:removeThing(self.sceneId, 0, rid);
			count = count - 1;
		end
	end
end

-- 根据rid移除生成的对象
function Lua_SceneOrnamentGenerater:removeThing(rid)
	if self._generateRIdToGId[rid] ~= nil then
		local gid = self._generateRIdToGId[rid];
		local behaviorVo = self._behaviorVosTab[rid];
		self._behaviorVosTab[rid] = nil;
		DataCenter.sceneOrnamentsManager:returnBehaviorVo(behaviorVo);
		self._generateRIdToGId[rid] = nil;
		self._generateThingsIds:removeItem(rid);
		self._generateThingsIdsTab[gid]:removeItem(rid);
		ThingUtil.instance:removeThing(self.sceneId, 0, rid);
		return true;
	elseif self._generateThingsIds:removeItem(rid) then
		local behaviorVo = self._behaviorVosTab[rid];
		self._behaviorVosTab[rid] = nil;
		DataCenter.sceneOrnamentsManager:returnBehaviorVo(behaviorVo);
		return true;
	end
	return false;
end

-- 根据rid移除数据
function Lua_SceneOrnamentGenerater:removeThingInfo(rid)
	if self._generateRIdToGId[rid] ~= nil then
		local gid = self._generateRIdToGId[rid];
		local behaviorVo = self._behaviorVosTab[rid];
		self._behaviorVosTab[rid] = nil;
		DataCenter.sceneOrnamentsManager:returnBehaviorVo(behaviorVo);
		self._generateRIdToGId[rid] = nil;
		self._generateThingsIds:removeItem(rid);
		self._generateThingsIdsTab[gid]:removeItem(rid);
		return true;
	elseif self._generateThingsIds:removeItem(rid) then
		local behaviorVo = self._behaviorVosTab[rid];
		self._behaviorVosTab[rid] = nil;
		DataCenter.sceneOrnamentsManager:returnBehaviorVo(behaviorVo);
		return true;
	end
	return false;
end

-- 思考：
-- 执行时机、完成回调（分场景对象、thing对象）
function Lua_SceneOrnamentGenerater:frameScript(dt)
	-- 物料（生成冷却）
	local canBorn = true;
	for i = 1, self._generateMatsArr:getSize() do
		local id = self._generateMatsArr[i];
		local vo = self._generateMatsTab[id];
		if vo ~= nil then
			vo:frameScript(dt);
			if not vo.canBorn then
				canBorn = false;
			end
		end
	end

	-- 行为（行为冷却）
	local len = self._generateThingsIds:getSize();
	while (len > 0) do
		local rid = self._generateThingsIds[len];
		local behaviorVo = self._behaviorVosTab[rid];
		if behaviorVo ~= nil then
			behaviorVo:frameScript(dt);
			if behaviorVo.isComplete then -- 完成行为回收
				-- print("完成行为回收", rid, behaviorVo.behaviorId);
				self:removeThing(rid);
			elseif (behaviorVo.behaviorAsBorn and behaviorVo.canOperateBehavior and behaviorVo.bOperating) or behaviorVo.bCooling then
				canBorn = false;
			end
		end
		len = len - 1;
	end

	if canBorn then
		self:generate() -- 生成
	end
end

-- 生成
function Lua_SceneOrnamentGenerater:generate()
	local idx = self._generateMatsArr:getSize();
	if idx > 1 then idx = Mathf.Random(1, idx); end
	local id = self._generateMatsArr[idx];
	local generatevo = self._generateMatsTab[id];
	if generatevo == nil then
		return;
	end

	if not generatevo.canBorn then 
		-- print("generate canBorn", generatevo.canBorn);
		return;
	end

	local cfg = generatevo.config;
	if cfg == nil then 
		-- print("generate", cfg == nil);
		return; 
	end

	local curNum = (self._generateThingsIdsTab[id] ~= nil) and self._generateThingsIdsTab[id]:getSize() or 0;
	if cfg.bornMaxNum ~= nil and curNum >= cfg.bornMaxNum then -- 超限
		-- print("generate 超限");
		return;
	end

	local modelName = cfg.modelName;
	if strIsNullOrEmpty(modelName) then
		-- print("generate modelName == nil");
		return;
	end

	local rate = cfg.bornRate;
	if rate ~= nil and Mathf.Random(0, 100) > rate then
		-- print("generate 概率不足");
		generatevo:coolBorn();
		return;
	end
	

	local roleId;
	local sceneId = cfg.sceneId;
	local resUrl = cfg.resUrl;
	if not strIsNullOrEmpty(resUrl) then -- 非场景对象
		local pos = cfg.pos;
		local randomPosRange = cfg.randomPosRange;
		local ox = (randomPosRange ~= nil and #randomPosRange > 0) and Mathf.Random(-randomPosRange[1], randomPosRange[1]) or 0;
		local oz = (randomPosRange ~= nil and #randomPosRange > 1) and Mathf.Random(-randomPosRange[2], randomPosRange[2]) or 0;
		local px = (pos ~= nil and #pos > 0) and (pos[1] + ox) or ox;
		local py = (pos ~= nil and #pos > 1) and pos[2] or 0;
		local pz = (pos ~= nil and #pos > 2) and (pos[3] + oz) or oz;
		if self._generateThingsIdsTab[id] == nil then self._generateThingsIdsTab[id] = Array(); end
		roleId = id * 100 + curNum;-- 分配rid
		self._generateThingsIds:add(roleId);
		self._generateThingsIdsTab[id]:add(roleId);
		self._generateRIdToGId[roleId] = id;
		local thingData = self:generateThingData(sceneId, roleId, resUrl, modelName, px, py, pz);
		if thingData ~= nil then ThingUtil.instance:addAnimal(thingData, true); end
	else -- 场景对象
		roleId = id;
		if self._generateThingsIds:indexOf(id) == -1 then
			self._generateThingsIds:add(id);
		end
	end

	generatevo:born();
	local behaviorVo = self._behaviorVosTab[roleId];
	local sceneModelName = self._generateRIdToGId[roleId] == nil and modelName or nil;
	if behaviorVo == nil then
		behaviorVo = DataCenter.sceneOrnamentsManager:createBehaviorVo(generatevo.behaviorId, generatevo.behaviorIntervalTime, generatevo.behaviorTime, generatevo.behaviorRate);
		behaviorVo:setOrnament(sceneId, roleId, sceneModelName, generatevo.behaviorAsBorn);
		self._behaviorVosTab[roleId] = behaviorVo;
	else
		behaviorVo:setBehaviorId(generatevo.behaviorId, generatevo.behaviorIntervalTime, generatevo.behaviorTime, generatevo.behaviorRate);
		behaviorVo:setOrnament(sceneId, roleId, sceneModelName, generatevo.behaviorAsBorn);
	end

	-- print("generate ", roleId, behaviorVo.behaviorId, generatevo.behaviorId, generatevo.behaviorRate)
	if behaviorVo:randomOperateBehavior() then
		behaviorVo:operateBehavior();
	end
	return;
end

function Lua_SceneOrnamentGenerater:generateThingData(sceneId, roleId, resUrl, modelName, px, py, pz)
    local data = SceneOrnamentData();
    data.kind = Lua_ThingDefine.SCENEN_ORNAMENT;
    data.roleId = roleId;
    data.sceneId = sceneId;
    data.resUrl = resUrl;
    data.modelName = modelName;
	data.isShowName = false;
	data.isShowHpBar = false;
	data.isShowBuffIcon = false;
    data.hp = 1;
    data.maxHp = 1;
    data.px = px;
    data.py = py;
    data.pz = pz;
    data:refreshShowPos();
    data.showShadow = false;
    data.bPlanarShadow = false;
    data.toLand = false;
    return data;
end

-- 场景对象完成动作
function Lua_SceneOrnamentGenerater:completeAction(sign)
	local rid = tonumber(tostring(sign));
	if self._behaviorVosTab[rid] ~= nil then -- 
		self._behaviorVosTab[rid]:completeAction();
		return true;
	end
	return false;
end

-- 对象完成移动
function Lua_SceneOrnamentGenerater:completeMove(rid)
	if self._behaviorVosTab[rid] ~= nil then -- 
		self._behaviorVosTab[rid]:completeMove();
		return true;
	end
	return false;
end

-- 完成行为
function Lua_SceneOrnamentGenerater:completeBehavior(rid)
	local behaviorVo = self._behaviorVosTab[rid];
	if behaviorVo ~= nil then
		local behaviorId = behaviorVo.behaviorId;
		behaviorVo.behaviorId = behaviorId + 1;
		behaviorVo:operateBehavior();
		return true;
	end
	return false;
end

-- 执行行为点击
function Lua_SceneOrnamentGenerater:operateBehaviorClick(rid)
	local behaviorVo = self._behaviorVosTab[rid];
	if behaviorVo ~= nil then
		behaviorVo:operateClickHandle();
		return true;
	end
	return false;
end

-- 隐藏切换回显示，动作重置到指定时间点
function Lua_SceneOrnamentGenerater:resetBackGroundGenerater()
	local len = self._generateThingsIds:getSize();
	for i = 1, len do
		local rid = self._generateThingsIds[i];
		local gid = self._generateRIdToGId[rid] ~= nil and self._generateRIdToGId[rid] or rid;
		local mat = self._generateMatsTab[gid];
		if mat ~= nil then
			local cfg = mat.config;
			if cfg ~= nil and cfg.showBackground ~= nil and cfg.showBackground == 1 then
				local behaviorVo = self._behaviorVosTab[rid];
				if behaviorVo ~= nil then
					behaviorVo:resetAction();
					behaviorVo:resetBubble();
				end
			end
		end
	end
end

-- 点击检测
function Lua_SceneOrnamentGenerater:ornamentClick(clickStr)
	local len = self._generateThingsIds:getSize();
	for i = 1, len do
		local rid = self._generateThingsIds[i];
		local gid = self._generateRIdToGId[rid];
		if gid ~= nil then -- 场景thing
			
		else -- 场景自带obj
			local mat = self._generateMatsTab[rid];
			if mat ~= nil then
				local config = DataCenter.sceneUtil:getOrnamentBubbleById(clickStr);
				if config ~= nil and config.ornamentId == rid then
					local behaviorVo = self._behaviorVosTab[rid];
					if behaviorVo ~= nil then
						behaviorVo:showQipao(config);
					end
					return true;
				end
			end
		end
	end
	return false;
end

function Lua_SceneOrnamentGenerater:getBehaviorVoById(id)
	if self._behaviorVosTab == nil then
		return nil;
	end
	return self._behaviorVosTab[id];
end