-- 场景装饰物管理类
checkRequire("Lua_SceneOrnamentGenerater");
checkRequire("Lua_SceneOrnamentGenerateVo");
checkRequire("Lua_SceneOrnamentBehaviorVo");
Lua_SceneOrnamentsManager = class("Lua_SceneOrnamentsManager");
function Lua_SceneOrnamentsManager:ctor()
	self:superCall("ctor");

	self._maxIdleGenerates = 10;
	self._idleGeneraters = Array();
	self._maxIdleGenerateVos = 20;
	self._idleGeneraterVos = Array();
	self._maxIdleBehaviorVos = 40;
	self._idleBehaviorVos = Array();

	self._generareScenesArr = Array();
	self._sceneOrnamentsState = {};
	self._sceneOrnamentGroupIdsTab = {};
	self._sceneOrnamentGeneratersTab = {};
end

function Lua_SceneOrnamentsManager:dispose()
	self:clearTimer();
	for k, v in pairs(self._sceneOrnamentsState) do
        self._sceneOrnamentsState[k] = 0;
    end

    local len = self._generareScenesArr:getSize();
    while (len > 0) do
    	local sceneId = self._generareScenesArr[len];
    	self._generareScenesArr:remove(len);
    	local gidsArr = self._sceneOrnamentGroupIdsTab[sceneId];
    	if gidsArr ~= nil then
    		local gidsArrLen = gidsArr:getSize();
    		while (gidsArrLen > 0) do
    			local groupId = gidsArr[gidsArrLen];
    			gidsArr:remove(gidsArrLen);
    			local generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
    			self._sceneOrnamentGeneratersTab[sceneId][groupId] = nil;
    			self:returnGenerater(generater);
    			gidsArrLen = gidsArrLen - 1;
    		end
    	end
    	len = len - 1;
    end

    self.ornamentBubbleIdx = nil;
end

function Lua_SceneOrnamentsManager:returnGenerater(generater)
	if generater == nil then return; end
	if self._idleGeneraters:getSize() < self._maxIdleGenerates then
		generater:clean();
		self._idleGeneraters:add(generater);
	else
		generater:dispose();
	end
end

function Lua_SceneOrnamentsManager:createGenerater(sceneId, groupId)
	local generater = nil;
	if self._idleGeneraters:getSize() > 0 then
		generater = self._idleGeneraters:remove();
		generater:setGenerateInfo(sceneId, groupId);
	else
		generater = Lua_SceneOrnamentGenerater.new(sceneId, groupId);
	end
	return generater;
end

function Lua_SceneOrnamentsManager:returnGeneraterVo(vo)
	if vo == nil then return; end
	if self._idleGeneraterVos:getSize() < self._maxIdleGenerateVos then
		vo:clean();
		self._idleGeneraterVos:add(vo);
	else
		vo:dispose();
	end
end

function Lua_SceneOrnamentsManager:createGeneraterVo(generaterId)
	local vo = nil;
	if self._idleGeneraterVos:getSize() > 0 then
		vo = self._idleGeneraterVos:remove();
		vo:setGenerateId(generaterId);
	else
		vo = Lua_SceneOrnamentGenerateVo.new(generaterId);
	end
	return vo;
end

function Lua_SceneOrnamentsManager:returnBehaviorVo(vo)
	if vo == nil then return; end
	if self._idleBehaviorVos:getSize() < self._maxIdleBehaviorVos then
		vo:clean();
		self._idleBehaviorVos:add(vo);
	else
		vo:dispose();
	end
end

function Lua_SceneOrnamentsManager:createBehaviorVo(behaviorId, behaviorIntervalTime, behaviorTime, behaviorRate)
	local vo = nil;
	if self._idleBehaviorVos:getSize() > 0 then
		vo = self._idleBehaviorVos:remove();
		vo:setBehaviorId(behaviorId, behaviorIntervalTime, behaviorTime, behaviorRate);
	else
		vo = Lua_SceneOrnamentBehaviorVo.new(behaviorId, behaviorIntervalTime, behaviorTime, behaviorRate);
	end
	return vo;
end

-- 场景对象移除的时候，检测生成器里面是否包含
function Lua_SceneOrnamentsManager:removeThing(sceneId, rid)
	if sceneId == nil then return; end
	local gidsArr = self._sceneOrnamentGroupIdsTab[sceneId];
	if gidsArr == nil then return; end
	local len = gidsArr:getSize();
	if len <= 0 then return; end
	while (len > 0) do
		local groupId = gidsArr[len];
		local generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
		if generater:removeThingInfo(rid) then
			break;
		end
		len = len - 1;
	end
end

function Lua_SceneOrnamentsManager:hideScene(sceneId)
	local val = self._sceneOrnamentsState[sceneId];
	if val == nil or val == 0 then return; end
	self._sceneOrnamentsState[sceneId] = 0;
	local gidsArr = self._sceneOrnamentGroupIdsTab[sceneId];
	if gidsArr == nil then return; end
	local len = gidsArr:getSize();
	if len <= 0 then return; end
	while (len > 0) do
		local groupId = gidsArr[len];
		local generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
		if not generater:removeMatBySceneHide(sceneId) then
			gidsArr:remove(len);
			self._sceneOrnamentGeneratersTab[sceneId][groupId] = nil;
			self:returnGenerater(generater);
		end
		len = len - 1;
	end

	if gidsArr:getSize() <= 0 then
		self._generareScenesArr:removeItem(sceneId);
		if self._generareScenesArr:getSize() <= 0 then
			self:clearTimer();
		end
	end
end

function Lua_SceneOrnamentsManager:showScene(sceneId)
	local val = self._sceneOrnamentsState[sceneId];
	if val ~= 1 then
		self:resetBackGroundGeneraters(sceneId);
		self:initSceneOrnaments(sceneId);
	end

	for i = 1, self._generareScenesArr:getSize() do
		local sid = self._generareScenesArr[i];
		if sid ~= sceneId then
			self:hideScene(sid);
		end
	end
end

function Lua_SceneOrnamentsManager:reserOrnamentsAct(sceneId)
	local val = self._sceneOrnamentsState[sceneId];
	if val == 1 then -- 显示中被隐藏才需重置动作
		-- print("显示中被隐藏才需重置动作");
		self:resetBackGroundGeneraters(sceneId);
	end
end

function Lua_SceneOrnamentsManager:resetBackGroundGeneraters(sceneId)
	local gidsArr = self._sceneOrnamentGroupIdsTab[sceneId];
	if gidsArr == nil then return; end
	local len = gidsArr:getSize();
	if len <= 0 then return; end
	while (len > 0) do
		local groupId = gidsArr[len];
		local generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
		generater:resetBackGroundGenerater();
		len = len - 1;
	end
end

function Lua_SceneOrnamentsManager:initSceneOrnaments(sceneId)
	local arr = DataCenter.sceneUtil:getOrnamentsBySceneId(sceneId);
	if arr == nil then return; end
	local len = arr:getSize();
	if len <= 0 then return; end
	self._sceneOrnamentsState[sceneId] = 1;
	local gidsArr = self._sceneOrnamentGroupIdsTab[sceneId];
	if gidsArr == nil then
		gidsArr = Array();
		self._sceneOrnamentGroupIdsTab[sceneId] = gidsArr;
	end

	if self._generareScenesArr:indexOf(sceneId) == -1 then
		self._generareScenesArr:add(sceneId);
	end

	for i = 1, len do
		local cfg = arr[i];
		if cfg.isShield == nil or cfg.isShield ~= 1 then
			local gid = cfg.id;
			local groupId = cfg.behaviorGroup;
			local sceneId = cfg.sceneId;
			local generater = nil;
			if gidsArr:indexOf(groupId) == -1 then
				gidsArr:add(groupId);
				-- print("initSceneOrnaments createGenerater", sceneId, groupId);
				generater = self:createGenerater(sceneId, groupId);
				if self._sceneOrnamentGeneratersTab[sceneId] == nil then
					self._sceneOrnamentGeneratersTab[sceneId] = {};
				end
				self._sceneOrnamentGeneratersTab[sceneId][groupId] = generater;
			else
				generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
			end

			if not generater:hasGenerateMat(gid) then
				local vo = self:createGeneraterVo(gid);
				generater:addGenerateMat(vo);
			end
		end
	end

	if self._generareScenesArr:getSize() > 0 then
		self:addTimer();
	end
end

function Lua_SceneOrnamentsManager:clearTimer()
	if self._bAddTimer then
		self._startTime = 0;
		self._bAddTimer = false;
		Lua_CommonUtil.removeTimer(self, self.frameScript);
	end
end

function Lua_SceneOrnamentsManager:addTimer()
	if not self._bAddTimer then
		self._bAddTimer = true;
		self._startTime = getTime();
		Lua_CommonUtil.addTimer("SceneOrnamentFrameScript", 1, self, self.frameScript);
	end
end

function Lua_SceneOrnamentsManager:frameScript()
	local t = getTime();
	local dt = t - self._startTime;
	self._startTime = t;
	local len = self._generareScenesArr:getSize()
	while (len > 0) do
		local sceneId = self._generareScenesArr[len];
		local arr = self._sceneOrnamentGroupIdsTab[sceneId];
		if arr ~= nil then
			local arrLen = arr:getSize();
			while (arrLen > 0) do
				local groupId = arr[arrLen];
				local generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
				if generater ~= nil then
					generater:frameScript(dt);
				end
				arrLen = arrLen - 1;
			end
		end
		len = len - 1;
	end
end

-- 场景对象动作完成
function Lua_SceneOrnamentsManager:sceneObjCompleteAction(sign)
	local len = self._generareScenesArr:getSize()
	while (len > 0) do
		local sceneId = self._generareScenesArr[len];
		local arr = self._sceneOrnamentGroupIdsTab[sceneId];
		if arr ~= nil then
			local arrLen = arr:getSize();
			while (arrLen > 0) do
				local groupId = arr[arrLen];
				local generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
				if generater ~= nil and generater:completeAction(sign) then
					return;
				end
				arrLen = arrLen - 1;
			end
		end
		len = len - 1;
	end
end

-- thing对象移动完成
function Lua_SceneOrnamentsManager:thingMoveComplete(rid)
	rid = tonumber(tostring(rid));
	local len = self._generareScenesArr:getSize()
	while (len > 0) do
		local sceneId = self._generareScenesArr[len];
		local arr = self._sceneOrnamentGroupIdsTab[sceneId];
		if arr ~= nil then
			local arrLen = arr:getSize();
			while (arrLen > 0) do
				local groupId = arr[arrLen];
				local generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
				if generater ~= nil and generater:completeMove(rid) then
					return;
				end
				arrLen = arrLen - 1;
			end
		end
		len = len - 1;
	end
end

-- 装饰物点击
function Lua_SceneOrnamentsManager:ornamentClick(clickStr)
	local sceneId = Scene3DManager.instance.curShowSceneId;
	local val = self._sceneOrnamentsState[sceneId];
	if val ~= 1 then return; end
	local arr = self._sceneOrnamentGroupIdsTab[sceneId];
	if arr ~= nil then
		local arrLen = arr:getSize();
		while (arrLen > 0) do
			local groupId = arr[arrLen];
			local generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
			if generater ~= nil and generater:ornamentClick(clickStr) then
				return;
			end
			arrLen = arrLen - 1;
		end
	end
end

function Lua_SceneOrnamentsManager:getBehaviorVoById(id)
	local sceneId = Scene3DManager.instance.curShowSceneId;
	local val = self._sceneOrnamentsState[sceneId];
	if val ~= 1 then return; end
	local arr = self._sceneOrnamentGroupIdsTab[sceneId];
	if arr ~= nil then
		local arrLen = arr:getSize();
		while (arrLen > 0) do
			local groupId = arr[arrLen];
			local generater = self._sceneOrnamentGeneratersTab[sceneId][groupId];
			local behaviorVo = generater:getBehaviorVoById(id);
			if behaviorVo ~= nil then
				return behaviorVo;
			end
			arrLen = arrLen - 1;
		end
	end
	return nil;
end