Lua_DialogueBubbleItem = class("Lua_DialogueBubbleItem", Lua_DisplayObject);
function Lua_DialogueBubbleItem:ctor(p, type)
	local skin;
	if type == 1 then
		skin = "LeftDialogueBubbleItem_skin";
	else
		skin = "RightDialogueBubbleItem_skin";
	end
	self:superCall("ctor", p, skin);
end

function Lua_DialogueBubbleItem:buildShowItem()
	self:superCall("buildShowItem");
end

function Lua_DialogueBubbleItem:dispose()
	self:stopTimer();
	self:playBubbleTween(false, true);
	self:superCall("dispose");
end

function Lua_DialogueBubbleItem:setData(data)
	self.data = data;
	
	self.playSpeed = 1;
	self:showContent();
	if self.bubbleContentId == nil then
		return;
	end
	self:starTimer();
end

function Lua_DialogueBubbleItem:showContent()
	local config = getConfig("shop_bubble", self.data);
	local anchor = config.anchor or 1;
	local posX = config.pos ~= nil and #config.pos > 0 and config.pos[1] or 145;
	local posY = config.pos ~= nil and #config.pos > 1 and config.pos[2] or -64;
	local width = config.size ~= nil and #config.size > 0 and config.size[1] or 289;
	local height = config.size ~= nil and #config.size > 1 and config.size[2] or 121;
	local pivot = config.type == 1 and 8 or 6;
	self:setRectPivot(pivot);
	self:setRectAnchor(anchor);
	self:setLocation(posX, posY);
	self:setRectSize(width, height);
	self.contentBg:SetDimensions(width, height);
	self.contentTxt:SetDimensions(width - 33, height - 46);

	self.existTime = config.show_time or 5;
	self.deltaTime = config.delta_time or 5;
	self.startTweenId = config.start_tween_id;
	self.endTweenId = config.end_tween_id;

	self:resetBubble();
end

function Lua_DialogueBubbleItem:resetBubble()
	self.bubbleContentId = self:getBubbleConId();

	if self.bubbleContentId == nil then
		return;
	end
	self:setContentData();
	self:playBubbleTween(true);
end

function Lua_DialogueBubbleItem:getBubbleConId()
	local id;
	local idList = getConfig("shop_bubble", self.data, "content_ids");
	if idList == nil then
		return;
	end

	if idList ~= nil and #idList < 2 then
		return idList[1];
	end

	while (true) do
		id = getRandomRange(#idList);
		id = idList[id];

		if id ~= self.bubbleContentId then
			break;
		end
	end

	return id;
end

function Lua_DialogueBubbleItem:setContentData()
	local config = getConfig("shop_bubble_content", self.bubbleContentId);
	self.delayedTime = config.delay_time or 0;
	self.contentPlayTime = config.play_time or 0;

	local atlas, sprite = "common_altas_4", "xui_xmb_widget_13";
	if config.content_atlas ~= nil and config.content_sprite ~= nil then
		atlas, sprite = config.content_atlas, config.content_sprite;
	end
	self.contentBg:setAltas(atlas, sprite);

	self.contentTxt.text = config.content;
	self.contentLength = self.contentTxt.showTextLen;
	if self.contentPlayTime <= 0 then
		self.contentTxt:setTextShowIndex(-1);
	else
		self.contentTxt:setTextShowIndex(0);
	end
	self.curFrameTime = getTime();
	self.curType = 1; -- 1:显示气泡  2：隐藏气泡  
	self.playTime = 0;
end

function Lua_DialogueBubbleItem:starTimer()
	if self.bStartTime ~= true then
		self.bStartTime = true;
		self:frameScript();
		Lua_CommonUtil.addTimer("BUBBLE_FRAMESCRIPT", 0.01, self, self.frameScript);
	end
end

function Lua_DialogueBubbleItem:stopTimer()
	if self.bStartTime == true then
		Lua_CommonUtil.removeTimer(self, self.frameScript);
	end
end

function Lua_DialogueBubbleItem:frameScript()
	local t = getTime();
    local dt = (t - self.curFrameTime); -- 等比加速
	local scaleDelayTime = dt * self.playSpeed;
    self.curFrameTime = t;

    self.playTime = self.playTime + scaleDelayTime;
    if self.playTime >= self.existTime and self.curType == 1 then
    	self:playBubbleTween(false);
    	self.curType = 2;
    end

    if self.playTime >= self.existTime + self.deltaTime and self.curType == 2 then
    	self:resetBubble();
    end

    if self.curType == 2 then
    	return;
    end

	if self.playTime <= self.delayedTime then
		return;
	end

	local time = self.playTime - self.delayedTime;
	if time < self.contentPlayTime then
		local rate = time / self.contentPlayTime;
		self.contentTxt:setTextShowIndex(Mathf.Floor(rate * self.contentLength));
	else
		self.contentTxt:setTextShowIndex(self.contentLength);
	end
end

function Lua_DialogueBubbleItem:playBubbleTween(bStart, bDispose)
	if self.tweenTf ~= nil then
		TweenUtil.getInstance():stop(self.tweenTf);
	end

	if bDispose then
		return;
	end

	if bStart and self.startTweenId ~= nil then
		self.tweenTf = DataCenter.winTweenManager:playWinTween(self.startTweenId, self.content.transform);
	elseif bStart == false and self.endTweenId ~= nil then
		self.tweenTf = DataCenter.winTweenManager:playWinTween(self.endTweenId, self.content.transform);
	end
end
