Lua_ShopItemVo = class("Lua_ShopItemVo")
function Lua_ShopItemVo:ctor(id, config)
	self.id = id;
	self.config = config;
	if self.config == nil then
		self.config = getConfig("shop", id);
	end

	local shopTypeVo = DataCenter.shopUtil:getShopTypeVo(self.shopType);
	if shopTypeVo ~= nil then
		self.shopModel = shopTypeVo.model;
	end
	self:resetServerInfo();
end

function Lua_ShopItemVo:dispose()
	self:resetServerInfo();
end

function Lua_ShopItemVo:resetServerInfo()
	self.sBuyTime = 0; -- 已购买次数

	self.sItemAmount = nil; -- 道具数量
	self.sMoneyType = nil; -- 消耗货币类型
	self.sMoneyCount = nil; -- 消耗货币数量
	self.sDiscount = nil; -- 折扣
	self.index = nil;--商品索引
	self.leftTimes = self.limitTimes; -- 剩余次数
end

function Lua_ShopItemVo:updateServerInfo(serverInfo)
	if serverInfo == nil or serverInfo.id ~= self.id then
		self:resetServerInfo();
		return;
	end

	self.sBuyTime = serverInfo.buy_time;
	if self.limitTimes > 0 then
		self.leftTimes = self.limitTimes - serverInfo.buy_time;
	end

	-- 随机刷新类的商店有用到
	if self.shopModel == 3 then
		self.sItemAmount = serverInfo.rand_item_info.amount;
		self.sMoneyType = serverInfo.rand_item_info.money_id;
		self.sMoneyCount = serverInfo.rand_item_info.money;
		self.sDiscount = serverInfo.rand_item_info.discount;
		self.index = serverInfo.rand_item_info.index;
	end
end

-- 次日零点已购买数量重置
function Lua_ShopItemVo:resetBuyTime()
	self.sBuyTime = 0;
	if self.limitTimes > 0 then
		self.leftTimes = self.limitTimes;
	end
end

function Lua_ShopItemVo.Get:isOpened()
	if self.role_level ~= nil then
		local roleLv = DataCenter.roleData.level;
		if roleLv < self.role_level[1] or roleLv > self.role_level[2] then
			return false;
		end
	end

	if self.role_world_level ~= nil then
		local worldLv = DataCenter.roleData.worldLv;
		if worldLv < self.role_world_level[1] or worldLv > self.role_world_level[2] then
			return false;
		end
	end

	if self.func_id ~= nil then
		if DataCenter.funcOpenData:isOpened(self.func_id) == false then
			return false;
		end
	end

	return true;
end

function Lua_ShopItemVo.Get:itemVo()
	if self._itemVo == nil then
		self._itemVo = Lua_ToolVo.createToolVo(self.itemId, self.itemAmount);
	end
	return self._itemVo;
end

-- 限购次数
function Lua_ShopItemVo.Get:limitTimes()
	local limitTimes = 0;
	local limitData = Lua_ShopLimitData.getConfigDataById(self.id * 100 + DataCenter.vipData.level);

	-- if self.shopType == Lua_ShopData.STAR_SHOP then
	-- 	limitTimes = self.sell_times;
	-- 	if limitData ~= nil then
	-- 		limitTimes = self.sell_times + limitData.limit_times;
	-- 	end
	-- elseif self.shopType == Lua_ShopData.LEAGUE_SHOP then
	-- 	limitTimes = self:getlimitTimes(DataCenter.allianceData.iAllianceLevel);
	-- 	if limitData ~= nil then
	-- 		limitTimes = self:getlimitTimes(DataCenter.allianceData.iAllianceLevel) + limitData.limit_times;
	-- 	end
	-- else
		-- if self.limit_type == 1 then
		-- 	limitTimes = self.daily_times;
		-- 	if limitData ~= nil and limitData.limit_type == self.limit_type then
		-- 		limitTimes = self.daily_times + limitData.limit_times;
		-- 	end
		-- elseif self.limit_type == 2 then
		-- 	limitTimes = self.weekly_times;
		-- 	if limitData ~= nil and limitData.limit_type == self.limit_type then
		-- 		limitTimes = self.weekly_times + limitData.limit_times;
		-- 	end
		-- elseif self.limit_type == 3 then
		-- 	limitTimes = self.monthly_times;
		-- 	if limitData ~= nil and limitData.limit_type == self.limit_type then
		-- 		limitTimes = self.monthly_times + limitData.limit_times;
		-- 	end
		-- end
	-- end

	if self.sell_times > 0 then
		limitTimes = self.sell_times;
	else
		if self.limit_type == 1 then
			limitTimes = self.daily_times;
		elseif self.limit_type == 2 then
			limitTimes = self.weekly_times;
		elseif self.limit_type == 3 then
			limitTimes = self.monthly_times;	
		elseif self.limit_type == 4 then
			limitTimes = self.ever_times;	
		end
	end
	if limitData ~= nil and limitData.limit_type == self.limit_type then
		limitTimes = limitTimes + limitData.limit_times;
	end
	return limitTimes;
end

-- 根据等级获得限购次数
function Lua_ShopItemVo:getlimitTimes(level)
	local buyNum = self.config.buy_num;
	if buyNum ~= nil then
		for k,v in pairs(buyNum) do
			if v.lvl == level then
				return v.number;
			end
		end
	end
	return 0;
end

-- 是否售罄
function Lua_ShopItemVo.Get:isSoldOut()
	-- 社区（联盟）中若未满足商品解锁条件（加入比先前联盟等级低时，即便已售罄，也按未售罄处理，满足时再显示）
	if self.shopType == 5 and not self.isLeagueOpen then
		return false;
	end

	if self.limitTimes <= 0 then
		return false;
	end
	self.leftTimes = self.limitTimes - self.sBuyTime;
	if self.leftTimes <= 0 then
		return true;
	end
	return false;
end

-- 推荐商品>打折商品>普通商品>售罄商品
function Lua_ShopItemVo.Get:sortIndex()
	-- if self.isSoldOut then
	-- 	return 999;
	-- elseif self.recommend > 0 then
	-- 	return -1;
	-- elseif self.discount > 0 then
	-- 	return 0;
	-- end
	return self.sort;
end

function Lua_ShopItemVo.Set:highlightEff(effName)
	self._highlightEff = effName;
end

function Lua_ShopItemVo.Get:highlightEff()
	return self._highlightEff;
end

-------------------- 配置数据 --------------------

function Lua_ShopItemVo.Get:name()
	return self.config.name;
end

-- 商店类型
function Lua_ShopItemVo.Get:shopType()
	return self.config.shop_type;
end

-- 折扣
function Lua_ShopItemVo.Get:discount()
	if self.sDiscount ~= nil then
		return self.sDiscount;
	end

	if self.config.discount == nil then
		return 0;
	end
	return self.config.discount;
end

-- 推荐
function Lua_ShopItemVo.Get:recommend()
	if self.config.recommend == nil then
		return 0;
	end
	return self.config.recommend;
end

-- 排序
function Lua_ShopItemVo.Get:sort()
	if self.config.sort == nil then
		return 1;
	end
	return self.config.sort;
end

-- 玩家等级
function Lua_ShopItemVo.Get:role_level()
	return self.config.role_level;
end

-- 玩家世界等级
function Lua_ShopItemVo.Get:role_world_level()
	return self.config.role_world_level;
end

-- VIP等级
function Lua_ShopItemVo.Get:vip_level()
	if self.config.vip_level == nil then
		return 0;
	end
	return self.config.vip_level;
end

-- 对应系统功能开启Id
function Lua_ShopItemVo.Get:func_id()
	return self.config.func_id;
end

-- 购买的道具
function Lua_ShopItemVo.Get:itemId()
	return self.config.item_id;
end

-- 道具数量
function Lua_ShopItemVo.Get:itemAmount()
	if self.sItemAmount ~= nil then
		return self.sItemAmount;
	end
	if self.config.item_amount == nil then
		return 1;
	end
	return self.config.item_amount;
end

-- 货币类型
function Lua_ShopItemVo.Get:moneyType()
	if self.sMoneyType ~= nil then
		return self.sMoneyType;
	end
	return self.config.use_item_id;
end

-- 出售价格
function Lua_ShopItemVo.Get:moneyCount()
	if self.sMoneyCount ~= nil then
		return self.sMoneyCount;
	end
	return self.config.use_item_amount;
end

-- 限购类型 0/nil不限购 1每日 2每周 3每月 4永久
function Lua_ShopItemVo.Get:limit_type()
	return self.config.limit_type;
end

function Lua_ShopItemVo.Get:sell_times()
	if self.config.sell_times == nil then
		return 0;
	end
	return self.config.sell_times;
end

function Lua_ShopItemVo.Get:daily_times()
	if self.config.daily_times == nil then
		return 0;
	end
	return self.config.daily_times;
end

function Lua_ShopItemVo.Get:weekly_times()
	if self.config.weekly_times == nil then
		return 0;
	end
	return self.config.weekly_times;
end

function Lua_ShopItemVo.Get:monthly_times()
	if self.config.monthly_times == nil then
		return 0;
	end
	return self.config.monthly_times;
end

function Lua_ShopItemVo.Get:ever_times()
	if self.config.ever_times == nil then
		return 0;
	end
	return self.config.ever_times;
end

function Lua_ShopItemVo.Get:hero_camp()
	if self.config.hero_camp == nil then
		return nil;
	end
	return self.config.hero_camp;
end

--魔灵岛难度限制
function Lua_ShopItemVo.Get:limit_grade()
	if self.config.limit_grade == nil then
		return 0;
	end
	return self.config.limit_grade;
end

--巅峰战力段位
function Lua_ShopItemVo.Get:peak_duel_lvl_id()
	if self.config.peak_duel_lvl_id == nil then
		return 0;
	end
	return self.config.peak_duel_lvl_id;
end

-- 商品的社区（联盟）开启等级
function Lua_ShopItemVo.Get:communityLevel()
	if self.shopType == 5 then -- 5为社区（联盟）
		return self.config.community_level or 0;
	end
end

-- 社区（联盟）的等级是否满足商品解锁条件
function Lua_ShopItemVo.Get:isLeagueOpen()
	if self.shopType == 5 then -- 5为社区（联盟）
		return DataCenter.roleData.league_level >= self.communityLevel;
	end
end

function Lua_ShopItemVo.Get:leagueSort()
	if self.shopType == 5 then -- 5为社区（联盟）
		--	已解锁				未解锁			已售罄
		-- sortIndex       sortIndex+1000      sortIndex + 10000
		return self.isSoldOut and self.sortIndex + 10000 or ( self.isLeagueOpen and self.sortIndex  or self.sortIndex + 1000);
	end
end