Lua_ShopTypeVo = class("Lua_ShopTypeVo")
function Lua_ShopTypeVo:ctor(shopType, config)
	self.shopType = shopType;
	self.config = config;
	if self.config == nil then
		self.config = getConfig("shopType", shopType);
	end

	self:resetServerInfo();
end

function Lua_ShopTypeVo:resetServerInfo()
	self.freeTimes = 0; -- 已使用的免费刷新次数
	self.times = 0; -- 已经自费刷新的次数
	self.buyTimes = 0; -- 已购买的刷新次数
	self.nextTicks = 0; -- 下一次自动刷新时间

	self.totalTimes = self.refresh_limit;
	self.leftTimes = 0; -- 剩余可自费次数
	self.leftFreeTimes = 0; -- 剩余免费刷新次数
	self.leftBuyTimes = 0; -- 剩余可购买的刷新次数
end

function Lua_ShopTypeVo:updateServerInfo(serverInfo)
	self.freeTimes = serverInfo.free_times;
	self.times = serverInfo.times;
	self.buyTimes = serverInfo.buy_times;
	self.nextTicks = tonumber(tostring(serverInfo.next_ticks));

	if self.shopType == Lua_ShopData.SPIRIT_SHOP then -- 精灵商店根据vip额外增加刷新次数
		local vipVo = DataCenter.vipData:getVipVoByLevel(DataCenter.vipData.level);
		self.totalTimes = self.buyTimes + vipVo.store_refresh_times;
		self.leftTimes = vipVo.store_refresh_times - self.times;
	else
		self.totalTimes = self.buyTimes + self.refresh_limit;
		self.leftTimes = self.totalTimes - self.times;
	end

	if self.free_refresh > 0 then
		self.leftFreeTimes = self.free_refresh - self.freeTimes;
		if self.shopType == Lua_ShopData.SPIRIT_SHOP then
			local redPointState = (self.leftFreeTimes > 0 or self.leftTimes > 0) and self.bOpenSpritShop ~= true
			DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Main_SpiritShop, redPointState);
		end
	end
	if self.buy_limit > 0 then
		self.leftBuyTimes = self.buy_limit - self.buyTimes;
	end
end

function Lua_ShopTypeVo:checkOpened()
	if self.funcId == nil then
		return true;
	end
	return DataCenter.funcOpenData:isOpened(self.funcId);
end

-------------------- 配置数据 --------------------

-- 商店名
function Lua_ShopTypeVo.Get:name()
	return self.config.name;
end

-- 对应系统功能开启Id
function Lua_ShopTypeVo.Get:funcId()
	return self.config.funcId;
end

-- 是否可刷新
function Lua_ShopTypeVo.Get:refresh()
	return self.config.refresh;
end

-- 刷新上限
function Lua_ShopTypeVo.Get:refresh_limit()
	local limit = self.config.refresh_limit;
	if limit == nil then
		limit = 0;
	end
	return limit;
end

-- 免费刷新次数
function Lua_ShopTypeVo.Get:free_refresh()
	local free = self.config.free_refresh;
	if free == nil then
		free = 0;
	end
	return free;
end

-- 消耗货币
function Lua_ShopTypeVo.Get:cost_money()
	return self.config.cost_money;
end

-- 消耗数量
function Lua_ShopTypeVo.Get:cost()
	return self.config.cost;
end

function Lua_ShopTypeVo.Get:cost_arr()
	return self.config.cost_arr;
end

-- 购买次数上限
function Lua_ShopTypeVo.Get:buy_limit()
	if self.config.buy_limit == nil then
		return 0;
	end
	return self.config.buy_limit;
end

-- 购买花费货币
function Lua_ShopTypeVo.Get:buy_money()
	return self.config.buy_money;
end

-- 购买次数花费
function Lua_ShopTypeVo.Get:buy_cost()
	return self.config.buy_cost;
end

-- 显示的货币
function Lua_ShopTypeVo.Get:moneyType()
	return self.config.moneyType;
end

--是否显示倒计时
function Lua_ShopTypeVo.Get:showTimer()
	return self.config.show_timer;
end

-- 规则说明
function Lua_ShopTypeVo.Get:langId()
	return self.config.langId;
end

function Lua_ShopTypeVo.Get:panelStr()
    return self.config.panel_str;
end

function Lua_ShopTypeVo.Get:model()
    return self.config.model;
end