-- 古城秘藏 Data
checkRequire("Lua_TownTreasureCellVo");
Lua_TownTreasureData = class("Lua_TownTreasureData");

function Lua_TownTreasureData:ctor()
	self:dispose();
end

function Lua_TownTreasureData:dispose()
	self.curVersion = 0;
	self.isOpenAuto = false;
	self.isMoving = false;
	self.isTheWorld = false;
	self.doubleNum = 0;
end

function Lua_TownTreasureData:fromServer50207001(msg)
	if self.curVersion ~= msg.round then
		self.curVersion = msg.round;
		self.keyWord = DataCenter.townTreasureUtil:getKeyWord(self.curVersion);
		self.lastTick = msg.close_tick;
		self.loopTime = nil;

		self.cellDic = {};
		self.buildIdArr = Array();
		local config = getConfig("tower_secrete_cell");
		local vo;
		for k, v in pairs(config) do
			if v.version_id == self.curVersion then
				vo = Lua_TownTreasureCellVo.new(v.id, v);
				self.cellDic[v.id] = vo;

				if vo.bind_build_id ~= nil and self.buildIdArr:indexOf(vo.bind_build_id) == -1 then
					self.buildIdArr:add(vo.bind_build_id);
				end
			end
		end

		self.roadArr = Array();
		local startId = DataCenter.townTreasureUtil:getVersionInfo(self.curVersion, "start_id");
		local curId = 0;
		vo = self.cellDic[startId];
		while vo ~= nil and startId ~= curId do
			self.roadArr:add(vo.id);
			curId = vo.next_stage[1];
			vo = self.cellDic[curId];
		end
	end

	local cellInfo = msg.cells;
	local cellVo;
	for i = 1, #cellInfo do
		self:updateCellInfo(cellInfo[i]);
	end

	self.lastLoopNum = self.loopTime;
	self.buildNum = 0;

	self.closeTick = msg.close_tick; -- 活动结束时间戳
	self.curCellId = msg.role_pos; -- 主角棋格位置id
	self.diceTime = msg.use_times; -- 可投掷次数
	self.totalTime = msg.curr_times; -- 累计投掷次数
	self.loopTime = msg.loop_times; -- 圈数
	self.version = msg.round; -- 周期
	self.buyTime = msg.buy_times; -- 剩余购买次数
	self.nearCellVo = self:getBuildCellVo(msg.near_cell); -- 临近升级建筑棋格
	self:updateBuffInfo(msg.buff_maps); -- 增益效果状态

	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.TownTreasure_Login, self.diceTime > 0);

	if self.lastTick ~= self.closeTick or (self.lastLoopNum ~= nil and self.lastLoopNum > self.loopTime) then
		self.lastTick = self.closeTick;
		self.diceInfoArr = nil;
		Lua_MsgUtil.callFunc("TOWN_TREASURE_TIME_RESET");
	end

	Lua_MsgUtil.callFunc("TOWN_TREASURE_INFO_UPDATE");
end

function Lua_TownTreasureData:updateCellInfo(data)
	if data == nil then return end

	local vo = self.cellDic[data.cell_id];
	if vo ~= nil then
		if vo.bind_build_id ~= nil then
			vo = self.cellDic[vo.bind_build_id];
		end
		vo:fromServerInfo(data);
	end 
end

function Lua_TownTreasureData:getBuildCellVo(id)
	local vo = self.cellDic[id];
	if vo ~= nil then
		return self.cellDic[vo.bind_build_id];
	end
	return nil;
end


function Lua_TownTreasureData.Get:getBuildCellArr()
	local arr = Array();
	local vo;
	for i = 1, self.buildIdArr:getSize() do
		vo = self.cellDic[self.buildIdArr[i]];
		if vo.level > 0 then
			arr:add(vo);
		end
	end
	return arr;
end

function Lua_TownTreasureData:updateBuffInfo(data)
	self.buffArr = Array();
	local config;
	local vo;
	self.doubleNum = 0;
	for i = 1, #data do
		config = DataCenter.townTreasureUtil:eventCfg(data[i].key);
		if config ~= nil then
			vo = {id = config.id, buff_icon = config.buff_icon, describe = config.describe, count = data[i].value, index = i};
			self.buffArr:add(vo);
		end

		if data[i].key == 3007 then
			self.doubleNum = data[i].value; -- 记录双倍奖励次数
		elseif data[i].key == 3006 then
			self.buildNum = data[i].value; -- 记录建筑双倍升级
		end
	end
end

function Lua_TownTreasureData:getLoopRewardConfig(loopTime)
	local level = DataCenter.townTreasureUtil:getLoopRewardLv(self.curVersion, loopTime);
	return getConfig("tower_secrete_loop_reward", level, self.keyWord);
end

function Lua_TownTreasureData:fromServer50207002(msg)
	self.totalTime = msg.curr_times; -- 累计投掷次数

	local taskStateDic = {};
	local minId;
	for i = 1, #msg.task_info do
		taskStateDic[msg.task_info[i].key] = msg.task_info[i].value;
		if minId == nil or minId > msg.task_info[i].key then
			minId = msg.task_info[i].key;
		end
	end

	self.taskMinId = minId;
	self.rewardState = #msg.task_info > 0;
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.TownTreasure_Task, self.rewardState);

	local arr = Array();
	local config = getConfig("tower_secrete_task");
	local vo;
	local state;
	for k, v in pairs(config) do
		state = v.times;
		if v.times <= self.totalTime then
			if taskStateDic[v.times] ~= nil then
				state = state - 1000;
			else
				state = state + 1000;
			end
		end
		vo = {id = v.times, reward = v[self.keyWord], state = state};
		arr:add(vo);
	end
	arr:sort("state", false);
	Lua_MsgUtil.callFunc("TOWN_TREASURE_TASK_INFO", arr);
end

function Lua_TownTreasureData.Get:getTaskRewardShow()
	local data;
	if self.taskMinId == nil then
		data = DataCenter.townTreasureUtil:getNextTask(self.totalTime);
	else
		data = DataCenter.townTreasureUtil:getTaskConfig(self.taskMinId);
	end

	local itemInfo = data[self.keyWord][1];
	return Lua_ToolVo.createToolVo(itemInfo[1], itemInfo[2]);
end

function Lua_TownTreasureData:fromServer80207003(msg)
	local cellVo;
	local data;
	local idArr = Array();
	for i = 1, #msg.building_info do
		data = msg.building_info[i];
		cellVo = self:getBuildCellVo(data.cell_id);
		if cellVo ~= nil then
			cellVo.curTime = data.rand_times;
			cellVo.value = data.output;
			idArr:add(cellVo.id);
		end
	end
	Lua_MsgUtil.callFunc("TOWN_TREASURE_BUILDING_UPDATE", idArr);
end

function Lua_TownTreasureData:fromServer50207004(msg)
	if Lua_WindowUtil.isWindowShowing("Lua_TownTreasureWindow") == false then
		return;
	end

	self.diceInfoArr = Array();
	for i = 1, #msg.dice_state do
		self.diceInfoArr:add(msg.dice_state[i]);
	end

	local data = self.diceInfoArr:shift();
	self.diceTime = data.use_times; -- 可投掷次数
	self:updateBuffInfo(data.buff_maps);
	self.nearCellVo = self:getBuildCellVo(msg.near_cell); -- 临近升级建筑棋格
	self.totalTime = msg.curr_times;
	self.loopTime = msg.loop_times;

	self.diceNum = self.diceInfoArr[1].dice; -- 骰子点数/步数
	self.diceIdx = 0;

	self.isMoving = true;
	Lua_MsgUtil.callFunc("TOWN_TREASURE_DICE_ANI");
end

function Lua_TownTreasureData:setDiceInfo()
	if self.diceInfoArr == nil or self.diceInfoArr:getSize() <= 0 then
		self.isMoving = false;
		Lua_MsgUtil.callFunc("TOWN_TREASURE_PLAY_END");
		return;
	end

	local data = self.diceInfoArr:shift();
	self.diceNum = data.dice; -- 骰子点数/步数
	self.targetCellId = data.new_cell.cell_id;
	self.diceIdx = self.diceIdx + 1;

	-- 保存数据，最后一个格子时才更新
	self.saveDiceTime = data.use_times;
	self.saveBuffList = data.buff_maps;

	self.playCellInfo = {};
	self.playCellInfo[self.targetCellId] = data.new_cell;
	for i = 1, #data.ori_cell do
		self.playCellInfo[data.ori_cell[i].cell_id] = data.ori_cell[i];
	end
	Lua_MsgUtil.callFunc("TOWN_TREASURE_PLAY");
end

function Lua_TownTreasureData:refreshCellData(id)
	if id == self.targetCellId then
		self.diceTime = self.saveDiceTime; -- 可投掷次数
		self:updateBuffInfo(self.saveBuffList); -- 增益效果状态
		Lua_MsgUtil.callFunc("TOWN_TREASURE_INFO_UPDATE");
	end
	local data = self.playCellInfo[id];
	self:updateCellInfo(data);
end

function Lua_TownTreasureData:fromServer50207007(msg)
	self.diceTime = msg.curr_times; -- 剩余投掷次数
	self.buyTime = msg.buy_times; -- 剩余购买次数
	Lua_MsgUtil.callFunc("TOWN_TREASURE_DICE_INFO");
end

function Lua_TownTreasureData.Get:magnetRewardArr()
	local dic = {};
	local count;
	for k, v in pairs(self.cellDic) do
		if v.is_building == false and v.eventId ~= nil and v.eventId ~= 0 and v.reward_type ~= 3 then -- 道具
			if dic[v.itemId] == nil then
				dic[v.itemId] = 0;
			end
			dic[v.itemId] = dic[v.itemId] + Mathf.Ceil(v.value / 2);
		end
	end

	local arr = Array();
	for k, v in pairs(dic) do
		arr:add(Lua_ToolVo.createToolVo(k, v));
	end
	return arr;
end