Lua_FightHpBar = class("Lua_FightHpBar", Lua_DisplayObject);
function Lua_FightHpBar:ctor(p, barAtlas, bgSprite, barSprite, barWidth, barHeight)
	self.barAtlas = barAtlas;
	self.bgSprite = bgSprite;
	self.barSprite = barSprite;
	self.barWidth = barWidth == nil and 97 or barWidth;
	self.barHeight = barHeight == nil and 13 or barHeight;
	self:init();
	self:superCall("ctor", p);
end

function Lua_FightHpBar:buildShowItem()
	self:superCall("buildShowItem");

	self.showBar = UICreate.createProgressBar(self.content, self.barAtlas, self.bgSprite, self.barAtlas, self.barSprite, 0, 0, self.barWidth, self.barHeight);
	self.showBar.typeValue = 3;
	self.showBar.fillMethodValue = 0;
	self.showBar.fillOrigin = 0;
	self.showBar:setMaxValue(1);
	self.showBar:setValue(0);
end

function Lua_FightHpBar:dispose()
	self:endTimer();
	self.showBar = nil;
	self:superCall("dispose");
end

function Lua_FightHpBar:setBarOffset(offsetWidth, offsetHeight)
	self.showBar:setBarOffset(offsetWidth, offsetHeight);
end

-- 回血
function Lua_FightHpBar:createSecondBar(atlas, sprite)
	self.showBar:createSecondBar(atlas, sprite);
end

-- 扣血
function Lua_FightHpBar:createAnimationBar(atlas, sprite)
	self.showBar:createAnimationBar(atlas, sprite);
end

function Lua_FightHpBar:init()
	self.speedRate = 1;
	self.curValue1 = 0;
	self.curValue2 = 0;
	self.curValue3 = 0;
	self.lessTarget = 0;
	self.plusTarget = 0;
	self.ani_interval_value = 0;
	self.ani_wait_value = 0;

	self.aniInterval = Lua_CommonUtil.getCommonConfig("bar_ani_interval", 4, 0.2);
	self.aniWait = Lua_CommonUtil.getCommonConfig("bar_ani_wait", 4, 0.2);
	self.animation_time_1 = Lua_CommonUtil.getCommonConfig("bar_ani_time1", 4, 0.25);
	self.animation_time_2 = Lua_CommonUtil.getCommonConfig("bar_ani_time2", 4, 0.25);

	self.timerName = "Lua_FightHpBar";
end

function Lua_FightHpBar:setSpeedRate(value)
	self.speedRate = value;
end

function Lua_FightHpBar:setMaxValue(value)
	self.showBar:setMaxValue(value);
	self.showBar:setSecondMaxValue(value);
end

function Lua_FightHpBar:setValue(value)
	if self.curValue1 == 0 and self.bFirst ~= false then -- 初次赋值
		self.bFirst = false
		self:endTimer();
		self.curValue1 = value;
		self.curValue2 = value;
		self.curValue3 = value;
		self.showBar:setValue(self.curValue1);
		self.showBar:setSecondValue(self.curValue2);
		self.showBar:setAnimationValue(self.curValue3);
	else
		if self.bTiming ~= true then
			self.bTiming = true;
			self.ani_interval_value = self.aniInterval;
			self.less_start_time = getTime();
			Lua_CommonUtil.addTimer(self.timerName, 0.01, self, self.onTimer);
		else
			local t;
			local rate;

			if self.ani_interval_value > 0 then
				self.less_start_time = getTime();
				self.ani_interval_value = self.aniInterval;
			else
				t = (getTime() - self.curTime1) * self.speedRate;
				rate = t / self.animation_time_1;
				if rate >= 1 then
					self.curValue1 = self.lessTarget;
				else
					self.curValue1 = self.curValue1 + (self.lessTarget - self.curValue1) * rate;
				end
			end

			t = (getTime() - self.curTime2) * self.speedRate;
			rate = t / self.animation_time_1;
			if rate >= 1 then
				self.curValue2 = self.plusTarget;
			else
				self.curValue2 = self.curValue2 + (self.plusTarget - self.curValue2) * rate;
			end

			if self.curTime3 ~= nil then
				t = (getTime() - self.curTime3) * self.speedRate;
				rate = t / self.animation_time_2;
				if rate >= 1 then
					self.curValue3 = self.lessTarget;
				else
					self.curValue3 = self.curValue3 + (self.lessTarget - self.curValue3) * rate;
				end
			end
		end

		if value > self.curValue1 then
			self.less_start_time = getTime();
			self.ani_interval_value = self.aniWait;
			self.plusTarget = value;
		else
			self.plusTarget = 0;
		end

		if self.bTiming == true then
			self.ani_wait_value = self.aniWait;
			self.curTime3 = getTime();
		end

		self.lessTarget = value;
		self.curTime1 = getTime() + self.ani_interval_value;
		self.curTime2 = getTime();
	end
end

function Lua_FightHpBar:endTimer()
	if self.bTiming == true then
		self.bTiming = false;
		Lua_CommonUtil.removeTimer(self, self.onTimer);
	end
end

function Lua_FightHpBar:onTimer()
	if self.curValue1 == self.lessTarget and self.curValue3 == self.lessTarget then
		self:endTimer();
	end

	local dt;
	local t;
	local rate;
	local value;

	-- 红色扣血条
	if self.ani_wait_value > 0 then
		dt = getTime() - self.curTime3;
		self.ani_wait_value = self.ani_wait_value - dt;
		if self.ani_wait_value <= 0 then
			self.ani_wait_value = 0;
		end
		self.curTime3 = getTime();
	else
		t = (getTime() - self.curTime3) * self.speedRate;
		rate = t / self.animation_time_2;
		if rate >= 1 then
			rate = 1;
			self.curValue3 = self.lessTarget;
		end
		value = self.curValue3 + (self.lessTarget - self.curValue3) * rate;
		self.showBar:setAnimationValue(value);
	end

	-- 白色加血条
	if self.plusTarget > 0 then
		t = (getTime() - self.curTime2) * self.speedRate;
		rate = t / self.animation_time_1;
		if rate >= 1 then
			rate = 1;
			self.curValue2 = self.plusTarget;
		end
		value = self.curValue2 + (self.plusTarget - self.curValue2) * rate;
		self.showBar:setSecondValue(value);
	else
		self.showBar:setSecondValue(0);
	end

	-- 绿色血条
	if self.ani_interval_value > 0 then
		dt = getTime() - self.less_start_time;
		self.ani_interval_value = self.ani_interval_value - dt;
		if self.ani_interval_value <= 0 then
			self.ani_interval_value = 0;
		end
		self.less_start_time = getTime();
	else
		t = (getTime() - self.curTime1) * self.speedRate;
		rate = t / self.animation_time_1;
		if rate >= 1 then
			rate = 1;
			self.curValue1 = self.lessTarget;
		end
		value = self.curValue1 + (self.lessTarget - self.curValue1) * rate;
		self.showBar:setValue(value);
		self.curValue2 = value;
	end
end