Lua_FightShieldBar = class("Lua_FightShieldBar", Lua_DisplayObject);
function Lua_FightShieldBar:ctor(p, barAtlas, bgSprite, barSprite, barWidth, barHeight)
	self.barAtlas = barAtlas;
	self.bgSprite = bgSprite;
	self.barSprite = barSprite;
	self.barWidth = barWidth == nil and 97 or barWidth;
	self.barHeight = barHeight == nil and 13 or barHeight;
	self:init();
	self:superCall("ctor", p);
end

function Lua_FightShieldBar:buildShowItem()
	self:superCall("buildShowItem");
	self.showBar = UICreate.createProgressBar(self.content, self.barAtlas, self.bgSprite, self.barAtlas, self.barSprite, 0, 0, self.barWidth, self.barHeight);
	self.showBar:setMaxValue(1);
	self.showBar:setValue(0);
end

function Lua_FightShieldBar:dispose()
	self:endTimer();
	self.showBar = nil;
	self:superCall("dispose");
end

function Lua_FightShieldBar:init()
	self.speedRate = 1;
	self.curValue = 0;
	self.targetValue = -1;
	self.ani_interval_value = 0;
	self.ani_wait_value = 0;

	self.aniInterval = Lua_CommonUtil.getCommonConfig("bar_ani_interval", 4, 0.2);
	self.animation_time = Lua_CommonUtil.getCommonConfig("bar_ani_time1", 4, 0.25);

	self.timerName = "Lua_FightShieldBar";
end

function Lua_FightShieldBar:setSpeedRate(value)
	self.speedRate = value;
end

function Lua_FightShieldBar:setMaxValue(value)
	self.showBar:setMaxValue(value);
end

function Lua_FightShieldBar:setValue(value)
	if self.targetValue ~= value then
		if self.bTiming ~= true then
			self.bTiming = true;
			self.ani_interval_value = self.aniInterval;
			self.ani_start_time = getTime();
			Lua_CommonUtil.addTimer(self.timerName, 0.01, self, self.onTimer);
		else
			local t;
			local rate;

			if self.ani_interval_value > 0 then
				self.ani_start_time = getTime();
				self.ani_interval_value = self.aniInterval;
			else
				t = (getTime() - self.curTime) * self.speedRate;
				rate = t / self.animation_time;
				if rate >= 1 then
					self.curValue = self.targetValue;
				else
					self.curValue = self.curValue + (self.targetValue - self.curValue) * rate;
				end
			end
		end

		self.targetValue = value;
		self.curTime = getTime() + self.ani_interval_value;
	end
end

function Lua_FightShieldBar:endTimer()
	if self.bTiming == true then
		self.bTiming = false;
		Lua_CommonUtil.removeTimer(self, self.onTimer);
	end
end

function Lua_FightShieldBar:onTimer()
	if  self.curValue == self.targetValue and self.curValue3 == self.targetValue then
		self:endTimer();
	end

	local dt;
	local t;
	local rate;
	local value;

	if self.ani_interval_value > 0 then
		dt = getTime() - self.ani_start_time;
		self.ani_interval_value = self.ani_interval_value - dt;
		if self.ani_interval_value <= 0 then
			self.ani_interval_value = 0;
		end
		self.ani_start_time = getTime();
	else
		t = (getTime() - self.curTime) * self.speedRate;
		rate = t / self.animation_time;
		if rate >= 1 then
			rate = 1;
			self.curValue = self.targetValue;
		end
		value = self.curValue + (self.targetValue - self.curValue) * rate;
		self.showBar:setValue(value);
	end
end