-- 场景装饰物气泡
Lua_SceneOrnamentBubble = class("Lua_SceneOrnamentBubble", Lua_DisplayObject)
function Lua_SceneOrnamentBubble:ctor(p)
	self:superCall("ctor", p, "SceneOrnamentBubble_skin");
end

function Lua_SceneOrnamentBubble:buildShowItem()
	self:superCall("buildShowItem");
end

function Lua_SceneOrnamentBubble:addTimer()
	if not self._bAddTimer then
		self._bAddTimer = true;
		self.curFrameTime = getTime();
		self.playSpeed = 1;
		Lua_CommonUtil.addTimer("SceneOrnamentBubble_Timer", 0.02, self, self.onBubbleTimer);
	end
end

function Lua_SceneOrnamentBubble:clearTimer()
	if self._bAddTimer then
		self._bAddTimer = false;
		Lua_CommonUtil.removeTimer(self, self.onBubbleTimer);
	end
end

function Lua_SceneOrnamentBubble:removeTween()
	DataCenter.winTweenManager:removeWinTween(self.tweenName);
	self.tweenName = nil;
end

function Lua_SceneOrnamentBubble:setData(behaviorVo)
	local t = behaviorVo.bubbleTime;
	if t <= 0 then
		Lua_MsgUtil.callFunc("SceneOrnament_Bubble_Hide", behaviorVo.ornamentRoleId);
		return;
	end

	if self.behaviorVo == behaviorVo then
		return;
	end
	self.behaviorVo = behaviorVo;
	self:updatePos();

	-- 气泡动画
	local config = behaviorVo.qipaoCfg;
	local bubbleTween = config.bubbleTween;
	if not strIsNullOrEmpty(bubbleTween) then
		self.dialogPlayDelayTime = DataCenter.winTweenManager:getWinTweenCfgCountTime(bubbleTween);
		if self.tweenName ~= nil then
			DataCenter.winTweenManager:removeWinTween(self.tweenName);
		end
		self.tweenName = DataCenter.winTweenManager:createWinTween(bubbleTween, self.content.transform);
	else
		self.dialogPlayDelayTime = 0;
	end

	-- 气泡内容
	local langId = nil;
	local bubbleContentLangIds = config.bubbleContentLangIds;
	if bubbleContentLangIds ~= nil and #bubbleContentLangIds > 0 then
		local _idx = DataCenter.sceneOrnamentsManager.ornamentBubbleIdx;
		local len = #bubbleContentLangIds;
		local idx = Mathf.Random(1, len);
		while idx == _idx do
			idx = Mathf.Random(1, len);
		end
		if idx == nil or idx <= 0 then idx = 1; end
		DataCenter.sceneOrnamentsManager.ornamentBubbleIdx = idx;
		langId = bubbleContentLangIds[idx];
	end
	if langId ~= nil then
		self.contentTxt:setLangValue(langId);
		self.contentTxt:resetSize();
		self.contentBg:SetDimensions(190, self.contentTxt.height + 26);
	else
		self.contentTxt.text = nil;
	end

	self.dialogPlayTime = config.dialogPlayTime == nil and 1 or config.dialogPlayTime / 1000
	self.dialogPlayTime = self.dialogPlayTime - self.dialogPlayDelayTime;
	if self.dialogPlayTime <= 0 then
		self.contentTxtLen = -1;
		self.dialogPlayedTime = -1;
		self.contentTxt:setTextShowIndex(-1);
	else -- 开始逐字播放
		self.dialogPlayedTime = 0;
		self.contentTxtLen = self.contentTxt.showTextLen;
		self.contentTxt:setTextShowIndex(0);
	end

	self:addTimer();
end

function Lua_SceneOrnamentBubble:onBubbleTimer()
	local t = self.behaviorVo.bubbleTime;
	if t <= 0 then
		self:clearTimer();
		self:removeTween();
		Lua_MsgUtil.callFunc("SceneOrnament_Bubble_Hide", self.behaviorVo.ornamentRoleId);
		return;
	end
	self:updatePos();

	t = getTime();
	local dt = (t - self.curFrameTime); -- 等比加速
	local scaleDelayTime = dt * self.playSpeed;
	self.curFrameTime = t;

	if self.dialogPlayDelayTime > 0 then
		self.dialogPlayDelayTime = self.dialogPlayDelayTime - scaleDelayTime;
		if self.dialogPlayDelayTime <= 0 then
			scaleDelayTime = -self.dialogPlayDelayTime;
			self.dialogPlayDelayTime = 0;
		else
			return;
		end
	end

	if self.dialogPlayedTime >= 0 then
		self.dialogPlayedTime = self.dialogPlayedTime + scaleDelayTime;
		local rate = self.dialogPlayedTime / self.dialogPlayTime;
		if rate >= 1 then
			self.dialogPlayedTime = -1;
			self.contentTxt:setTextShowIndex(self.contentTxtLen);
		else
			local num = Mathf.Floor(rate * self.contentTxtLen);
			self.contentTxt:setTextShowIndex(num);
		end
	end
end

-- 更新位置
function Lua_SceneOrnamentBubble:updatePos()
	if strIsNullOrEmpty(self.behaviorVo.sceneModelName) then -- 非场景对象
		local thing = DataCenter.thingData:getThingPlayer(self.behaviorVo.ornamentSceneId, 0, self.behaviorVo.ornamentRoleId);
		if thing ~= nil then
			local bubbleOffset = self.behaviorVo.qipaoCfg.bubbleOffset;
			local bubbleOffsetX = (bubbleOffset ~= nil and #bubbleOffset > 0) and bubbleOffset[1] or 0;
			local bubbleOffsetY = (bubbleOffset ~= nil and #bubbleOffset > 1) and bubbleOffset[2] or 0;
			local bubbleOffsetZ = (bubbleOffset ~= nil and #bubbleOffset > 2) and bubbleOffset[3] or 0;
			LuaUtil.setUIObjPos(self.parent, self.content, thing.data.px * 0.01, thing.data.py * 0.01, thing.data.pz * 0.01, 
				bubbleOffsetX, bubbleOffsetY, bubbleOffsetZ);
		end
	else
		local bubbleHangRoot = self.behaviorVo.qipaoCfg.bubbleHangRoot;
		if bubbleHangRoot == nil then
			bubbleHangRoot = self.behaviorVo.sceneModelName;
		end
		if bubbleHangRoot ~= nil then
			local bubbleOffset = self.behaviorVo.qipaoCfg.bubbleOffset;
			local bubbleOffsetX = (bubbleOffset ~= nil and #bubbleOffset > 0) and bubbleOffset[1] or 0;
			local bubbleOffsetY = (bubbleOffset ~= nil and #bubbleOffset > 1) and bubbleOffset[2] or 0;
			local bubbleOffsetZ = (bubbleOffset ~= nil and #bubbleOffset > 2) and bubbleOffset[3] or 0;
			LuaUtil.setUIObjPos(self.parent, self.content, self.behaviorVo.ornamentSceneId, bubbleHangRoot,
				bubbleOffsetX, bubbleOffsetY, bubbleOffsetZ);
		end
	end
end

function Lua_SceneOrnamentBubble:dispose()
	self:clearTimer();
	self:removeTween();
	self.behaviorVo = nil;

	self:superCall("dispose");
end