-- 性能设置
Lua_DeviceQuality = class("Lua_DeviceQuality")
function Lua_DeviceQuality:ctor()
	self:superCall("ctor");
	
	self.deviceQuality = DeviceQuality.instance;
	self.supportSceneDepthTex = false;-- 场景默认关闭
	self.supportSceneOpaqueTex = false;-- 场景默认关闭
	self.allowHDR = false;-- 场景默认关闭
	self.supportSceneShadow = false;-- 场景默认关闭
end

function Lua_DeviceQuality:dispose()
	if not self.supportSceneHDR then
		self.supportSceneHDR = false;
		self:updateHDR();
	end
	if not self.supportSceneShadow then
		self.supportSceneShadow = false;
		self:updateRenderShadow();
	end
	if not self.supportSceneDepthTex then
		self.supportSceneDepthTex = false;
		self:updateDepthTexture();
	end
	if not self.supportSceneOpaqueTex then
		self.supportSceneOpaqueTex = false;
		self:updateOpaqueTexture();
	end
end

-- 根据场景变化调整部分性能设置开关
function Lua_DeviceQuality:updateSceneQuality()
	local curShowSceneId = Scene3DManager.instance.curShowSceneId;
	if curShowSceneId ~= nil then
		local scenecfg = Lua_SceneConfigData.getSceneConfig(curShowSceneId);
		if scenecfg ~= nil and scenecfg.sceneResId ~= nil then
			local resCfg = getConfig("tb_scene_res", scenecfg.sceneResId);
			if resCfg ~= nil then
				local depthTexture = resCfg.depthTexture; -- depthTexture
				local supportDepthTex = depthTexture ~= nil and depthTexture == 1;
				local opaqueTexture = resCfg.opaqueTexture; -- opaqueTexture
				local supportOpaqueTex = opaqueTexture ~= nil and opaqueTexture == 1;
				local allowHDR = resCfg.allowHDR; -- allowHDR
				local supportHDR = allowHDR ~= nil and allowHDR == 1;
				local bCastShadow = resCfg.bCastShadow; -- bCastShadow
				local supportShadow = bCastShadow ~= nil and bCastShadow == 1;
				if self.supportSceneHDR ~= supportHDR then
					self.supportSceneHDR = supportHDR;
					self:updateHDR();
				end
				if self.supportSceneShadow ~= supportShadow then
					self.supportSceneShadow = supportShadow;
					self:updateRenderShadow();
				end
				if self.supportSceneDepthTex ~= supportDepthTex then
					self.supportSceneDepthTex = supportDepthTex;
					self:updateDepthTexture();
				end
				if self.supportSceneOpaqueTex ~= supportOpaqueTex then
					self.supportSceneOpaqueTex = supportOpaqueTex;
					self:updateOpaqueTexture();
				end
				return;
			end
		end
	end

	if not self.supportSceneHDR then
		self.supportSceneHDR = false;
		self:updateHDR();
	end
	if not self.supportSceneShadow then
		self.supportSceneShadow = false;
		self:updateRenderShadow();
	end
	if not self.supportSceneDepthTex then
		self.supportSceneDepthTex = false;
		self:updateDepthTexture();
	end
	if not self.supportSceneOpaqueTex then
		self.supportSceneOpaqueTex = false;
		self:updateOpaqueTexture();
	end
end

-- 性能
function Lua_DeviceQuality:updateQuality()
    local qualityLv = PlayerPrefsUtil.instance:GetUnInt(Lua_SetUpDefine.quality_lv, -1);
    if qualityLv == -1 then qualityLv = self.suggestQualityLv; end
    self.deviceQuality.qualityLevel = qualityLv;
    self:updateRenderScale(qualityLv);-- 分辨率
    self:updateMSAA(qualityLv);-- 抗锯齿
    self:updateHDR(qualityLv); -- hdr
    self:updatePostprocessing(qualityLv); -- 后处理
    self:updateRenderShadow(qualityLv); -- 阴影
    self:updateSkinWeights(qualityLv); -- 骨骼权重
end

function Lua_DeviceQuality.Get:qualityLevel()
	return self.deviceQuality.qualityLevel;
end

function Lua_DeviceQuality.Set:qualityLevel(val)
	if self.deviceQuality.qualityLevel ~= val then
		self.deviceQuality.qualityLevel = val;
		self:updateRenderScale(val);-- 分辨率
	    self:updateMSAA(val);-- 抗锯齿
	    self:updateHDR(val); -- hdr
	    self:updatePostprocessing(val); -- 后处理
	    self:updateRenderShadow(val); -- 阴影
	    self:updateSkinWeights(val); -- 骨骼权重
	end
end

function Lua_DeviceQuality.Get:renderScale()
	return self.deviceQuality.renderScale;
end

function Lua_DeviceQuality.Get:msaaCount()
	return self.deviceQuality.msaaSampleCount;
end

-- 分辨率
function Lua_DeviceQuality:updateRenderScale(qualityLv)
	local renderScaleIdx = PlayerPrefsUtil.instance:GetUnInt(Lua_SetUpDefine.quality_renderscale, -1);
	if renderScaleIdx == -1 then
		if qualityLv == nil then qualityLv = self.qualityLevel; end
		self.deviceQuality.renderScale = self:getRenderScaleByQualityLv(qualityLv);
	else
		self.deviceQuality.renderScale = self:getRenderScaleByIdx(renderScaleIdx);
	end
end

-- 抗锯齿
function Lua_DeviceQuality:updateMSAA(qualityLv)
	local msaa = PlayerPrefsUtil.instance:GetUnInt(Lua_SetUpDefine.quality_msaa, -1);
	if msaa == -1 then 
		if qualityLv == nil then qualityLv = self.qualityLevel; end
		msaa = self:getMsaaCountByIdx(qualityLv); 
	end
    self.deviceQuality.msaaSampleCount = msaa;
end

-- hdr
function Lua_DeviceQuality:updateHDR(qualityLv)
	if not self.supportSceneHDR then -- 场景不需要支持则直接关闭
		self.deviceQuality.supportsHDR = false;
		return;
	end

	if qualityLv == nil then qualityLv = self.qualityLevel; end
    local supportsHDR = self:getSupportHDRByQualityLv(qualityLv);
    self.deviceQuality.supportsHDR = supportsHDR;
end

-- 后处理
function Lua_DeviceQuality:updatePostprocessing(qualityLv)
	if qualityLv == nil then qualityLv = self.qualityLevel; end
    local bEnablePostProcessing = self:getSupportPostProcessingByQualityLv(qualityLv);
    self.deviceQuality.bEnablePostProcessing = bEnablePostProcessing;
end

-- 阴影
function Lua_DeviceQuality:updateRenderShadow(qualityLv)
	if not self.supportSceneShadow then -- 场景不需要支持则直接关闭
		self.deviceQuality.bRenderShadows = false;
		return;
	end

	if qualityLv == nil then qualityLv = self.qualityLevel; end
    local bRenderShadows = self:getSupportRenderShadowByQualityLv(qualityLv);
    self.deviceQuality.bRenderShadows = bRenderShadows;
end

-- 骨骼权重
function Lua_DeviceQuality:updateSkinWeights(qualityLv)
	if qualityLv == nil then qualityLv = self.qualityLevel; end
    local skinWeights = self:getSkinWeightsByQualityLv(qualityLv);
    self.deviceQuality.skinWeights = skinWeights;
end

-- DepthTexture
function Lua_DeviceQuality:updateDepthTexture()
    self.deviceQuality.supportsCameraDepthTexture = self.supportSceneDepthTex;
end

-- OpaqueTexture
function Lua_DeviceQuality:updateOpaqueTexture()
    self.deviceQuality.supportsCameraOpaqueTexture = self.supportSceneOpaqueTex;
end

-- 根据cpu推荐的性能等级
function Lua_DeviceQuality.Get:suggestQualityLv()
	if self._suggestQualityLv == nil then
	    local gpu = ServerConfig.graphicsDeviceName;--gpu
	    print("GUP型号 = ", gpu);
	    if strIsNullOrEmpty(gpu) then
	    	self._suggestQualityLv = 1;
	    else
	        local defaultCfg = nil;
		    local list = getConfig("set_quality");
		    if list ~= nil then
		        for i = 1, #list do
		            if list[i].gpu ~= nil then
		            	if list[i].gpu == gpu then
			                self._suggestQualityLv = list[i].quality;
			                print("getConfigQualityLevel", list[i].quality)
			                break;
			            elseif list[i].gpu == "Default" then
			            	defaultCfg = list[i];
			            end
		            end
		        end
		    end

		    if self._suggestQualityLv == nil and defaultCfg ~= nil then
			    self._suggestQualityLv = defaultCfg.quality;
			end
	    end
	    
	    if self._suggestQualityLv == nil then
	    	self._suggestQualityLv = 1;
	    end
	end
    return self._suggestQualityLv;
end

-- 0 省电
-- 1 流畅
-- 2 高性能
function Lua_DeviceQuality:getQualityCfgByIdx(qualityLv)
	if qualityLv == nil then qualityLv = 1; end
	if qualityLv > 2 then qualityLv = 2; end
	if qualityLv < 0 then qualityLv = 0; end
	return getConfig("quality_config", qualityLv);
end

-- 根据性能分级获取msaa
function Lua_DeviceQuality:getMsaaCountByIdx(qualityLv)
	local cfg = self:getQualityCfgByIdx(qualityLv);
	if cfg ~= nil and cfg.msaa ~= nil then 
		local msaa = cfg.msaa;
		if msaa <= 0 then
			return 0;
		elseif msaa < 4 then
			return 2;
		elseif msaa < 8 then
			return 4;
		else
			return 8;
		end
	end
	return 0;
end

-- 根据性能分级获取renderscale
function Lua_DeviceQuality:getRenderScaleByQualityLv(qualityLv)
	local idx = self:getRenderScaleIdxByQualityLv(qualityLv);
	if idx ~= nil then
		return self:getRenderScaleByIdx(idx);
	end
	return 1;
end

-- 根据性能分级获取renderscale对应索引
function Lua_DeviceQuality:getRenderScaleIdxByQualityLv(qualityLv)
	local cfg = self:getQualityCfgByIdx(qualityLv);
	if cfg ~= nil and cfg.renderscale ~= nil then 
		return cfg.renderscale;
	end
	return 1;
end

-- 根据索引获取对应分辨率
function Lua_DeviceQuality:getRenderScaleByIdx(idx)
	local tab = getConfig("personal_data_const", "renderscale_options", "json_value")
	if tab ~= nil and #tab >= idx then
		return tab[idx];
	end
	return 1;
end

-- 根据性能分级获取是否支持HDR
function Lua_DeviceQuality:getSupportHDRByQualityLv(qualityLv)
	local cfg = self:getQualityCfgByIdx(qualityLv);
	if cfg ~= nil and cfg.hdr ~= nil then 
		return cfg.hdr == 1;
	end
	return false;
end

-- 根据性能分级获取是否支持后处理
function Lua_DeviceQuality:getSupportPostProcessingByQualityLv(qualityLv)
	local cfg = self:getQualityCfgByIdx(qualityLv);
	if cfg ~= nil and cfg.postprocessing ~= nil then 
		return cfg.postprocessing == 1;
	end
	return false;
end

-- 根据性能分级获取是否支持实时阴影
function Lua_DeviceQuality:getSupportRenderShadowByQualityLv(qualityLv)
	local cfg = self:getQualityCfgByIdx(qualityLv);
	if cfg ~= nil and cfg.rendershadow ~= nil then 
		return cfg.rendershadow == 1;
	end
	return false;
end

-- 根据性能分级获取骨骼动画权重
function Lua_DeviceQuality:getSkinWeightsByQualityLv(qualityLv)
	local cfg = self:getQualityCfgByIdx(qualityLv);
	if cfg ~= nil and cfg.skinWeights ~= nil then 
		local skinWeights = cfg.skinWeights;
		if skinWeights <= 1 then
			return 1;
		elseif skinWeights < 4 then
			return 2;
		elseif skinWeights < 255 then
			return 4;
		else
			return 255;
		end
	end
	return 1;
end